@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowCarbon.cpp,v 1.7 2003/07/09 16:01:27 ton Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.9 2003/11/23 23:04:35 ton Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowCarbon.cpp,v 1.7 2003/07/09 16:01:27 ton Exp $
+ * $Id: GHOST_WindowCarbon.cpp,v 1.9 2003/11/23 23:04:35 ton Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -42,7 +42,6 @@
 #endif
 
 #include "GHOST_WindowCarbon.h"
-
 #include "GHOST_Debug.h"
 
 AGLContext GHOST_WindowCarbon::s_firstaglCtx = NULL;
@@ -67,6 +66,8 @@ AGL_DEPTH_SIZE,		16,
 AGL_NONE,
 };
 
+WindowRef ugly_hack=NULL;
+
 GHOST_WindowCarbon::GHOST_WindowCarbon(
 	const STR_String& title,
 	GHOST_TInt32 left,
@@ -106,6 +107,13 @@ GHOST_WindowCarbon::GHOST_WindowCarbon(
             updateDrawingContext();
             activateDrawingContext();
         }
+		if(ugly_hack==NULL) {
+			ugly_hack= m_windowRef;
+			// when started from commandline, window remains in the back... also for play anim
+			ProcessSerialNumber psn;
+			GetCurrentProcess(&psn);
+			SetFrontProcess(&psn);
+		}
     }
     else {
     /*
@@ -133,8 +141,10 @@ GHOST_WindowCarbon::~GHOST_WindowCarbon()
 {
 	if (m_customCursor) delete m_customCursor;
 
-    //GHOST_PRINT("GHOST_WindowCarbon::~GHOST_WindowCarbon(): removing drawing context\n");
-	setDrawingContextType(GHOST_kDrawingContextTypeNone);
+	if(ugly_hack==m_windowRef) ugly_hack= NULL;
+	
+	// printf("GHOST_WindowCarbon::~GHOST_WindowCarbon(): removing drawing context\n");
+	if(ugly_hack==NULL) setDrawingContextType(GHOST_kDrawingContextTypeNone);
     if (m_windowRef) {
         ::DisposeWindow(m_windowRef);
 		m_windowRef = 0;
@@ -333,6 +343,7 @@ GHOST_TSuccess GHOST_WindowCarbon::setOrder(GHOST_TWindowOrder order)
 		::SelectWindow(m_windowRef);
     }
     else {
+		/* doesnt work if you do this with a mouseclick */
         ::SendBehind(m_windowRef, nil);
     }
     return GHOST_kSuccess;
