@@ -1,5 +1,5 @@
 /**
- * $Id: GHOST_WindowWin32.cpp,v 1.5 2003/07/05 11:15:49 ton Exp $
+ * $Id: GHOST_WindowWin32.cpp,v 1.7 2003/10/27 00:03:18 rwenzlaff Exp $
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
  * This program is free software; you can redistribute it and/or
@@ -31,7 +31,7 @@
 
 /**
 
- * $Id: GHOST_WindowWin32.cpp,v 1.5 2003/07/05 11:15:49 ton Exp $
+ * $Id: GHOST_WindowWin32.cpp,v 1.7 2003/10/27 00:03:18 rwenzlaff Exp $
  * Copyright (C) 2001 NaN Technologies B.V.
  * @author	Maarten Gribnau
  * @date	May 10, 2001
@@ -506,27 +506,28 @@ void GHOST_WindowWin32::loadCursor(bool visible, GHOST_TStandardCursor cursor) c
 		bool success = true;
 		LPCSTR id;
 		switch (cursor) {
-			case GHOST_kStandardCursorDefault:				id = IDC_ARROW;
-			case GHOST_kStandardCursorRightArrow:			id = IDC_ARROW;		break;
-			case GHOST_kStandardCursorLeftArrow:			id = IDC_ARROW;		break;
-			case GHOST_kStandardCursorInfo:					id = IDC_SIZEALL;	break;	// Four-pointed arrow pointing north, south, east, and west
-			case GHOST_kStandardCursorDestroy:				id = IDC_NO;		break;	// Slashed circle
-			case GHOST_kStandardCursorHelp:					id = IDC_HELP;		break;	// Arrow and question mark
-			case GHOST_kStandardCursorCycle:				id = IDC_NO;		break;	// Slashed circle
-			case GHOST_kStandardCursorSpray:				id = IDC_SIZEALL;	break;	// Four-pointed arrow pointing north, south, east, and west
-			case GHOST_kStandardCursorWait:					id = IDC_WAIT;		break;	// Hourglass
-			case GHOST_kStandardCursorText:					id = IDC_IBEAM;		break;	// I-beam
-			case GHOST_kStandardCursorCrosshair:			id = IDC_CROSS;		break;	// Crosshair
-			case GHOST_kStandardCursorUpDown:				id = IDC_SIZENS;	break;	// Double-pointed arrow pointing north and south
-			case GHOST_kStandardCursorLeftRight:			id = IDC_SIZEWE;	break;	// Double-pointed arrow pointing west and east
-			case GHOST_kStandardCursorTopSide:				id = IDC_UPARROW;	break;	// Vertical arrow
-			case GHOST_kStandardCursorBottomSide:			id = IDC_SIZENS;	break;
-			case GHOST_kStandardCursorLeftSide:				id = IDC_SIZEWE;	break;
-			case GHOST_kStandardCursorTopLeftCorner:		id = IDC_SIZENWSE;	break;
-			case GHOST_kStandardCursorTopRightCorner:		id = IDC_SIZENESW;	break;
+			case GHOST_kStandardCursorDefault:		id = IDC_ARROW; 	break;
+			case GHOST_kStandardCursorRightArrow:		id = IDC_ARROW;		break;
+			case GHOST_kStandardCursorLeftArrow:		id = IDC_ARROW;		break;
+			case GHOST_kStandardCursorInfo:			id = IDC_SIZEALL;	break;	// Four-pointed arrow pointing north, south, east, and west
+			case GHOST_kStandardCursorDestroy:		id = IDC_NO;		break;	// Slashed circle
+			case GHOST_kStandardCursorHelp:			id = IDC_HELP;		break;	// Arrow and question mark
+			case GHOST_kStandardCursorCycle:		id = IDC_NO;		break;	// Slashed circle
+			case GHOST_kStandardCursorSpray:		id = IDC_SIZEALL;	break;	// Four-pointed arrow pointing north, south, east, and west
+			case GHOST_kStandardCursorWait:			id = IDC_WAIT;		break;	// Hourglass
+			case GHOST_kStandardCursorText:			id = IDC_IBEAM;		break;	// I-beam
+			case GHOST_kStandardCursorCrosshair:		id = IDC_CROSS;		break;	// Crosshair
+			case GHOST_kStandardCursorUpDown:		id = IDC_SIZENS;	break;	// Double-pointed arrow pointing north and south
+			case GHOST_kStandardCursorLeftRight:		id = IDC_SIZEWE;	break;	// Double-pointed arrow pointing west and east
+			case GHOST_kStandardCursorTopSide:		id = IDC_UPARROW;	break;	// Vertical arrow
+			case GHOST_kStandardCursorBottomSide:		id = IDC_SIZENS;	break;
+			case GHOST_kStandardCursorLeftSide:		id = IDC_SIZEWE;	break;
+			case GHOST_kStandardCursorTopLeftCorner:	id = IDC_SIZENWSE;	break;
+			case GHOST_kStandardCursorTopRightCorner:	id = IDC_SIZENESW;	break;
 			case GHOST_kStandardCursorBottomRightCorner:	id = IDC_SIZENWSE;	break;
-			case GHOST_kStandardCursorBottomLeftCorner:		id = IDC_SIZENESW;	break;
-		default:
+			case GHOST_kStandardCursorBottomLeftCorner:	id = IDC_SIZENESW;	break;
+			case GHOST_kStandardCursorPencil:		id = IDC_ARROW; 	break;
+			default:
 			success = false;
 		}
 		
