@@ -5,7 +5,7 @@
  * any data in Blender. This block is not serialized, but built anew
  * for every fresh Blender run.
  *
- * $Id: BKE_global.h,v 1.8 2003/07/20 20:25:28 ton Exp $ 
+ * $Id: BKE_global.h,v 1.9 2003/10/19 20:52:33 rwenzlaff Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -62,6 +62,17 @@ struct Object;
 struct bSoundListener;
 struct BMF_Font;
 
+#define UNDO_MAX	64
+#define UNDO_EDIT_MAX 	64
+
+typedef struct UndoBufferEdit {
+
+	void *datablock;
+	char name[64];
+
+} UndoBufferEdit;
+
+
 typedef struct Global {
 
     /* active pointers */
@@ -143,7 +154,15 @@ typedef struct Global {
     short special1, special2;
 
     int flags;
-    
+
+    /* editmode undo - written by intrr, ported by Det. Thorn  */
+    struct UndoBufferEdit undo_edit[UNDO_EDIT_MAX+1];/* need one more for undoing first undo */
+    int undo_edit_level;			/* index of last undo buffer created */
+    int undo_edit_highest;			/* index of highest undo buffer in use */
+    void *undo_last_data;			/* pointer to last datablock edited */
+    void (*undo_clear)(void);		/* pointer to function to free the undo data */
+
+        
 } Global;
 
 /* **************** GLOBAL ********************* */
