@@ -1,7 +1,7 @@
 /**
  * blenlib/BKE_mesh.h (mar-2001 nzc)
  *	
- * $Id: BKE_mesh.h,v 1.5 2002/12/27 13:10:09 mein Exp $ 
+ * $Id: BKE_mesh.h,v 1.7 2003/03/24 16:46:45 zuster Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -109,6 +109,12 @@ int rendermesh_uses_displist(struct Mesh *me);
 float get_mvert_weight (struct Object *ob, int vert, int defgroup);
 int update_realtime_texture(struct TFace *tface, double time);
 
+	/** Generate the mesh vertex normals by averaging over connected faces.
+	 *
+	 * @param me The mesh to update.
+	 */
+void mesh_calculate_vertex_normals	(struct Mesh *me);
+
 #ifdef __cplusplus
 }
 #endif
