@@ -1,5 +1,5 @@
 /**
- * $Id: armature.c,v 1.3 2002/11/25 12:01:52 mein Exp $
+ * $Id: armature.c,v 1.5 2003/04/24 23:13:58 hos Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -73,7 +73,6 @@
 /*	Function prototypes */
 
 static void apply_pose_bonechildren (Bone* bone, bPose* pose, int doit);
-static float dist_to_bone (float vec[3], float b1[3], float b2[3]);
 static Bone *get_named_bone_bonechildren (Bone *bone, const char *name);
 static Bone *get_indexed_bone_bonechildren (Bone *bone, int *index);
 /*void make_bone_parent_matrix (Bone* bone);*/
@@ -568,7 +567,7 @@ static int verify_boneptr_children (Bone *cBone, Bone *tBone)
 }
 
 
-static float dist_to_bone (float vec[3], float b1[3], float b2[3])
+float dist_to_bone (float vec[3], float b1[3], float b2[3])
 {
 /*  	float dist=0; */
 	float bdelta[3];
@@ -888,11 +887,27 @@ void make_boneMatrixvr (float outmatrix[][4],float delta[3], float roll)
 
 	/*	Find Axis & Amount for bone matrix*/
 	Crossf (axis,target,nor);
-	Normalise (axis);
-	theta=(float) acos (Inpf (target,nor));
 
-	/*	Make Bone matrix*/
-	VecRotToMat3(axis, theta, bMatrix);
+	if (Inpf(axis,axis) > 0.0000000000001) {
+		/* if nor is *not* a multiple of target ... */
+		Normalise (axis);
+		theta=(float) acos (Inpf (target,nor));
+
+		/*	Make Bone matrix*/
+		VecRotToMat3(axis, theta, bMatrix);
+	}
+	else {
+		/* if nor is a multiple of target ... */
+		float updown;
+
+		/* point same direction, or opposite? */
+		updown = ( Inpf (target,nor) > 0 ) ? 1.0 : -1.0;
+
+		/* I think this should work ... */
+		bMatrix[0][0]=updown; bMatrix[0][1]=0.0;    bMatrix[0][2]=0.0;
+		bMatrix[1][0]=0.0;    bMatrix[1][1]=updown; bMatrix[1][2]=0.0;
+		bMatrix[2][0]=0.0;    bMatrix[2][1]=0.0;    bMatrix[2][2]=1.0;
+	}
 
 	/*	Make Roll matrix*/
 	VecRotToMat3(nor, roll, rMatrix);
