@@ -1,9 +1,9 @@
 
 /*  blender.c   jan 94     MIXED MODEL
  * 
- * algemene hulp funkties en data
+ * common help functions and data
  * 
- * $Id: blender.c,v 1.5 2003/02/10 14:13:09 phase Exp $
+ * $Id: blender.c,v 1.11 2003/11/06 14:13:49 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -93,7 +93,7 @@ UserDef U;
 char versionstr[48]= "";
 
 /* ************************************************ */
-/* pushpop faciliteit: om tijdelijk data te bewaren */
+/* pushpop facility: to store data temporally, FIFO! */
 
 ListBase ppmain={0, 0};
 
@@ -162,7 +162,7 @@ void pushpop_test()
 
 
 
-/* ********** vrijgeven ********** */
+/* ********** free ********** */
 
 void free_blender(void)
 {
@@ -172,7 +172,7 @@ void free_blender(void)
 	IMB_freeImBufdata();		/* imbuf lib */
 }
 
-void duplicatelist(ListBase *list1, ListBase *list2)  /* kopie van 2 naar 1 */
+void duplicatelist(ListBase *list1, ListBase *list2)  /* copy from 2 to 1 */
 {
 	struct Link *link1, *link2;
 	
@@ -207,6 +207,10 @@ void initglobals(void)
 
 	sprintf(versionstr, "www.blender.org %d", G.version);
 
+#ifdef _WIN32	// FULLSCREEN
+	G.windowstate = G_WINDOWSTATE_USERDEF;
+#endif
+
 	clear_workob();	/* object.c */
 }
 
@@ -217,10 +221,10 @@ static void clear_global(void) {
 
 	freeAllRad();
 	free_main(G.main); /* free all lib data */
-	freefastshade();	/* anders oude lampgegevens */
+	freefastshade();	/* othwerwise old lamp settings stay active */
 
 
-	/* hangende vars voorkomen */	
+	/* prevent hanging vars */	
 	R.backbuf= 0;
 	
 	/* force all queues to be left */
@@ -261,17 +265,28 @@ static void setup_app_data(BlendFileData *bfd, char *filename) {
 	if (bfd->user) {
 		U= *bfd->user;
 		MEM_freeN(bfd->user);
+		
+		/* the UserDef struct is not corrected with do_versions() .... ugh! */
 		if(U.wheellinescroll == 0) U.wheellinescroll = 3;
+		if(U.menuthreshold1==0) {
+			U.menuthreshold1= 5;
+			U.menuthreshold2= 2;
+		}
+		if(U.tb_leftmouse==0) {
+			U.tb_leftmouse= 5;
+			U.tb_rightmouse= 5;
+		}
+		if(U.mixbufsize==0) U.mixbufsize= 2048;
 	}
 	
 	R.winpos= bfd->winpos;
 	R.displaymode= bfd->displaymode;
 	G.curscreen= bfd->curscreen;
 	G.fileflags= bfd->fileflags;
-	
+	G.f= bfd->globalf;
 	G.scene= G.curscreen->scene;
 	
-		/* weinig DispListen, wel text_to_curve */
+		/* few DispLists, but do text_to_curve */
 	// this should be removed!!! But first a better displist system (ton)
 	for (ob= G.main->object.first; ob; ob= ob->id.next) {
 		if(ob->type==OB_FONT) {
@@ -295,7 +310,7 @@ static void setup_app_data(BlendFileData *bfd, char *filename) {
 	}
 	
 	strcpy(G.sce, filename);
-	strcpy(G.main->name, filename); /* is gegarandeerd current file */
+	strcpy(G.main->name, filename); /* is guaranteed current file */
 	
 	MEM_freeN(bfd);
 }
