@@ -1,10 +1,8 @@
 
-/*  displist.c      GRAPHICS
+/*  displist.c
  * 
- *  
- *  maart 95
  * 
- * $Id: displist.c,v 1.3 2002/11/25 10:13:52 mein Exp $
+ * $Id: displist.c,v 1.10 2003/11/21 12:30:14 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -49,6 +47,7 @@
 #include "MEM_guardedalloc.h"
 
 #include "nla.h" /* For __NLA: Please do not remove yet */
+#include "render.h"
 
 #include "IMB_imbuf_types.h"
 
@@ -235,7 +234,7 @@ static void initfastshade(void)
 	if(G.scene->camera==0) G.scene->camera= scene_find_camera(G.scene);
 	if(G.scene->camera==0) return;
 
-	/* uit roteerscene gejat */
+	/* copied from 'roteerscene' (does that function still exist? (ton) */
 	where_is_object(G.scene->camera);
 	Mat4CpyMat4(R.viewinv, G.scene->camera->obmat);
 	Mat4Ortho(R.viewinv);
@@ -303,7 +302,7 @@ void freefastshade()
 static void fastshade(float *co, float *nor, float *orco, Material *ma, char *col1, char *col2, char *vertcol)
 {
 	FastLamp *fl;
-	float i, t, inp, soft, inpr, inpg, inpb, isr=0, isg=0, isb=0, lv[3], lampdist, ld;
+	float i, t, inp, soft, inpr, inpg, inpb, isr=0, isg=0, isb=0, lv[3], view[3], lampdist, ld;
 	float inpr1, inpg1, inpb1, isr1=0, isg1=0, isb1=0;
 	int a, back;
 
@@ -386,9 +385,11 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 		inpr= inpg= inpb= inpr1= inpg1= inpb1= ma->emit;
 	}
 	
-	/* col[0]= (255.0*ma->r); */
-	/* col[1]= (255.0*ma->g); */
-	/* col[2]= (255.0*ma->b); */
+	view[0]= 0.0;
+	view[1]= 0.0;
+	view[2]= 1.0;
+	
+	Normalise(view);
 
 	for (fl= fastlamplist; fl; fl= fl->next) {
 		/* if(fl->mode & LA_LAYER) if((fl->lay & ma->lay)==0) continue; */
@@ -436,7 +437,7 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 				i= 1.0;
 				soft= 1.0;
 				if(t<fl->spotbl && fl->spotbl!=0.0) {
-					/* zachte gebied */
+					/* soft area */
 					i= t/fl->spotbl;
 					t= i*i;
 					soft= (3.0*t-2.0*t*i);
@@ -447,8 +448,14 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 			}
 		}
 
-		inp= nor[0]*lv[0]+ nor[1]*lv[1]+ nor[2]*lv[2];
-
+		if(fl->mode & LA_NO_DIFF) inp= 0.0;
+		else {
+			inp= nor[0]*lv[0]+ nor[1]*lv[1]+ nor[2]*lv[2];
+	
+			if(ma->diff_shader==MA_DIFF_ORENNAYAR) inp= OrenNayar_Diff(nor, lv, view, ma->roughness);
+			else if(ma->diff_shader==MA_DIFF_TOON) inp= Toon_Diff(nor, lv, view, ma->param[0], ma->param[1]);
+		}
+		
 		back= 0;
 		if(inp<0.0) {
 			back= 1;
@@ -465,13 +472,19 @@ static void fastshade(float *co, float *nor, float *orco, Material *ma, char *co
 			inpg1+= inp*fl->g;
 			inpb1+= inp*fl->b;
 		}
-		if(ma->spec) {
+		if(ma->spec && (fl->mode & LA_NO_SPEC)==0) {
+			
+			if(ma->spec_shader==MA_SPEC_PHONG) 
+				t= Phong_Spec(nor, lv, view, ma->har);
+			else if(ma->spec_shader==MA_SPEC_COOKTORR) 
+				t= CookTorr_Spec(nor, lv, view, ma->har);
+			else if(ma->spec_shader==MA_SPEC_BLINN) 
+				t= Blinn_Spec(nor, lv, view, ma->refrac, (float)ma->har);
+			else 
+				t= Toon_Spec(nor, lv, view, ma->param[2], ma->param[3]);
 			
-			lv[2]+= 1.0;
-			Normalise(lv);
-			t= nor[0]*lv[0]+nor[1]*lv[1]+nor[2]*lv[2];
 			if(t>0) {
-				t= ma->spec*lampdist*RE_Spec(t, ma->har);
+				t*= ma->spec*lampdist;
 				if(back==0) {
 					isr+= t*(fl->r * ma->specr);
 					isg+= t*(fl->g * ma->specg);
@@ -650,7 +663,7 @@ void shadeDispList(Object *ob)
 	Mesh *me;
 	Curve *cu;
 /*  	extern Material defmaterial;	 *//* initrender.c, already in bad lev calls*/
-	float *orco, imat[3][3], tmat[4][4], mat[4][4], vec[3], xn, yn, zn;
+	float *orco=NULL, imat[3][3], tmat[4][4], mat[4][4], vec[3], xn, yn, zn;
 	float *fp, *nor, n1[3];
 	unsigned int *col1, *col2, *vertcol;
 	int a, lastmat= -1, need_orco = 0;
@@ -663,11 +676,11 @@ void shadeDispList(Object *ob)
 	Mat4Invert(tmat, mat);
 	Mat3CpyMat4(imat, tmat);
 	
-	/* we halen de dl_verts eruit, deform info */
+	/* we extract dl_verts, deform info */
 	dldeform= find_displist(&ob->disp, DL_VERTS);
 	if(dldeform) BLI_remlink(&ob->disp, dldeform);
 	
-	/* Metaballs hebben de standaard displist aan het Object zitten */
+	/* Metaballs have the standard displist in the Object */
 	if(ob->type!=OB_MBALL) freedisplist(&ob->disp);
 
 	if((R.flag & R_RENDERING)==0) {
@@ -692,8 +705,9 @@ void shadeDispList(Object *ob)
 
 			dl= me->disp.first;
 			while(dl) {
-				if(dl->type==DL_MESH) {
+				if(dl->type==DL_MESH && dl->mesh && dl->mesh->totvert) {
 					DispListMesh *dlm= dl->mesh;
+					float *vnors, *vn;
 					int i;
 					
 					dlob= MEM_callocN(sizeof(DispList), "displistshade");
@@ -704,6 +718,25 @@ void shadeDispList(Object *ob)
 					if (me->flag & ME_TWOSIDED)
 						dlob->col2= MEM_mallocN(sizeof(*dlob->col2)*dlm->totface*4, "col1");
 					
+					/* vertexnormals */
+					vn=vnors= MEM_mallocN(dlm->totvert*3*sizeof(float), "vnors disp");
+					mvert= dlm->mvert;
+					a= dlm->totvert;
+					while(a--) {
+						
+						xn= mvert->no[0]; 
+						yn= mvert->no[1]; 
+						zn= mvert->no[2];
+						
+						/* transpose ! */
+						vn[0]= imat[0][0]*xn+imat[0][1]*yn+imat[0][2]*zn;
+						vn[1]= imat[1][0]*xn+imat[1][1]*yn+imat[1][2]*zn;
+						vn[2]= imat[2][0]*xn+imat[2][1]*yn+imat[2][2]*zn;
+						Normalise(vn);
+						
+						mvert++; vn+=3;
+					}		
+			
 					for (i=0; i<dlm->totface; i++) {
 						MFaceInt *mf= &dlm->mface[i];
 
@@ -740,14 +773,17 @@ void shadeDispList(Object *ob)
 								
 								VECCOPY(vec, mv->co);
 								Mat4MulVecfl(mat, vec);
+								if(mf->flag & ME_SMOOTH) vn= vnors+3*vidx[j];
+								else vn= n1;
 							
 								if (need_orco && orco)
-									fastshade(vec, n1, &orco[vidx[j]*3], ma, (char *)col1, (char*)col2, (char*) mcol);
+									fastshade(vec, vn, &orco[vidx[j]*3], ma, (char *)col1, (char*)col2, (char*) mcol);
 								else
-									fastshade(vec, n1, mv->co, ma, (char *)col1, (char*)col2, (char*) mcol);
+									fastshade(vec, vn, mv->co, ma, (char *)col1, (char*)col2, (char*) mcol);
 							}
 						}
 					}
+					MEM_freeN(vnors);
 				}
 				dl= dl->next;
 			}
@@ -758,7 +794,8 @@ void shadeDispList(Object *ob)
 			}
 		}
 		else if(me->totvert>0) {
-		
+			float *vnors, *vn;
+			
 			if(me->orco==0 && need_orco) {
 				make_orco_mesh(me);
 			}
@@ -783,25 +820,23 @@ void shadeDispList(Object *ob)
 				col2= dl->col2= MEM_mallocN(4*sizeof(int)*me->totface, "col2");
 			}
 			
-			/* even geen puno's */
+			/* vertexnormals */
+			vn=vnors= MEM_mallocN(me->totvert*3*sizeof(float), "vnors disp");
 			mvert= me->mvert;
 			a= me->totvert;
-			while(FALSE || a--) {
-				
-				VECCOPY(vec, mvert->co);
-				Mat4MulVecfl(mat, vec);
+			while(a--) {
 				
 				xn= mvert->no[0]; 
 				yn= mvert->no[1]; 
 				zn= mvert->no[2];
 				
 				/* transpose ! */
-				n1[0]= imat[0][0]*xn+imat[0][1]*yn+imat[0][2]*zn;
-				n1[1]= imat[1][0]*xn+imat[1][1]*yn+imat[1][2]*zn;
-				n1[2]= imat[2][0]*xn+imat[2][1]*yn+imat[2][2]*zn;
-				Normalise(n1);
+				vn[0]= imat[0][0]*xn+imat[0][1]*yn+imat[0][2]*zn;
+				vn[1]= imat[1][0]*xn+imat[1][1]*yn+imat[1][2]*zn;
+				vn[2]= imat[2][0]*xn+imat[2][1]*yn+imat[2][2]*zn;
+				Normalise(vn);
 				
-				mvert++;
+				mvert++; vn+=3;
 			}		
 			
 			mface= me->mface;
@@ -825,9 +860,11 @@ void shadeDispList(Object *ob)
 					mvert= me->mvert+mface->v1;
 					VECCOPY(vec, mvert->co);
 					Mat4MulVecfl(mat, vec);
+					if(mface->flag & ME_SMOOTH) vn= vnors+3*mface->v1;
+					else vn= n1;
 					
-					if(orco)  fastshade(vec, n1, orco+3*mface->v1, ma, (char *)col1, (char *)col2, (char *)vertcol);
-					else fastshade(vec, n1, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					if(orco)  fastshade(vec, vn, orco+3*mface->v1, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					else fastshade(vec, vn, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
 					col1++;
 					if(vertcol) vertcol++; 
 					if(col2) col2++;
@@ -835,9 +872,11 @@ void shadeDispList(Object *ob)
 					mvert= me->mvert+mface->v2;
 					VECCOPY(vec, mvert->co);
 					Mat4MulVecfl(mat, vec);
+					if(mface->flag & ME_SMOOTH) vn= vnors+3*mface->v2;
+					else vn= n1;
 					
-					if(orco)  fastshade(vec, n1, orco+3*mface->v2, ma, (char *)col1, (char *)col2, (char *)vertcol);
-					else fastshade(vec, n1, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					if(orco)  fastshade(vec, vn, orco+3*mface->v2, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					else fastshade(vec, vn, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
 					col1++;
 					if(vertcol) vertcol++; 
 					if(col2) col2++;
@@ -845,9 +884,11 @@ void shadeDispList(Object *ob)
 					mvert= me->mvert+mface->v3;
 					VECCOPY(vec, mvert->co);
 					Mat4MulVecfl(mat, vec);
+					if(mface->flag & ME_SMOOTH) vn= vnors+3*mface->v3;
+					else vn= n1;
 					
-					if(orco)  fastshade(vec, n1, orco+3*mface->v3, ma, (char *)col1, (char *)col2, (char *)vertcol);
-					else fastshade(vec, n1, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					if(orco)  fastshade(vec, vn, orco+3*mface->v3, ma, (char *)col1, (char *)col2, (char *)vertcol);
+					else fastshade(vec, vn, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
 					col1++;
 					if(vertcol) vertcol++; 
 					if(col2) col2++;
@@ -856,9 +897,11 @@ void shadeDispList(Object *ob)
 						mvert= me->mvert+mface->v4;
 						VECCOPY(vec, mvert->co);
 						Mat4MulVecfl(mat, vec);
+						if(mface->flag & ME_SMOOTH) vn= vnors+3*mface->v4;
+						else vn= n1;
 						
-						if(orco)  fastshade(vec, n1, orco+3*mface->v4, ma, (char *)col1, (char *)col2, (char *)vertcol);
-						else fastshade(vec, n1, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
+						if(orco)  fastshade(vec, vn, orco+3*mface->v4, ma, (char *)col1, (char *)col2, (char *)vertcol);
+						else fastshade(vec, vn, mvert->co, ma, (char *)col1, (char *)col2, (char *)vertcol);
 					}
 					col1++;
 					if(vertcol) vertcol++; 
@@ -875,6 +918,8 @@ void shadeDispList(Object *ob)
 				mface++;
 			}
 			
+			MEM_freeN(vnors);
+			
 			if(me->orco) {
 				MEM_freeN(me->orco);
 				me->orco= 0;
@@ -887,7 +932,7 @@ void shadeDispList(Object *ob)
 	}
 	else if ELEM3(ob->type, OB_CURVE, OB_SURF, OB_FONT) {
 	
-		/* nu hebben we wel de normalen nodig */
+		/* now we need the normals */
 		cu= ob->data;
 		dl= cu->disp.first;
 		
@@ -911,7 +956,7 @@ void shadeDispList(Object *ob)
 
 			if(dl->type==DL_INDEX3) {
 				if(dl->nors) {
-					/* er is maar 1 normaal */
+					/* there's just one normal */
 					n1[0]= imat[0][0]*dl->nors[0]+imat[0][1]*dl->nors[1]+imat[0][2]*dl->nors[2];
 					n1[1]= imat[1][0]*dl->nors[0]+imat[1][1]*dl->nors[1]+imat[1][2]*dl->nors[2];
 					n1[2]= imat[2][0]*dl->nors[0]+imat[2][1]*dl->nors[1]+imat[2][2]*dl->nors[2];
@@ -955,7 +1000,7 @@ void shadeDispList(Object *ob)
 		}
 	}
 	else if(ob->type==OB_MBALL) {
-		/* normalen zijn er al */
+		/* there are normals already */
 		dl= ob->disp.first;
 		
 		while(dl) {
@@ -1000,7 +1045,7 @@ void shadeDispList(Object *ob)
 		}
 	}
 
-	/* deze was er tijdelijk uitgehaald */
+	/* this one was temporally removed */
 	if(dldeform) BLI_addtail(&ob->disp, dldeform);
 }
 
@@ -1018,11 +1063,11 @@ void reshadeall_displist(void)
 			
 			ob= base->object;
 			
-			/* we halen de dl_verts eruit, deform info */
+			/* we extract dl_verts, deform info */
 			dldeform= find_displist(&ob->disp, DL_VERTS);
 			if(dldeform) BLI_remlink(&ob->disp, dldeform);
 			
-			/* Metaballs hebben de standaard displist aan het Object zitten */
+			/* Metaballs have standard displist at the Object */
 			if(ob->type==OB_MBALL) shadeDispList(ob);
 			else freedisplist(&ob->disp);
 			
@@ -1073,7 +1118,7 @@ static void curve_to_displist(ListBase *nubase, ListBase *dispbase)
 		if(nu->hide==0) {
 			if((nu->type & 7)==CU_BEZIER) {
 				
-				/* tellen */
+				/* count */
 				len= 0;
 				a= nu->pntsu-1;
 				if(nu->flagu & 1) a++;
@@ -1093,7 +1138,7 @@ static void curve_to_displist(ListBase *nubase, ListBase *dispbase)
 				}
 				
 				dl= MEM_callocN(sizeof(DispList), "makeDispListbez");
-				/* len+1 i.v.m. maakbez */
+				/* len+1 because of 'maakbez' function */
 				dl->verts= MEM_callocN( (len+1)*3*sizeof(float), "dlverts");
 				BLI_addtail(dispbase, dl);
 				dl->parts= 1;
@@ -1210,7 +1255,7 @@ static void filldisplist(ListBase *dispbase, ListBase *to)
 			
 					colnr= dl->col;
 		
-					/* editverts en edges maken */
+					/* make editverts and edges */
 					f1= dl->verts;
 					a= dl->nr;
 					eve= v1= 0;
@@ -1245,7 +1290,7 @@ static void filldisplist(ListBase *dispbase, ListBase *to)
 
 		if(totvert && BLI_edgefill(0)!=0) {
 
-			/* vlakken tellen */
+			/* count faces (vlak in dutch!) */
 			tot= 0;
 			evl= fillvlakbase.first;
 			while(evl) {
@@ -1263,7 +1308,7 @@ static void filldisplist(ListBase *dispbase, ListBase *to)
 				dlnew->index= MEM_mallocN(tot*3*sizeof(int), "dlindex");
 				dlnew->verts= MEM_mallocN(totvert*3*sizeof(float), "dlverts");
 				
-				/* vertdata */
+				/* vert data */
 				f1= dlnew->verts;
 				totvert= 0;
 				eve= fillvertbase.first;
@@ -1271,14 +1316,14 @@ static void filldisplist(ListBase *dispbase, ListBase *to)
 					VECCOPY(f1, eve->co);
 					f1+= 3;
 	
-					/* indexnummer */
+					/* index number */
 					eve->vn= (EditVert *)totvert;
 					totvert++;
 					
 					eve= eve->next;
 				}
 				
-				/* indexdata */
+				/* index data */
 				evl= fillvlakbase.first;
 				index= dlnew->index;
 				while(evl) {
@@ -1299,7 +1344,7 @@ static void filldisplist(ListBase *dispbase, ListBase *to)
 		colnr++;
 	}
 	
-	/* poly's niet vrijgeven. nodig voor wireframe display */
+	/* do not free polys, needed for wireframe display */
 	
 	/* same as above ... */
 /*  	if(G.f & G_PLAYANIM == 0) waitcursor(0); */
@@ -1443,10 +1488,13 @@ void makeDispList(Object *ob)
 		}
 	}
 	else if(ob->type==OB_MBALL) {
-		ob= find_basis_mball(ob);
 		
+		ob= find_basis_mball(ob);
+
 		metaball_polygonize(ob);
 		tex_space_mball(ob);
+
+		object_deform(ob);
 	}
 	else if(ob->type==OB_SURF) {
 		
@@ -1504,9 +1552,9 @@ void makeDispList(Object *ob)
 							if(nu->flagv & 1) dl->flag|= 2;
 						}
 						else {
-							dl->parts= nu->resolu;	/* andersom want makeNurbfaces gaat zo */
+							dl->parts= nu->resolu;	/* in reverse, because makeNurbfaces works that way */
 							dl->nr= nu->resolv;
-							if(nu->flagv & 1) dl->flag|= 1;	/* ook andersom ! */
+							if(nu->flagv & 1) dl->flag|= 1;	/* reverse too! */
 							if(nu->flagu & 1) dl->flag|= 2;
 						}
 						dl->col= nu->mat_nr;
@@ -1564,7 +1612,7 @@ void makeDispList(Object *ob)
 				if(ob->dt!=0) makebevelcurve(ob, &dlbev);
 			}
 
-			/* met bevellist werken */
+			/* work with bevellist */
 			widfac= cu->width-1.0;
 			bl= cu->bev.first;
 			nu= cu->nurb.first;
@@ -1591,7 +1639,7 @@ void makeDispList(Object *ob)
 					}
 				}
 				else {
-					/* voor iedere stuk van de bevel een aparte dispblok maken */
+					/* for each part of the bevel use a separate displblock */
 					dlb= dlbev.first;
 					while(dlb) {
 						dl= MEM_callocN(sizeof(DispList), "makeDispListbev1");
@@ -1611,9 +1659,9 @@ void makeDispList(Object *ob)
 						data= dl->verts;
 						bevp= (BevPoint *)(bl+1);
 						a= bl->nr;
-						while(a--) {	/* voor ieder punt van poly een bevelstuk maken */
+						while(a--) {	/* for each point of poly make a bevel piece */
 
-							/* roteer bevelstuk en schrijf in data */
+							/* rotate bevel piece and write in data */
 							fp1= dlb->verts;
 							b= dlb->nr;
 
@@ -1673,7 +1721,7 @@ typedef struct Sample{
 } Sample;
 
 typedef struct Segment{
-	/* coordinaten */
+	/* coordinates */
 	struct Segment * next, * prev;
 	float co[2];
 } Segment;
@@ -1706,9 +1754,9 @@ static Sample * outline(struct ImBuf * ibuf,
 	int startx = 0, starty = 0;
 	Sample * samp, * oldsamp;
 	
-	/* wat erin gaat:
-	 * 1 - plaatje waarvan outline berekent moet worden, 
-	 * 2 - pointer naar functie die bepaalt welke pixel in of uit is
+	/* input:
+	 * 1 - image 
+	 * 2 - pointer to function that defines which pixel 'in' or 'out' is
 	 */
 	
 	if (ibuf == 0) return (0);
@@ -1717,11 +1765,11 @@ static Sample * outline(struct ImBuf * ibuf,
 	if (in_or_out == 0) in_or_out = dflt_in_out;
 	in = in_or_out(ibuf, 0, 0);
 	
-	/* zoek naar eerste overgang en ga van daar uit 'zoeken' */	
+	/* search for first transition, and continue from there */	
 	for (y = 0; y < ibuf->y; y++) {
 		for (x = 0; x < ibuf->x; x++) {
 			if (in_or_out(ibuf, x, y) != in) {
-				/* eerste 'andere' punt gevonden !! */
+				/* found first 'other' point !! */
 				
 				if (x != startx) dir = 0;
 				else dir = 6;
@@ -1750,7 +1798,7 @@ static Sample * outline(struct ImBuf * ibuf,
 					}
 					
 					if (i >= 8) {
-						/* dit moet een losse punt geweest zijn */
+						/* this has to be a loose point */
 						break;
 					}
 					
@@ -1760,7 +1808,7 @@ static Sample * outline(struct ImBuf * ibuf,
 				} while(x != startx || y != starty);
 				
 				if (i >= 8) {
-					/* losse punten patch */
+					/* patch for loose points */
 					MEM_freeN(samp);
 				} else {
 					count = count - 1;
@@ -1771,7 +1819,7 @@ static Sample * outline(struct ImBuf * ibuf,
 			}
 		}
 	}
-	/* printf("geen overgang \n"); */
+	/* printf("no transition \n"); */
 	return(0);
 }
 
@@ -1782,7 +1830,7 @@ static Sample * outline(struct ImBuf * ibuf,
 /*******************************/
 
 
-static float DistToLine2D(short *v1, short *v2, short *v3)   /* met formule van Hesse :GEEN LIJNSTUK! */
+static float DistToLine2D(short *v1, short *v2, short *v3)   /* using Hesse formula :NO LINE PIECE! */
 {
 	float a[2],deler;
 
@@ -1864,7 +1912,7 @@ static void ibuf2wire(ListBase * wireframe, struct ImBuf * ibuf)
 	int count;
 	Sample * samp;
 	
-	/* eerst een lijst met samples maken */
+	/* first make a list of samples */
 	
 	samp = outline(ibuf, 0);
 	if (samp == 0) return;
@@ -1908,7 +1956,7 @@ void imagestodisplist(void)
 				if(ma && ma->mtex[0] && ma->mtex[0]->tex) {
 					tex= ma->mtex[0]->tex;
 					
-					/* dit zorgt voor correct laden van nieuwe imbufs */
+					/* this takes care of correct loading of new imbufs */
 					externtex(ma->mtex[0], vec);
 					
 					if(tex->type==TEX_IMAGE && tex->ima && tex->ima->ibuf) {				
