@@ -2,7 +2,7 @@
  * lattice.c
  *
  *
- * $Id: lattice.c,v 1.6 2003/04/26 11:56:44 ton Exp $
+ * $Id: lattice.c,v 1.7 2003/10/14 12:09:57 jiri Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -418,6 +418,15 @@ int object_deform(Object *ob)
 				}
 			}
 		}
+		else if(ob->type==OB_MBALL) {
+			dl=ob->disp.first;
+			while(dl) {
+				fp = dl->verts;
+				for(a=0;a<dl->nr;a++,fp+=3)
+					calc_latt_deform(fp);
+				dl=dl->next;
+			}
+		}
 		else if ELEM(ob->type, OB_CURVE, OB_SURF) {
 		
 			cu= ob->data;
