@@ -1,7 +1,7 @@
 /* object.c
  *
  * 
- * $Id: object.c,v 1.4 2003/04/26 13:07:59 ton Exp $
+ * $Id: object.c,v 1.8 2003/11/23 14:31:35 michel Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -92,7 +92,6 @@
 #include "BKE_mball.h"
 #include "BKE_effect.h"
 #include "BKE_sca.h"
-#include "BPY_extern.h"
 #include "BKE_displist.h"
 #include "BKE_property.h"
 #include "BKE_anim.h"
@@ -101,6 +100,8 @@
 #include "BKE_constraint.h"
 #include "BKE_scene.h"
 
+#include "BPY_extern.h"
+
 /* Local function protos */
 static void solve_parenting (Object *ob, Object *par, float slowmat[][4], int simul);
 
@@ -1444,7 +1445,7 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 	bConstraint *con;
 	float tmat[4][4], focusmat[4][4], lastmat[4][4];
 	int i, clear=1, tot=0;
-	float	a;
+	float	a=0;
 	float	aquat[4], quat[4];
 	float	aloc[3], loc[3];
 	float	asize[3], size[3];
@@ -1472,6 +1473,7 @@ void solve_constraints (Object *ob, short obtype, void *obdata, float ctime)
 				enf = con->enforce;
 
 			/* Get the targetmat */
+			//get_constraint_target(con, obtype, obdata, tmat, size, ctime - ob->sf);
 			get_constraint_target(con, obtype, obdata, tmat, size, ctime);
 			
 			Mat4CpyMat4(focusmat, tmat);
