@@ -4,7 +4,7 @@
  *  jun 2001
  *  
  * 
- * $Id: subsurf.c,v 1.6 2003/04/26 13:07:59 ton Exp $
+ * $Id: subsurf.c,v 1.14 2003/11/21 20:52:27 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -132,11 +132,7 @@ static float *Vec2Add(float *ta, float *b) {
 	return ta;
 }
 
-static float *Vec2MulNT(float *t, float *a, float n) {
-	t[0]= a[0]*n;
-	t[1]= a[1]*n;
-	return t;
-}
+
 static float *Vec3MulNT(float *t, float *a, float n) {
 	t[0]= a[0]*n;
 	t[1]= a[1]*n;
@@ -174,7 +170,8 @@ struct _HyperVert {
 	HyperVert *next;
 	
 	float co[3];
-	
+	int flag;		// added for drawing optim
+	float *orig;	// if set, pointer to original vertex, for handles
 	HyperVert *nmv;
 	LinkNode *edges, *faces;
 };
@@ -184,7 +181,7 @@ struct _HyperEdge {
 
 	HyperVert *v[2];
 	HyperVert *ep;
-
+	int flag;		// added for drawing optim
 	LinkNode *faces;
 };
 
@@ -251,14 +248,15 @@ static HyperVert *hyperedge_other_vert(HyperEdge *e, HyperVert *a) {
 	return (a==e->v[0])?e->v[1]:e->v[0];
 }
 
-static HyperVert *hypermesh_add_vert(HyperMesh *hme, float *co) {
+static HyperVert *hypermesh_add_vert(HyperMesh *hme, float *co, float *orig) {
 	HyperVert *hv= BLI_memarena_alloc(hme->arena, sizeof(*hv));
 	
 	hv->nmv= NULL;
 	hv->edges= NULL;
 	hv->faces= NULL;
-
+	
 	Vec3Cpy(hv->co, co);
+	hv->orig= orig;
 	
 	hv->next= hme->verts;
 	hme->verts= hv;
@@ -266,7 +264,7 @@ static HyperVert *hypermesh_add_vert(HyperMesh *hme, float *co) {
 	return hv;
 }
 
-static HyperEdge *hypermesh_add_edge(HyperMesh *hme, HyperVert *v1, HyperVert *v2) {
+static HyperEdge *hypermesh_add_edge(HyperMesh *hme, HyperVert *v1, HyperVert *v2, int flag) {
 	HyperEdge *he= BLI_memarena_alloc(hme->arena, sizeof(*he));
 	
 	BLI_linklist_prepend_arena(&v1->edges, he, hme->arena);
@@ -276,14 +274,15 @@ static HyperEdge *hypermesh_add_edge(HyperMesh *hme, HyperVert *v1, HyperVert *v
 	he->v[1]= v2;
 	he->ep= NULL;
 	he->faces= NULL;
-
+	he->flag= flag;
+	
 	he->next= hme->edges;
 	hme->edges= he;
 	
 	return he;
 }
 
-static HyperFace *hypermesh_add_face(HyperMesh *hme, HyperVert **verts, int nverts) {
+static HyperFace *hypermesh_add_face(HyperMesh *hme, HyperVert **verts, int nverts, int flag) {
 	HyperFace *f= BLI_memarena_alloc(hme->arena, sizeof(*f));
 	HyperVert *last;
 	int j;
@@ -302,7 +301,7 @@ static HyperFace *hypermesh_add_face(HyperMesh *hme, HyperVert **verts, int nver
 		HyperEdge *e= hypervert_find_edge(v, last);
 
 		if (!e)
-			e= hypermesh_add_edge(hme, v, last);
+			e= hypermesh_add_edge(hme, v, last, flag);
 
 		f->verts[j]= v;
 		f->edges[j]= e;
@@ -348,9 +347,9 @@ static HyperMesh *hypermesh_from_mesh(Mesh *me, DispList *dlverts) {
 	
 	for (i= 0; i<me->totvert; i++) {
 		if (dlverts)
-			vert_tbl[i]= hypermesh_add_vert(hme, &dlverts->verts[i*3]);
+			vert_tbl[i]= hypermesh_add_vert(hme, &dlverts->verts[i*3], NULL);
 		else
-			vert_tbl[i]= hypermesh_add_vert(hme, me->mvert[i].co);
+			vert_tbl[i]= hypermesh_add_vert(hme, me->mvert[i].co, NULL);
 	}
 
 	for (i=0; i<me->totface; i++) {
@@ -367,7 +366,7 @@ static HyperMesh *hypermesh_from_mesh(Mesh *me, DispList *dlverts) {
 			if (nverts>3)
 				verts[3]= vert_tbl[mf->v4];
 		
-			f= hypermesh_add_face(hme, verts, nverts);
+			f= hypermesh_add_face(hme, verts, nverts, 1);
 			f->orig.ind= i;
 
 			if (hme->hasuvco) {
@@ -388,7 +387,7 @@ static HyperMesh *hypermesh_from_mesh(Mesh *me, DispList *dlverts) {
 					*((unsigned int*) f->vcol[j])= *((unsigned int*) &mcol[j]);
 			}
 		} else {
-			hypermesh_add_edge(hme, vert_tbl[mf->v1], vert_tbl[mf->v2]);
+			hypermesh_add_edge(hme, vert_tbl[mf->v1], vert_tbl[mf->v2], 1);
 		}
 	}
 
@@ -403,29 +402,45 @@ static HyperMesh *hypermesh_from_editmesh(EditVert *everts, EditEdge *eedges, Ed
 	EditEdge *ee;
 	EditVlak *ef;
 
+		/* we only add vertices with edges, 'f1' is a free flag */
+		/* added: check for hide flag in vertices */
+	for (ev= everts; ev; ev= ev->next) ev->f1= 1;	
+
 		/* hack, tuck the new hypervert pointer into
 		 * the ev->prev link so we can find it easy, 
 		 * then restore real prev links later.
 		 */
-	for (ev= everts; ev; ev= ev->next)
-		ev->prev= (EditVert*) hypermesh_add_vert(hme, ev->co);
-
-	for (ee= eedges; ee; ee= ee->next)
-		hypermesh_add_edge(hme, (HyperVert*) ee->v1->prev, (HyperVert*) ee->v2->prev);
-	
+	for (ee= eedges; ee; ee= ee->next) {
+		if(ee->v1->h==0 && ee->v2->h==0) {
+			if(ee->v1->f1) {
+				ee->v1->prev= (EditVert*) hypermesh_add_vert(hme, ee->v1->co, ee->v1->co);
+				ee->v1->f1= 0;
+			}
+			if(ee->v2->f1) {
+				ee->v2->prev= (EditVert*) hypermesh_add_vert(hme, ee->v2->co, ee->v2->co);
+				ee->v2->f1= 0;
+			}
+				
+			hypermesh_add_edge(hme, (HyperVert*) ee->v1->prev, (HyperVert*) ee->v2->prev, 1);
+		}
+	}
 	for (ef= efaces; ef; ef= ef->next) {
-		int nverts= ef->v4?4:3;
-		HyperVert *verts[4];
-		HyperFace *f;
-		
-		verts[0]= (HyperVert*) ef->v1->prev;
-		verts[1]= (HyperVert*) ef->v2->prev;
-		verts[2]= (HyperVert*) ef->v3->prev;
-		if (nverts>3)
-			verts[3]= (HyperVert*) ef->v4->prev;
-
-		f= hypermesh_add_face(hme, verts, nverts);
-		f->orig.ef= ef;
+		if(ef->v1->h || ef->v2->h || ef->v3->h);
+		else if(ef->v4 && ef->v4->h);
+		else {
+			int nverts= ef->v4?4:3;
+			HyperVert *verts[4];
+			HyperFace *f;
+			
+			verts[0]= (HyperVert*) ef->v1->prev;
+			verts[1]= (HyperVert*) ef->v2->prev;
+			verts[2]= (HyperVert*) ef->v3->prev;
+			if (nverts>3)
+				verts[3]= (HyperVert*) ef->v4->prev;
+	
+			f= hypermesh_add_face(hme, verts, nverts, 1);
+			f->orig.ef= ef;
+		}
 	}
 
 		/* see hack above, restore the prev links */
@@ -456,7 +471,7 @@ static void hypermesh_subdivide(HyperMesh *me, HyperMesh *nme) {
 			Vec3Add(co, f->verts[j]->co);
 		Vec3MulN(co, (float)(1.0/f->nverts));
 
-		f->mid= hypermesh_add_vert(nme, co);
+		f->mid= hypermesh_add_vert(nme, co, NULL);
 	}
 		
 	for (e= me->edges; e; e= e->next) {
@@ -471,7 +486,7 @@ static void hypermesh_subdivide(HyperMesh *me, HyperMesh *nme) {
 			Vec3MulN(co, (float)(1.0/count));
 		}
 		
-		e->ep= hypermesh_add_vert(nme, co);
+		e->ep= hypermesh_add_vert(nme, co, NULL);
 	}
 
 	for (v= me->verts; v; v= v->next) {
@@ -518,12 +533,12 @@ static void hypermesh_subdivide(HyperMesh *me, HyperMesh *nme) {
 			Vec3MulN(s, (float)(1.0/count));
 		}
 
-		v->nmv= hypermesh_add_vert(nme, s);
+		v->nmv= hypermesh_add_vert(nme, s, v->orig);
 	}
 
 	for (e= me->edges; e; e= e->next) {
-		hypermesh_add_edge(nme, e->v[0]->nmv, e->ep);
-		hypermesh_add_edge(nme, e->v[1]->nmv, e->ep);
+		hypermesh_add_edge(nme, e->v[0]->nmv, e->ep, e->flag);
+		hypermesh_add_edge(nme, e->v[1]->nmv, e->ep, e->flag);
 	}
 
 	for (f= me->faces; f; f= f->next) {
@@ -570,7 +585,7 @@ static void hypermesh_subdivide(HyperMesh *me, HyperMesh *nme) {
 			nv[2]= f->mid;
 			nv[3]= f->edges[last]->ep;
 			
-			nf= hypermesh_add_face(nme, nv, 4);
+			nf= hypermesh_add_face(nme, nv, 4, 0);
 			nf->orig= f->orig;
 			
 			if (me->hasvcol) {
@@ -648,6 +663,16 @@ static int hypermesh_get_nverts(HyperMesh *hme) {
 	return count;
 }
 
+static int hypermesh_get_nverts_handles(HyperMesh *hme) {
+	HyperVert *v;
+	int count= 0;
+	
+	for (v= hme->verts; v; v= v->next)
+		if(v->orig) count++;
+	
+	return count;
+}
+
 static int hypermesh_get_nfaces(HyperMesh *hme) {
 	HyperFace *f;
 	int count= 0;
@@ -688,7 +713,8 @@ static int hypermesh_get_nhidden(HyperMesh *hme) {
 	return count;
 }
 
-static DispList *hypermesh_to_displist(HyperMesh *hme) {
+/* flag is me->flag, for handles and 'optim' */
+static DispList *hypermesh_to_displist(HyperMesh *hme, short flag) {
 	int nverts= hypermesh_get_nverts(hme);
 	int nfaces= hypermesh_get_nfaces(hme) + hypermesh_get_nlines(hme) - hypermesh_get_nhidden(hme);
 	DispList *dl= MEM_callocN(sizeof(*dl), "dl");
@@ -699,7 +725,7 @@ static DispList *hypermesh_to_displist(HyperMesh *hme) {
 	TFace *tfaces;
 	MFace *mfaces;
 	MFaceInt *mf;
-	int i, j;
+	int i, j, handles=0;
 
 		/* hme->orig_me==NULL if we are working on an editmesh */
 	if (hme->orig_me) {
@@ -710,13 +736,20 @@ static DispList *hypermesh_to_displist(HyperMesh *hme) {
 		mfaces= NULL;
 	}
 
+	/* added: handles for editmode */
+	if (hme->orig_me==NULL && (flag & ME_OPT_EDGES)) {
+		handles= hypermesh_get_nverts_handles(hme);
+	}
+
 	dl->type= DL_MESH;
 	dl->mesh= dlm;
 	
-	dlm->totvert= nverts;
-	dlm->totface= nfaces;
-	dlm->mvert= MEM_mallocN(dlm->totvert*sizeof(*dlm->mvert), "dlm->mvert");
+	dlm->totvert= nverts+handles;
+	dlm->totface= nfaces+handles;
+	/* calloc for clear flag and nor in mvert */
+	dlm->mvert= MEM_callocN(dlm->totvert*sizeof(*dlm->mvert), "dlm->mvert");
 	dlm->mface= MEM_mallocN(dlm->totface*sizeof(*dlm->mface), "dlm->mface");
+	if(hme->orig_me) dlm->flag= hme->orig_me->flag;
 	
 	if (hme->hasuvco)
 		dlm->tface= MEM_callocN(dlm->totface*sizeof(*dlm->tface), "dlm->tface");
@@ -740,9 +773,9 @@ static DispList *hypermesh_to_displist(HyperMesh *hme) {
 			 * nverts by face vertices, if necessary.
 			 */
 		
-		mf->v1= (int) f->verts[(0+voff)%4]->nmv;
-		mf->v2= (int) f->verts[(1+voff)%4]->nmv;
-		mf->v3= (int) f->verts[(2+voff)%4]->nmv;
+		mf->v1= (int) f->verts[(0+voff)]->nmv;
+		mf->v2= (int) f->verts[(1+voff)]->nmv;
+		mf->v3= (int) f->verts[(2+voff)]->nmv;
 		mf->v4= (int) f->verts[(3+voff)%4]->nmv;
 
 		if (hme->orig_me) {			
@@ -751,23 +784,57 @@ static DispList *hypermesh_to_displist(HyperMesh *hme) {
 			mf->mat_nr= origmf->mat_nr;
 			mf->flag= origmf->flag;
 			mf->puno= 0;
-			mf->edcode= ME_V1V2|ME_V2V3|ME_V3V4|ME_V4V1;
+			
+			
 		} else {
 			EditVlak *origef= f->orig.ef;
 			
 			mf->mat_nr= origef->mat_nr;
 			mf->flag= origef->flag;
 			mf->puno= 0;
-			mf->edcode= ME_V1V2|ME_V2V3|ME_V3V4|ME_V4V1;
 		}
-
+		
+		{ 	// draw flag			
+			mf->edcode= 0;
+			
+			f->verts[0]->flag= 0;
+			f->verts[1]->flag= 0;
+			f->verts[2]->flag= 0;
+			f->verts[3]->flag= 0;
+			
+			if(f->edges[0]->flag) {
+				f->edges[0]->flag= 0;
+				f->edges[0]->v[0]->flag++;
+				f->edges[0]->v[1]->flag++;
+			}
+			if(f->edges[1]->flag) {
+				f->edges[1]->flag= 0;
+				f->edges[1]->v[0]->flag++;
+				f->edges[1]->v[1]->flag++;
+			}
+			if(f->edges[2]->flag) {
+				f->edges[2]->flag= 0;
+				f->edges[2]->v[0]->flag++;
+				f->edges[2]->v[1]->flag++;
+			}
+			if(f->edges[3]->flag) {
+				f->edges[3]->flag= 0;
+				f->edges[3]->v[0]->flag++;
+				f->edges[3]->v[1]->flag++;
+			}
+			if( f->verts[0+voff]->flag && f->verts[1+voff]->flag ) mf->edcode|=ME_V1V2;
+			if( f->verts[1+voff]->flag && f->verts[2+voff]->flag ) mf->edcode|=ME_V2V3;
+			if( f->verts[2+voff]->flag && f->verts[(3+voff)%4]->flag ) mf->edcode|=ME_V3V4;
+			if( f->verts[(3+voff)%4]->flag && f->verts[0+voff]->flag ) mf->edcode|=ME_V4V1;
+		}
+			
 		if (hme->hasuvco) {
 			TFace *origtf, *tf= &dlm->tface[i];
 			
-			if (hme->orig_me)
+			//if (hme->orig_me)
 				origtf= &tfaces[f->orig.ind];
-			else
-				origtf= f->orig.ef->tface;
+			//else		ton: removed, hme->hasuvco doesn't happen in editmode (yet?)
+			//	origtf= f->orig.ef->tface;
 			
 			for (j=0; j<4; j++) {
 				Vec2Cpy(tf->uv[j], f->uvco[(j+voff)%4]);
@@ -805,12 +872,39 @@ static DispList *hypermesh_to_displist(HyperMesh *hme) {
 		}
 	}
 	
+	/* and we add the handles */
+	if(handles) {
+		MVert *mv= dlm->mvert+nverts;
+		mf= dlm->mface+nfaces;
+		i= nverts;
+		for (v= hme->verts; v; v= v->next) {
+			if(v->orig) {
+				/* new vertex */
+				Vec3Cpy(mv->co, v->orig);
+
+				/* new face */
+				mf->v1= (int) v->nmv;
+				mf->v2= i;
+				mf->v3= 0;
+				mf->v4= 0;
+				
+				mf->mat_nr= 0;
+				mf->flag= 0;
+				mf->puno= 0;
+				mf->edcode= ME_V1V2;
+					
+				mf++; i++; mv++;
+			}
+		}
+	}	
+	
 	add_mvert_normals_from_mfaces(dlm->mvert, dlm->totvert, dlm->mface, dlm->totface);
 
 	return dl;
 }
 
-static DispList *subsurf_subdivide_to_displist(HyperMesh *hme, short subdiv) {
+/* flag is me->flag, for handles and 'optim' */
+static DispList *subsurf_subdivide_to_displist(HyperMesh *hme, short subdiv, short flag) {
 	DispList *dl;
 	int i;
 
@@ -826,20 +920,26 @@ static DispList *subsurf_subdivide_to_displist(HyperMesh *hme, short subdiv) {
 		hme= tmp;
 	}
 
-	dl= hypermesh_to_displist(hme);
+	dl= hypermesh_to_displist(hme, flag);
 	hypermesh_free(hme);
 	
 	return dl;
 }
 
 void subsurf_make_editmesh(Object *ob) {
+	DispList *dl;
+	
 	if (G.eded.first) {
 		Mesh *me= ob->data;
 		HyperMesh *hme= hypermesh_from_editmesh(G.edve.first, G.eded.first, G.edvl.first);
 
 		free_displist_by_type(&me->disp, DL_MESH);
-		BLI_addtail(&me->disp, subsurf_subdivide_to_displist(hme, me->subdiv));
+		BLI_addtail(&me->disp, subsurf_subdivide_to_displist(hme, me->subdiv, me->flag));
+		
+		dl= me->disp.first;
+		if(dl && dl->mesh) dl->mesh->flag= me->flag;
 	}
+	
 }
 
 void subsurf_make_mesh(Object *ob, short subdiv) {
@@ -849,16 +949,16 @@ void subsurf_make_mesh(Object *ob, short subdiv) {
 		HyperMesh *hme= hypermesh_from_mesh(me, find_displist(&ob->disp, DL_VERTS));
 
 		free_displist_by_type(&me->disp, DL_MESH);
-		BLI_addtail(&me->disp, subsurf_subdivide_to_displist(hme, subdiv));
+		BLI_addtail(&me->disp, subsurf_subdivide_to_displist(hme, subdiv, me->flag));
 	}
 }
 
 void subsurf_to_mesh(Object *oldob, Mesh *me) {
 	Mesh *oldme= oldob->data;
 	
-	if (oldme->totface) {
+	if (oldme->totface && oldme->subdiv) {
 		HyperMesh *hme= hypermesh_from_mesh(oldme, NULL);
-		DispList *dl= subsurf_subdivide_to_displist(hme, oldme->subdiv);
+		DispList *dl= subsurf_subdivide_to_displist(hme, oldme->subdiv, oldme->flag);
 		DispListMesh *dlm= dl->mesh;
 		MFace *mfaces;
 		int i;
@@ -899,7 +999,7 @@ DispList* subsurf_mesh_to_displist(Mesh *me, DispList *dl, short subdiv)
 	
 	hme= hypermesh_from_mesh(me, dl);
 
-	return subsurf_subdivide_to_displist(hme, subdiv);
+	return subsurf_subdivide_to_displist(hme, subdiv, me->flag);
 }
 
 void subsurf_calculate_limit_positions(Mesh *me, float (*positions_r)[3]) 
