@@ -1,7 +1,7 @@
 /* texture.c
  *
  *
- * $Id: texture.c,v 1.4 2003/04/26 13:07:59 ton Exp $
+ * $Id: texture.c,v 1.10 2003/11/25 18:45:05 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -325,6 +325,10 @@ void free_texture(Tex *tex)
 
 void default_tex(Tex *tex)
 {
+        PluginTex *pit;
+        VarStruct *varstr;
+        int a;
+
 	tex->stype= 0;
 	tex->imaflag= TEX_INTERPOL+TEX_MIPMAP;
 	tex->extend= TEX_REPEAT;
@@ -344,6 +348,23 @@ void default_tex(Tex *tex)
 	tex->gfac= 1.0;
 	tex->bfac= 1.0;
 
+        if (tex->env) {
+           tex->env->stype=ENV_STATIC;
+           tex->env->clipsta=0.1;
+           tex->env->clipend=100;
+           tex->env->cuberes=100;
+           tex->env->depth=0;
+        }
+
+        pit = tex->plugin;
+        if (pit) {
+             varstr= pit->varstr;
+             if(varstr) {
+                for(a=0; a<pit->vars; a++, varstr++) {
+		      pit->data[a] = varstr->def;
+                }
+             }
+        }
 }
 
 /* ------------------------------------------------------------------------- */
@@ -629,10 +650,6 @@ void init_render_texture(Tex *tex)
 		tex->extend= TEX_CLIP;
 		
 		if(tex->env) {
-			/* temporal solution: layer 21 is to indicate an anvmap object */
-			tex->env->notlay |= (1<<20);
-			if(tex->env->object) tex->env->object->lay |= (1<<20);
-			
 			if(R.flag & R_RENDERING) {
 				if(tex->env->stype==ENV_ANIM) RE_free_envmapdata(tex->env);
 			}
@@ -1337,17 +1354,18 @@ void do_material_tex()
 	Tex *tex;
 	float *co = NULL, *dx = NULL, *dy = NULL, fact, 
 		facm, factt, facmm, facmul = 0.0, stencilTin=1.0;
-	float texvec[3], dxt[3], dyt[3], tempvec[3], norvec[3];
+	float texvec[3], dxt[3], dyt[3], tempvec[3], norvec[3], Tnor=1.0;
 	int tex_nr, rgbnor= 0;
 
 	/* here: test flag if there's a tex (todo) */
 	
 	mat_col=mat_colspec=mat_colmir=mat_ref=mat_spec=mat_har=mat_emit=mat_alpha= R.mat;
 	
-	tex_nr= 0;
-	if(R.mat->septex) tex_nr= R.mat->texact;
-	
-	for(; tex_nr<8; tex_nr++) {
+	for(tex_nr=0; tex_nr<8; tex_nr++) {
+		
+		/* separate tex switching */
+		if(R.mat->septex & (1<<tex_nr)) continue;
+		
 		if(R.mat->mtex[tex_nr]) {
 			mtex= R.mat->mtex[tex_nr];
 			
@@ -1508,8 +1526,9 @@ void do_material_tex()
 				}
 			}
 			else {
-				if(rgbnor & TEX_RGB) Ta*= stencilTin;
-				else Tin*= stencilTin;
+				Ta*= stencilTin;
+				Tnor*= stencilTin;
+				Tin*= stencilTin;
 			}
 
 			if(tex->nor && (rgbnor & TEX_NOR)==0) {
@@ -1618,9 +1637,9 @@ void do_material_tex()
 					if(mtex->maptoneg & MAP_NORM) tex->norfac= -mtex->norfac;
 					else tex->norfac= mtex->norfac;
 
-					R.vn[0]+= tex->norfac*tex->nor[0];
-					R.vn[1]+= tex->norfac*tex->nor[1];
-					R.vn[2]+= tex->norfac*tex->nor[2];
+					R.vn[0]+= Tnor*tex->norfac*tex->nor[0];
+					R.vn[1]+= Tnor*tex->norfac*tex->nor[1];
+					R.vn[2]+= Tnor*tex->norfac*tex->nor[2];
 					
 					Normalise(R.vn);
 					
@@ -1718,8 +1737,6 @@ void do_material_tex()
 				}
 			}
 		}
-		
-		if(R.mat->septex) break;
 	}
 }
 
