@@ -3,7 +3,7 @@
  * various string, file, list operations.
  *
  *
- * $Id: util.c,v 1.8 2003/08/12 20:37:45 aphex Exp $
+ * $Id: util.c,v 1.9 2003/10/25 14:09:17 aphex Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -507,12 +507,12 @@ char *BLI_gethome(void) {
 
 		if (BLI_exists(dir))
 		{
-			//strcat(dir,"/.blender/");
+			//strcat(dir,".blender/");
 			strcpy(tmpdir,dir);
-			strcat(tmpdir,"/.blender/.B.blend");
+			strcat(tmpdir,".blender/.B.blend");
 			if (BLI_exists(tmpdir))
 			{
-				strcat(dir,"/.blender/");
+				strcat(dir,".blender/");
 				return(dir);
 			}
 		}
