@@ -2,7 +2,7 @@
  *
  * .blend file writing
  *
- * $Id: writefile.c,v 1.9 2003/07/21 19:41:05 ton Exp $
+ * $Id: writefile.c,v 1.13 2003/10/25 14:09:17 aphex Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -472,7 +472,15 @@ static void write_renderinfo(WriteData *wd)		/* for renderdaemon */
 
 static void write_userdef(WriteData *wd)
 {
+	bTheme *btheme;
+	
 	writestruct(wd, USER, "UserDef", 1, &U);
+	
+	btheme= U.themes.first;
+	while(btheme) {
+		writestruct(wd, DATA, "bTheme", 1, btheme);
+		btheme= btheme->next;
+	}
 }
 
 static void write_effects(WriteData *wd, ListBase *lb)
@@ -683,6 +691,12 @@ static void write_constraints(WriteData *wd, ListBase *conlist)
 		case CONSTRAINT_TYPE_ACTION:
 			writestruct(wd, DATA, "bActionConstraint", 1, con->data);
 			break;
+		case CONSTRAINT_TYPE_LOCKTRACK:
+			writestruct(wd, DATA, "bLockTrackConstraint", 1, con->data);
+			break;
+		case CONSTRAINT_TYPE_FOLLOWPATH:
+			writestruct(wd, DATA, "bFollowPathConstraint", 1, con->data);
+			break;
 		default:
 			break;
 		}
@@ -1201,9 +1215,16 @@ static void write_screens(WriteData *wd, ListBase *scrbase)
 		sa= sc->areabase.first;
 		while(sa) {
 			SpaceLink *sl;
+			Panel *pa;
 
 			writestruct(wd, DATA, "ScrArea", 1, sa);
 			
+			pa= sa->panels.first;
+			while(pa) {
+				writestruct(wd, DATA, "Panel", 1, pa);
+				pa= pa->next;
+			}
+			
 			sl= sa->spacedata.first;
 			while(sl) {
 				if(sl->spacetype==SPACE_VIEW3D) {
@@ -1572,6 +1593,9 @@ int BLO_write_file(char *dir, int write_flags, char **error_r)
 	char userfilename[FILE_MAXDIR+FILE_MAXFILE];
 	char tempname[FILE_MAXDIR+FILE_MAXFILE];
 	int file, fout, write_user_block;
+#ifdef WIN32
+	char tmpdir[FILE_MAXDIR+FILE_MAXFILE];
+#endif
 	
 	sprintf(tempname, "%s@", dir);
 
@@ -1581,7 +1605,17 @@ int BLO_write_file(char *dir, int write_flags, char **error_r)
 		return 0;
 	}
 	
+#ifdef WIN32
+	BLI_getInstallationDir(tmpdir);
+
+	if(BLI_exists(tmpdir))
+		strcat(tmpdir,".blender/");
+
+	BLI_make_file_string(G.sce, userfilename, tmpdir, ".B.blend");
+#else
 	BLI_make_file_string(G.sce, userfilename, BLI_gethome(), ".B.blend");
+#endif
+
 	write_user_block= BLI_streq(dir, userfilename);
 
 	fout= write_file_handle(file, write_user_block, write_flags);
