@@ -11,7 +11,7 @@
  * are self-explanatory (e.g. IS_amiga tests whether the buffer
  * contains an Amiga-format file).
  *
- * $Id: IMB_imbuf_types.h,v 1.4 2002/12/27 13:10:17 mein Exp $ 
+ * $Id: IMB_imbuf_types.h,v 1.10 2003/06/06 11:55:16 sgefant Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -41,6 +41,12 @@
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
+/**
+ * \file IMB_imbuf_types.h
+ * \ingroup imbuf
+ * \brief Contains defines and structs used throughout the imbuf module.
+ * \todo Clean up includes.
+ */
 
 #ifndef IMB_IMBUF_TYPES_H
 #define IMB_IMBUF_TYPES_H
@@ -50,34 +56,44 @@
 struct _AviMovie;
 struct Mdec;
 
-/* the basic imbuf type */
-
+/**
+ * \brief The basic imbuf type
+ * \ingroup imbuf
+ * This is the abstraction of an image.  ImBuf is the basic type used for all
+ * imbuf operations.
+ *
+ */
 typedef struct ImBuf{
-	short	x,y;		/* breedte in pixels, hoogte in scanlines */
-	short	skipx;		/* breedte in ints om bij volgende scanline te komen */
-	unsigned char	depth;		/* actieve aantal bits/bitplanes */
-	unsigned char	cbits;		/* aantal active bits in cmap */
-	unsigned short	mincol;
-	unsigned short	maxcol;
-	int	type;		/* 0=abgr, 1=bitplanes */
-	int	ftype;
-	unsigned int	*cmap;		/* int array van kleuren */
-	unsigned int	*rect;		/* databuffer */
-	unsigned int	**planes;	/* bitplanes */
-	int	flags;
-	int	mall;		/* wat is er intern gemalloced en mag weer vrijgegeven worden */
-	short	xorig, yorig;
-	char	name[1023];
-	char	namenull;
-	int	userflags;
-	int	*zbuf;
-	void *userdata;
-	unsigned char *encodedbuffer;
-	unsigned int   encodedsize;
-	unsigned int   encodedbuffersize;
+		/** Width in pixels */
+	short	x, y;		/**< Height in scanlines */
+	short	skipx;		/**< Width in ints to get to the next scanline */
+	unsigned char	depth;		/**< Active amount of bits/bitplanes */
+	unsigned char	cbits;		/**< Amount of active bits in cmap */
+	unsigned short	mincol;		/**< First active color?*/
+	unsigned short	maxcol;		/**< Last active color?*/
+	int	type;		/**< 0=abgr, 1=bitplanes */
+	int	ftype;		/**< File type */
+	unsigned int	*cmap;		/**< Color map data. */
+	unsigned int	*rect;		/**< databuffer */
+	unsigned int	**planes;	/**< bitplanes */
+	int	flags;		/**< Controls which components should exist. */
+	int	mall;		/**< what is malloced internal, and can be freed */
+		/** X origin.  What is this relative to? */
+	short	xorig, yorig;		/**< Y origin.  What is this relative to? */
+	char	name[1023];		/**< The file name */
+	char	namenull;		/**< What does this do?*/
+	int	userflags;		/**< What does this do? Holds an enum ImBuf_userflagsMask?*/
+	int	*zbuf;		/**< A z buffer */
+	void *userdata;		/**< What does this do?*/
+	unsigned char *encodedbuffer; 		/**< Does encoded mean compressed? */
+	unsigned int   encodedsize;		/**< Compressed size?? */
+	unsigned int   encodedbuffersize;		/**< Uncompressed size?? */
 } ImBuf;
 
 /* Moved from BKE_bmfont_types.h because it is a userflag bit mask. */
+/**
+ * \brief Flags for the user?
+ */
 typedef enum {
 	IB_BITMAPFONT = 1 << 0,
 	IB_BITMAPDIRTY = 1 << 1
@@ -90,6 +106,12 @@ typedef enum {
  * probably imbuf_intern only. This will need to be merged later
  * on. */
 
+/**
+ * \name Imbuf Component flags
+ * \brief These flags determine the components of an ImBuf struct.
+ */
+/**@{*/
+/** \brief Flag defining the components of the ImBuf struct. */
 #define IB_rect			(1 << 0)
 #define IB_planes		(1 << 1)
 #define IB_cmap			(1 << 2)
@@ -104,13 +126,31 @@ typedef enum {
 #define IB_zbuf			(1 << 13)
 
 #define IB_mem			(1 << 14)
+/**@}*/
 
+/** \name imbuf_formats Image file formats
+ * \brief These defines are bit flags for the various image file formats.
+ */
+/**@{*/
+/** \brief Identifier for an image file format.
+ *
+ * The bit flag is stored in the ImBuf.ftype variable.
+ */
 #define AMI	    (1 << 31)
 #define PNG	    (1 << 30)
 #define Anim	(1 << 29)
 #define TGA	    (1 << 28)
 #define JPG		(1 << 27)
 #define BMP		(1 << 26)
+#ifdef WITH_QUICKTIME
+#define QUICKTIME	(1 << 25)
+#endif
+#ifdef WITH_FREEIMAGE
+#define FREEIMAGE	(1 << 24)
+#endif
+#ifdef WITH_IMAGEMAGICK
+#define IMAGEMAGICK	(1 << 23)
+#endif
 
 #define RAWTGA	(TGA | 1)
 
@@ -132,7 +172,13 @@ typedef enum {
 #define AN_yuvx			(Anim | YUVX)
 #define AN_hamx			(Anim | HAMX)
 #define AN_tanx			(Anim | TANX)
+/**@}*/
 
+/** \name Imbuf File Type Tests
+ * \brief These macros test if an ImBuf struct is the corresponding file type.
+ */
+/**@{*/
+/** \brief Tests the ImBuf.ftype variable for the file format. */
 #define IS_amiga(x)		(x->ftype & AMI)
 #define IS_ham(x)		((x->ftype & AM_ham) == AM_ham)
 #define IS_hbrite(x)	((x->ftype & AM_hbrite) == AM_hbrite)
@@ -151,6 +197,10 @@ typedef enum {
 #define IS_vidjpg(x)	((x->ftype & JPG_MSK) == JPG_VID)
 #define IS_jstjpg(x)	((x->ftype & JPG_MSK) == JPG_JST)
 #define IS_maxjpg(x)	((x->ftype & JPG_MSK) == JPG_MAX)
+/**@}*/
 
 #endif
 
+
+
+
