@@ -1,5 +1,5 @@
 /**
- * $Id: BIF_editarmature.h,v 1.5 2003/04/24 00:48:42 hos Exp $
+ * $Id: BIF_editarmature.h,v 1.6 2003/10/10 17:28:59 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -83,7 +83,7 @@ void	adduplicate_armature(void);
 void	addvert_armature(void);
 void	add_primitiveArmature(int type);
 void	apply_rot_armature (struct Object *ob, float mat[3][3]);
-void	armaturebuts(void);
+
 void	clear_armature(struct Object *ob, char mode);
 void	clever_numbuts_armature (void);
 void	clever_numbuts_posearmature (void);
@@ -105,6 +105,8 @@ void	remake_editArmature(void);
 void	selectconnected_armature(void);
 void	selectconnected_posearmature(void);
 void	select_bone_by_name (struct bArmature *arm, char *name, int select);
+void    unique_editbone_name (char* name);
+
 struct	Bone *get_first_selected_bone (void);
 void auto_align_armature(void);
 void create_vgroups_from_armature(Object *ob, Object *par);
