@@ -1,5 +1,5 @@
 /* toolbox (SPACEKEY) related
- * $Id: BIF_toolbox.h,v 1.8 2003/05/03 12:17:08 phase Exp $ 
+ * $Id: BIF_toolbox.h,v 1.11 2003/10/29 00:23:15 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -103,6 +103,9 @@ void tbox_getactive (int *x, int *y);
 void drawtoolbox (void);
 void toolbox (void);
 
+void toolbox_n(void);
+void toolbox_n_add(void);
+
 void notice (char *str, ...);
 void error (char *fmt, ...);
 
@@ -115,6 +118,7 @@ short sbutton (char *var, float min, float max, char *str);	/* __NLA */
 int movetolayer_buts (unsigned int *lay);
 void draw_numbuts_tip (char *str, int x1, int y1, int x2, int y2);
 int do_clever_numbuts (char *name, int tot, int winevent);
+void clever_numbuts_buts(void);
 void add_numbut (int nr, int type, char *str, float min, float max, void *poin, char *tip);
 void clever_numbuts (void);
 void replace_names_but (void);
