@@ -9,7 +9,7 @@
  *   mouse / timer / window: until 0x020
  *   custom codes: 0x4...
  * 
- * $Id: mydevice.h,v 1.7 2003/04/27 10:05:36 ton Exp $
+ * $Id: mydevice.h,v 1.9 2003/10/13 14:57:43 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -202,24 +202,22 @@
 
 /* REDRAWVIEW3D has to be the first one (lowest number) for buttons! */
 #define REDRAWVIEW3D		0x4010
+#define REDRAWVIEWCAM		0x4011
+#define REDRAWVIEW3D_Z		0x4012
 
-#define REDRAWBUTSHEAD		0x4011
-#define REDRAWBUTSALL		0x4012
-#define REDRAWBUTSVIEW		0x4013
-#define REDRAWBUTSLAMP		0x4014
-#define REDRAWBUTSMAT		0x4015
-#define REDRAWBUTSTEX		0x4016
-#define REDRAWBUTSANIM		0x4017
-#define REDRAWBUTSWORLD		0x4018
-#define REDRAWBUTSRENDER	0x4019
-#define REDRAWBUTSEDIT		0x401A
-#define REDRAWVIEWCAM		0x401B
-#define REDRAWHEADERS		0x401C
-#define REDRAWBUTSGAME		0x401D
-#define REDRAWBUTSRADIO		0x401E
-#define REDRAWVIEW3D_Z		0x401F
+#define REDRAWALL			0x4013
+#define REDRAWHEADERS		0x4014
+
+#define REDRAWBUTSHEAD		0x4015
+#define REDRAWBUTSALL		0x4016
+
+#define REDRAWBUTSSCENE		0x4017
+#define REDRAWBUTSOBJECT	0x4018
+#define REDRAWBUTSEDIT		0x4019
+#define REDRAWBUTSSCRIPT	0x401A
+#define REDRAWBUTSLOGIC		0x401B
+#define REDRAWBUTSSHADING	0x401C
 
-#define REDRAWALL			0x4020
 #define REDRAWINFO			0x4021
 #define RENDERPREVIEW		0x4022
 #define REDRAWIPO			0x4023
@@ -233,13 +231,10 @@
 #define AFTERIMAWRITE       0x402B
 #define IMALEFTMOUSE		0x402C
 #define AFTERPIBREAD        0x402D
-#define REDRAWTEXT	        0x402F
-#define REDRAWBUTSSCRIPT	0x4030
-#define REDRAWSOUND			0x4031
-#define REDRAWBUTSSOUND		0x4032
-#define REDRAWACTION		0x4033
-#define REDRAWBUTSCONSTRAINT	0x4034
-#define REDRAWNLA			0x4035
+#define REDRAWTEXT	        0x402E
+#define REDRAWSOUND			0x402F
+#define REDRAWACTION		0x4030
+#define REDRAWNLA			0x4031
 
 #endif	/* !__MYDEVICE_H__ */
 
