@@ -3,7 +3,7 @@
  *
  * Mesh stuff.
  *
- * $Id: DNA_mesh_types.h,v 1.6 2003/04/27 11:55:09 ton Exp $ 
+ * $Id: DNA_mesh_types.h,v 1.9 2003/11/19 16:21:37 intrr Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -143,9 +143,9 @@ typedef struct Mesh {
 	float rot[3];
 	
 	short smoothresh, flag;
-	short subdiv, subdivr;
+	short subdiv, subdivr, subdivdone;
 	short totcol;
-	short reserved1, reserved2, reserved3;	/* Padding */
+	short reserved1, reserved2;	/* Padding */
 
 	float cubemapsize, rtf;
 
@@ -167,6 +167,7 @@ typedef struct Mesh {
 #define ME_AUTOSMOOTH	32
 #define ME_SMESH		64
 #define ME_SUBSURF		128
+#define ME_OPT_EDGES	256
 
 /* puno = vertexnormal (mface) */
 #define ME_FLIPV1		1
@@ -223,6 +224,7 @@ typedef struct Mesh {
 /* mvert->flag (1=select) */
 #define ME_SPHERETEST	2
 #define ME_SPHERETEMP	4
+#define ME_HIDE			16
 
 #endif
 
