@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_screen_types.h (mar-2001 nzc)
  *
- * $Id: DNA_screen_types.h,v 1.7 2003/06/27 15:29:36 ton Exp $ 
+ * $Id: DNA_screen_types.h,v 1.15 2003/10/27 13:23:46 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -78,6 +78,19 @@ typedef struct ScrEdge {
 typedef unsigned short dna_ushort_fix;
 #endif
 
+
+
+typedef struct Panel {		/* the part from uiBlock that needs saved in file */
+	struct Panel *next, *prev;
+	char panelname[64], tabname[64];	/* defined as UI_MAX_NAME_STR */
+	short ofsx, ofsy, sizex, sizey;
+	short flag, active;					/* active= used currently by a uiBlock */
+	short control, pad;
+	short old_ofsx, old_ofsy;		/* for stow */
+	int pad2;
+	struct Panel *paneltab;		/* this panel is tabbed in *paneltab */
+} Panel;
+
 typedef struct ScrArea {
 	struct ScrArea *next, *prev;
 	ScrVert *v1, *v2, *v3, *v4;
@@ -93,20 +106,24 @@ typedef struct ScrArea {
 	char win_swap, win_equal;
 	
 	short headbutlen, headbutofs;
-	short cursor, rt;
+	short cursor, flag;
 
 	ListBase spacedata;
 	ListBase uiblocks;
+	ListBase panels;
 } ScrArea;
 
 #define MAXWIN		128
 
+/* area->flag */
+#define HEADER_NO_PULLDOWN	1
+
 /* If you change EDGEWIDTH, also do the global arrat edcol[]  */
-#define EDGEWIDTH	5
-#define EDGEWIDTH2	(2)
+#define EDGEWIDTH	1
+#define EDGEWIDTH2	0
 #define AREAGRID	4
 #define AREAMINX	32
-#define HEADERY		21
+#define HEADERY		26
 #define AREAMINY	(HEADERY+EDGEWIDTH)
 
 #define HEADERDOWN	1
