@@ -1,7 +1,7 @@
 /**
  * blenlib/DNA_space_types.h (mar-2001 nzc)
  *	
- * $Id: DNA_space_types.h,v 1.11 2003/07/13 20:16:49 larstiq Exp $ 
+ * $Id: DNA_space_types.h,v 1.16 2003/10/28 14:20:02 ton Exp $ 
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -55,21 +55,29 @@ struct BlendHandle;
 typedef struct SpaceLink SpaceLink;
 struct SpaceLink {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
+	short blockhandler[8];
 };
 
 typedef struct SpaceInfo {
 	SpaceLink *next, *prev;
-	int spacetype, pad1;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
+
+	short blockhandler[8];
 } SpaceInfo;
 
 typedef struct SpaceIpo {
 	SpaceLink *next, *prev;
-	int spacetype, pad1;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+	
 	unsigned int rowbut, pad2; 
 	View2D v2d;
 	
@@ -83,17 +91,20 @@ typedef struct SpaceIpo {
 	short showkey, blocktype;
 	short menunr, lock;
 	int flag;
-	int	reserved1;
+	float median[3];
 	rctf tot;
 } SpaceIpo;
 
 typedef struct SpaceButs {
 	SpaceLink *next, *prev;
-	int spacetype, pad1;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
 	short cursens, curact;
-	int pad2;
+	short align, tabo;		/* align for panels, tab is old tab */
 	View2D v2d;
 	
 	short mainb, menunr;	/* texnr and menunr have to remain shorts */
@@ -109,20 +120,21 @@ typedef struct SpaceButs {
 
 	short scriptblock;
 	short scaflag;
+	short re_align, pad1;
+	int pad2;
 	
-	char texact, pad3[7];
-	
-		/* a hackish link the anim buts keep
-		 * to a SpaceIpo.
-		 */
-	struct SpaceIpo *anim_linked_sipo;
+	char texact, tab[7];	/* storing tabs for each context */
+		
 } SpaceButs;
 
 typedef struct SpaceSeq {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
 	View2D v2d;
 	
 	short mainb, zoom;
@@ -132,9 +144,12 @@ typedef struct SpaceSeq {
 
 typedef struct SpaceFile {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 	
+	short blockhandler[8];
+
 	struct direntry *filelist;
 	int totfile;
 	char title[24];
@@ -166,9 +181,12 @@ typedef struct SpaceFile {
 
 typedef struct SpaceOops {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
 	View2D v2d;
 	
 	ListBase oops;
@@ -179,9 +197,12 @@ typedef struct SpaceOops {
 
 typedef struct SpaceImage {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
 	View2D v2d;
 	
 	struct Image *image;
@@ -197,17 +218,25 @@ typedef struct SpaceImage {
 typedef struct SpaceNla{
 	struct SpaceLink *next, *prev;
 	int spacetype;
-	short menunr, lock;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
+	short menunr, lock;
+	int pad;
+	
 	View2D v2d;	
 } SpaceNla;
 
 typedef struct SpaceText {
 	SpaceLink *next, *prev;
-	int spacetype, pad;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 
+	short blockhandler[8];
+
 	struct Text *text;	
 
 	int top, viewlines;
@@ -262,7 +291,8 @@ typedef struct ImaDir {
 
 typedef struct SpaceImaSel {
 	SpaceLink *next, *prev;
-	int spacetype, pad1;
+	int spacetype;
+	float blockscale;
 	struct ScrArea *area;
 	
 	char   title[28];
@@ -337,32 +367,7 @@ typedef struct SpaceImaSel {
 #define V3D_LOCAL		2
 */
 
-/* buts->mainb */
-#define BUTS_VIEW			0
-#define BUTS_LAMP			1
-#define BUTS_MAT			2
-#define BUTS_TEX			3
-#define BUTS_ANIM			4
-#define BUTS_WORLD			5
-#define BUTS_RENDER			6
-#define BUTS_EDIT			7
-#define BUTS_GAME			8
-#define BUTS_FPAINT			9
-#define BUTS_RADIO			10
-#define BUTS_SCRIPT			11
-#define BUTS_SOUND			12
-#define BUTS_CONSTRAINT		13
-
-/* buts->scaflag */		
-#define BUTS_SENS_SEL		1
-#define BUTS_SENS_ACT		2
-#define BUTS_SENS_LINK		4
-#define BUTS_CONT_SEL		8
-#define BUTS_CONT_ACT		16
-#define BUTS_CONT_LINK		32
-#define BUTS_ACT_SEL		64
-#define BUTS_ACT_ACT		128
-#define BUTS_ACT_LINK		256
+/* buts defines in BIF_butspace.h */
 
 /* these values need to be hardcoded in blender.h SpaceFile: struct dna does not recognize defines */
 #define FILE_MAXDIR			160
