@@ -1,5 +1,5 @@
 /**
- * $Id: BPY_extern.h,v 1.6 2003/07/03 01:41:58 ianwill Exp $
+ * $Id: BPY_extern.h,v 1.7 2003/11/23 14:31:36 michel Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -64,4 +64,8 @@ void BPY_copy_scriptlink(struct ScriptLink *scriptlink);
 /* format importer hook */
 int BPY_call_importloader(char *name);
 
+int BPY_spacetext_is_pywin(struct SpaceText *st);
+void BPY_spacetext_do_pywin_draw(struct SpaceText *st);
+void BPY_spacetext_do_pywin_event(struct SpaceText *st, unsigned short event, short val);
+
 void init_syspath(void);
