@@ -24,7 +24,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Michel Selten, Willian P. Germano
+ * Contributor(s): Michel Selten, Willian P. Germano, Stephen Swaney
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -61,6 +61,11 @@
 #include "BPY_extern.h"
 #include "api2_2x/EXPP_interface.h"
 
+/* bpy_registryDict is declared in api2_2x/Registry.h and defined
+ * here.  This Python dictionary will be used to store data that scripts
+ * choose to preserve after they are executed, so user changes can be
+ * restored next time the script is used.  Check the Blender.Registry module. */
+extern PyObject *bpy_registryDict;
 
 /*****************************************************************************/
 /* Structure definitions                                                     */
@@ -96,6 +101,11 @@ PyObject *blender_import(PyObject *self, PyObject *args);
 /*****************************************************************************/
 void BPY_start_python(void)
 {
+  bpy_registryDict = PyDict_New(); /* check comment at start of this file */
+
+  if (!bpy_registryDict)
+    printf("Error: Couldn't create the Registry Python Dictionary!");
+
 /* TODO: Shouldn't "blender" be replaced by PACKAGE ?? (config.h) */
   Py_SetProgramName("blender");
 
@@ -115,6 +125,11 @@ void BPY_start_python(void)
 /*****************************************************************************/
 void BPY_end_python(void)
 {
+  if (bpy_registryDict) {
+    Py_DECREF (bpy_registryDict);
+    bpy_registryDict = NULL;
+  }
+
   Py_Finalize();
   return;
 }
@@ -184,7 +199,47 @@ void init_syspath(void)
     syspath_append(p);  /* append to module search path */
   }
 
-  /* set sys.executable to the Blender exe */
+  /* 
+   * bring in the site module so we can add 
+   * site-package dirs to sys.path 
+   */
+
+  mod = PyImport_ImportModule("site"); /* new ref */
+
+  if (mod) {
+    PyObject* item;
+    int size = 0;
+    int index;
+
+    /* get the value of 'sitedirs' from the module */
+
+    /* the ref man says GetDict() never fails!!! */
+    d = PyModule_GetDict (mod); /* borrowed ref */
+    p = PyDict_GetItemString (d, "sitedirs");  /* borrowed ref */
+
+    if( p ) {  /* we got our string */
+      /* append each item in sitedirs list to path */
+      size = PyList_Size (p);
+
+      for (index = 0; index < size; index++) {
+	item  = PySequence_GetItem (p, index);  /* new ref */
+	if( item )
+	  syspath_append (item);
+      }
+    }
+    Py_DECREF(mod);
+  }
+  else {  /* import 'site' failed */
+    PyErr_Clear();
+    printf("sys_init:warning - no sitedirs added from site module.\n");
+  }
+
+  /* 
+   * initialize the sys module
+   * set sys.executable to the Blender exe 
+   * set argv[0] to the Blender exe
+   */
+
   mod = PyImport_ImportModule("sys"); /* new ref */
 
   if (mod) {
@@ -265,7 +320,9 @@ void BPY_Err_Handle(Text *text)
     v = PyObject_GetAttrString(err, "lineno");
     g_script_error.lineno = PyInt_AsLong(v);
     Py_XDECREF(v);
-    return; 
+    /* this avoids an abort in Python 2.3's garbage collecting: */
+    PyErr_Clear(); 
+    return;
   } else {
     PyErr_NormalizeException(&exception, &err, &tb);
     PyErr_Restore(exception, err, tb); // takes away reference!
