@@ -42,22 +42,30 @@ static PyObject *M_Ipo_New(PyObject *self, PyObject *args)
 	Ipo *add_ipo(char *name, int idcode);
 	char*name = NULL,*code=NULL;
 	int idcode = -1;
-  C_Ipo    *pyipo;
+  BPy_Ipo    *pyipo;
   Ipo      *blipo;
 
-	if (!PyArg_ParseTuple(args, "ss", &code,&name))
+  if (!PyArg_ParseTuple(args, "ss", &code,&name))
     return (EXPP_ReturnPyObjError (PyExc_TypeError,"expected string int arguments"));
-		if (!strcmp(code,"Object"))idcode = ID_OB;
-		if (!strcmp(code,"Camera"))idcode = ID_CA;
-		if (!strcmp(code,"World"))idcode = ID_WO;
-		if (!strcmp(code,"Material"))idcode = ID_MA;
-		if (idcode == -1)
-    return (EXPP_ReturnPyObjError (PyExc_TypeError,"Bad code"));
 
+  if (!strcmp(code,"Object"))idcode = ID_OB;
+  if (!strcmp(code,"Camera"))idcode = ID_CA;
+  if (!strcmp(code,"World"))idcode = ID_WO;
+  if (!strcmp(code,"Material"))idcode = ID_MA;
+  
+  if (idcode == -1)
+    return (EXPP_ReturnPyObjError (PyExc_TypeError,"Bad code"));
+  
+  printf("%d %d %d \n", ID_OB, ID_CA, ID_WO);
+  
   blipo = add_ipo(name,idcode);
 
-  if (blipo) 
-		pyipo = (C_Ipo *)PyObject_NEW(C_Ipo, &Ipo_Type);
+  if (blipo){
+    /* return user count to zero because add_ipo() inc'd it */
+    blipo->id.us = 0;
+    /* create python wrapper obj */
+    pyipo = (BPy_Ipo *)PyObject_NEW(BPy_Ipo, &Ipo_Type);
+  }
   else
     return (EXPP_ReturnPyObjError (PyExc_RuntimeError,
 																	 "couldn't create Ipo Data in Blender"));
@@ -84,7 +92,7 @@ static PyObject *M_Ipo_Get(PyObject *self, PyObject *args)
   char   *name = NULL;
   Ipo *ipo_iter;
   PyObject *ipolist, *pyobj;
-  C_Ipo *wanted_ipo = NULL;
+  BPy_Ipo *wanted_ipo = NULL;
   char error_msg[64];
 
   if (!PyArg_ParseTuple(args, "|s", &name))
@@ -96,7 +104,7 @@ static PyObject *M_Ipo_Get(PyObject *self, PyObject *args)
   if (name) { /* (name) - Search ipo by name */
     while ((ipo_iter) && (wanted_ipo == NULL)) {
       if (strcmp (name, ipo_iter->id.name+2) == 0) {
-        wanted_ipo = (C_Ipo *)PyObject_NEW(C_Ipo, &Ipo_Type);
+        wanted_ipo = (BPy_Ipo *)PyObject_NEW(BPy_Ipo, &Ipo_Type);
         if (wanted_ipo) wanted_ipo->ipo = ipo_iter;
       }
       ipo_iter = ipo_iter->id.next;
@@ -141,12 +149,11 @@ static PyObject *M_Ipo_Get(PyObject *self, PyObject *args)
 static PyObject * M_Ipo_Recalc(PyObject *self, PyObject *args)
 {	
 	void testhandles_ipocurve(IpoCurve *icu);
-
-IpoCurve *icu;
-	//C_IpoCurve *p = (C_IpoCurve*)args;
- icu = IpoCurve_FromPyObject(args);
- printf("%d\n",IpoCurve_CheckPyObject(args));
- printf("%p\n",icu);
+ PyObject  *a;
+IpoCurve *icu; 
+if (!PyArg_ParseTuple(args, "O", &a))
+    return (EXPP_ReturnPyObjError (PyExc_TypeError,"expected ipo argument)"));
+icu = IpoCurve_FromPyObject(a);
  testhandles_ipocurve(icu); 
  
 Py_INCREF(Py_None);
@@ -168,9 +175,9 @@ PyObject *Ipo_Init (void)
 }
 
 /*****************************************************************************/
-/* Python C_Ipo methods:                                                  */
+/* Python BPy_Ipo methods:                                                  */
 /*****************************************************************************/
-static PyObject *Ipo_getName(C_Ipo *self)
+static PyObject *Ipo_getName(BPy_Ipo *self)
 {
   PyObject *attr = PyString_FromString(self->ipo->id.name+2);
 
@@ -186,7 +193,7 @@ static PyObject *Ipo_getName(C_Ipo *self)
 
 
 
-static PyObject *Ipo_setName(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_setName(BPy_Ipo *self, PyObject *args)
 {
   char *name;
   char buf[21];
@@ -202,7 +209,7 @@ static PyObject *Ipo_setName(C_Ipo *self, PyObject *args)
   return Py_None;
 }
 
-static PyObject *Ipo_getBlocktype(C_Ipo *self)
+static PyObject *Ipo_getBlocktype(BPy_Ipo *self)
 {
   PyObject *attr = PyInt_FromLong(self->ipo->blocktype);
   if (attr) return attr;
@@ -210,7 +217,7 @@ static PyObject *Ipo_getBlocktype(C_Ipo *self)
 }
 
 
-static PyObject *Ipo_setBlocktype(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_setBlocktype(BPy_Ipo *self, PyObject *args)
 {
   int blocktype = 0;
 
@@ -225,7 +232,7 @@ static PyObject *Ipo_setBlocktype(C_Ipo *self, PyObject *args)
 
 
 
-static PyObject *Ipo_getRctf(C_Ipo *self)
+static PyObject *Ipo_getRctf(BPy_Ipo *self)
 {
         PyObject* l = PyList_New(0);
   PyList_Append( l, PyFloat_FromDouble(self->ipo->cur.xmin));
@@ -237,7 +244,7 @@ static PyObject *Ipo_getRctf(C_Ipo *self)
 }
 
 
-static PyObject *Ipo_setRctf(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_setRctf(BPy_Ipo *self, PyObject *args)
 {
         float v[4];
   if (!PyArg_ParseTuple(args, "ffff",v,v+1,v+2,v+3))
@@ -252,7 +259,7 @@ static PyObject *Ipo_setRctf(C_Ipo *self, PyObject *args)
   return Py_None;
 }
 
-static PyObject *Ipo_getNcurves(C_Ipo *self)
+static PyObject *Ipo_getNcurves(BPy_Ipo *self)
 { 
         int i = 0;
 
@@ -265,27 +272,88 @@ static PyObject *Ipo_getNcurves(C_Ipo *self)
 }
 
 
+	 char*nametab_ob[24] = {"LocX","LocY","LocZ","dLocX","dLocY","dLocZ","RotX","RotY","RotZ","dRotX","dRotY","dRotZ","SizeX","SizeY","SizeZ","dSizeX","dSizeY","dSizeZ","Layer","Time","ColR","ColG","ColB","ColA"};
+	char*nametab_ca[3] = {"Lens","ClSta","ClEnd"};
+	char*nametab_wo[29] = {"HorR","HorG","HorB","ZenR","ZenG","ZenB","Expos","Misi","MisDi","MisSta","MisHi","StaR","StaG","StaB","StarDi","StarSi","OfsX","OfsY","OfsZ","SizeX","SizeY","SizeZ","TexR","TexG","TexB","DefVar","Col","Nor","Var"};
+	char*nametab_ma[32] = {"R","G","B","SpecR","SpecG","SpecB","MirR","MirG","MirB","Ref","Alpha","Emit","Amb","Spec","Hard","SpTra","Ang","Mode","HaSize","OfsX","OfsY","OfsZ","SizeX","SizeY","SizeZ","TexR","TexG","TexB","DefVar","Col","Nor","Var"};
 
-static PyObject *Ipo_addCurve(C_Ipo *self, PyObject *args)
+
+
+int num_from_type(char*type,int ipotype)
+{
+	int i = 0,typenumber = -1,tabsize;
+	if (ipotype == ID_OB)
+		{
+			tabsize = 24;
+	for(i = 0;i<tabsize;i++)
+		if(!strcmp(nametab_ob[i],type))
+			typenumber=i+1;
+
+		}
+	if (ipotype == ID_CA)
+		{
+			tabsize = 3;
+	for(i = 0;i<tabsize;i++)
+		if(!strcmp(nametab_ca[i],type))
+			typenumber=i+1;
+		}
+	if (ipotype == ID_WO)
+		{
+			tabsize = 29;
+	for(i = 0;i<tabsize;i++)
+		if(!strcmp(nametab_wo[i],type))
+			typenumber=i+1;
+		}
+	if (ipotype == ID_MA)
+		{
+			tabsize = 32;
+	for(i = 0;i<tabsize;i++)
+		if(!strcmp(nametab_ma[i],type))
+			typenumber=i+1;
+		}
+
+	return typenumber;
+}
+
+char* type_from_num(int num,int ipotype)
+{
+	
+	if (ipotype == ID_CA)
+		{
+	return nametab_ca[num-1];
+		}
+	if (ipotype == ID_OB)
+		{
+	return nametab_ob[num-1];
+		}
+	if (ipotype == ID_WO)
+		{
+	return nametab_wo[num-1];
+		}
+	if (ipotype == ID_MA)
+		{
+	return nametab_ma[num-1];
+		}
+	return 0;
+}
+
+
+static PyObject *Ipo_addCurve(BPy_Ipo *self, PyObject *args)
 { 
 	void *MEM_callocN(unsigned int len, char *str);
     IpoCurve *ptr;
-	int i = 0,typenumber = -1;
+	int typenumber = -1;
 	char*type = 0;
-	static char *name="mmm";	
-	char * nametab[24] = {"LocX","LocY","LocZ","dLocX","dLocY","dLocZ","RotX","RotY","RotZ","dRotX","dRotY","dRotZ","SizeX","SizeY","SizeZ","dSizeX","dSizeY","dSizeZ","Layer","Time","ColR","ColG","ColB","ColA"};
+	static char *name="mmm";
 
 	if (!PyArg_ParseTuple(args, "s",&type))
 		return (EXPP_ReturnPyObjError (PyExc_TypeError, "expected string argument"));
-	for(i = 0;i<24;i++)
-		if(!strcmp(nametab[i],type))
-			typenumber=i+1;
-
+	typenumber = num_from_type(type,self->ipo->blocktype);
 	if (typenumber == -1)
 		return (EXPP_ReturnPyObjError (PyExc_TypeError, "unknown type"));
 
 	ptr = (IpoCurve*)MEM_callocN(sizeof(IpoCurve),name);
-	ptr->blocktype = 16975;
+	ptr->blocktype = ID_OB;
 	ptr->totvert = 0;
 	ptr->adrcode = typenumber;
 	ptr->ipo = 2;
@@ -296,8 +364,22 @@ static PyObject *Ipo_addCurve(C_Ipo *self, PyObject *args)
 	return IpoCurve_CreatePyObject (ptr);
 }
 
+static PyObject *Ipo_getCurve(BPy_Ipo *self, PyObject *args)
+{
+	//int num = 0 , i = 0;
+	char*str;
+	IpoCurve *icu = 0;
+	if (!PyArg_ParseTuple(args, "s",&str))
+		return (EXPP_ReturnPyObjError (PyExc_TypeError, "expected string argument"));
+ for (icu=self->ipo->curve.first; icu; icu=icu->next){
+	 if (!strcmp(type_from_num(icu->adrcode, icu->blocktype),str))return IpoCurve_CreatePyObject(icu);
+        }
 
-static PyObject *Ipo_getCurves(C_Ipo *self)
+  Py_INCREF(Py_None);
+  return Py_None;
+}
+
+static PyObject *Ipo_getCurves(BPy_Ipo *self)
 {
   PyObject *attr = PyList_New(0);
   IpoCurve *icu;
@@ -308,7 +390,7 @@ static PyObject *Ipo_getCurves(C_Ipo *self)
 }
 
 
-static PyObject *Ipo_getNBezPoints(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_getNBezPoints(BPy_Ipo *self, PyObject *args)
 { 
 	int num = 0 , i = 0;
 	IpoCurve *icu = 0;
@@ -325,7 +407,7 @@ static PyObject *Ipo_getNBezPoints(C_Ipo *self, PyObject *args)
  return (PyInt_FromLong(icu->totvert) );
 }
 
-static PyObject *Ipo_DeleteBezPoints(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_DeleteBezPoints(BPy_Ipo *self, PyObject *args)
 { 
 	int num = 0 , i = 0;
 	IpoCurve *icu = 0;
@@ -347,7 +429,7 @@ static PyObject *Ipo_DeleteBezPoints(C_Ipo *self, PyObject *args)
 
 
 
-static PyObject *Ipo_getCurveBP(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_getCurveBP(BPy_Ipo *self, PyObject *args)
 {       
 	struct BPoint *ptrbpoint;
 	int num = 0,i;
@@ -373,7 +455,7 @@ static PyObject *Ipo_getCurveBP(C_Ipo *self, PyObject *args)
 	return l;
 }
 
-static PyObject *Ipo_getCurveBeztriple(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_getCurveBeztriple(BPy_Ipo *self, PyObject *args)
 {       
         struct BezTriple *ptrbt;
         int num = 0,pos,i,j;
@@ -404,7 +486,7 @@ static PyObject *Ipo_getCurveBeztriple(C_Ipo *self, PyObject *args)
 }
 
 
-static PyObject *Ipo_setCurveBeztriple(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_setCurveBeztriple(BPy_Ipo *self, PyObject *args)
 {       
         struct BezTriple *ptrbt;
         int num = 0,pos,i;
@@ -412,8 +494,9 @@ static PyObject *Ipo_setCurveBeztriple(C_Ipo *self, PyObject *args)
         PyObject *listargs=0;
 
   if (!PyArg_ParseTuple(args, "iiO",&num,&pos,&listargs))
-    return (EXPP_ReturnPyObjError (PyExc_TypeError,
-                                                                                                                                         "expected int argument"));
+    return (EXPP_ReturnPyObjError (PyExc_TypeError,"expected int int object argument"));
+        if(!PyTuple_Check(listargs)) 
+return (EXPP_ReturnPyObjError (PyExc_TypeError,"3rd arg should be a tuple"));
         icu =self->ipo->curve.first;
         if(!icu) return (EXPP_ReturnPyObjError (PyExc_TypeError,"No IPO curve"));
         for(i = 0;i<num;i++)
@@ -430,7 +513,8 @@ static PyObject *Ipo_setCurveBeztriple(C_Ipo *self, PyObject *args)
         
         for(i=0;i<9;i++)
                 {
-                        PyObject * xx = PyList_GetItem(listargs,i);
+                        PyObject * xx = PyTuple_GetItem(listargs,i);
+												printf("%f\n", PyFloat_AsDouble(xx));
                         ptrbt->vec[i/3][i%3] = PyFloat_AsDouble(xx);
                 }
 
@@ -441,14 +525,35 @@ static PyObject *Ipo_setCurveBeztriple(C_Ipo *self, PyObject *args)
 
 
 
-static PyObject *Ipo_getCurvecurval(C_Ipo *self, PyObject *args)
+static PyObject *Ipo_EvaluateCurveOn(BPy_Ipo *self, PyObject *args)
+{  
+	float eval_icu(IpoCurve *icu, float ipotime) ;
+     
+        int num = 0,i;
+        IpoCurve *icu;
+				float time = 0;
+
+  if (!PyArg_ParseTuple(args, "if",&num,&time))
+    return (EXPP_ReturnPyObjError (PyExc_TypeError,"expected int argument"));
+        icu =self->ipo->curve.first;
+        if(!icu) return (EXPP_ReturnPyObjError (PyExc_TypeError,"No IPO curve"));
+        for(i = 0;i<num;i++)
+                {
+                        if(!icu) return (EXPP_ReturnPyObjError (PyExc_TypeError,"Bad ipo number"));
+                        icu=icu->next;
+                 
+    }
+        return PyFloat_FromDouble(eval_icu(icu,time));
+}
+
+
+static PyObject *Ipo_getCurvecurval(BPy_Ipo *self, PyObject *args)
 {       
         int num = 0,i;
         IpoCurve *icu;
 
   if (!PyArg_ParseTuple(args, "i",&num))
-    return (EXPP_ReturnPyObjError (PyExc_TypeError,
-                                                                                                                                         "expected int argument"));
+    return (EXPP_ReturnPyObjError (PyExc_TypeError,"expected int argument"));
         icu =self->ipo->curve.first;
         if(!icu) return (EXPP_ReturnPyObjError (PyExc_TypeError,"No IPO curve"));
         for(i = 0;i<num;i++)
@@ -464,65 +569,60 @@ static PyObject *Ipo_getCurvecurval(C_Ipo *self, PyObject *args)
 
 /*****************************************************************************/
 /* Function:    IpoDeAlloc                                                   */
-/* Description: This is a callback function for the C_Ipo type. It is        */
+/* Description: This is a callback function for the BPy_Ipo type. It is        */
 /*              the destructor function.                                     */
 /*****************************************************************************/
-static void IpoDeAlloc (C_Ipo *self)
+static void IpoDeAlloc (BPy_Ipo *self)
 {
   PyObject_DEL (self);
 }
 
 /*****************************************************************************/
 /* Function:    IpoGetAttr                                                   */
-/* Description: This is a callback function for the C_Ipo type. It is        */
-/*              the function that accesses C_Ipo "member variables" and      */
+/* Description: This is a callback function for the BPy_Ipo type. It is        */
+/*              the function that accesses BPy_Ipo "member variables" and      */
 /*              methods.                                                     */
 /*****************************************************************************/
-static PyObject *IpoGetAttr (C_Ipo *self, char *name)
+static PyObject *IpoGetAttr (BPy_Ipo *self, char *name)
 {
 if (strcmp (name, "curves") == 0)return Ipo_getCurves (self);
-  return Py_FindMethod(C_Ipo_methods, (PyObject *)self, name);
+  return Py_FindMethod(BPy_Ipo_methods, (PyObject *)self, name);
 }
 
 /*****************************************************************************/
 /* Function:    IpoSetAttr                                                */
-/* Description: This is a callback function for the C_Ipo type. It is the */
+/* Description: This is a callback function for the BPy_Ipo type. It is the */
 /*              function that sets Ipo Data attributes (member variables).*/
 /*****************************************************************************/
-static int IpoSetAttr (C_Ipo *self, char *name, PyObject *value)
+static int IpoSetAttr (BPy_Ipo *self, char *name, PyObject *value)
 {
   return 0; /* normal exit */
 }
 
 /*****************************************************************************/
 /* Function:    IpoRepr                                                      */
-/* Description: This is a callback function for the C_Ipo type. It           */
+/* Description: This is a callback function for the BPy_Ipo type. It           */
 /*              builds a meaninful string to represent ipo objects.          */
 /*****************************************************************************/
-static PyObject *IpoRepr (C_Ipo *self)
+static PyObject *IpoRepr (BPy_Ipo *self)
 {
-  return PyString_FromFormat("[Ipo \"%s\"]", self->ipo->id.name+2);
+  return PyString_FromFormat("[Ipo \"%s\" %d]", self->ipo->id.name+2,self->ipo->blocktype);
 }
 
 /* Three Python Ipo_Type helper functions needed by the Object module: */
 
 /*****************************************************************************/
 /* Function:    Ipo_CreatePyObject                                           */
-/* Description: This function will create a new C_Ipo from an existing       */
+/* Description: This function will create a new BPy_Ipo from an existing       */
 /*              Blender ipo structure.                                       */
 /*****************************************************************************/
 PyObject *Ipo_CreatePyObject (Ipo *ipo)
 {
-	C_Ipo *pyipo;
-
-	pyipo = (C_Ipo *)PyObject_NEW (C_Ipo, &Ipo_Type);
-
+	BPy_Ipo *pyipo;
+	pyipo = (BPy_Ipo *)PyObject_NEW (BPy_Ipo, &Ipo_Type);
 	if (!pyipo)
-		return EXPP_ReturnPyObjError (PyExc_MemoryError,
-						"couldn't create C_Ipo object");
-
+		return EXPP_ReturnPyObjError (PyExc_MemoryError,"couldn't create BPy_Ipo object");
 	pyipo->ipo = ipo;
-
 	return (PyObject *)pyipo;
 }
 
@@ -543,5 +643,5 @@ int Ipo_CheckPyObject (PyObject *pyobj)
 /*****************************************************************************/
 Ipo *Ipo_FromPyObject (PyObject *pyobj)
 {
-	return ((C_Ipo *)pyobj)->ipo;
+	return ((BPy_Ipo *)pyobj)->ipo;
 }
