@@ -24,7 +24,7 @@
  *
  * This is a new part of Blender.
  *
- * Contributor(s): Willian P. Germano
+ * Contributor(s): Willian P. Germano, Alex Mole
  *
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
 */
@@ -36,12 +36,15 @@
 
 extern PyTypeObject Button_Type, Material_Type;
 
+extern PyTypeObject Texture_Type, MTex_Type;
+
 extern PyTypeObject Object_Type;
 extern PyTypeObject Scene_Type;
 extern PyTypeObject NMesh_Type, NMFace_Type, NMVert_Type, NMCol_Type;
 extern PyTypeObject Camera_Type, Lamp_Type, Image_Type, Text_Type;
 extern PyTypeObject Armature_Type, Bone_Type;
 extern PyTypeObject Curve_Type, Ipo_Type, Metaball_Type;
+extern PyTypeObject Lattice_Type;
 
 extern PyTypeObject vector_Type, buffer_Type, rgbTuple_Type,
 				            constant_Type;
