@@ -31,7 +31,7 @@
  * 
  *  may 1999
  * 
- * $Id: envmap.c,v 1.4 2003/04/27 14:05:41 ton Exp $
+ * $Id: envmap.c,v 1.7 2003/11/08 14:41:56 ton Exp $
  */
 
 #include <math.h>
@@ -203,13 +203,16 @@ void envmap_renderdata(EnvMap *env)
 {
 	static RE_Render envR;
 	static Object *camera;
+	int cuberes;
 	
 	if(env) {
 		envR= R;
 		camera= G.scene->camera;
 		
-		env->cuberes &= 0xFFFC;
-		R.rectx= R.r.xsch= R.recty= R.r.ysch= env->cuberes;
+		cuberes = (env->cuberes * R.r.size) / 100;
+		cuberes &= 0xFFFC;
+		env->lastsize= R.r.size;
+		R.rectx= R.r.xsch= R.recty= R.r.ysch= cuberes;
 		R.afmx= R.afmy= R.r.xsch/2;
 		R.xstart= R.ystart= -R.afmx;
 		R.xend= R.yend= R.xstart+R.rectx-1;
@@ -280,7 +283,7 @@ void env_rotate_scene(float mat[][4], int mode)
 	VertRen *ver = NULL;
 	LampRen *lar = NULL;
 	HaloRen *har = NULL;
-	float xn, yn, zn, imat[3][3], pmat[4][4], smat[4][4], tmat[4][4];
+	float xn, yn, zn, imat[3][3], pmat[4][4], smat[4][4], tmat[4][4], cmat[3][3];
 	int a;
 	
 	if(mode==0) {
@@ -333,19 +336,13 @@ void env_rotate_scene(float mat[][4], int mode)
 	
 	for(a=0; a<R.totlamp; a++) {
 		lar= R.la[a];
-
-		/* smat should actually be a 3x3 matrix, the 4x4 declaration is  */
-		/* just for confusion.                                           */
 		
-		/* Only the left-top 3x3 is copied...? smat is 4by4! imat is 3x3 */
-		/* Actually, it is not: Mat3CpyMat3 copies the first 9 floats!   */
-		/* What should happen is mat4cpymat3                             */
-/*          Mat3CpyMat3(smat, lar->imat); */
-  		MTC_Mat4CpyMat3nc(smat, lar->imat); 
-		/* This would be lar->imat = smat * imat, so 3d = 4d * 3d?       */
-		/* the mat3mulmat3 does a multiply on the wrong elements....     */
-/*    		Mat3MulMat3(lar->imat, smat, imat);  */
-  		MTC_Mat4MulMat33(lar->imat, smat, imat); 
+		/* removed here some horrible code of someone in NaN who tried to fix
+		   prototypes... just solved by introducing a correct cmat[3][3] instead
+		   of using smat. this works, check square spots in reflections  (ton) */
+		Mat3CpyMat3(cmat, lar->imat); 
+		Mat3MulMat3(lar->imat, cmat, imat); 
+
 		MTC_Mat3MulVecfl(imat, lar->vec);
 		MTC_Mat4MulVecfl(tmat, lar->co);
 
@@ -381,6 +378,18 @@ void env_layerflags(unsigned int notlay)
 	}
 }
 
+void env_hideobject(Object *ob)
+{
+	VlakRen *vlr = NULL;
+	int a;
+	
+	for(a=0; a<R.totvlak; a++) {
+		if((a & 255)==0) vlr= R.blovl[a>>8];
+		else vlr++;
+		if(vlr->ob == ob) vlr->flag &= ~R_VISIBLE;
+	}
+}
+
 /* ------------------------------------------------------------------------- */
 
 void env_set_imats()
@@ -443,6 +452,7 @@ void render_envmap(EnvMap *env)
 		init_render_world();
 		setzbufvlaggen(RE_projectverto);
 		env_layerflags(env->notlay);
+		env_hideobject(env->object);
 		env_set_imats();
 				
 		if(RE_local_test_break()==0) {
@@ -492,36 +502,60 @@ void render_envmap(EnvMap *env)
 void make_envmaps()
 {
 	Tex *tex;
-	int do_init= 0;
+	int do_init= 0, depth= 0;
 	
-	tex= G.main->tex.first;
-	while(tex) {
-		if(tex->id.us && tex->type==TEX_ENVMAP) {
-			if(tex->env && tex->env->object) {
-				if(tex->env->object->lay & G.scene->lay) {
-					if(tex->env->stype!=ENV_LOAD) {
-						
-						if(tex->env->ok==0) {
-							do_init= 1;
-							render_envmap(tex->env);
-						}
-						else if((R.r.mode & R_OSA) && tex->env->ok==ENV_NORMAL) {
-							do_init= 1;
-							RE_free_envmapdata(tex->env);
-							render_envmap(tex->env);
+	if (!(R.r.mode & R_ENVMAP)) return;
+
+
+	/* 5 = hardcoded max recursion level */
+	while(depth<5) {
+		tex= G.main->tex.first;
+		while(tex) {
+			if(tex->id.us && tex->type==TEX_ENVMAP) {
+				if(tex->env && tex->env->object) {
+					if(tex->env->object->lay & G.scene->lay) {
+						if(tex->env->stype!=ENV_LOAD) {
+							
+							/* decide if to render an envmap (again) */
+							if(tex->env->depth >= depth) {
+								
+								/* set 'recalc' to make sure it does an entire loop of recalcs */
+								
+								if(tex->env->ok) {
+										/* free when OSA, and old one isn't OSA */
+									if((R.r.mode & R_OSA) && tex->env->ok==ENV_NORMAL) 
+										RE_free_envmapdata(tex->env);
+										/* free when size larger */
+									else if(tex->env->lastsize < R.r.size) 
+										RE_free_envmapdata(tex->env);
+										/* free when env is in recalcmode */
+									else if(tex->env->recalc)
+										RE_free_envmapdata(tex->env);
+								}
+								
+								if(tex->env->ok==0 && depth==0) tex->env->recalc= 1;
+								
+								if(tex->env->ok==0) {
+									do_init= 1;
+									render_envmap(tex->env);
+									
+									if(depth==tex->env->depth) tex->env->recalc= 0;
+								}
+							}
 						}
 					}
 				}
 			}
+			tex= tex->id.next;
 		}
-		tex= tex->id.next;
+		depth++;
 	}
 
 	if(do_init) {
 		RE_local_init_render_display();
 		RE_local_clear_render_display(R.win);
-		allqueue(REDRAWBUTSTEX, 0);
-	}
+		allqueue(REDRAWBUTSSHADING, 0);			// bad!
+	}	
 }
 
 /* ------------------------------------------------------------------------- */
