@@ -1,5 +1,5 @@
 /**
- * $Id: initrender.c,v 1.8 2003/05/24 20:04:33 hos Exp $
+ * $Id: initrender.c,v 1.12 2003/11/08 12:50:39 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -136,10 +136,10 @@ void RE_init_render_data(void)
 	O.dxwin[0]= 1.0;
 	O.dywin[1]= 1.0;
 	
-	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(MAXVERT>>8),"Blove");
-	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(MAXVLAK>>8),"Blovl");
-	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(MAXVERT>>8),"Bloha");
-	R.la= (LampRen **)MEM_mallocN(MAXLAMP*sizeof(void *),"renderlamparray");
+	R.blove= (VertRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blove");
+	R.blovl= (VlakRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Blovl");
+	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(TABLEINITSIZE),"Bloha");
+	R.la= (LampRen **)MEM_mallocN(LAMPINITSIZE*sizeof(void *),"renderlamparray");
 
 	init_def_material();
 }
@@ -178,15 +178,16 @@ float  calc_weight(float *weight, int i, int j)
 
 		weight[a]= 0.0;
 
-		if(R.r.mode & R_GAUSS) {
-			if(dist<1.5) {
-				x = dist*1.5;
-				weight[a]= (1.0/exp(x*x) - 1.0/exp(1.5*1.5*1.5*1.5));
-			}
-		}
-		else {
+		/* gaussian weighting has been cancelled */
+		//if(R.r.mode & R_GAUSS) {
+		//	if(dist<1.5) {
+		//		x = dist*1.5;
+		//		weight[a]= (1.0/exp(x*x) - 1.0/exp(1.5*1.5*1.5*1.5));
+		//	}
+		//}
+		//else {
 			if(i==0 && j==0) weight[a]= 1.0;
-		}
+		//}
 
 		totw+= weight[a];
 
@@ -794,18 +795,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 	unsigned int *rt, *rt1, *rt2;
 	int len, y;
 	short blur, a,fields,fi,parts;  /* pa is a global because of print */
-	unsigned int *border_buf= NULL;
-	unsigned int border_x= 0;
-	unsigned int border_y= 0;
 	
-	if((R.r.mode & R_BORDER) && !(R.r.mode & R_MOVIECROP)) {
-		border_buf= R.rectot;
-		border_x= R.rectx;
-		border_y= R.recty;
-		R.rectot= 0;
-	}
-
-
 	if (R.rectz) MEM_freeN(R.rectz);
 	R.rectz = 0;
 
@@ -908,11 +898,11 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 						part= MEM_callocN(sizeof(Part), "part");
 						BLI_addtail(&R.parts, part);
 						part->rect= R.rectot;
-						R.rectot= 0;
+						R.rectot= NULL;
 						
 						if (R.rectz) {
 							MEM_freeN(R.rectz);
-							R.rectz= 0;
+							R.rectz= NULL;
 						}
 					}
 				}
@@ -930,13 +920,8 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 
 				if(parts>1 || (R.r.mode & R_BORDER)) {
 					if(R.rectot) MEM_freeN(R.rectot);
-					if(R.r.mode & R_BORDER) {
-						if(border_x<R.rectx || border_y<R.recty || border_buf==NULL)
-							R.rectot= (unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
-						else 
-							R.rectot= border_buf;
-					}
-					else R.rectot=(unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectot");
+					
+					R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
 					
 					part= R.parts.first;
 					for(pa=0; pa<parts; pa++) {
@@ -964,7 +949,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 			}
 
 			if( (R.flag & R_HALO)) {
-				add_halo_flare();
+				if(RE_local_test_break()==0) add_halo_flare();
 			}
 
 			if(R.r.mode & R_MBLUR) {
@@ -998,7 +983,7 @@ void oldRenderLoop(void)  /* here the PART and FIELD loops */
 		R.r.yasp/=2;
 
 		if(R.rectot) MEM_freeN(R.rectot);	/* happens when a render has been stopped */
-		R.rectot=(unsigned int *)MEM_mallocN(sizeof(int)*R.rectx*R.recty, "rectot");
+		R.rectot=(unsigned int *)MEM_callocN(sizeof(int)*R.rectx*R.recty, "rectot");
 
 		if(RE_local_test_break()==0) {
 			rt= R.rectot;
@@ -1075,7 +1060,14 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	R.r= G.scene->r;
 	R.r.postigamma= 1.0/R.r.postgamma;
 	
-	/* to be sure: when a premature return */
+	/* WINDOW size (sch='scherm' dutch for screen...) */
+	R.r.xsch= (R.r.size*R.r.xsch)/100;
+	R.r.ysch= (R.r.size*R.r.ysch)/100;
+
+	R.afmx= R.r.xsch/2;
+	R.afmy= R.r.ysch/2;
+
+	/* to be sure: when a premature return (rectx can differ from xsch) */
 	R.rectx= R.r.xsch;
 	R.recty= R.r.ysch;
 
@@ -1146,13 +1138,6 @@ void RE_initrender(struct View3D *ogl_render_view3d)
 	}
 	else R.osa= 0;
 
-	/* WINDOW */
-	R.r.xsch= (R.r.size*R.r.xsch)/100;
-	R.r.ysch= (R.r.size*R.r.ysch)/100;
-
-	R.afmx= R.r.xsch/2;
-	R.afmy= R.r.ysch/2;
-
 	/* when rendered without camera object */
 	/* it has to done here because of envmaps */
 	R.near= 0.1;
