@@ -29,7 +29,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  * Prepare the scene data for rendering.
  * 
- * $Id: renderPreAndPost.c,v 1.4 2003/04/28 11:25:42 ton Exp $
+ * $Id: renderPreAndPost.c,v 1.5 2003/08/31 20:33:46 ton Exp $
  */
 
 #include "MEM_guardedalloc.h"
@@ -43,6 +43,7 @@
 #include "envmap.h"
 #include "renderHelp.h"
 #include "shadowBuffer.h"
+#include "radio.h"
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -79,6 +80,9 @@ void prepareScene()
 		}
 	}
 
+	/* RADIO */
+	if(R.r.mode & R_RADIO) do_radio_render();
+
 	/* ENVIRONMENT MAPS */
 	make_envmaps();
 }
