@@ -1,5 +1,5 @@
 /**
- * $Id: rendercore.c,v 1.12 2003/08/11 18:18:09 ton Exp $
+ * $Id: rendercore.c,v 1.17 2003/11/07 19:23:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -1054,10 +1054,8 @@ void scanlinehaloPS(unsigned int *rectz, long *rectdelta, unsigned int *rectt, s
 		if((a & 255)==0) har= R.bloha[a>>8];
 		else har++;
 
-		if( !(a % 256) && RE_local_test_break() ) break;  /* Hos, RPW - fix slow render bug, */
-													/* !(loopvar%256) keeps checking for */
-													/* ESC too often and bogging down render */
-													/* (Based on discovery by Rob Haarsma) */
+		if( RE_local_test_break() ) break;  
+
 		if(ys>har->maxy);
 		else if(ys<har->miny);
 		else {
@@ -1163,7 +1161,7 @@ void scanlinehalo(unsigned int *rectz, unsigned int *rectt, short ys)
 		if((a & 255)==0) har= R.bloha[a>>8];
 		else har++;
 
-		if( !(a % 256) && RE_local_test_break() ) break; /* Hos, RPW, fixes Slow Render Bug */
+		if(RE_local_test_break() ) break; 
 
 		if(ys>har->maxy);
 		else if(ys<har->miny);
@@ -1217,7 +1215,7 @@ void halovert()
 		if((a & 255)==0) har= R.bloha[a>>8];
 		else har++;
 
-		if( !(a % 256) && RE_local_test_break() ) break; /* Hos, RPW, fixes slow render bug */
+		if(RE_local_test_break() ) break; 
 
 		if(har->maxy<0);
 		else if(R.recty<har->miny);
@@ -1271,7 +1269,7 @@ void halovert()
 					rectt+= R.rectx;
 					rectz+= R.rectx;
 					
-					if( !(y % 256) && RE_local_test_break() ) break; /* Hos,RPW, Fixes slow render bug */
+					if(RE_local_test_break() ) break; 
 				}
 
 			}
@@ -1636,16 +1634,9 @@ void shade_lamp_loop()
 	}
 	
 	if(ma->mode & MA_SHLESS) {
-		if( (ma->mode & (MA_VERTEXCOL+MA_VERTEXCOLP+MA_FACETEXTURE) )) {
-			ir= R.vcol[0]*ma->r;
-			ig= R.vcol[1]*ma->g;
-			ib= R.vcol[2]*ma->b;
-		}
-		else {
-			ir= ma->r;
-			ig= ma->g;
-			ib= ma->b;
-		}
+		ir= ma->r;
+		ig= ma->g;
+		ib= ma->b;
 	
 		if(usegamtab) {
 			a= 65535*ir;
@@ -1681,7 +1672,7 @@ void shade_lamp_loop()
 
 		/* test for lamp layer */
 		if(lar->mode & LA_LAYER) if((lar->lay & R.vlr->lay)==0) continue;
-
+		
 		/* lampdist calculation */
 		if(lar->type==LA_SUN || lar->type==LA_HEMI) {
 			VECCOPY(lv, lar->vec);
@@ -1757,7 +1748,7 @@ void shade_lamp_loop()
 					soft= (3.0*t-2.0*t*i);
 					inpr*= soft;
 				}
-				if(lar->mode & LA_ONLYSHADOW && lar->shb) {
+				if((lar->mode & LA_ONLYSHADOW) && lar->shb) {
 					if(ma->mode & MA_SHADOW) {
 						/* dot product positive: front side face! */
 						inp= vn[0]*lv[0] + vn[1]*lv[1] + vn[2]*lv[2];
@@ -1850,9 +1841,6 @@ void shade_lamp_loop()
 				else {
 					/* specular shaders */
 					float specfac;
-
-					/* we force a different lamp vector for sun light */
-					if(lar->type==LA_SUN) lv[2]-= 1.0;
 										
 					if(ma->spec_shader==MA_SPEC_PHONG) 
 						specfac= Phong_Spec(vn, lv, view, ma->har);
@@ -1896,26 +1884,26 @@ void shade_lamp_loop()
 			else ma->alpha= (1.0-t)*R.mat->alpha+t;
 		}
 	}
-		
+
 	if(R.refcol[0]==0.0) {
-		a= 65535.0*( ma->r*ir +ma->ambr +isr);
+		a= 65535.0*( ma->r*ir +ma->ambr +isr +ma->amb*R.rad[0]);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[0]= a;
-		a= 65535.0*(ma->g*ig +ma->ambg +isg);
+		a= 65535.0*(ma->g*ig +ma->ambg +isg +ma->amb*R.rad[1]);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[1]= a;
-		a= 65535*(ma->b*ib +ma->ambb +isb);
+		a= 65535*(ma->b*ib +ma->ambb +isb + ma->amb*R.rad[2]);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[2]= a;
 	}
 	else {
-		a= 65535.0*( ma->mirr*R.refcol[1] + (1.0 - ma->mirr*R.refcol[0])*(ma->r*ir +ma->ambr) +isr);
+		a= 65535.0*( ma->mirr*R.refcol[1] + (1.0 - ma->mirr*R.refcol[0])*(ma->r*ir +ma->ambr +ma->amb*R.rad[0]) +isr);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[0]= a;
-		a= 65535.0*( ma->mirg*R.refcol[2] + (1.0 - ma->mirg*R.refcol[0])*(ma->g*ig +ma->ambg) +isg);
+		a= 65535.0*( ma->mirg*R.refcol[2] + (1.0 - ma->mirg*R.refcol[0])*(ma->g*ig +ma->ambg +ma->amb*R.rad[1]) +isg);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[1]= a;
-		a= 65535.0*( ma->mirb*R.refcol[3] + (1.0 - ma->mirb*R.refcol[0])*(ma->b*ib +ma->ambb) +isb);
+		a= 65535.0*( ma->mirb*R.refcol[3] + (1.0 - ma->mirb*R.refcol[0])*(ma->b*ib +ma->ambb +ma->amb*R.rad[2]) +isb);
 		if(a>65535) a=65535; else if(a<0) a= 0;
 		shortcol[2]= a;
 	}
@@ -2266,6 +2254,14 @@ void shadepixel(float x, float y, int vlaknr)
 					R.vcol[2]= 0.0;
 				}
 			}
+			if(R.matren->mode & MA_RADIO) {
+				R.rad[0]= (l*v3->rad[0] - u*v1->rad[0] - v*v2->rad[0]);
+				R.rad[1]= (l*v3->rad[1] - u*v1->rad[1] - v*v2->rad[1]);
+				R.rad[2]= (l*v3->rad[2] - u*v1->rad[2] - v*v2->rad[2]);
+			}
+			else {
+				R.rad[0]= R.rad[1]= R.rad[2]= 0.0;
+			}
 			if(R.matren->mode & MA_FACETEXTURE) {
 				if((R.matren->mode & MA_VERTEXCOL)==0) {
 					R.vcol[0]= 1.0;
@@ -2312,6 +2308,7 @@ void shadepixel(float x, float y, int vlaknr)
 		}
 		else {
 			VECCOPY(R.vn, vlr->n);
+			R.rad[0]= R.rad[1]= R.rad[2]= 0.0;
 		}
 		if(R.matren->texco & TEXCO_WINDOW) {
 			R.winco[0]= (x+(R.xstart))/(float)R.afmx;
@@ -2675,7 +2672,7 @@ void zbufshadeDA(void)	/* Delta Accum Pixel Struct */
 			/* 1 is for osa */
 		if(R.r.mode & R_EDGE) edge_enhance();
 		
-		if( !(v % 256) && RE_local_test_break()) break; /*Hos,RPW, fixes slow render bug*/
+		if(RE_local_test_break()) break; 
 	}
 	if(R.flag & (R_ZTRA+R_HALO) ) {	 /* to get back correct values of zbuffer Z for transp and halos */
 		xd= jit[0][0];
@@ -2807,7 +2804,7 @@ void zbufshadeDA(void)	/* Delta Accum Pixel Struct */
 			}
 			rz+= R.rectx;
 		}
-		if(!(y % 256) && RE_local_test_break()) break; /* Hos,RPW, fixes slow render bug */
+		if(RE_local_test_break()) break; 
 	}
 
 	if( (R.r.mode & R_EDGE) && RE_local_test_break()==0) {
@@ -2906,7 +2903,7 @@ void zbufshade(void)
 									R.rectot);
 		}
 		
-		if(!(y % 256) && RE_local_test_break()) break; /*Hos,RPW, Fixes Slow render bug */
+		if(RE_local_test_break()) break; 
 	}
 
 	if(R.flag & R_ZTRA) endaccumbuf();
@@ -2969,7 +2966,7 @@ void renderhalo(HaloRen *har)	/* postprocess version */
 	
 				rectt+= R.rectx;
 				
-				if( !(y % 256) && RE_local_test_break()) break; /* Hos,RPW, fixes slow render bug */
+				if(RE_local_test_break()) break; 
 			}
 	
 		}
