@@ -1,5 +1,5 @@
 /**
- * $Id: convertBlenderScene.c,v 1.7 2003/05/20 17:09:55 sirdude Exp $
+ * $Id: convertBlenderScene.c,v 1.17 2003/11/28 21:09:41 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -98,10 +98,10 @@
 
 #include "BSE_sequence.h"
 
-#include "BPY_extern.h"
-
 #include "nla.h"
 
+#include "BPY_extern.h"
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -301,13 +301,13 @@ void RE_make_stars(void (*initfunc)(void),
 					}
 				}
 			}
-			if(done > MAXVERT) {
-				printf("Too many stars\n");
-				break;
-			}
+//			if(done > MAXVERT) {
+//				printf("Too many stars\n");
+//				break;
+//			}
 			if(blender_test_break()) break;
 		}
-		if(done > MAXVERT) break;
+//		if(done > MAXVERT) break;
 
 		if(blender_test_break()) break;
 	}
@@ -1114,23 +1114,17 @@ static void init_render_displist_mesh(Object *ob)
 	dl= me->disp.first;
 
 	/* Force a displist rebuild if this is a subsurf and we have a different subdiv level */
-#if 1
-	if((dl==0) || ((me->subdiv != me->subdivr))){
-		object_deform(ob);
-		subsurf_make_mesh(ob, me->subdivr);
-		dl = me->disp.first;
-		
+
+	if((dl==0) || ((me->subdiv != me->subdivr))) {
+		/* prevent subsurf called again for duplicate use of mesh, tface pointers change */
+		if(me->subdivdone!=me->subdivr) {
+			object_deform(ob);
+			subsurf_make_mesh(ob, me->subdivr);
+			me->subdivdone= me->subdivr;
+			dl = me->disp.first;
+		}
 	}
-	else{
-		makeDispList(ob);
-		dl= me->disp.first;
-	}
-#else
-	tempdiv = me->subdiv;
-	me->subdiv = me->subdivr;
-	makeDispList(ob);
-	dl= me->disp.first;
-#endif
+
 	if(dl==0) return;
 
 	if(need_orco) {
@@ -1138,10 +1132,6 @@ static void init_render_displist_mesh(Object *ob)
 		orco= me->orco;
 	}
 
-#if 0
-	me->subdiv = tempdiv;
-#endif
-
 	while(dl) {
 		if(dl->type==DL_SURF) {
 			startvert= R.totvert;
@@ -1194,6 +1184,7 @@ static void init_render_displist_mesh(Object *ob)
 					flen= CalcNormFloat4(v1->co, v3->co, v4->co, v2->co, n1);
 					if(flen!=0.0) {
 						vlr= RE_findOrAddVlak(R.totvlak++);
+						vlr->ob= ob;
 						vlr->v1= v1;
 						vlr->v2= v3;
 						vlr->v3= v4;
@@ -1272,6 +1263,7 @@ static void init_render_displist_mesh(Object *ob)
 
 				if(flen!=0.0) {
 					vlr= RE_findOrAddVlak(R.totvlak++);
+					vlr->ob= ob;
 					vlr->v1= v1;
 					vlr->v2= v2;
 					vlr->v3= v3;
@@ -1323,7 +1315,7 @@ static int verghalo(const void *a1, const void *a2)
 }
 
 /* ------------------------------------------------------------------------- */
-
+extern int rblohalen;
 static void sort_halos(void)
 {
 	struct halosort *hablock, *haso;
@@ -1348,7 +1340,7 @@ static void sort_halos(void)
 	/* re-assamble R.bloha */
 
 	bloha= R.bloha;
-	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(MAXVERT>>8),"Bloha");
+	R.bloha= (HaloRen **)MEM_callocN(sizeof(void *)*(rblohalen),"Bloha");
 
 	haso= hablock;
 	for(a=0; a<R.tothalo; a++) {
@@ -1445,6 +1437,7 @@ static void init_render_mball(Object *ob)
 	for(a=0; a<dl->parts; a++, index+=4) {
 
 		vlr= RE_findOrAddVlak(R.totvlak++);
+		vlr->ob= ob;
 		vlr->v1= RE_findOrAddVert(startvert+index[0]);
 		vlr->v2= RE_findOrAddVert(startvert+index[1]);
 		vlr->v3= RE_findOrAddVert(startvert+index[2]);
@@ -1499,7 +1492,7 @@ static void init_render_mesh(Object *ob)
 	float xn, yn, zn, nor[3], imat[3][3], mat[4][4];
 	float *extverts=0, *orco;
 	int a, a1, ok, do_puno, need_orco=0, totvlako, totverto, vertofs;
-	int start, end, flipnorm;
+	int start, end, flipnorm, do_autosmooth=0;
 
 	me= ob->data;
 	if (rendermesh_uses_displist(me) && me->subdivr>0) {
@@ -1613,7 +1606,7 @@ static void init_render_mesh(Object *ob)
 
 			ma= give_render_material(ob, a1+1);
 			if(ma==0) ma= &defmaterial;
-
+			
 			/* test for 100% transparant */
 			ok= 1;
 			if(ma->alpha==0.0 && ma->spectra==0.0) {
@@ -1627,7 +1620,11 @@ static void init_render_mesh(Object *ob)
 			}
 
 			if(ok) {
-
+				/* radio faces need autosmooth, to separate shared vertices in corners */
+				if(R.r.mode & R_RADIO)
+					if(ma->mode & MA_RADIO) 
+						do_autosmooth= 1;
+				
 				start= 0;
 				end= me->totface;
 				set_buildvars(ob, &start, &end);
@@ -1646,6 +1643,7 @@ static void init_render_mesh(Object *ob)
 						if(mface->v3) {
 
 							vlr= RE_findOrAddVlak(R.totvlak++);
+							vlr->ob= ob;
 							vlr->v1= RE_findOrAddVert(vertofs+mface->v1);
 							vlr->v2= RE_findOrAddVert(vertofs+mface->v2);
 							vlr->v3= RE_findOrAddVert(vertofs+mface->v3);
@@ -1725,6 +1723,7 @@ static void init_render_mesh(Object *ob)
 						}
 						else if(mface->v2 && (ma->mode & MA_WIRE)) {
 							vlr= RE_findOrAddVlak(R.totvlak++);
+							vlr->ob= ob;
 							vlr->v1= RE_findOrAddVert(vertofs+mface->v1);
 							vlr->v2= RE_findOrAddVert(vertofs+mface->v2);
 							vlr->v3= vlr->v2;
@@ -1747,7 +1746,7 @@ static void init_render_mesh(Object *ob)
 		}
 	}
 	
-	if(me->flag & ME_AUTOSMOOTH) {
+	if(do_autosmooth || (me->flag & ME_AUTOSMOOTH)) {
 		autosmooth(totverto, totvlako, me->smoothresh);
 		do_puno= 1;
 	}
@@ -1761,14 +1760,22 @@ static void init_render_mesh(Object *ob)
 void RE_add_render_lamp(Object *ob, int doshadbuf)
 {
 	Lamp *la;
-	LampRen *lar;
+	LampRen *lar, **temp;
 	float mat[4][4], hoek, xn, yn;
 	int c;
-
-	if(R.totlamp>=MAXLAMP) {
-		printf("lamp overflow\n");
-		return;
+	static int rlalen=LAMPINITSIZE; /*number of currently allocated lampren pointers*/
+	
+	if(R.totlamp>=rlalen) { /* Need more Lamp pointers....*/
+		printf("Alocating %i more lamp groups, %i total.\n", 
+			LAMPINITSIZE, rlalen+LAMPINITSIZE);
+		temp=R.la;
+		R.la=(LampRen**)MEM_callocN(sizeof(void*)*(rlalen+LAMPINITSIZE) , "renderlamparray");
+		memcpy(R.la, temp, rlalen*sizeof(void*));
+		memset(&(R.la[R.totlamp]), 0, LAMPINITSIZE*sizeof(void*));
+		rlalen+=LAMPINITSIZE;  
+		MEM_freeN(temp);	
 	}
+	
 	la= ob->data;
 	lar= (LampRen *)MEM_callocN(sizeof(LampRen),"lampren");
 	R.la[R.totlamp++]= lar;
@@ -1806,9 +1813,12 @@ void RE_add_render_lamp(Object *ob, int doshadbuf)
 	lar->r= lar->energy*la->r;
 	lar->g= lar->energy*la->g;
 	lar->b= lar->energy*la->b;
-	lar->spotsi= 0.5;
-
-	lar->spotsi= cos( M_PI*la->spotsize/360.0 );
+	
+	lar->spotsi= la->spotsize;
+	if(lar->mode & LA_HALO) {
+		if(lar->spotsi>170.0) lar->spotsi= 170.0;
+	}
+	lar->spotsi= cos( M_PI*lar->spotsi/360.0 );
 	lar->spotbl= (1.0-lar->spotsi)*la->spotblend;
 
 	memcpy(lar->mtex, la->mtex, 8*4);
@@ -1831,7 +1841,6 @@ void RE_add_render_lamp(Object *ob, int doshadbuf)
 
 		if(lar->mode & LA_ONLYSHADOW) {
 			if((lar->mode & LA_SHAD)==0) lar->mode -= LA_ONLYSHADOW;
-			else if((R.r.mode & R_SHADOW)==0) lar->mode -= LA_ONLYSHADOW;
 		}
 
 	}
@@ -2115,6 +2124,7 @@ static void init_render_surf(Object *ob)
 
 //					if(flen!=0.0) {
 						vlr= RE_findOrAddVlak(R.totvlak++);
+						vlr->ob= ob;
 						vlr->v1= v1; vlr->v2= v2; vlr->v3= v3; vlr->v4= v4;
 						VECCOPY(vlr->n, n1);
 						vlr->len= flen;
@@ -2251,6 +2261,7 @@ static void init_render_surf(Object *ob)
 					flen= CalcNormFloat4(v1->co, v3->co, v4->co, v2->co, n1);
 					if(flen!=0.0) {
 						vlr= RE_findOrAddVlak(R.totvlak++);
+						vlr->ob= ob;
 						vlr->v1= v1;
 						vlr->v2= v3;
 						vlr->v3= v4;
@@ -2488,6 +2499,7 @@ static void init_render_curve(Object *ob)
 					for(; b<bl->nr; b++) {
 
 						vlr= RE_findOrAddVlak(R.totvlak++);
+						vlr->ob= ob;
 						vlr->v1= RE_findOrAddVert(p2);
 						vlr->v2= RE_findOrAddVert(p1);
 						vlr->v3= RE_findOrAddVert(p3);
@@ -2750,8 +2762,9 @@ void RE_freeRotateBlenderScene(void)
 				MEM_freeN(me->orco);
 				me->orco= 0;
 			}
-			if (rendermesh_uses_displist(me) && (me->subdiv!=me->subdivr)){
+			if (rendermesh_uses_displist(me) && (me->subdiv!=me->subdivr)) {
 				makeDispList(ob);
+				me->subdivdone= 0;	/* needed to prevent multiple used meshes being recalculated */
 			}
 		}
 		else if(ob->type==OB_MBALL) {
@@ -2767,7 +2780,8 @@ void RE_freeRotateBlenderScene(void)
 
 	end_render_textures();
 	end_render_materials();
-
+	end_radio_render();
+	
 	R.totvlak=R.totvert=R.totlamp=R.tothalo= 0;
 }
 
@@ -2856,6 +2870,7 @@ void RE_rotateBlenderScene(void)
 
 	/* layers: render in foreground current 3D window */
 	lay= G.scene->lay;
+	if(G.vd) lay= G.vd->lay;
 	sce= G.scene;
 
 	base= G.scene->base.first;
