@@ -1,5 +1,5 @@
 /**
- * $Id: drawimage.c,v 1.6 2003/04/28 14:43:20 ton Exp $
+ * $Id: drawimage.c,v 1.11 2003/11/19 22:00:12 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -45,7 +46,6 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_editVert.h"
 
 #include "IMB_imbuf_types.h"
 
@@ -68,6 +68,7 @@
 #include "BIF_screen.h"
 #include "BIF_mywindow.h"
 #include "BIF_drawimage.h"
+#include "BIF_resources.h"
 
 /* Modules used */
 #include "mydevice.h"
@@ -258,7 +259,7 @@ void image_changed(SpaceImage *sima, int dotile)
 					tface++;
 				}
 				allqueue(REDRAWVIEW3D, 0);
-				allqueue(REDRAWBUTSGAME, 0);
+				allqueue(REDRAWBUTSEDIT, 0);
 			}
 		}
 	}
@@ -304,7 +305,6 @@ void draw_tfaces(void)
 	TFace *tface;
 	MFace *mface;
 	Mesh *me;
-	unsigned int col;
 	int a;
 	
 	glPointSize(2.0);
@@ -334,7 +334,8 @@ void draw_tfaces(void)
 					setlinestyle(2);
 					/* colors: R=x G=y */
 					
-					if(tface->flag & TF_ACTIVE) cpack(0xFF00); else cpack(0xFFFFFF);
+					if(tface->flag & TF_ACTIVE) cpack(0xFF00); 
+					else cpack(0xFFFFFF);
 	
 					glBegin(GL_LINE_STRIP);
 						glVertex2fv( tface->uv[0] );
@@ -360,21 +361,21 @@ void draw_tfaces(void)
 					
 					glBegin(GL_POINTS);
 					
-					if(tface->flag & TF_SEL1) col= 0x77FFFF; else col= 0xFF70FF;
-					cpack(col);
+					if(tface->flag & TF_SEL1) BIF_ThemeColor(TH_VERTEX_SELECT); 
+					else BIF_ThemeColor(TH_VERTEX); 
 					glVertex2fv(tface->uv[0]);
 					
-					if(tface->flag & TF_SEL2) col= 0x77FFFF; else col= 0xFF70FF;
-					cpack(col);
+					if(tface->flag & TF_SEL2) BIF_ThemeColor(TH_VERTEX_SELECT); 
+					else BIF_ThemeColor(TH_VERTEX); 
 					glVertex2fv(tface->uv[1]);
 					
-					if(tface->flag & TF_SEL3) col= 0x77FFFF; else col= 0xFF70FF;
-					cpack(col);
+					if(tface->flag & TF_SEL3) BIF_ThemeColor(TH_VERTEX_SELECT); 
+					else BIF_ThemeColor(TH_VERTEX); 
 					glVertex2fv(tface->uv[2]);
 					
 					if(mface->v4) {
-						if(tface->flag & TF_SEL4) col= 0x77FFFF; else col= 0xFF70FF;
-						cpack(col);
+						if(tface->flag & TF_SEL4) BIF_ThemeColor(TH_VERTEX_SELECT); 
+						else BIF_ThemeColor(TH_VERTEX); 
 						glVertex2fv(tface->uv[3]);
 					}
 					glEnd();
@@ -413,13 +414,16 @@ static unsigned int *get_part_from_ibuf(ImBuf *ibuf, short startx, short starty,
 void drawimagespace(ScrArea *sa, void *spacedata)
 {
 	ImBuf *ibuf= NULL;
+	float col[3];
 	unsigned int *rect;
 	int x1, y1, xmin, xmax, ymin, ymax;
 	short sx, sy, dx, dy;
 	
-	glClearColor(.1875, .1875, .1875, 0.0); 
+	BIF_GetThemeColor3fv(TH_BACK, col);
+	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 
+
 	curarea->win_swap= WIN_BACK_OK;
 	
 	xmin= curarea->winrct.xmin; xmax= curarea->winrct.xmax;
@@ -441,77 +445,79 @@ void drawimagespace(ScrArea *sa, void *spacedata)
 		cpack(0x404040);
 		glRectf(0.0, 0.0, 1.0, 1.0);
 		draw_tfaces();
-		
-		return;
 	}
-
-	/* calc location */
-	x1= xmin+(curarea->winx-G.sima->zoom*ibuf->x)/2;
-	y1= ymin+(curarea->winy-G.sima->zoom*ibuf->y)/2;
-
-	x1-= G.sima->zoom*G.sima->xof;
-	y1-= G.sima->zoom*G.sima->yof;
-
+	else {
+		/* calc location */
+		x1= xmin+(curarea->winx-G.sima->zoom*ibuf->x)/2;
+		y1= ymin+(curarea->winy-G.sima->zoom*ibuf->y)/2;
+	
+		x1-= G.sima->zoom*G.sima->xof;
+		y1-= G.sima->zoom*G.sima->yof;
 	
-	if(G.sima->flag & SI_EDITTILE) {
-		rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom, (float)G.sima->zoom, ibuf->rect);
 		
-		dx= ibuf->x/G.sima->image->xrep;
-		dy= ibuf->y/G.sima->image->yrep;
-		sy= (G.sima->curtile / G.sima->image->xrep);
-		sx= G.sima->curtile - sy*G.sima->image->xrep;
-
-		sx*= dx;
-		sy*= dy;
-		
-		calc_image_view(G.sima, 'p');	/* pixel */
-		myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
-		
-		cpack(0x0);
-		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRects(sx,  sy,  sx+dx-1,  sy+dy-1); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-		cpack(0xFFFFFF);
-		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRects(sx+1,  sy+1,  sx+dx,  sy+dy); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
-	}
-	else if(G.sima->mode==SI_TEXTURE) {
-		if(G.sima->image->tpageflag & IMA_TILES) {
-			
-			
-			/* just leave this a while */
-			if(G.sima->image->xrep<1) return;
-			if(G.sima->image->yrep<1) return;
-			
-			if(G.sima->curtile >= G.sima->image->xrep*G.sima->image->yrep) 
-				G.sima->curtile = G.sima->image->xrep*G.sima->image->yrep - 1; 
+		if(G.sima->flag & SI_EDITTILE) {
+			rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom, (float)G.sima->zoom, ibuf->rect);
 			
 			dx= ibuf->x/G.sima->image->xrep;
 			dy= ibuf->y/G.sima->image->yrep;
-			
 			sy= (G.sima->curtile / G.sima->image->xrep);
 			sx= G.sima->curtile - sy*G.sima->image->xrep;
 	
 			sx*= dx;
 			sy*= dy;
 			
-			rect= get_part_from_ibuf(ibuf, sx, sy, sx+dx, sy+dy);
+			calc_image_view(G.sima, 'p');	/* pixel */
+			myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
 			
-			/* rect= ibuf->rect; */
-			for(sy= 0; sy+dy<=ibuf->y; sy+= dy) {
-				for(sx= 0; sx+dx<=ibuf->x; sx+= dx) {
-					
-					rectwrite_part(xmin, ymin, xmax, ymax, 
-						x1+sx*G.sima->zoom, y1+sy*G.sima->zoom, dx, dy, (float)G.sima->zoom, (float)G.sima->zoom, rect);
+			cpack(0x0);
+			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRects(sx,  sy,  sx+dx-1,  sy+dy-1); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+			cpack(0xFFFFFF);
+			glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRects(sx+1,  sy+1,  sx+dx,  sy+dy); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
+		}
+		else if(G.sima->mode==SI_TEXTURE) {
+			if(G.sima->image->tpageflag & IMA_TILES) {
+				
+				
+				/* just leave this a while */
+				if(G.sima->image->xrep<1) return;
+				if(G.sima->image->yrep<1) return;
+				
+				if(G.sima->curtile >= G.sima->image->xrep*G.sima->image->yrep) 
+					G.sima->curtile = G.sima->image->xrep*G.sima->image->yrep - 1; 
+				
+				dx= ibuf->x/G.sima->image->xrep;
+				dy= ibuf->y/G.sima->image->yrep;
+				
+				sy= (G.sima->curtile / G.sima->image->xrep);
+				sx= G.sima->curtile - sy*G.sima->image->xrep;
+		
+				sx*= dx;
+				sy*= dy;
+				
+				rect= get_part_from_ibuf(ibuf, sx, sy, sx+dx, sy+dy);
+				
+				/* rect= ibuf->rect; */
+				for(sy= 0; sy+dy<=ibuf->y; sy+= dy) {
+					for(sx= 0; sx+dx<=ibuf->x; sx+= dx) {
+						
+						rectwrite_part(xmin, ymin, xmax, ymax, 
+							x1+sx*G.sima->zoom, y1+sy*G.sima->zoom, dx, dy, (float)G.sima->zoom, (float)G.sima->zoom, rect);
+					}
 				}
+				
+				MEM_freeN(rect);
 			}
-			
-			MEM_freeN(rect);
+			else 
+				rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom,(float)G.sima->zoom, ibuf->rect);
+		
+			draw_tfaces();
 		}
-		else 
-			rectwrite_part(xmin, ymin, xmax, ymax, x1, y1, ibuf->x, ibuf->y, (float)G.sima->zoom,(float)G.sima->zoom, ibuf->rect);
 	
-		draw_tfaces();
+		calc_image_view(G.sima, 'f');	/* float */
 	}
-
-	calc_image_view(G.sima, 'f');	/* float */
+	myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
+	draw_area_emboss(sa);
+	myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
 }
 
 void image_viewmove(void)
