@@ -1,5 +1,5 @@
 /**
- * $Id: drawipo.c,v 1.9 2003/07/20 23:04:08 hos Exp $
+ * $Id: drawipo.c,v 1.25 2003/11/18 11:22:16 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,6 +32,7 @@
 
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -47,6 +48,7 @@
 #include "BMF_Api.h"
 
 #include "BLI_blenlib.h"
+#include "BLI_arithb.h"
 
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
@@ -62,6 +64,7 @@
 #include "BKE_curve.h"
 #include "BKE_ipo.h"
 #include "BKE_global.h"
+#include "BKE_key.h"
 
 #include "BIF_gl.h"
 #include "BIF_resources.h"
@@ -77,9 +80,9 @@
 #include "BSE_editnla_types.h"
 
 #include "mydevice.h"
-#include "interface.h"
 #include "ipo.h" /* retains old stuff */
 #include "blendef.h"
+#include "butspace.h"	// shouldnt be...
 
 /* local define... also used in editipo ... */
 #define ISPOIN(a, b, c)                       ( (a->b) && (a->c) )  
@@ -115,7 +118,6 @@ static void scroll_prstr(float x, float y, float val, char dir, int disptype)
 	
 	len= strlen(str);
 	if(dir=='h') x-= 4*len;
-	else y-= 4*len;
 	
 	if(dir=='v' && disptype==IPO_DISPDEGR) {
 		str[len]= 186; /* Degree symbol */
@@ -207,7 +209,7 @@ void draw_ipogrid(void)
 	step= (G.v2d->mask.xmax-G.v2d->mask.xmin+1)/IPOSTEP;
 	
 	if(curarea->spacetype==SPACE_SOUND) glColor3ub(0x70, 0x70, 0x60);
-	else glColor3ub(0x40, 0x40, 0x40);
+	else BIF_ThemeColor(TH_GRID);
 	
 	for(a=0; a<step; a++) {
 		glBegin(GL_LINE_STRIP);
@@ -219,7 +221,7 @@ void draw_ipogrid(void)
 	vec2[0]= vec1[0]-= 0.5*ipogrid_dx;
 	
 	if(curarea->spacetype==SPACE_SOUND) glColor3ub(0x80, 0x80, 0x70);
-	else glColor3ub(0x50, 0x50, 0x50);
+	else  BIF_ThemeColorShade(TH_GRID, 16);
 	
 	step++;
 	for(a=0; a<=step; a++) {
@@ -236,7 +238,7 @@ void draw_ipogrid(void)
 		
 		step= (curarea->winy+1)/IPOSTEP;
 		
-		glColor3ub(0x40, 0x40, 0x40);
+		BIF_ThemeColor(TH_GRID);
 		for(a=0; a<=step; a++) {
 			glBegin(GL_LINE_STRIP);
 			glVertex2fv(vec1); glVertex2fv(vec2);
@@ -247,7 +249,7 @@ void draw_ipogrid(void)
 		step++;
 		
 		if(curarea->spacetype==SPACE_IPO) {
-			glColor3ub(0x50, 0x50, 0x50);
+			BIF_ThemeColorShade(TH_GRID, 16);
 			for(a=0; a<step; a++) {
 				glBegin(GL_LINE_STRIP);
 				glVertex2fv(vec1); glVertex2fv(vec2);
@@ -257,7 +259,7 @@ void draw_ipogrid(void)
 		}
 	}
 	
-	glColor3ub(0, 0, 0);
+	BIF_ThemeColorShade(TH_GRID, -50);
 	
 	if (curarea->spacetype!=SPACE_ACTION && curarea->spacetype!=SPACE_NLA)
 	{	/* Horizontal axis */
@@ -368,6 +370,7 @@ void view2d_zoom(View2D *v2d, float factor, int winx, int winy) {
 	test_view2d(v2d, winx, winy);
 }
 
+
 void test_view2d(View2D *v2d, int winx, int winy)
 {
 	/* cur is not allowed to be larger than max, smaller than min, or outside of tot */
@@ -388,8 +391,8 @@ void test_view2d(View2D *v2d, int winx, int winy)
 	if (v2d->keepzoom & V2D_LOCKZOOM_X)
 		v2d->cur.xmax=v2d->cur.xmin+((float)winx);
 
-	if(v2d->keepzoom & V2D_KEEPZOOM) {
-		/* do not test for min/max: usiig curarea try to fixate zoom */
+	if(v2d->keepzoom) {
+		
 		zoom= ((float)winx)/dx;
 		
 		if(zoom<v2d->minzoom || zoom>v2d->maxzoom) {
@@ -414,45 +417,43 @@ void test_view2d(View2D *v2d, int winx, int winy)
 			cur->ymin= temp-0.5*dy;
 			cur->ymax= temp+0.5*dy;
 		}
-		
-		
 	}
-	
-	else{
+
+	if(v2d->keepaspect) {
+		short do_x=0, do_y=0;
 		
-		/* end test */
+		/* when a window edge changes, the aspect ratio can't be used to
+		   find which is the best new 'cur' rect. thats why it stores 'old' */
+		if(winx!=v2d->oldwinx) do_x= 1;
+		if(winy!=v2d->oldwiny) do_y= 1;
+		v2d->oldwinx= winx; 
+		v2d->oldwiny= winy;
 		
-		if(dx<v2d->min[0]) {
-			dx= v2d->min[0];
-			temp= 0.5*(cur->xmax+cur->xmin);
-			cur->xmin= temp-0.5*dx;
-			cur->xmax= temp+0.5*dx;
+		dx= (cur->ymax-cur->ymin)/(cur->xmax-cur->xmin);
+		dy= ((float)winy)/((float)winx);
+		
+		if(do_x==do_y) {
+			if( dy > 1.0) do_x= 0; else do_x= 1;
 		}
-		else if(dx>v2d->max[0]) {
-			dx= v2d->max[0];
-			temp= 0.5*(cur->xmax+cur->xmin);
-			cur->xmin= temp-0.5*dx;
-			cur->xmax= temp+0.5*dx;
-		}		
-		
-		if(dy<v2d->min[1]) {
-			dy= v2d->min[1];
-			temp= 0.5*(cur->ymax+cur->ymin);
-			cur->ymin= temp-0.5*dy;
-			cur->ymax= temp+0.5*dy;
+
+		if( do_x ) {
+			
+			/* portrait window: correct for x */
+			dx= cur->ymax-cur->ymin;
+			temp= (cur->xmax+cur->xmin);
+			
+			cur->xmin= temp/2.0 - 0.5*dx/dy;
+			cur->xmax= temp/2.0 + 0.5*dx/dy;
+		}
+		else {
+			dx= cur->xmax-cur->xmin;
+			temp= (cur->ymax+cur->ymin);
+			
+			cur->ymin= temp/2.0 - 0.5*dy*dx;
+			cur->ymax= temp/2.0 + 0.5*dy*dx;
 		}
-		else if(dy>v2d->max[1]) {
-			dy= v2d->max[1];
-			temp= 0.5*(cur->ymax+cur->ymin);
-			cur->ymin= temp-0.5*dy;
-			cur->ymax= temp+0.5*dy;
-		}		
-		
 	}
 	
-	
-	
-	
 	if(v2d->keeptot) {
 		dx= cur->xmax-cur->xmin;
 		dy= cur->ymax-cur->ymin;
@@ -519,36 +520,9 @@ void test_view2d(View2D *v2d, int winx, int winy)
 			}
 		}
 	}
-	
-	if(v2d->keepaspect) {
-		dx= (cur->ymax-cur->ymin)/(cur->xmax-cur->xmin);
-		dy= ((float)winy)/((float)winx);
-		
-		/* dx/dy is the total aspect */
-		
-		/* this exception is for buttons...keepzoom doesnt work proper */
-		if(v2d->keepzoom) fac= dy;
-		else fac= dx/dy;
-		
-		if(fac>1.0) {
-			
-			/* portrait window: correct for x */
-			dx= cur->ymax-cur->ymin;
-			temp= (cur->xmax+cur->xmin);
-			
-			cur->xmin= temp/2.0 - 0.5*dx/dy;
-			cur->xmax= temp/2.0 + 0.5*dx/dy;
-		}
-		else {
-			dx= cur->xmax-cur->xmin;
-			temp= (cur->ymax+cur->ymin);
-			
-			cur->ymin= temp/2.0 - 0.5*dy*dx;
-			cur->ymax= temp/2.0 + 0.5*dy*dx;
-		}
-	}
 }
 
+
 void calc_scrollrcts(View2D *v2d, int winx, int winy)
 {
 	v2d->mask.xmin= v2d->mask.ymin= 0;
@@ -574,23 +548,23 @@ void calc_scrollrcts(View2D *v2d, int winx, int winy)
 		if(v2d->scroll & L_SCROLL) {
 			v2d->vert= v2d->mask;
 			v2d->vert.xmax= SCROLLB;
-			v2d->mask.xmin= SCROLLB+1;
+			v2d->mask.xmin= SCROLLB;
 		}
 		else if(v2d->scroll & R_SCROLL) {
 			v2d->vert= v2d->mask;
 			v2d->vert.xmin= v2d->vert.xmax-SCROLLB;
-			v2d->mask.xmax= v2d->vert.xmin-1;
+			v2d->mask.xmax= v2d->vert.xmin;
 		}
 		
 		if(v2d->scroll & B_SCROLL) {
 			v2d->hor= v2d->mask;
 			v2d->hor.ymax= SCROLLH;
-			v2d->mask.ymin= SCROLLH+1;
+			v2d->mask.ymin= SCROLLH;
 		}
 		else if(v2d->scroll & T_SCROLL) {
 			v2d->hor= v2d->mask;
 			v2d->hor.ymin= v2d->hor.ymax-SCROLLH;
-			v2d->mask.ymax= v2d->hor.ymin-1;
+			v2d->mask.ymax= v2d->hor.ymin;
 		}
 	}
 }
@@ -643,42 +617,39 @@ void drawscroll(int disptype)
 {
 	rcti vert, hor;
 	float fac, dfac, val, fac2, tim;
-	unsigned int dark, darker, light;
+	int darker, dark, light, lighter;
 	
 	vert= (G.v2d->vert);
 	hor= (G.v2d->hor);
 	
-	darker= 0x525252;
-	dark= 0x656565;
-	light= 0x989898;
+	darker= -40;
+	dark= 0;
+	light= 20;
+	lighter= 50;
 	
-	cpack(dark);
 	if(G.v2d->scroll & HOR_SCROLL) {
+		
+		BIF_ThemeColorShade(TH_SHADE1, light);
 		glRecti(hor.xmin,  hor.ymin,  hor.xmax,  hor.ymax);
-		glColor3ub(0, 0, 0);
-		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRecti(hor.xmin,  hor.ymin,  hor.xmax,  hor.ymax); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 		
 		/* slider */
 		fac= (G.v2d->cur.xmin- G.v2d->tot.xmin)/(G.v2d->tot.xmax-G.v2d->tot.xmin);
 		if(fac<0.0) fac= 0.0;
-		horxmin= hor.xmin+fac*(hor.xmax-hor.xmin) + 1;
+		horxmin= hor.xmin+fac*(hor.xmax-hor.xmin);
 		
 		fac= (G.v2d->cur.xmax- G.v2d->tot.xmin)/(G.v2d->tot.xmax-G.v2d->tot.xmin);
 		if(fac>1.0) fac= 1.0;
-		horxmax= hor.xmin+fac*(hor.xmax-hor.xmin) -1;
+		horxmax= hor.xmin+fac*(hor.xmax-hor.xmin);
 		
-		if(horxmin > horxmax-2) horxmin= horxmax-2;
-		
-		glColor3ub(0x78, 0x78, 0x78);
-		glRecti(horxmin,  hor.ymin+1,  horxmax,  hor.ymax-1);
-		
-		cpack(light);
-		sdrawline(horxmin,  hor.ymax-1,  horxmax,  hor.ymax-1);  /* top */
-		sdrawline(horxmin,  hor.ymin+1,  horxmin,  hor.ymax-1);  /* left */
-		cpack(darker);
-		sdrawline(horxmin,  hor.ymin+1,  horxmax,  hor.ymin+1);  /* bottom */
-		sdrawline(horxmax,  hor.ymin+1,  horxmax,  hor.ymax-1);  /* right */
+		if(horxmin > horxmax) horxmin= horxmax;
 		
+		BIF_ThemeColorShade(TH_SHADE1, dark);
+		glRecti(horxmin,  hor.ymin,  horxmax,  hor.ymax);
+
+		/* decoration bright line */
+		BIF_ThemeColorShade(TH_SHADE1, lighter);
+		sdrawline(hor.xmin, hor.ymax, hor.xmax, hor.ymax);
+
 		/* the numbers: convert ipogrid_startx and -dx to scroll coordinates */
 		fac= (ipogrid_startx- G.v2d->cur.xmin)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
 		fac= hor.xmin+fac*(hor.xmax-hor.xmin);
@@ -686,7 +657,7 @@ void drawscroll(int disptype)
 		dfac= (ipogrid_dx)/(G.v2d->cur.xmax-G.v2d->cur.xmin);
 		dfac= dfac*(hor.xmax-hor.xmin);
 		
-		glColor3ub(0, 0, 0);
+		BIF_ThemeColor(TH_TEXT);
 		val= ipogrid_startx;
 		while(fac < hor.xmax) {
 			
@@ -697,8 +668,16 @@ void drawscroll(int disptype)
 				scroll_prstr(fac, 3.0+(float)(hor.ymin), tim+G.scene->r.frs_sec*fac2/100.0, 'h', disptype);
 			}
 			else if(curarea->spacetype==SPACE_SOUND) {
-				fac2= val/(float)G.scene->r.frs_sec;
-				scroll_prstr(fac, 3.0+(float)(hor.ymin), fac2, 'h', disptype);
+				SpaceSound *ssound= curarea->spacedata.first;
+				
+				if(ssound->flag & SND_DRAWFRAMES) {
+					ipomachtx= 1;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
+				}
+				else {
+					fac2= val/(float)G.scene->r.frs_sec;
+					scroll_prstr(fac, 3.0+(float)(hor.ymin), fac2, 'h', disptype);
+				}
 			}
 			else {
 				scroll_prstr(fac, 3.0+(float)(hor.ymin), val, 'h', disptype);
@@ -708,35 +687,32 @@ void drawscroll(int disptype)
 			val+= ipogrid_dx;
 		}
 	}
-	cpack(dark);
+	
 	if(G.v2d->scroll & VERT_SCROLL) {
+		BIF_ThemeColorShade(TH_SHADE1, light);
 		glRecti(vert.xmin,  vert.ymin,  vert.xmax,  vert.ymax);
-		glColor3ub(0, 0, 0);
-		
-		glPolygonMode(GL_FRONT_AND_BACK, GL_LINE); glRecti(vert.xmin,  vert.ymin,  vert.xmax,  vert.ymax); glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 		
 		/* slider */
 		fac= (G.v2d->cur.ymin- G.v2d->tot.ymin)/(G.v2d->tot.ymax-G.v2d->tot.ymin);
 		if(fac<0.0) fac= 0.0;
-		vertymin= vert.ymin+fac*(vert.ymax-vert.ymin) + 1;
+		vertymin= vert.ymin+fac*(vert.ymax-vert.ymin);
 		
 		fac= (G.v2d->cur.ymax- G.v2d->tot.ymin)/(G.v2d->tot.ymax-G.v2d->tot.ymin);
 		if(fac>1.0) fac= 1.0;
-		vertymax= vert.ymin+fac*(vert.ymax-vert.ymin) -1;
+		vertymax= vert.ymin+fac*(vert.ymax-vert.ymin);
 		
-		if(vertymin > vertymax-2) vertymin= vertymax-2;
-		
-		glColor3ub(0x78, 0x78, 0x78);
-		glRecti(vert.xmin+1,  vertymin,  vert.xmax-1,  vertymax);
-		
-		cpack(light);
-		
-		sdrawline(vert.xmin+1,  vertymax,  vert.xmax-1,  vertymax);  /* top */
-		sdrawline(vert.xmin+1,  vertymin,  vert.xmin+1,  vertymax);  /* left */
-		cpack(darker);
-		sdrawline(vert.xmin+1,  vertymin,  vert.xmax-1,  vertymin);  /* bottom */
-		sdrawline(vert.xmax-1,  vertymin,  vert.xmax-1,  vertymax);  /* right */
+		if(vertymin > vertymax) vertymin= vertymax;
 		
+		BIF_ThemeColorShade(TH_SHADE1, dark);
+		glRecti(vert.xmin,  vertymin,  vert.xmax,  vertymax);
+
+		/* decoration black line */
+		BIF_ThemeColorShade(TH_SHADE1, darker);
+		if(G.v2d->scroll & HOR_SCROLL) 
+			sdrawline(vert.xmax, vert.ymin+SCROLLH, vert.xmax, vert.ymax);
+		else 
+			sdrawline(vert.xmax, vert.ymin, vert.xmax, vert.ymax);
+
 		/* the numbers: convert ipogrid_starty and -dy to scroll coordinates */
 		fac= (ipogrid_starty- G.v2d->cur.ymin)/(G.v2d->cur.ymax-G.v2d->cur.ymin);
 		fac= vert.ymin+SCROLLH+fac*(vert.ymax-vert.ymin-SCROLLH);
@@ -745,7 +721,7 @@ void drawscroll(int disptype)
 		dfac= dfac*(vert.ymax-vert.ymin-SCROLLH);
 		
 		if(curarea->spacetype==SPACE_SEQ) {
-			glColor3ub(0, 0, 0);
+			BIF_ThemeColor(TH_TEXT);
 			val= ipogrid_starty;
 			fac+= 0.5*dfac;
 			while(fac < vert.ymax) {
@@ -760,10 +736,10 @@ void drawscroll(int disptype)
 			/* No digits on vertical axis in action mode! */
 		}
 		else {
-			glColor3ub(0, 0, 0);
+			BIF_ThemeColor(TH_TEXT);
 			val= ipogrid_starty;
 			while(fac < vert.ymax) {
-				scroll_prstr((float)(vert.xmax)-3.0, fac, val, 'v', disptype);
+				scroll_prstr((float)(vert.xmax)-14.0, fac, val, 'v', disptype);
 				fac+= dfac;
 				val+= ipogrid_dy;
 			}
@@ -788,27 +764,25 @@ static void draw_ipobuts(SpaceIpo *sipo)
 		if(tot<area->winy) sipo->butofs= 0;
 	}
 	
-	drawedge(v2d->mask.xmax+3, 0, v2d->mask.xmax+3, area->winy);
-	glColor3ub(0x7f, 0x70, 0x70);
-	
-	glRects(v2d->mask.xmax+6,  0,  area->winx,  area->winy);
+	BIF_ThemeColor(TH_SHADE2);
+	glRects(v2d->mask.xmax,  0,  area->winx,  area->winy);
 	
+	cpack(0x0);
+	sdrawline(v2d->mask.xmax, 0, v2d->mask.xmax, area->winy);
+
 	if(sipo->totipo==0) return;
 	if(sipo->editipo==0) return;
 	
 	sprintf(naam, "ipowin %d", area->win);
 	block= uiNewBlock(&area->uiblocks, naam, UI_EMBOSSN, UI_HELV, area->win);
-	uiBlockSetCol(block, BUTRUST);
-	
+
 	ei= sipo->editipo;
 	y= area->winy-30+sipo->butofs;
 	for(a=0; a<sipo->totipo; a++, ei++, y-=IPOBUTY) {
 		
 		but= uiDefButI(block, TOG|BIT|a, a+1, ei->name,  v2d->mask.xmax+18, y, IPOBUTX-15, IPOBUTY-1, &(sipo->rowbut), 0, 0, 0, 0, "");
-			/* XXXXX, is this, the sole caller
-			 * of this function, really necessary?
-			 */
-		uiButSetFlag(but, UI_TEXT_LEFT);
+		// no hilite, its not visible, but most of all the winmatrix is not correct later on...
+		uiButSetFlag(but, UI_TEXT_LEFT|UI_NO_HILITE);
 		
 		if(ei->icu) {
 			cpack(ei->col);
@@ -866,29 +840,36 @@ static void draw_ipovertices(int sel)
 	EditIpo *ei;
 	BezTriple *bezt;
 	float v1[2];
-	unsigned int col;
 	int val, ok, nr, a, b;
 	
 	if(G.f & G_PICKSEL) return;
 	
 	glPointSize(3.0);
-	glBegin(GL_POINTS);
 	
 	ei= G.sipo->editipo;
 	for(nr=0; nr<G.sipo->totipo; nr++, ei++) {
 		if ISPOIN(ei, flag & IPO_VISIBLE, icu) {
 			
 			if(G.sipo->showkey) {
-				if(sel) col= 0xFFFFFF; else col= 0x0;
+				if(sel) BIF_ThemeColor(TH_TEXT_HI);
+				 else BIF_ThemeColor(TH_TEXT);
 			} else if(ei->flag & IPO_EDIT) {
-				if(sel) col= 0x77FFFF; else col= 0xFF70FF;
+				if(sel) BIF_ThemeColor(TH_VERTEX_SELECT); 
+				else BIF_ThemeColor(TH_VERTEX);
 			} else {
-				if(sel) col= 0xFFFFFF; else col= 0x0;
+				if(sel) BIF_ThemeColor(TH_TEXT_HI);
+				 else BIF_ThemeColor(TH_TEXT);
+				 
 				val= (ei->icu->flag & IPO_SELECT)!=0;
 				if(sel != val) continue;
 			}
+
+			/* We can't change the color in the middle of
+			 * GL_POINTS because then Blender will segfault
+			 * on TNT2 / Linux with NVidia's drivers
+			 * (at least up to ver. 4349) */		
 			
-			cpack(col);
+			glBegin(GL_POINTS);
 			
 			bezt= ei->icu->bezt;
 			a= ei->icu->totvert;
@@ -935,10 +916,11 @@ static void draw_ipovertices(int sel)
 				
 				bezt++;
 			}
+
+			glEnd();
 		}
 	}
 	
-	glEnd();
 	glPointSize(1.0);
 }
 
@@ -1037,7 +1019,8 @@ static void draw_ipocurves(int sel)
 				icu= ei->icu;	
 				
 				/* curve */
-				if(G.sipo->showkey) glColor3ub(0, 0, 0); else cpack(ei->col);
+				if(G.sipo->showkey) BIF_ThemeColor(TH_TEXT); 
+				else cpack(ei->col);
 				
 				/* cyclic */
 				if(icu->extrap & IPO_CYCL) {
@@ -1232,7 +1215,7 @@ static void draw_cfra(SpaceIpo *sipo)
 	vec[0]*= G.scene->r.framelen;
 	
 	vec[1]= v2d->cur.ymin;
-	glColor3ub(0x60, 0xc0, 0x40);
+	glColor3ub(0x60, 0xc0, 0x40);	// no theme, should be global color once...
 	glLineWidth(2.0);
 	
 	glBegin(GL_LINE_STRIP);
@@ -1246,7 +1229,7 @@ static void draw_cfra(SpaceIpo *sipo)
 		if(ob && ob->sf!=0.0 && (ob->ipoflag & OB_OFFS_OB) ) {
 			vec[0]-= ob->sf;
 			
-			glColor3ub(0x10, 0x60, 0);
+			BIF_ThemeColorShade(TH_HILITE, -30);
 			
 			glBegin(GL_LINE_STRIP);
 			glVertex2fv(vec);
@@ -1326,28 +1309,317 @@ static void draw_key(SpaceIpo *sipo, int visible)
 	setlinestyle(0);
 }
 
+/* ************************** buttons *********************** */
+
+
+#define B_SETSPEED		3401
+#define B_MUL_IPO		3402
+#define B_TRANS_IPO		3403
+#define B_IPO_NONE		3404
+
+static float hspeed= 0;
+
+
+static void boundbox_ipo_visible(SpaceIpo *si)
+{
+	EditIpo *ei;
+	Key *key;
+	KeyBlock *kb;
+	int a, first= 1;
+
+	ei= si->editipo;
+	if(ei==0)
+		return;
+
+	for(a=0; a<si->totipo; a++, ei++) {
+		
+		if(ei->icu) {
+			if(ei->flag & IPO_VISIBLE) {
+	
+				boundbox_ipocurve(ei->icu);
+				if(first) {
+					si->v2d.tot= ei->icu->totrct;
+					first= 0;
+				}
+				else BLI_union_rctf(&(si->v2d.tot), &(ei->icu->totrct));
+			}
+		}
+	}
+	/* keylines? */
+	if(si->blocktype==ID_KE) {
+		key= (Key *)si->from;
+		if(key && key->block.first) {
+			kb= key->block.first;
+			if(kb->pos < si->v2d.tot.ymin) si->v2d.tot.ymin= kb->pos;
+			kb= key->block.last;
+			if(kb->pos > si->v2d.tot.ymax) si->v2d.tot.ymax= kb->pos;
+		}
+	}
+	si->tot= si->v2d.tot;
+}
+
+
+/* is used for both read and write... */
+static void ipo_editvertex_buts(uiBlock *block, SpaceIpo *si, float min, float max)
+{
+	EditIpo *ei;
+	BezTriple *bezt;
+	float median[3];
+	int a, b, tot, iskey=0;
+	
+	median[0]= median[1]= median[2]= 0.0;
+	tot= 0;
+	
+	ei= G.sipo->editipo;
+	for(a=0; a<G.sipo->totipo; a++, ei++) {
+		
+		if ISPOIN(ei, flag & IPO_VISIBLE, icu) {
+			if( (ei->flag & IPO_EDIT) || G.sipo->showkey) {
+
+				if(ei->icu->bezt) {
+					bezt= ei->icu->bezt;
+					b= ei->icu->totvert;
+					while(b--) {
+						// all three selected 
+						if(bezt->f2 & 1) {
+							VecAddf(median, median, bezt->vec[1]);
+							tot++;
+						}
+						else {
+							if(bezt->f1 & 1) {
+								VecAddf(median, median, bezt->vec[0]);
+								tot++;
+							}
+							if(bezt->f3 & 1) {
+								VecAddf(median, median, bezt->vec[2]);
+								tot++;
+							}
+						}
+						bezt++;
+					}
+					
+				}
+			}
+		}
+	}
+	/* check for keys */
+	if(tot==0) {
+		if(G.sipo->blocktype==ID_KE) {
+			Key *key= (Key *)G.sipo->from;
+			KeyBlock *kb;
+			
+			if(key==0) return;
+			iskey= 1;
+			
+			kb= key->block.first;
+			while(kb) {
+				if(kb->flag & SELECT) {
+					median[1]+= kb->pos;
+					tot++;
+				}
+				kb= kb->next;
+			}
+		}
+	}
+	if(tot==0) return;
+
+	median[0] /= (float)tot;
+	median[1] /= (float)tot;
+	median[2] /= (float)tot;
+	
+	if(block) {	// buttons
+	
+		VECCOPY(si->median, median);
+		
+		if(tot==1) {
+			if(iskey) 
+				uiDefButF(block, NUM, B_TRANS_IPO, "Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
+			else {
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Vertex Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+			}
+		}
+		else {
+			if(iskey) 
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median Key Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 10, 0, "");
+			else {
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median X:",	10, 100, 300, 19, &(si->median[0]), min, max, 100, 0, "");
+				uiDefButF(block, NUM, B_TRANS_IPO, "Median Y:",	10, 80, 300, 19, &(si->median[1]), min, max, 100, 0, "");
+			}
+		}
+	}
+	else if(iskey) {	// apply
+		VecSubf(median, si->median, median);
+
+		if(G.sipo->blocktype==ID_KE) {
+			Key *key= (Key *)G.sipo->from;
+			KeyBlock *kb;
+			
+			if(key==0) return;
+			
+			kb= key->block.first;
+			while(kb) {
+				if(kb->flag & SELECT) {
+					kb->pos+= median[1];
+					tot++;
+				}
+				kb= kb->next;
+			}			
+			sort_keys(key);
+		}
+	}
+	else {
+		
+		VecSubf(median, si->median, median);
+
+		ei= G.sipo->editipo;
+		for(a=0; a<G.sipo->totipo; a++, ei++) {
+			
+			if ISPOIN(ei, flag & IPO_VISIBLE, icu) {
+				if( (ei->flag & IPO_EDIT) || G.sipo->showkey) {
+
+					if(ei->icu->bezt) {
+						bezt= ei->icu->bezt;
+						b= ei->icu->totvert;
+						while(b--) {
+							// all three selected
+							if(bezt->f2 & 1) {
+								VecAddf(bezt->vec[0], bezt->vec[0], median);
+								VecAddf(bezt->vec[1], bezt->vec[1], median);
+								VecAddf(bezt->vec[2], bezt->vec[2], median);
+							}
+							else {
+								if(bezt->f1 & 1) {
+									VecAddf(bezt->vec[0], bezt->vec[0], median);
+								}
+								if(bezt->f3 & 1) {
+									VecAddf(bezt->vec[2], bezt->vec[2], median);
+								}
+							}
+							bezt++;
+						}
+						
+					}
+				}
+			}
+		}
+	}
+}
+
+void do_ipobuts(unsigned short event)
+{
+
+	switch(event) {
+	case B_SETSPEED:
+		set_speed_editipo(hspeed);
+		break;
+	case B_MUL_IPO:
+		scale_editipo();
+		allqueue(REDRAWIPO, 0);
+		break;
+	case B_TRANS_IPO:
+		ipo_editvertex_buts(NULL, G.sipo, 0.0, 0.0);
+		editipo_changed(G.sipo, 1);
+		allqueue(REDRAWIPO, 0);
+		break;
+	}
+}
+
+
+static void ipo_panel_properties(short cntrl)	// IPO_HANDLER_PROPERTIES
+{
+	extern int totipo_vis;	// editipo.c
+	uiBlock *block;
+	float min, max;
+	
+	block= uiNewBlock(&curarea->uiblocks, "ipo_panel_properties", UI_EMBOSS, UI_HELV, curarea->win);
+	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
+	uiSetPanelHandler(IPO_HANDLER_PROPERTIES);  // for close and esc
+	if(uiNewPanel(curarea, block, "Transform Properties", "Ipo", 10, 230, 318, 204)==0) return;
+
+
+	boundbox_ipo_visible(G.sipo);	// should not be needed... transform/draw calls should update
+	
+	/* calculate a nice range for the button */
+	min= MIN2(G.sipo->tot.xmin, G.sipo->tot.ymin)-100.0;
+	min= MIN2(min, -100);
+	max= MAX2(G.sipo->tot.xmax, G.sipo->tot.ymax)+100.0;
+	max= MAX2(max, 100);
+
+	if(G.sipo->ipo && G.sipo->ipo->curve.first && totipo_vis) {
+		extern int totipo_vertsel;	// editipo.c
+		uiDefBut(block, LABEL, 0, "Visible curves",		10, 200, 150, 19, NULL, 1.0, 0.0, 0, 0, "");
+		
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmin:",		10, 180, 150, 19, &G.sipo->tot.xmin, min, max, 100, 0, "");
+		uiDefButF(block, NUM, B_MUL_IPO, "Xmax:",		160, 180, 150, 19, &G.sipo->tot.xmax, min, max, 100, 0, "");
+		
+		uiDefButF(block, NUM, B_MUL_IPO, "Ymin:",		10, 160, 150, 19, &G.sipo->tot.ymin, min, max, 100, 0, "");
+		uiDefButF(block, NUM, B_MUL_IPO, "Ymax:",		160, 160, 150, 19, &G.sipo->tot.ymax, min, max, 100, 0, "");
+
+		/* SPEED BUTTON */
+		if(totipo_vertsel) {
+			uiDefButF(block, NUM, B_IPO_NONE, "Speed:",			10,130,150,19, &hspeed, 0.0, 180.0, 1, 0, "");
+			uiDefBut(block, BUT, B_SETSPEED,"SET",			160,130,50,19, 0, 0, 0, 0, 0, "");
+		}
+	}
+
+	/* this one also does keypositions */
+	if(G.sipo->ipo) ipo_editvertex_buts(block, G.sipo, min, max);
+}
+
+static void ipo_blockhandlers(ScrArea *sa)
+{
+	SpaceIpo *sipo= sa->spacedata.first;
+	short a;
+
+	/* warning; blocks need to be freed each time, handlers dont remove (for ipo moved to drawipospace) */
+
+	for(a=0; a<SPACE_MAXHANDLER; a+=2) {
+		switch(sipo->blockhandler[a]) {
+
+		case IPO_HANDLER_PROPERTIES:
+			ipo_panel_properties(sipo->blockhandler[a+1]);
+			break;
+		
+		}
+		/* clear action value for event */
+		sipo->blockhandler[a+1]= 0;
+	}
+	uiDrawBlocksPanels(sa, 0);
+
+}
+
+
 void drawipospace(ScrArea *sa, void *spacedata)
 {
-	SpaceIpo *sipo= curarea->spacedata.first;
+	SpaceIpo *sipo= sa->spacedata.first;
 	View2D *v2d= &sipo->v2d;
 	EditIpo *ei;
+	float col[3];
 	int ofsx, ofsy, a, disptype;
 
+	bwin_clear_viewmat(sa->win);	/* clear buttons view */
+	glLoadIdentity();
+	
+	uiFreeBlocksWin(&sa->uiblocks, sa->win);	/* for panel handler to work */
+	
 	v2d->hor.xmax+=IPOBUTX;
-	calc_scrollrcts(G.v2d, curarea->winx, curarea->winy);
+	calc_scrollrcts(G.v2d, sa->winx, sa->winy);
 
+	BIF_GetThemeColor3fv(TH_BACK, col);
+	glClearColor(col[0], col[1], col[2], 0.0); 
 
 	if (sipo->pin)
-		glClearColor(.45, .40, .40, 0.0);	// litepink
+		glClearColor(col[0]+0.05,col[1],col[2], 0.0);	// litepink
 	else
-		glClearColor(.45, .45, .45, 0.0);
+		glClearColor(col[0],col[1],col[2], 0.0);
 
 	glClear(GL_COLOR_BUFFER_BIT);
 	
-	if(curarea->winx>SCROLLB+10 && curarea->winy>SCROLLH+10) {
+	if(sa->winx>SCROLLB+10 && sa->winy>SCROLLH+10) {
 		if(v2d->scroll) {	
-			ofsx= curarea->winrct.xmin;	// ivm mywin 
-			ofsy= curarea->winrct.ymin;
+			ofsx= sa->winrct.xmin;	// ivm mywin 
+			ofsy= sa->winrct.ymin;
 			glViewport(ofsx+v2d->mask.xmin,  ofsy+v2d->mask.ymin, ( ofsx+v2d->mask.xmax-1)-(ofsx+v2d->mask.xmin)+1, ( ofsy+v2d->mask.ymax-1)-( ofsy+v2d->mask.ymin)+1); 
 			glScissor(ofsx+v2d->mask.xmin,  ofsy+v2d->mask.ymin, ( ofsx+v2d->mask.xmax-1)-(ofsx+v2d->mask.xmin)+1, ( ofsy+v2d->mask.ymax-1)-( ofsy+v2d->mask.ymin)+1);
 		} 
@@ -1399,12 +1671,12 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		draw_ipovertices(1);
 		
 		/* restore viewport */
-		mywinset(curarea->win);
+		mywinset(sa->win);
 		
-		if(curarea->winx>SCROLLB+10 && curarea->winy>SCROLLH+10) {
+		if(sa->winx>SCROLLB+10 && sa->winy>SCROLLH+10) {
 			
-			/* ortho at pixel level curarea */
-			myortho2(-0.5, curarea->winx+0.5, -0.5, curarea->winy+0.5);
+			/* ortho at pixel level sa */
+			myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
 			
 			if(v2d->scroll) {
 				drawscroll(disptype);
@@ -1419,7 +1691,14 @@ void drawipospace(ScrArea *sa, void *spacedata)
 		draw_ipogrid();
 	}
 	
-	curarea->win_swap= WIN_BACK_OK;
+	myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
+	draw_area_emboss(sa);
+
+	/* it is important to end a view in a transform compatible with buttons */
+	bwin_scalematrix(sa->win, sipo->blockscale, sipo->blockscale, sipo->blockscale);
+	ipo_blockhandlers(sa);
+
+	sa->win_swap= WIN_BACK_OK;
 }
 
 void scroll_ipobuts()
@@ -1449,78 +1728,79 @@ void scroll_ipobuts()
 	}
 }
 
-
-
+/* total mess function, especially with mousewheel, needs cleanup badly (ton) */
 int view2dzoom(unsigned short event)
 {
 	float fac, dx, dy, wtemp;
 	short mval[2], mvalo[2];
-
+	
 	areawinset(curarea->win);	/* from buttons */
 	curarea->head_swap= 0;
 	getmouseco_areawin(mvalo);
 	
-	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || 
-          (event==WHEELUPMOUSE) ||
-          (event==WHEELDOWNMOUSE) ) {
-
-    /* regular mousewheel:   zoom regular
-     * alt-shift mousewheel: zoom y only
-     * alt-ctrl mousewheel:  zoom x only
-     */
+	while( (get_mbut()&(L_MOUSE|M_MOUSE)) || (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) {
+	
+		/* regular mousewheel:   zoom regular
+		* alt-shift mousewheel: zoom y only
+		* alt-ctrl mousewheel:  zoom x only
+		*/
 		if (event==WHEELUPMOUSE) {
 			if(U.uiflag & WHEELZOOMDIR)
-				wtemp = -0.1154;
+				wtemp = -0.0375;
 			else
-				wtemp = 0.1154;
-
+				wtemp = 0.03;
+			if(curarea->spacetype!=SPACE_BUTS) wtemp*= 3;
+			
 			dx= (float)(wtemp*(G.v2d->cur.xmax-G.v2d->cur.xmin));
 			dy= (float)(wtemp*(G.v2d->cur.ymax-G.v2d->cur.ymin));
-
-      switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
-      case 0:
-        break;
-      case (LR_SHIFTKEY|LR_ALTKEY):
-        dx = 0;
-        break;
-      case (LR_CTRLKEY|LR_ALTKEY):
-        dy = 0;
-        break;
-      default:
-        return 0;
-        break;
-      }
-    }
+			
+			switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
+			case 0:
+				break;
+			case (LR_SHIFTKEY|LR_ALTKEY):
+				dx = 0;
+				break;
+			case (LR_CTRLKEY|LR_ALTKEY):
+				dy = 0;
+				break;
+			default:
+				if(curarea->spacetype==SPACE_BUTS);	// exception
+				else return 0;
+				break;
+			}
+		}
 		else if (event==WHEELDOWNMOUSE) {
 			if(U.uiflag & WHEELZOOMDIR)
-				wtemp = 0.1154;
+				wtemp = 0.03;
 			else
-				wtemp = -0.1154;
+				wtemp = -0.0375;
+			if(curarea->spacetype!=SPACE_BUTS) wtemp*= 3;
+			
 			dx= (float)(wtemp*(G.v2d->cur.xmax-G.v2d->cur.xmin));
 			dy= (float)(wtemp*(G.v2d->cur.ymax-G.v2d->cur.ymin));
-
-      switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
-      case 0:
-        break;
-      case (LR_SHIFTKEY|LR_ALTKEY):
-        dx = 0;
-        break;
-      case (LR_CTRLKEY|LR_ALTKEY):
-        dy = 0;
-        break;
-      default:
-        return 0;
-        break;
-      }
-
-    }
-    else {
-      getmouseco_areawin(mval);
+			
+			switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
+				case 0:
+				break;
+			case (LR_SHIFTKEY|LR_ALTKEY):
+				dx = 0;
+				break;
+			case (LR_CTRLKEY|LR_ALTKEY):
+				dy = 0;
+				break;
+			default:
+				if(curarea->spacetype==SPACE_BUTS);
+				else return 0;
+				break;
+			}
+		}
+		else {
+			getmouseco_areawin(mval);
 			fac= 0.001*(mval[0]-mvalo[0]);
 			dx= fac*(G.v2d->cur.xmax-G.v2d->cur.xmin);
 			fac= 0.001*(mval[1]-mvalo[1]);
 			dy= fac*(G.v2d->cur.ymax-G.v2d->cur.ymin);
-    }
+		}
 		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 			
 			G.v2d->cur.xmin+= dx;
@@ -1529,17 +1809,17 @@ int view2dzoom(unsigned short event)
 				G.v2d->cur.ymin+= dy;
 				G.v2d->cur.ymax-= dy;
 			}
-				
+		
 			test_view2d(G.v2d, curarea->winx, curarea->winy);	/* cur min max rects */
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		}
 		else BIF_wait_for_statechange();
-    /* return if we were using the mousewheel
-     */
-    if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
+		/* return if we were using the mousewheel
+		*/
+		if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
 	}
-  return 1;
+	return 1;
 }
 
 void center_currframe(void)
@@ -1562,6 +1842,7 @@ void center_currframe(void)
 	curarea->head_swap= 0;
 }
 
+/* total mess function, especially with mousewheel, needs cleanup badly (ton) */
 int view2dmove(unsigned short event)
 {
 	/* return 1 when something was done */
@@ -1577,10 +1858,12 @@ int view2dmove(unsigned short event)
 	 * or if the mousewheel is being used.
 	 * Return if zooming was done.
 	 */
-	if ( (G.qual & LR_CTRLKEY) ||
-		(event==WHEELUPMOUSE) || 
-		(event==WHEELDOWNMOUSE) ) {
-		if (view2dzoom(event)) {
+	 
+	 
+	if ( (G.qual & LR_CTRLKEY) || (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) {
+		/* patch for buttonswin, standard scroll no zoom */
+		if(curarea->spacetype==SPACE_BUTS && (G.qual & LR_CTRLKEY)==0);
+		else if (view2dzoom(event)) {
 			curarea->head_swap= 0;
 			return 0;
 		}
@@ -1635,63 +1918,59 @@ int view2dmove(unsigned short event)
        * is used with the ctrl key then scroll left
        * and right.
        */
-      if (event==WHEELUPMOUSE) {
-        switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
-        case (LR_SHIFTKEY):
-          dx = 0.0;
-          dy= facy*10.0;
-          break;
-        case (LR_CTRLKEY):
-          dx= facx*10.0;
-          dy = 0.0;
-          break;
-        default:
-          return 0;
-          break;
-        }
-      }
-      else if (event==WHEELDOWNMOUSE) {
-        switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
-        case (LR_SHIFTKEY):
-          dx = 0.0;
-          dy= -facy*10.0;
-          break;
-        case (LR_CTRLKEY):
-          dx= -facx*10.0;
-          dy = 0.0;
-          break;
-        default:
-          return 0;
-          break;
-        }
-      }
-      else {
-        getmouseco_areawin(mval);
-        dx= facx*(mvalo[0]-mval[0]);
-        dy= facy*(mvalo[1]-mval[1]);
-      }
+		if (event==WHEELUPMOUSE || event==WHEELDOWNMOUSE) {
+			if(event==WHEELDOWNMOUSE) {	
+				facx= -facx; facy= -facy;
+			}
+			
+			switch (G.qual & (LR_CTRLKEY|LR_SHIFTKEY|LR_ALTKEY)) {
+			case (LR_SHIFTKEY):
+				dx = 0.0;
+				dy= facy*20.0;
+				break;
+			case (LR_CTRLKEY):
+				dx= facx*20.0;
+				dy = 0.0;
+				break;
+			default:
+				if(curarea->spacetype==SPACE_BUTS) {
+					if(G.buts->align==BUT_HORIZONTAL) {
+						dx= facx*30; dy= 0.0;
+					} else {
+						dx= 0.0; dy= facy*30;
+					}
+				}
+				else return 0;
+				break;
+			}
+		}
+		else {
+			getmouseco_areawin(mval);
+			dx= facx*(mvalo[0]-mval[0]);
+			dy= facy*(mvalo[1]-mval[1]);
+		}
 
-      if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
+		if(mval[0]!=mvalo[0] || mval[1]!=mvalo[1]) {
 
-        G.v2d->cur.xmin+= left*dx;
-        G.v2d->cur.xmax+= right*dx;
-        G.v2d->cur.ymin+= left*dy;
-        G.v2d->cur.ymax+= right*dy;
+			G.v2d->cur.xmin+= left*dx;
+			G.v2d->cur.xmax+= right*dx;
+			G.v2d->cur.ymin+= left*dy;
+			G.v2d->cur.ymax+= right*dy;
 				
-        test_view2d(G.v2d, curarea->winx, curarea->winy);
+			test_view2d(G.v2d, curarea->winx, curarea->winy);
 				
-        scrarea_do_windraw(curarea);
-        screen_swapbuffers();
+			scrarea_do_windraw(curarea);
+			screen_swapbuffers();
 				
-        mvalo[0]= mval[0];
-        mvalo[1]= mval[1];
+			mvalo[0]= mval[0];
+			mvalo[1]= mval[1];
 				
-      }
-      else BIF_wait_for_statechange();
-      /* return if we were using the mousewheel
-       */
-      if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
-    }
+		}
+		else BIF_wait_for_statechange();
+			/* return if we were using the mousewheel
+			*/
+		if ( (event==WHEELUPMOUSE) || (event==WHEELDOWNMOUSE) ) return 1;
+	}
 
     curarea->head_swap= 0;
     return 1;
@@ -1731,9 +2010,16 @@ EditIpo *select_proj_ipo(rctf *rectf, int event)
 	glInitNames();	/* whatfor? but otherwise it does not work */
 	glPushName(-1);
 	
+	/* get rid of buttons view */
+	glPushMatrix();
+	glLoadIdentity();
+	
 	init_pickselcode();	/* drawipo.c */
 	draw_ipocurves(0);	
 	
+	/* restore buttons view */
+	glPopMatrix();
+
 	G.f -= G_PICKSEL;
 	
 	hits= glRenderMode(GL_RENDER);
