@@ -1,5 +1,5 @@
 /**
- * $Id: drawobject.c,v 1.15 2003/07/21 17:46:55 ton Exp $
+ * $Id: drawobject.c,v 1.44 2003/11/23 20:28:35 theeth Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -46,14 +46,12 @@
 
 #include "IMB_imbuf.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "MTC_matrixops.h"
 
 #include "DNA_camera_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_constraint_types.h" // for drawing constraint
 #include "DNA_effect_types.h"
 #include "DNA_ipo_types.h"
 #include "DNA_lamp_types.h"
@@ -68,12 +66,18 @@
 #include "DNA_view3d_types.h"
 #include "DNA_world_types.h"
 
+#include "BLI_blenlib.h"
+#include "BLI_arithb.h"
+#include "BLI_editVert.h"
+
 #include "BKE_utildefines.h"
 #include "BKE_curve.h"
+#include "BKE_constraint.h" // for the get_constraint_target function
 #include "BKE_object.h"
 #include "BKE_global.h"
 #include "BKE_displist.h"
 #include "BKE_material.h"
+#include "BKE_mball.h"
 #include "BKE_ipo.h"
 #include "BKE_mesh.h"
 #include "BKE_effect.h"
@@ -86,6 +90,7 @@
 #include "BIF_editarmature.h"
 #include "BIF_editika.h"
 #include "BIF_editmesh.h"
+#include "BIF_resources.h"
 
 #include "BDR_drawmesh.h"
 #include "BDR_drawobject.h"
@@ -99,9 +104,7 @@
 #include "mydevice.h"
 #include "nla.h"
 
-#ifdef __NLA
 #include "BKE_deform.h"
-#endif
 
 /* pretty stupid */
 /*  extern Lattice *editLatt; already in BKE_lattice.h  */
@@ -183,12 +186,6 @@ unsigned int rectllib_sel[81]= {0xff777777,0xff777777,0xfff64c,0xfff64c,0xfff64c
 
 unsigned int rectl_set[81]= {0xff777777,0xff777777,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xff777777,0xff777777,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0xff777777,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0xaaaaaa,0xaaaaaa,0x4e4e4e,0x10100,0x202020,0x0,0x4e4e4d,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xff777777,0x0,0xaaa100,0xaaaaaa,0xaaa100,0x0,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0x202020,0xfffde2,0xffffff,0xaaaaaa,0x202020,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0x10100,0xaaaaaa,0xfffde2,0xaaa100,0x0,0xff777777,0xaaaaaa,0xaaaaaa,0xaaaaaa,0x4f4f4f,0x0,0x202020,0x10100,0x4e4e4e,0xaaaaaa,0xaaaaaa,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0xff777777,0xff777777,0xaaaaaa,0xaaaaaa,0xff777777,0xff777777,0xff777777,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xaaaaaa,0xff777777,0xff777777};
 
-#define B_YELLOW	0x77FFFF
-#define B_PURPLE	0xFF70FF
-
-
-unsigned int selcol= 0xFF88FF;
-unsigned int actselcol= 0xFFBBFF;
 
 static unsigned int colortab[24]=
 	{0x0,		0xFF88FF, 0xFFBBFF, 
@@ -258,7 +255,7 @@ void helpline(float *vec)
 	getmouseco_areawin(mval);
 	project_short_noclip(vecrot, mval1);
 
-	persp(0);
+	persp(PERSP_WIN);
 	
 	glDrawBuffer(GL_FRONT);
 	
@@ -271,8 +268,8 @@ void helpline(float *vec)
 	glEnd();
 	setlinestyle(0);
 	
-	persp(1);
-
+	persp(PERSP_VIEW);
+	glFinish(); // flush display for frontbuffer
 	glDrawBuffer(GL_BACK);
 }
 
@@ -720,7 +717,7 @@ void drawcamera(Object *ob)
 		mygetsingmatrix(G.vd->persmat);
 
 		if(cam->flag & CAM_SHOWLIMITS) 
-			draw_limit_line(cam->clipsta, cam->clipend, B_YELLOW);
+			draw_limit_line(cam->clipsta, cam->clipend, 0x77FFFF);
 
 		wrld= G.scene->world;
 		if(cam->flag & CAM_SHOWMIST) 
@@ -734,12 +731,14 @@ static void tekenvertslatt(short sel)
 {
 	Lattice *lt;
 	BPoint *bp;
+	float size;
 	int a, uxt, u, vxt, v, wxt, w;
 
-	glPointSize(3.0);
+	size= BIF_GetThemeValuef(TH_VERTEX_SIZE);
+	glPointSize(size);
 
-	if(sel) cpack(B_YELLOW);
-	else cpack(B_PURPLE);
+	if(sel) BIF_ThemeColor(TH_VERTEX_SELECT);
+	else BIF_ThemeColor(TH_VERTEX);
 
 	glBegin(GL_POINTS);
 
@@ -804,6 +803,7 @@ void calc_lattverts_ext(void)
 {
 
 	areawinset(curarea->win);
+	persp(PERSP_VIEW);
 	mymultmatrix(G.obedit->obmat);
 	calc_lattverts();
 	myloadmatrix(G.vd->viewmat);
@@ -944,6 +944,8 @@ void calc_meshverts_ext(void)
 {
 
 	areawinset(curarea->win);
+	persp(PERSP_VIEW);
+	
 	mymultmatrix(G.obedit->obmat);
 	calc_meshverts();
 	myloadmatrix(G.vd->viewmat);
@@ -958,6 +960,7 @@ void calc_meshverts_ext_f2(void)
 	
 	/* matrices */
 	areawinset(curarea->win);
+	persp(PERSP_VIEW);
 	mymultmatrix(G.obedit->obmat);
 	
 	if(G.edve.first==0) return;
@@ -1025,6 +1028,7 @@ void calc_nurbverts_ext(void)
 {
 
 	areawinset(curarea->win);
+	persp(PERSP_VIEW);
 	mymultmatrix(G.obedit->obmat);
 	calc_Nurbverts(editNurb.first);
 	myloadmatrix(G.vd->viewmat);
@@ -1034,73 +1038,46 @@ void calc_nurbverts_ext(void)
 void tekenvertices(short sel)
 {
 	EditVert *eve;
-
-	glPointSize(2.0);
+	float size;
+	char col[3];
 	
-	if(sel) cpack(B_YELLOW);
-	else cpack(B_PURPLE);
-#ifdef __NLA /* __TEKENTEST */
-//	if (sel==2)
-//		cpack (0x0000FF);
-#endif	/* __TEKENTEST */
-	glBegin(GL_POINTS);
+	/* draws in zbuffer mode twice, to show invisible vertices transparent */
 
-	eve= (EditVert *)G.edve.first;
-	while(eve) {
-		if(eve->h==0 && (eve->f & 1)==sel ) {
-			glVertex3fv(eve->co);
+	size= BIF_GetThemeValuef(TH_VERTEX_SIZE);
+	if(sel) BIF_GetThemeColor3ubv(TH_VERTEX_SELECT, col);
+	else BIF_GetThemeColor3ubv(TH_VERTEX, col);
+	
+	if(G.zbuf) {
+		glPointSize(size>2.1?size/2.0: size);
+
+		glDisable(GL_DEPTH_TEST);
+		glColor4ub(col[0], col[1], col[2], 100);
+		
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		glEnable(GL_BLEND);
+
+		glBegin(GL_POINTS);
+		for(eve= G.edve.first; eve; eve= eve->next) {
+			if(eve->h==0 && (eve->f & 1)==sel ) glVertex3fv(eve->co);
 		}
-#ifdef __NLA /* __TEKENTEST */
-//	if (eve->totweight && sel==2)
-//		glVertex3fv(eve->co);
+		glEnd();
+		
+		glDisable(GL_BLEND);
+		glEnable(GL_DEPTH_TEST);
+	}
 
-#endif	/* __TEKENTEST */
-		eve= eve->next;
+	glPointSize(size);
+	glColor3ub(col[0], col[1], col[2]);
+
+	glBegin(GL_POINTS);
+	for(eve= G.edve.first; eve; eve= eve->next) {
+		if(eve->h==0 && (eve->f & 1)==sel ) glVertex3fv(eve->co);
 	}
 	glEnd();
 
 	glPointSize(1.0);
 }
 
-void tekenvertices_ext(int mode)
-{
-	ScrArea *tempsa, *sa;
-	View3D *vd;
-	
-	if(G.f & (G_FACESELECT+G_DRAWFACES)) {
-		allqueue(REDRAWVIEW3D, 0);
-		return;
-	}
-	
-	if(G.zbuf) glDisable(GL_DEPTH_TEST);
-	
-	glDrawBuffer(GL_FRONT);
-
-	/* check all views */
-	tempsa= curarea;
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		if(sa->spacetype==SPACE_VIEW3D) {
-			vd= sa->spacedata.first;
-			if(G.obedit->lay & vd->lay) {
-				areawinset(sa->win);
-				mymultmatrix(G.obedit->obmat);
-
-				calc_meshverts();
-				if(mode==0 || mode==2) tekenvertices(0);
-				if(mode==1 || mode==2) tekenvertices(1);
-				sa->win_swap= WIN_FRONT_OK;
-				
-				myloadmatrix(G.vd->viewmat);
-			}
-		}
-		sa= sa->next;
-	}
-	if(curarea!=tempsa) areawinset(tempsa->win);
-	
-	glDrawBuffer(GL_BACK);
-	if(G.zbuf) glEnable(GL_DEPTH_TEST);
-}
 
 /* ************** DRAW DISPLIST ****************** */
 
@@ -1112,15 +1089,40 @@ static void displistmesh_draw_wire(DispListMesh *dlm) {
 	for (i=0; i<dlm->totface; i++) {
 		MFaceInt *mf= &dlm->mface[i];
 		
-		glBegin(GL_LINE_LOOP);
-		glVertex3fv(dlm->mvert[mf->v1].co);
-		glVertex3fv(dlm->mvert[mf->v2].co);
-		if (mf->v3) {
-			glVertex3fv(dlm->mvert[mf->v3].co);
-			if (mf->v4)
-				glVertex3fv(dlm->mvert[mf->v4].co);
+		if(dlm->flag & ME_OPT_EDGES) {
+			int test= mf->edcode;
+			if(test) {
+				glBegin(GL_LINES);
+				if(test & ME_V1V2) {
+					glVertex3fv(dlm->mvert[mf->v1].co); 
+					glVertex3fv(dlm->mvert[mf->v2].co);
+				}
+				if(test & ME_V2V3) {
+					glVertex3fv(dlm->mvert[mf->v2].co); 
+					glVertex3fv(dlm->mvert[mf->v3].co);
+				}
+				if(test & ME_V3V4) {
+					glVertex3fv(dlm->mvert[mf->v3].co); 
+					glVertex3fv(dlm->mvert[mf->v4].co);
+				}
+				if(test & ME_V4V1) {
+					glVertex3fv(dlm->mvert[mf->v4].co); 
+					glVertex3fv(dlm->mvert[mf->v1].co);
+				}
+				glEnd();
+			}
+		}
+		else {	// old method
+			glBegin(GL_LINE_LOOP);
+			glVertex3fv(dlm->mvert[mf->v1].co);
+			glVertex3fv(dlm->mvert[mf->v2].co);
+			if (mf->v3) {
+				glVertex3fv(dlm->mvert[mf->v3].co);
+				if (mf->v4)
+					glVertex3fv(dlm->mvert[mf->v4].co);
+			}
+			glEnd();
 		}
-		glEnd();
 	}
 }
 
@@ -1675,38 +1677,6 @@ static void drawDispListshaded(ListBase *lb, Object *ob)
 	glShadeModel(GL_FLAT);
 }
 
-/* wrappers for shaded+wire and solid+wire */
-static void drawMeshWireExtra(Object *ob) {
-	GLfloat origwidth;
-	GLint origcolor[4];
-	
-	glGetFloatv(GL_LINE_WIDTH, &origwidth);
-	glGetIntegerv(GL_CURRENT_COLOR, origcolor);
-	
-	glLineWidth(1.2);
-
-	cpack(0x4F4F4F);
-	drawmeshwire(ob);
-
-	glColor4iv(origcolor);
-	glLineWidth(origwidth);
-}
-
-static void drawDispListWireExtra(ListBase *lb) {
-	GLfloat origwidth;
-	GLint origcolor[4];
-
-	glGetFloatv(GL_LINE_WIDTH, &origwidth);
-	glGetIntegerv(GL_CURRENT_COLOR, origcolor);
-
-	glLineWidth(1.2);
-
-	cpack(0x4F4F4F);
-	drawDispListwire(lb);
-
-	glLineWidth(origwidth);
-	glColor4iv(origcolor);
-}
 
 static void drawmeshsolid(Object *ob, float *nors)
 {
@@ -1779,18 +1749,6 @@ static void drawmeshsolid(Object *ob, float *nors)
 		
 		glDisable(GL_LIGHTING);
 		glShadeModel(GL_FLAT);
-		
-		if(ob==G.obedit) {
-			calc_meshverts();
-		
-			if(G.zbuf) glDisable(GL_DEPTH_TEST);
-			tekenvertices(0);
-			tekenvertices(1);
-			if(G.zbuf) glEnable(GL_DEPTH_TEST);
-		}
-		
-		if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
-
 	}
 	else {		/* [nors] should never be zero, but is weak code... the displist 
 				   system needs a make over (ton)
@@ -2054,8 +2012,6 @@ static void drawmeshshaded(Object *ob, unsigned int *col1, unsigned int *col2)
 	glEnd();
 	glShadeModel(GL_FLAT);
 	if(twoside) glDisable(GL_CULL_FACE);
-	
-	if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 }
 
 static void drawDispList(Object *ob, int dt)
@@ -2093,22 +2049,18 @@ static void drawDispList(Object *ob, int dt)
 					/* vertexpaint only true when selecting */
 				if (vertexpaint) {
 					drawmeshsolid(ob, NULL);
-					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);					
 				} else {
 					init_gl_materials(ob);
 					two_sided(me->flag & ME_TWOSIDED);
 					drawDispListsolid(lb, ob);
-					if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 				}
 			}
 			else {
 				drawmeshsolid(ob, dl->nors);
-				if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 			}
 			
 		}
 		else if(dt==OB_SHADED) {
-#ifdef __NLA
 			if( G.f & G_WEIGHTPAINT && me->dvert) {
 				unsigned char *wtcol, *curwt;
 				MFace *curface;
@@ -2139,26 +2091,20 @@ static void drawDispList(Object *ob, int dt)
 				}
 				
 				drawmeshshaded(ob, (unsigned int*)wtcol, 0);
-				if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 				
 				MEM_freeN (wtcol);
 				
 			}
 			else
-#endif
+
 			if( G.f & (G_VERTEXPAINT+G_TEXTUREPAINT)) {
 				/* in order: vertexpaint already made mcol */
-///*
-
-//*/
 				if(me->mcol) {
 					drawmeshshaded(ob, (unsigned int *)me->mcol, 0);
-					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 				} else if(me->tface) {
 					tface_to_mcol(me);
 					drawmeshshaded(ob, (unsigned int *)me->mcol, 0);	
 					MEM_freeN(me->mcol); me->mcol= 0;
-					if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 				}
 				else 
 					drawmeshwire(ob);
@@ -2174,10 +2120,8 @@ static void drawDispList(Object *ob, int dt)
 				if(dl) {
 					if(mesh_uses_displist(me)) {
 						drawDispListshaded(&me->disp, ob);
-						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
 					} else {
 						drawmeshshaded(ob, dl->col1, dl->col2);
-						if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 					}
 				}
 			}
@@ -2211,14 +2155,11 @@ static void drawDispList(Object *ob, int dt)
 			if(dt==OB_SHADED) {
 				if(ob->disp.first==0) shadeDispList(ob);
 				drawDispListshaded(lb, ob);
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 			else {
 				init_gl_materials(ob);
 				two_sided(0);
 				drawDispListsolid(lb, ob);
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
-
 			}
 			index3_nors_incr= 1;
 		}
@@ -2242,14 +2183,12 @@ static void drawDispList(Object *ob, int dt)
 			if(dt==OB_SHADED) {
 				if(ob->disp.first==0) shadeDispList(ob);
 				drawDispListshaded(lb, ob);
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 			else {
 				init_gl_materials(ob);
 				two_sided(0);
 			
 				drawDispListsolid(lb, ob);
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
 			}
 		}
 		else {
@@ -2257,27 +2196,27 @@ static void drawDispList(Object *ob, int dt)
 		}
 		break;
 	case OB_MBALL:
-
-		lb= &ob->disp;
-		if(lb->first==0) makeDispList(ob);
-
-		if(solid) {
-			
-			if(dt==OB_SHADED) {
-				dl= lb->first;
-				if(dl && dl->col1==0) shadeDispList(ob);
-				drawDispListshaded(lb, ob);
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
-			}
-			else {
-				init_gl_materials(ob);
-				two_sided(0);
-			
-				drawDispListsolid(lb, ob);	
-				if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(lb);
+		
+		if( is_basis_mball(ob)) {
+			lb= &ob->disp;
+			if(lb->first==0) makeDispList(ob);
+	
+			if(solid) {
+				
+				if(dt==OB_SHADED) {
+					dl= lb->first;
+					if(dl && dl->col1==0) shadeDispList(ob);
+					drawDispListshaded(lb, ob);
+				}
+				else {
+					init_gl_materials(ob);
+					two_sided(0);
+				
+					drawDispListsolid(lb, ob);	
+				}
 			}
+			else drawDispListwire(lb);
 		}
-		else drawDispListwire(lb);
 		break;
 	}
 	
@@ -2397,24 +2336,31 @@ static void drawmeshwire(Object *ob)
 	EditEdge *eed;
 	EditVlak *evl;
 	float fvec[3], cent[3], *f1, *f2, *f3, *f4, *extverts=0;
-	int a, start, end, test, /*  colbcol=0, */ ok;
+	int a, start, end, test, ok, handles=0;
 
 	me= get_mesh(ob);
 
 	if(ob==G.obedit || (G.obedit && ob->data==G.obedit->data)) {
 		
-		if(G.f & (G_FACESELECT+G_DRAWFACES)) {	/* faces */
+		if( (me->flag & ME_OPT_EDGES) && (me->flag & ME_SUBSURF) && me->subdiv) handles= 1;
+		
+		if(handles==0 && (G.f & (G_FACESELECT+G_DRAWFACES))) {	/* faces */
+			char col1[4], col2[4];
+			
+			BIF_GetThemeColor4ubv(TH_FACE, col1);
+			BIF_GetThemeColor4ubv(TH_FACE_SELECT, col2);
 			
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 			glEnable(GL_BLEND);
-
+			glDepthMask(0);		// disable write in zbuffer, needed for nice transp
+			
 			evl= G.edvl.first;
 			while(evl) {
 				if(evl->v1->h==0 && evl->v2->h==0 && evl->v3->h==0) {
 					
 					if(1) {
-						if(vlakselectedAND(evl, 1)) glColor4ub(200, 100, 200, 60);
-						else glColor4ub(0, 50, 150, 30);
+						if(vlakselectedAND(evl, 1)) glColor4ub(col2[0], col2[1], col2[2], col2[3]); 
+						else glColor4ub(col1[0], col1[1], col1[2], col1[3]);
 						
 						glBegin(evl->v4?GL_QUADS:GL_TRIANGLES);
 						glVertex3fv(evl->v1->co);
@@ -2424,13 +2370,13 @@ static void drawmeshwire(Object *ob)
 						glEnd();
 						
 					} else {
-						if(vlakselectedAND(evl, 1)) cpack(0x559999);
-						else cpack(0x664466);
+						if(vlakselectedAND(evl, 1)) glColor4ub(col2[0], col2[1], col2[2], col2[3]); 
+						else glColor4ub(col1[0], col1[1], col1[2], col1[3]);
 					
 						if(evl->v4 && evl->v4->h==0) {
 						
 							CalcCent4f(cent, evl->v1->co, evl->v2->co, evl->v3->co, evl->v4->co);
-							glBegin(GL_LINE_LOOP);
+							glBegin(GL_QUADS);
 								VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
 								VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
 								VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
@@ -2440,7 +2386,7 @@ static void drawmeshwire(Object *ob)
 						else {
 	
 							CalcCent3f(cent, evl->v1->co, evl->v2->co, evl->v3->co);
-							glBegin(GL_LINE_LOOP);
+							glBegin(GL_TRIANGLES);
 								VecMidf(fvec, cent, evl->v1->co); glVertex3fv(fvec);
 								VecMidf(fvec, cent, evl->v2->co); glVertex3fv(fvec);
 								VecMidf(fvec, cent, evl->v3->co); glVertex3fv(fvec);
@@ -2451,24 +2397,33 @@ static void drawmeshwire(Object *ob)
 				evl= evl->next;
 			}
 			glDisable(GL_BLEND);
+			glDepthMask(1);		// restore write in zbuffer
 		}
 
-		if(G.zbuf==0 && mesh_uses_displist(me)) {
-			cpack(0x505050);
-			drawDispListwire(&me->disp);
+		if(mesh_uses_displist(me)) {
+			/* dont draw the subsurf when solid... then this is a 'drawextra' */
+			if(handles==0 && ob->dt>OB_WIRE && G.vd->drawtype>OB_WIRE);
+			else {
+				if(handles) BIF_ThemeColor(TH_WIRE);
+				else BIF_ThemeColorBlend(TH_WIRE, TH_BACK, 0.7);
+					
+				drawDispListwire(&me->disp);
+			}
 		}
 		cpack(0x0);
 		
-		if(G.f & G_DRAWEDGES) {	/* Use edge Highlighting */
+		if(handles==0 && (G.f & G_DRAWEDGES)) {	/* Use edge Highlighting */
+			char col[4];
+			BIF_GetThemeColor3ubv(TH_EDGE_SELECT, col);
 			glShadeModel(GL_SMOOTH);
 			
 			eed= G.eded.first;
 			glBegin(GL_LINES);
 			while(eed) {
 				if(eed->h==0) {
-					if(eed->v1->f & 1) cpack(0x559999); else cpack(0x0);
+					if(eed->v1->f & 1) glColor3ub(col[0], col[1], col[2]); else cpack(0x0);
 					glVertex3fv(eed->v1->co);
-					if(eed->v2->f & 1) cpack(0x559999); else cpack(0x0);
+					if(eed->v2->f & 1) glColor3ub(col[0], col[1], col[2]); else cpack(0x0);
 					glVertex3fv(eed->v2->co);
 				}
 				eed= eed->next;
@@ -2476,7 +2431,7 @@ static void drawmeshwire(Object *ob)
 			glEnd();
 			glShadeModel(GL_FLAT);
 		}
-		else {
+		else if(handles==0) {
 			eed= G.eded.first;
 			glBegin(GL_LINES);
 			while(eed) {
@@ -2492,13 +2447,8 @@ static void drawmeshwire(Object *ob)
 		
 		calc_meshverts();
 
-		if(G.zbuf) glDisable(GL_DEPTH_TEST);
 		tekenvertices(0);
 		tekenvertices(1);
-#ifdef __NLA
-		tekenvertices(2);	/* __TEKENTEST */
-#endif
-		if(G.zbuf) glEnable(GL_DEPTH_TEST);
 
 		if(G.f & G_DRAWNORMALS) {	/* normals */
 			cpack(0xDDDD22);
@@ -2780,13 +2730,16 @@ static void tekenvertsN(Nurb *nu, short sel)
 {
 	BezTriple *bezt;
 	BPoint *bp;
+	float size;
 	int a;
 
 	if(nu->hide) return;
 
-	if(sel) cpack(B_YELLOW);
-	else cpack(B_PURPLE);
-	glPointSize(3.0);
+	if(sel) BIF_ThemeColor(TH_VERTEX_SELECT);
+	else BIF_ThemeColor(TH_VERTEX);
+
+	size= BIF_GetThemeValuef(TH_VERTEX_SIZE);
+	glPointSize(size);
 	
 	glBegin(GL_POINTS);
 	
@@ -3022,24 +2975,86 @@ static void tekentextcurs(void)
 
 void drawcircball(float *cent, float rad, float tmat[][4])
 {
-	float si, co, phi, dphi, vec[3], vx[3], vy[3];
+	float vec[3], vx[3], vy[3];
 	int a, tot=32;
+
+	/* 32 values of sin function (still same result!) */
+	static float si[32] = {0.00000000,
+		0.20129852,
+		0.39435585,
+		0.57126821,
+		0.72479278,
+		0.84864425,
+		0.93775213,
+		0.98846832,
+		0.99871650,
+		0.96807711,
+		0.89780453,
+		0.79077573,
+		0.65137248,
+		0.48530196,
+		0.29936312,
+		0.10116832,
+		-0.10116832,
+                -0.29936312,
+                -0.48530196,
+                -0.65137248,
+                -0.79077573,
+                -0.89780453,
+                -0.96807711,
+                -0.99871650,
+                -0.98846832,
+                -0.93775213,
+                -0.84864425,
+                -0.72479278,
+                -0.57126821,
+                -0.39435585,
+                -0.20129852,
+		0.00000000};
+	/* 32 values of cos function (still same result!) */
+	static float co[32] ={1.00000000,
+                0.97952994,
+                0.91895781,
+                0.82076344,
+                0.68896691,
+                0.52896401,
+                0.34730525,
+                0.15142777,
+                -0.05064916,
+                -0.25065253,
+                -0.44039415,
+                -0.61210598,
+                -0.75875812,
+                -0.87434661,
+                -0.95413925,
+                -0.99486932,
+                -0.99486932,
+                -0.95413925,
+                -0.87434661,
+                -0.75875812,
+                -0.61210598,
+                -0.44039415,
+                -0.25065253,
+                -0.05064916,
+                0.15142777,
+                0.34730525,
+                0.52896401,
+                0.68896691,
+                0.82076344,
+                0.91895781,
+                0.97952994,
+                1.00000000};
 		
 	VECCOPY(vx, tmat[0]);
 	VECCOPY(vy, tmat[1]);
 	VecMulf(vx, rad);
 	VecMulf(vy, rad);
 	
-	dphi= 2.0*M_PI/tot;
-	phi= 0.0;
-	
 	glBegin(GL_LINE_LOOP);
-	for(a=0; a<tot; a++, phi+= dphi) {
-		si= sin(phi);
-		co= cos(phi);
-		vec[0]= cent[0]+si*vx[0]+co*vy[0];
-		vec[1]= cent[1]+si*vx[1]+co*vy[1];
-		vec[2]= cent[2]+si*vx[2]+co*vy[2];
+	for(a=0; a<tot; a++) {
+		vec[0]= cent[0] + *(si+a) * vx[0] + *(co+a) * vy[0];
+		vec[1]= cent[1] + *(si+a) * vx[1] + *(co+a) * vy[1];
+		vec[2]= cent[2] + *(si+a) * vx[2] + *(co+a) * vy[2];
 		glVertex3fv(vec);
 	}
 	glEnd();
@@ -3292,6 +3307,55 @@ static int ob_from_decimator(Object *ob)
 	return 0;
 }
 
+
+static void drawWireExtra(Object *ob, ListBase *lb) 
+{
+	float winmat[16], ofs;
+
+	if(ob!=G.obedit && (ob->flag & SELECT)) {
+		if(ob==OBACT) BIF_ThemeColor(TH_ACTIVE);
+		else BIF_ThemeColor(TH_SELECT);
+	}
+	else BIF_ThemeColor(TH_WIRE);
+
+	glMatrixMode(GL_PROJECTION);
+	glGetFloatv(GL_PROJECTION_MATRIX, (float *)winmat);
+	
+	if(winmat[15]>0.5) ofs= 0.00005*G.vd->dist;  // ortho tweaking
+	else ofs= 0.001;
+	winmat[14]-= ofs;
+	glLoadMatrixf(winmat);
+	glMatrixMode(GL_MODELVIEW);
+
+	if(ob->type==OB_MESH) drawmeshwire(ob);
+	else drawDispListwire(lb);
+
+	glMatrixMode(GL_PROJECTION);
+	winmat[14]+= ofs;
+	glLoadMatrixf(winmat);
+	glMatrixMode(GL_MODELVIEW);
+
+}
+
+static void draw_extra_wire(Object *ob) 
+{
+	Curve *cu;
+
+	switch(ob->type) {
+	case OB_MESH:
+		drawWireExtra(ob, NULL);
+		break;
+	case OB_CURVE:
+	case OB_SURF:
+		cu= ob->data;
+		if(boundbox_clip(ob->obmat, cu->bb)) drawWireExtra(ob, &cu->disp);
+		break;
+	case OB_MBALL:
+		drawWireExtra(ob, &ob->disp);
+		break;
+	}
+}
+
 void draw_object(Base *base)
 {
 	PartEff *paf;
@@ -3388,22 +3452,33 @@ void draw_object(Base *base)
 	if((G.f & (G_BACKBUFSEL+G_PICKSEL)) == 0) {
 		project_short(ob->obmat[3], &base->sx);
 		
-		if(G.moving==1 && (base->flag & (SELECT+BA_PARSEL))) colindex= 12;
+		if(G.moving==1 && (base->flag & (SELECT+BA_PARSEL))) BIF_ThemeColor(TH_TRANSFORM);
 		else {
+		
+			BIF_ThemeColor(TH_WIRE);
 			if((G.scene->basact)==base) {
-				if(base->flag & (SELECT+BA_WASSEL)) colindex= 2;
+				if(base->flag & (SELECT+BA_WASSEL)) BIF_ThemeColor(TH_ACTIVE);
 			}
 			else {
-				if(base->flag & (SELECT+BA_WASSEL)) colindex= 1;
+				if(base->flag & (SELECT+BA_WASSEL)) BIF_ThemeColor(TH_SELECT);
 			}
-			if(ob->id.lib) colindex+= 3;
-			else if(warning_recursive==1) colindex+= 6;
-			else if(ob->flag & OB_FROMGROUP) colindex+= 9;
+			
+			// no theme yet
+			if(ob->id.lib) {
+				if(base->flag & (SELECT+BA_WASSEL)) colindex = 4;
+				else colindex = 3;
+			}
+			else if(warning_recursive==1) {
+				if(base->flag & (SELECT+BA_WASSEL)) colindex = 7;
+				else colindex = 6;
+			}
+
 		}	
 		
-		col= colortab[colindex];
-		cpack(col);
-
+		if(colindex) {
+			col= colortab[colindex];
+			cpack(col);
+		}
 	}
 	
 	/* maximum drawtype */
@@ -3434,20 +3509,23 @@ void draw_object(Base *base)
 			}
 		}
 	}
-	else if(dt>=OB_WIRE ) {
+	if(dt>=OB_WIRE ) {
 
 		if(dt>OB_SOLID) if(G.f & G_BACKBUFSEL) dt= OB_SOLID;
 
 		dtx= ob->dtx;
 		if(G.obedit==ob) {
-			if(dtx & OB_TEXSPACE) dtx= OB_TEXSPACE;
-			else dtx= 0;
+			// the only 2 extra drawtypes alowed in editmode
+			dtx= dtx & (OB_DRAWWIRE|OB_TEXSPACE);
+		}
+		else if(G.f & (G_FACESELECT)) {
+			// unwanted combo
+			dtx &= ~OB_DRAWWIRE;
 		}
 		
 		if(G.f & G_DRAW_EXT) {
 			if(ob->type==OB_EMPTY || ob->type==OB_CAMERA || ob->type==OB_LAMP) dt= OB_WIRE;
 		}
-		
 	}
 
 	if( (G.f & G_DRAW_EXT) && dt>OB_WIRE) {
@@ -3485,15 +3563,11 @@ void draw_object(Base *base)
 						init_gl_materials(ob);
 						two_sided( me->flag & ME_TWOSIDED );
 						drawDispListsolid(&me->disp, ob);
-						/* this seems to be the place where the wire for subsurfs 
-						 * gets drawn.. so we draw an extra wire in grey here (editmode) */
-						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
-						drawmeshwire(ob);
 					}
 					else {
 						drawmeshsolid(ob, 0);
-						if(ob->dtx & OB_DRAWWIRE) drawDispListWireExtra(&me->disp);
 					}
+					dtx |= OB_DRAWWIRE;	// draws edges, transp faces, subsurf handles, vertices
 				}
 				if(ob==G.obedit && (G.f & G_PROPORTIONAL)) draw_prop_circle();
 			}
@@ -3508,14 +3582,12 @@ void draw_object(Base *base)
 					if(G.f & G_BACKBUFSEL) drawmeshsolid(ob, 0);					
 					else if(G.f & G_FACESELECT || G.vd->drawtype==OB_TEXTURE) {
 						draw_tface_mesh(ob, ob->data, dt);
-						if(ob->dtx & OB_DRAWWIRE) drawMeshWireExtra(ob);
 					}
 					else drawDispList(ob, dt);
 				}
 				else drawDispList(ob, dt);
 			}
-			if( (ob!=G.obedit) 
-				&& ((G.f & (G_BACKBUFSEL+G_PICKSEL)) == 0) ) {
+			if( (ob!=G.obedit) && ((G.f & (G_BACKBUFSEL+G_PICKSEL)) == 0) ) {
 				paf = give_parteff(ob);
 				if( paf ) {
 					if(col) cpack(0xFFFFFF);	/* for visibility */
@@ -3568,14 +3640,9 @@ void draw_object(Base *base)
 			drawlattice(ob);
 			if(ob==G.obedit && (G.f & G_PROPORTIONAL)) draw_prop_circle();
 			break;
-		case OB_IKA:
-			draw_ika(ob, base->flag & SELECT);
-			break;
-#ifdef __NLA
 		case OB_ARMATURE:
 			draw_armature (ob);
 			break;
-#endif
 		default:
 			drawaxes(1.0);
 		}
@@ -3597,6 +3664,7 @@ void draw_object(Base *base)
 			BMF_DrawString(G.font, ob->id.name+2);
 		}
 		if(dtx & OB_DRAWIMAGE) drawDispListwire(&ob->disp);
+		if((dtx & OB_DRAWWIRE) && dt>=OB_SOLID) draw_extra_wire(ob);
 	}
 
 	if(dt<OB_SHADED) {
@@ -3623,6 +3691,8 @@ void draw_object(Base *base)
 	if(G.f & G_SIMULATION) return;
 
 	if((G.f & (G_BACKBUFSEL+G_PICKSEL))==0) {
+		ListBase *list;
+
 		/* help lines and so */
 		if(ob->parent && (ob->parent->lay & G.vd->lay)) {
 			setlinestyle(3);
@@ -3633,6 +3703,31 @@ void draw_object(Base *base)
 			setlinestyle(0);
 		}
 		
+		/* Drawing the constraint lines */
+		list = &ob->constraints;
+		if (list){
+			extern void make_axis_color(char *col, char *col2, char axis);	// drawview.c
+			bConstraint *curcon;
+			float size[3], tmat[4][4];
+			char col[4], col2[4];
+			
+			BIF_GetThemeColor3ubv(TH_GRID, col);
+			make_axis_color(col, col2, 'z');
+			glColor3ubv(col2);
+			
+			for (curcon = list->first; curcon; curcon=curcon->next){
+				if ((curcon->flag & CONSTRAINT_EXPAND)&&(curcon->type!=CONSTRAINT_TYPE_NULL)&&(constraint_has_target(curcon))){
+					get_constraint_target(curcon, TARGET_OBJECT, NULL, tmat, size, bsystem_time(ob, 0, (float)(G.scene->r.cfra), ob->sf));
+					setlinestyle(3);
+					glBegin(GL_LINES);
+					glVertex3fv(tmat[3]);
+					glVertex3fv(ob->obmat[3]);
+					glEnd();
+					setlinestyle(0);
+				}
+			}
+		}
+
 		/* object centers */
 		if(G.zbuf) glDisable(GL_DEPTH_TEST);
 		if(ob->type == OB_LAMP) {
@@ -3679,8 +3774,6 @@ void draw_object(Base *base)
 
 void draw_object_ext(Base *base)
 {
-	ScrArea *tempsa, *sa;
-	View3D *vd;
 	
 	if(G.vd==NULL || base==NULL) return;
 	
@@ -3692,33 +3785,18 @@ void draw_object_ext(Base *base)
 	G.f |= G_DRAW_EXT;
 
 	glDrawBuffer(GL_FRONT);
+	persp(PERSP_VIEW);
 
-	/* check all views */
-	tempsa= curarea;
-	sa= G.curscreen->areabase.first;
-	while(sa) {
-		if(sa->spacetype==SPACE_VIEW3D) {
-			/* limited drawing in both buffers: selectbuffer! */
-			
-			vd= sa->spacedata.first;
-			if(base->lay & vd->lay) {
-				areawinset(sa->win);
+	draw_object(base);
 
-				draw_object(base);
-
-				sa->win_swap= WIN_FRONT_OK;
-			}
-		}
-		sa= sa->next;
-	}
-	if(curarea!=tempsa) areawinset(tempsa->win);
-	
 	G.f &= ~G_DRAW_EXT;
 
+	glFinish();		/* reveil frontbuffer drawing */
 	glDrawBuffer(GL_BACK);
 	
 	if(G.zbuf) {
 		G.zbuf= 0;
 		glDisable(GL_DEPTH_TEST);
 	}
+	curarea->win_swap= WIN_FRONT_OK;
 }
