@@ -1,5 +1,5 @@
 /**
- * $Id: drawtext.c,v 1.12 2003/07/09 20:09:07 ton Exp $
+ * $Id: drawtext.c,v 1.28 2003/11/25 21:38:37 michel Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,6 +32,8 @@
 
 #include <stdlib.h>
 #include <math.h>
+#include <string.h>
+#include <ctype.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -50,7 +52,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "DNA_text_types.h"
 #include "DNA_space_types.h"
@@ -62,8 +63,6 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 
-#include "BPY_extern.h"
-
 #include "BIF_gl.h"
 #include "BIF_keyval.h"
 #include "BIF_interface.h"
@@ -75,12 +74,14 @@
 #include "BIF_toolbox.h"
 #include "BIF_space.h"
 #include "BIF_mywindow.h"
+#include "BIF_resources.h"
 
 #include "BSE_filesel.h"
 
+#include "BPY_extern.h"
+
 #include "mydevice.h"
 #include "blendef.h" 
-#include "interface.h"
 
 #define TEXTXLOC	38
 
@@ -251,7 +252,7 @@ static void draw_cursor(SpaceText *st) {
 		if (x) {
 			h= txt_get_span(text->lines.first, text->curl) - st->top;
 
-			glColor3f(1.0, 0.0, 0.0);
+			BIF_ThemeColor(TH_HILITE);
 			
 			glRecti(x-1, curarea->winy-st->lheight*(h)-2, x+1, curarea->winy-st->lheight*(h+1)-2);
 		}
@@ -288,13 +289,15 @@ static void draw_cursor(SpaceText *st) {
 	
 		x= text_draw(st, linef->line, st->left, charf, 0, 0, 0);
 
-		glColor3f(0.75, 0.44, 0.44);
+		BIF_ThemeColor(TH_SHADE2);
 
-		if(st->showlinenrs)
+		if(st->showlinenrs) {
 			if (!x) x= TXT_OFFSET + TEXTXLOC -4;
-		else
+		}
+		else {
 			if (!x) x= TXT_OFFSET - 4;
-
+		}
+		
 		if (!x) x= TXT_OFFSET-10;
 		while (linef && linef != linel) {
 			h= txt_get_span(text->lines.first, linef) - st->top;
@@ -318,13 +321,10 @@ static void draw_cursor(SpaceText *st) {
 
 		i= text_draw(st, linel->line, st->left, charl, 0, 0, 0);
 		if(i) glRecti(x, curarea->winy-st->lheight*(h)-2, i, curarea->winy-st->lheight*(h+1)-2);
-// draw cursor in selection
-//		h= txt_get_span(text->lines.first, text->curl) - st->top;
-//		glColor3f(1.0, 0.0, 0.0);
-//		glRecti(x2-1, curarea->winy-st->lheight*(h)-2, x2+1, curarea->winy-st->lheight*(h+1)-2);
-	}
 
-	glColor3f(0.0, 0.0, 0.0);
+	}
+	
+	BIF_ThemeColor(TH_TEXT);
 }
 
 static void calc_text_rcts(SpaceText *st)
@@ -376,14 +376,14 @@ static void draw_textscroll(SpaceText *st)
 
 	calc_text_rcts(st);
 	
-	cpack(0x707070);
+	BIF_ThemeColorShade(TH_SHADE1, -20);
 	glRecti(2, 2, 20, curarea->winy-6);
 	uiEmboss(2, 2, 20, curarea->winy-6, 1);
 
-	cpack(0x909090);
+	BIF_ThemeColor(TH_SHADE1);
 	glRecti(st->txtbar.xmin, st->txtbar.ymin, st->txtbar.xmax, st->txtbar.ymax);
 
-	cpack(0x7777c6);
+	BIF_ThemeColor(TH_SHADE2);
 	glRecti(st->txtscroll.xmin, st->txtscroll.ymin, st->txtscroll.xmax, st->txtscroll.ymax);
 
 	uiEmboss(st->txtbar.xmin, st->txtbar.ymin, st->txtbar.xmax, st->txtbar.ymax, st->flags & ST_SCROLL_SELECT);
@@ -540,6 +540,7 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 	int i;
 	TextLine *tmp;
 	char linenr[12];
+	float col[3];
 	int linecount = 0;
 
 	if (BPY_spacetext_is_pywin(st)) {
@@ -547,9 +548,12 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 		return;
 	}
 	
-	glClearColor(0.6, 0.6,  0.6, 1.0);
+	BIF_GetThemeColor3fv(TH_BACK, col);
+	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
-	myortho2(-0.5, curarea->winrct.xmax-curarea->winrct.xmin-0.5, -0.5, curarea->winrct.ymax-curarea->winrct.ymin-0.5);
+	myortho2(-0.375, (float)(sa->winx)-0.375, -0.375, (float)(sa->winy)-0.375);
+
+	draw_area_emboss(sa);
 
 	text= st->text;
 	if(!text) return;
@@ -566,7 +570,7 @@ void drawtextspace(ScrArea *sa, void *spacedata)
 		glRecti(23,  0, (st->lheight==15)?63:59,  curarea->winy - 2);
 	}
 
-	glColor3f(0.0, 0.0, 0.0);
+	BIF_ThemeColor(TH_TEXT);
 
 	draw_cursor(st);
 
@@ -730,7 +734,7 @@ void unlink_text(Text *text)
 	}
 }
 
-static int jumptoline_interactive(SpaceText *st) {
+int jumptoline_interactive(SpaceText *st) {
 	short nlines= txt_get_span(st->text->lines.first, st->text->lines.last)+1;
 	short tmp= txt_get_span(st->text->lines.first, st->text->curl)+1;
 
@@ -831,6 +835,42 @@ void txt_copy_selectbuffer (Text *text)
 }
 
 
+#ifdef _WIN32
+char *unixNewLine(char *buffer)
+{
+	char *p, *p2, *output;
+	
+	/* we can afford the few extra bytes */
+	output= MEM_callocN(strlen(buffer)+1, "unixnewline");
+	for (p= buffer, p2= output; *p; p++)
+		if (*p != '\r') *(p2++)= *p;
+	
+	*p2= 0;
+	return(output);
+}
+
+char *winNewLine(char *buffer)
+{
+	char *p, *p2, *output;
+	int add= 0;
+	
+	for (p= buffer; *p; p++)
+		if (*p == '\n') add++;
+		
+	bufferlength= p-buffer+add+1;
+	output= MEM_callocN(bufferlength, "winnewline");
+	for (p= buffer, p2= output; *p; p++, p2++) {
+		if (*p == '\n') { 
+			*(p2++)= '\r'; *p2= '\n';
+		} else *p2= *p;
+	}
+	*p2= 0;
+	
+	return(output);
+}
+#endif
+
+
 void txt_paste_clipboard(Text *text) {
 #ifdef _WIN32
 	char * buffer = NULL;
@@ -838,6 +878,7 @@ void txt_paste_clipboard(Text *text) {
 	if ( OpenClipboard(NULL) ) {
 		HANDLE hData = GetClipboardData( CF_TEXT );
 		buffer = (char*)GlobalLock( hData );
+		buffer = unixNewLine(buffer);
 		txt_insert_buf(text, buffer);
 		GlobalUnlock( hData );
 		CloseClipboard();
@@ -853,6 +894,8 @@ void txt_copy_clipboard(Text *text) {
 		HLOCAL clipbuffer;
 		char* buffer;
 
+		copybuffer = winNewLine(copybuffer);
+
 		EmptyClipboard();
 		clipbuffer = LocalAlloc(LMEM_FIXED,((bufferlength+1)));
 		buffer = (char *) LocalLock(clipbuffer);
@@ -871,6 +914,66 @@ void txt_copy_clipboard(Text *text) {
 #endif
 }
 
+/*
+ * again==0 show find panel or find
+ * again==1 find text again */
+void txt_find_panel(SpaceText *st, int again)
+{
+	Text *text=st->text;
+	char *findstr= last_txt_find_string;
+			
+	if (again==0) {
+		findstr= txt_sel_to_buf(text);
+	} else if (again==1) {
+		char buf[256];
+
+	if (findstr && strlen(findstr)<(sizeof(buf)-1))
+		strcpy(buf, findstr);
+	else
+		buf[0]= 0;
+		
+	if (sbutton(buf, 0, sizeof(buf)-1, "Find: ") && buf[0])
+		findstr= BLI_strdup(buf);
+	else
+		findstr= NULL;
+	}
+
+	if (findstr!=last_txt_find_string) {
+		if (last_txt_find_string)
+			MEM_freeN(last_txt_find_string);
+		last_txt_find_string= findstr;
+	}
+				
+	if (findstr) {
+		if (txt_find_string(text, findstr))
+			pop_space_text(st);
+		else
+			error("Not found: %s", findstr);
+	}
+}
+
+void run_python_script(SpaceText *st)
+{
+	char *py_filename;
+	Text *text=st->text;
+	if (!BPY_txt_do_python(st)) {
+		int lineno = BPY_Err_getLinenumber();
+		// jump to error if happened in current text:
+		py_filename = (char*) BPY_Err_getFilename();
+		if (!strcmp(py_filename, st->text->id.name+2)) {
+			error("Python script error, check console");
+		if (lineno >= 0) {
+			txt_move_toline(text, lineno-1, 0);
+			txt_sel_line(text);
+			pop_space_text(st);
+		}	
+	} else {
+		error("Error in other (possibly external) file, "\
+		"check console");
+	}	
+	}
+}
+
 
 void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 {
@@ -879,7 +982,6 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	char ascii= evt->ascii;
 	SpaceText *st= curarea->spacedata.first;
 	Text *text= st->text;
-	char *py_filename;
 	int do_draw=0, p;
 
 	if (BPY_spacetext_is_pywin(st)) {
@@ -893,21 +995,21 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 	text= st->text;
 	
 	if (!text) {
-			if (event==RIGHTMOUSE) {
-				switch (pupmenu("File %t|New %x0|Open... %x1")) {
-				case 0:
-					st->text= add_empty_text();
-					st->top= 0;
-				
-					allqueue(REDRAWTEXT, 0);
-					allqueue(REDRAWHEADERS, 0);
-					break;
-				case 1:
-					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
-					break;
-				}
+		if (event==RIGHTMOUSE) {
+			switch (pupmenu("File %t|New %x0|Open... %x1")) {
+			case 0:
+				st->text= add_empty_text();
+				st->top= 0;
+			
+				allqueue(REDRAWTEXT, 0);
+				allqueue(REDRAWHEADERS, 0);
+				break;
+			case 1:
+				activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
+				break;
 			}
-			if (val && !ELEM(G.qual, 0, LR_SHIFTKEY)) {
+		}
+		if (val && !ELEM(G.qual, 0, LR_SHIFTKEY)) {
 			if (event==FKEY && (G.qual & LR_ALTKEY) && (G.qual & LR_SHIFTKEY)) {
 				switch (pupmenu("File %t|New %x0|Open... %x1")) {
 				case 0:
@@ -920,12 +1022,26 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 				case 1:
 					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
 					break;
+				}
+			} 
+			else if (event==QKEY) {
+					if(okee("QUIT BLENDER")) exit_usiblender();
 			}
-		} else if (event==QKEY) {
-				if(okee("QUIT BLENDER")) exit_usiblender();
+			else if (event==NKEY) {
+				if (G.qual & LR_ALTKEY) {
+					st->text= add_empty_text();
+					st->top= 0;
+				
+					allqueue(REDRAWTEXT, 0);
+					allqueue(REDRAWHEADERS, 0);
+				}
+			}
+			else if (event==OKEY) {
+				if (G.qual & LR_ALTKEY) {
+					activate_fileselect(FILE_SPECIAL, "LOAD TEXT FILE", G.sce, add_text_fs);
+				}
 			}
 		}
-		
 		return;
 	}
 	
@@ -1011,37 +1127,12 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 					break;
 				}
 			} else if (G.qual & LR_ALTKEY) {
-				char *findstr= last_txt_find_string;
-				
-				if (txt_has_sel(text) && !(G.qual & LR_CTRLKEY)) {
-					findstr= txt_sel_to_buf(text);
+					if (txt_has_sel(text) && !(G.qual & LR_CTRLKEY)) {
+					txt_find_panel(st,0);
 				} else if (!last_txt_find_string || (G.qual & LR_CTRLKEY)) {
-					char buf[256];
-
-					if (findstr && strlen(findstr)<(sizeof(buf)-1))
-						strcpy(buf, findstr);
-					else
-						buf[0]= 0;
-					
-					if (sbutton(buf, 0, sizeof(buf)-1, "Find: ") && buf[0])
-						findstr= BLI_strdup(buf);
-					else
-						findstr= NULL;
+					txt_find_panel(st,1);
 				}
-
-				if (findstr!=last_txt_find_string) {
-					if (last_txt_find_string)
-						MEM_freeN(last_txt_find_string);
-					last_txt_find_string= findstr;
-				}
-				
-				if (findstr) {
-					if (txt_find_string(text, findstr))
-						pop_space_text(st);
-					else
-						error("Not found: %s", findstr);
-				}
-					
+	
 				do_draw= 1;
 			}
 			
@@ -1194,25 +1285,21 @@ void winqreadtextspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			}
 			break;
 
+		case NKEY:
+			if (G.qual & LR_ALTKEY) {
+				st->text= add_empty_text();
+				st->top= 0;
+			
+				allqueue(REDRAWTEXT, 0);
+				allqueue(REDRAWHEADERS, 0);
+
+			}
+			break;
+
 		case PKEY:
 			if (G.qual & LR_ALTKEY) {
-				if (!BPY_txt_do_python(st)) {
-					int lineno = BPY_Err_getLinenumber();
-					// jump to error if happened in current text:
-					py_filename = (char*) BPY_Err_getFilename();
-					if (!strcmp(py_filename, st->text->id.name+2)) {
-						error("Python script error, check console");
-						if (lineno >= 0) {
-							txt_move_toline(text, lineno-1, 0);
-							txt_sel_line(text);
-							do_draw= 1;
-							pop_space_text(st);
-						}	
-					} else {
-						error("Error in other (possibly external) file, "\
-						"check console");
-					}	
-				}
+				run_python_script(st);
+				do_draw= 1;
 			}
 			break;
 			
