@@ -1,5 +1,5 @@
 /**
- * $Id: drawview.c,v 1.14 2003/07/21 17:46:55 ton Exp $
+ * $Id: drawview.c,v 1.50 2003/11/23 14:28:46 michel Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -33,6 +33,7 @@
 //#define NAN_LINEAR_PHYSICS
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -52,27 +53,37 @@
 
 #include "BMF_Api.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_arithb.h"
-#include "BLI_editVert.h"
-
 #include "IMB_imbuf_types.h"
 
-#include "DNA_object_types.h"
+#include "DNA_action_types.h"
+#include "DNA_armature_types.h"
 #include "DNA_constraint_types.h"
+#include "DNA_curve_types.h"
 #include "DNA_group_types.h"
 #include "DNA_image_types.h"
+#include "DNA_lattice_types.h"
+#include "DNA_mesh_types.h"
+#include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_texture_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_userdef_types.h"
+#include "DNA_space_types.h"
+
+#include "BLI_blenlib.h"
+#include "BLI_arithb.h"
+#include "BLI_editVert.h"
 
 #include "BKE_action.h"
+#include "BKE_armature.h"
 #include "BKE_anim.h"
 #include "BKE_constraint.h"
+#include "BKE_curve.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_ika.h"
+#include "BKE_lattice.h"
+#include "BKE_library.h"
 #include "BKE_image.h"
 #include "BKE_ipo.h"
 #include "BKE_key.h"
@@ -86,10 +97,12 @@
 #include "BIF_screen.h"
 #include "BIF_interface.h"
 #include "BIF_space.h"
-#include "BIF_buttons.h"
+#include "BIF_butspace.h"
 #include "BIF_drawimage.h"
 #include "BIF_editgroup.h"
 #include "BIF_mywindow.h"
+#include "BIF_editarmature.h"
+#include "BIF_poseobject.h"
 
 #include "BDR_drawmesh.h"
 #include "BDR_drawobject.h"
@@ -98,15 +111,15 @@
 #include "BSE_drawview.h"
 #include "BSE_headerbuttons.h"
 #include "BSE_seqaudio.h"
-
+#include "BSE_filesel.h"
 
 #include "RE_renderconverter.h"
 
-#include "BPY_extern.h" // Blender Python library
+#include "BPY_extern.h"
 
-#include "interface.h"
 #include "blendef.h"
 #include "mydevice.h"
+#include "butspace.h"  // event codes
 
 /* Modules used */
 #include "render.h"
@@ -154,22 +167,57 @@ void setalpha_bgpic(BGpic *bgpic)
 }
 
 
-static float light_pos1[] = { -0.3, 0.3, 0.90, 0.0 }; 
-/*  static float light_pos2[] = { 0.3, -0.3, -0.90, 0.0 };  never used? */
-
 void default_gl_light(void)
 {
-	float mat_specular[] = { 0.5, 0.5, 0.5, 1.0 };
-	float light_col1[] = { 0.8, 0.8, 0.8, 0.0 }; 
-
 	int a;
+	
+	/* initialize */
+	if(U.light[0].flag==0 && U.light[1].flag==0 && U.light[2].flag==0) {
+		U.light[0].flag= 1;
+		U.light[0].vec[0]= -0.3; U.light[0].vec[1]= 0.3; U.light[0].vec[2]= 0.9;
+		U.light[0].col[0]= 0.8; U.light[0].col[1]= 0.8; U.light[0].col[2]= 0.8;
+		U.light[0].spec[0]= 0.5; U.light[0].spec[1]= 0.5; U.light[0].spec[2]= 0.5;
+		U.light[0].spec[3]= 1.0;
 		
-	glLightfv(GL_LIGHT0, GL_POSITION, light_pos1); 
-	glLightfv(GL_LIGHT0, GL_DIFFUSE, light_col1); 
-	glLightfv(GL_LIGHT0, GL_SPECULAR, mat_specular); 
+		U.light[1].flag= 0;
+		U.light[1].vec[0]= 0.5; U.light[1].vec[1]= 0.5; U.light[1].vec[2]= 0.1;
+		U.light[1].col[0]= 0.4; U.light[1].col[1]= 0.4; U.light[1].col[2]= 0.8;
+		U.light[1].spec[0]= 0.3; U.light[1].spec[1]= 0.3; U.light[1].spec[2]= 0.5;
+		U.light[1].spec[3]= 1.0;
+	
+		U.light[2].flag= 0;
+		U.light[2].vec[0]= 0.3; U.light[2].vec[1]= -0.3; U.light[2].vec[2]= -0.2;
+		U.light[2].col[0]= 0.8; U.light[2].col[1]= 0.5; U.light[2].col[2]= 0.4;
+		U.light[2].spec[0]= 0.5; U.light[2].spec[1]= 0.4; U.light[2].spec[2]= 0.3;
+		U.light[2].spec[3]= 1.0;
+	}
+	
 
-	glEnable(GL_LIGHT0);
-	for(a=1; a<8; a++) glDisable(GL_LIGHT0+a);
+	glLightfv(GL_LIGHT0, GL_POSITION, U.light[0].vec); 
+	glLightfv(GL_LIGHT0, GL_DIFFUSE, U.light[0].col); 
+	glLightfv(GL_LIGHT0, GL_SPECULAR, U.light[0].spec); 
+
+	glLightfv(GL_LIGHT1, GL_POSITION, U.light[1].vec); 
+	glLightfv(GL_LIGHT1, GL_DIFFUSE, U.light[1].col); 
+	glLightfv(GL_LIGHT1, GL_SPECULAR, U.light[1].spec); 
+
+	glLightfv(GL_LIGHT2, GL_POSITION, U.light[2].vec); 
+	glLightfv(GL_LIGHT2, GL_DIFFUSE, U.light[2].col); 
+	glLightfv(GL_LIGHT2, GL_SPECULAR, U.light[2].spec); 
+
+	for(a=0; a<8; a++) {
+		if(a<3) {
+			if(U.light[a].flag) glEnable(GL_LIGHT0+a);
+			else glDisable(GL_LIGHT0+a);
+			
+			// clear stuff from other opengl lamp usage
+			glLightf(GL_LIGHT0+a, GL_SPOT_CUTOFF, 180.0);
+			glLightf(GL_LIGHT0+a, GL_CONSTANT_ATTENUATION, 1.0);
+			glLightf(GL_LIGHT0+a, GL_LINEAR_ATTENUATION, 0.0);
+		}
+		else glDisable(GL_LIGHT0+a);
+	}
+	
 	glDisable(GL_LIGHTING);
 
 	glDisable(GL_COLOR_MATERIAL);
@@ -401,13 +449,59 @@ void timestr(double time, char *str)
 	str[11]=0;
 }
 
+static void drawgrid_draw(float wx, float wy, float x, float y, float dx)
+{
+	float fx, fy;
+	
+	x+= (wx); 
+	y+= (wy);
+	fx= x/dx;
+	fx= x-dx*floor(fx);
+	
+	while(fx< curarea->winx) {
+		fdrawline(fx,  0.0,  fx,  (float)curarea->winy); 
+		fx+= dx; 
+	}
+
+	fy= y/dx;
+	fy= y-dx*floor(fy);
+	
+
+	while(fy< curarea->winy) {
+		fdrawline(0.0,  fy,  (float)curarea->winx,  fy); 
+		fy+= dx;
+	}
+
+}
+
+// not intern, called in editobject for constraint axis too
+void make_axis_color(char *col, char *col2, char axis)
+{
+	if(axis=='x') {
+		col2[0]= col[0]>219?255:col[0]+36;
+		col2[1]= col[1]<26?0:col[1]-26;
+		col2[2]= col[2]<26?0:col[2]-26;
+	}
+	else if(axis=='y') {
+		col2[0]= col[0]<46?0:col[0]-36;
+		col2[1]= col[1]>189?255:col[1]+66;
+		col2[2]= col[2]<46?0:col[2]-36; 
+	}
+	else {
+		col2[0]= col[0]<26?0:col[0]-26; 
+		col2[1]= col[1]<26?0:col[1]-26; 
+		col2[2]= col[2]>209?255:col[2]+46;
+	}
+	
+}
 
 static void drawgrid(void)
 {
 	/* extern short bgpicmode; */
 	float wx, wy, x, y, fw, fx, fy, dx;
 	float vec4[4];
-
+	char col[3], col2[3];
+	
 	vec4[0]=vec4[1]=vec4[2]=0.0; 
 	vec4[3]= 1.0;
 	Mat4MulVec4fl(G.vd->persmat, vec4);
@@ -431,55 +525,93 @@ static void drawgrid(void)
 
 	dx= fabs(x-(wx)*fx/fw);
 	if(dx==0) dx= fabs(y-(wy)*fy/fw);
+	
+	glDepthMask(0);		// disable write in zbuffer
+	
+	/* check zoom out */
+	BIF_ThemeColor(TH_GRID);
+	persp(PERSP_WIN);
 
 	if(dx<6.0) {
 		dx*= 10.0;
-		setlinestyle(3);
-		if(dx<6.0) {
+		
+		if(dx<6.0) {	
 			dx*= 10.0;
+			
 			if(dx<6.0) {
-				setlinestyle(0);
-				return;
+				dx*=10;
+				if(dx<6.0);
+				else {
+					BIF_ThemeColor(TH_GRID);
+					drawgrid_draw(wx, wy, x, y, dx);
+				}
+			}
+			else {	// start blending out
+				BIF_ThemeColorBlend(TH_BACK, TH_GRID, dx/60.0);
+				drawgrid_draw(wx, wy, x, y, dx);
+			
+				BIF_ThemeColor(TH_GRID);
+				drawgrid_draw(wx, wy, x, y, 10*dx);
 			}
 		}
+		else {	// start blending out (6 < dx < 60)
+			BIF_ThemeColorBlend(TH_BACK, TH_GRID, dx/60.0);
+			drawgrid_draw(wx, wy, x, y, dx);
+			
+			BIF_ThemeColor(TH_GRID);
+			drawgrid_draw(wx, wy, x, y, 10*dx);
+		}
+	}
+	else {
+		if(dx>60.0) {		// start blending in
+			dx/= 10.0;			
+			if(dx>60.0) {		// start blending in
+				dx/= 10.0;
+				if(dx>60.0) {
+					BIF_ThemeColor(TH_GRID);
+					drawgrid_draw(wx, wy, x, y, dx);
+				}
+				else {
+					BIF_ThemeColorBlend(TH_BACK, TH_GRID, dx/60.0);
+					drawgrid_draw(wx, wy, x, y, dx);
+					BIF_ThemeColor(TH_GRID);
+					drawgrid_draw(wx, wy, x, y, dx*10);
+				}
+			}
+			else {
+				BIF_ThemeColorBlend(TH_BACK, TH_GRID, dx/60.0);
+				drawgrid_draw(wx, wy, x, y, dx);
+				BIF_ThemeColor(TH_GRID);
+				drawgrid_draw(wx, wy, x, y, dx*10);
+			}
+		}
+		else {
+			BIF_ThemeColorBlend(TH_BACK, TH_GRID, dx/60.0);
+			drawgrid_draw(wx, wy, x, y, dx);
+			BIF_ThemeColor(TH_GRID);
+			drawgrid_draw(wx, wy, x, y, dx*10);
+		}
 	}
-	
-	persp(0);
 
-	cpack(0x606060);
-	
 	x+= (wx); 
 	y+= (wy);
-	fx= x/dx;
-	fx= x-dx*floor(fx);
+	BIF_GetThemeColor3ubv(TH_GRID, col);
 	
-	while(fx< curarea->winx) {
-		fdrawline(fx,  0.0,  fx,  (float)curarea->winy); 
-		fx+= dx; 
-	}
-
-	fy= y/dx;
-	fy= y-dx*floor(fy);
-	
-
-	while(fy< curarea->winy) {
-		fdrawline(0.0,  fy,  (float)curarea->winx,  fy); 
-		fy+= dx;
-	}
-
 	/* center cross */
-	if(G.vd->view==3) cpack(0xA0D0A0); /* y-as */
-	else cpack(0xA0A0D0);	/* x-as */
-
+	if(G.vd->view==3) make_axis_color(col, col2, 'y');
+	else make_axis_color(col, col2, 'x');
+	glColor3ubv(col2);
+	
 	fdrawline(0.0,  y,  (float)curarea->winx,  y); 
 	
-	if(G.vd->view==7) cpack(0xA0D0A0);	/* y-as */
-	else cpack(0xE0A0A0);	/* z-as */
+	if(G.vd->view==7) make_axis_color(col, col2, 'y');
+	else make_axis_color(col, col2, 'z');
+	glColor3ubv(col2);
 
-	fdrawline(x,  0.0,  x,  (float)curarea->winy); 
+	fdrawline(x, 0.0, x, (float)curarea->winy); 
 
-	persp(1);
-	setlinestyle(0);
+	glDepthMask(1);		// enable write in zbuffer
+	persp(PERSP_VIEW);
 }
 
 
@@ -488,6 +620,7 @@ static void drawfloor(void)
 	View3D *vd;
 	float vert[3], grid;
 	int a, gridlines;
+	char col[3], col2[3];
 	
 	vd= curarea->spacedata.first;
 
@@ -498,17 +631,18 @@ static void drawfloor(void)
 	gridlines= vd->gridlines/2;
 	grid= gridlines*vd->grid;
 	
-	cpack(0x606060);
-
+	BIF_GetThemeColor3ubv(TH_GRID, col);
+	
 	for(a= -gridlines;a<=gridlines;a++) {
 
 		if(a==0) {
-			if(vd->persp==0) cpack(0xA0D0A0);
-			else cpack(0x402000);
+			make_axis_color(col, col2, 'y');
+			glColor3ubv(col2);
 		}
-		else if(a==1) {
-			cpack(0x606060);
+		else if( (a % 10)==0) {
+			BIF_ThemeColorShade(TH_GRID, -10);
 		}
+		else BIF_ThemeColorShade(TH_GRID, 10);
 		
 	
 		glBegin(GL_LINE_STRIP);
@@ -520,16 +654,15 @@ static void drawfloor(void)
 		glEnd();
 	}
 	
-	cpack(0x606060);
-	
 	for(a= -gridlines;a<=gridlines;a++) {
 		if(a==0) {
-			if(vd->persp==0) cpack(0xA0A0D0);
-			else cpack(0);
+			make_axis_color(col, col2, 'x');
+			glColor3ubv(col2);
 		}
-		else if(a==1) {
-			cpack(0x606060);
+		else if( (a % 10)==0) {
+			BIF_ThemeColorShade(TH_GRID, -10);
 		}
+		else BIF_ThemeColorShade(TH_GRID, 10);
 	
 		glBegin(GL_LINE_STRIP);
         vert[1]= a*vd->grid;
@@ -705,6 +838,9 @@ void backdrawview3d(int test)
 			return;
 		}
 	}
+	
+	persp(PERSP_VIEW);
+	
 #ifdef __APPLE__
 	glDrawBuffer(GL_AUX0);
 #endif	
@@ -785,7 +921,7 @@ static void draw_selected_name(char *name)
 
 	sprintf(info, "(%d) %s", CFRA, name);
 
-	cpack(0xFFFFFF);
+	BIF_ThemeColor(TH_TEXT_HI);
 	glRasterPos2i(30,  10);
 	BMF_DrawString(G.fonts, info);
 }
@@ -810,13 +946,579 @@ static void draw_view_icon(void)
 	glDisable(GL_BLEND);
 }
 
+/* ******************* view3d space & buttons ************** */
+
+static void view3d_change_bgpic_ima(View3D *v3d, Image *newima) {
+	if (v3d->bgpic && v3d->bgpic->ima!=newima) {
+		if (newima)
+			id_us_plus((ID*) newima);
+		if (v3d->bgpic->ima)
+			v3d->bgpic->ima->id.us--;
+		v3d->bgpic->ima= newima;
+
+		if(v3d->bgpic->rect) MEM_freeN(v3d->bgpic->rect);
+		v3d->bgpic->rect= NULL;
+		
+		allqueue(REDRAWVIEW3D, 0);
+	}
+}
+static void view3d_change_bgpic_tex(View3D *v3d, Tex *newtex) {
+	if (v3d->bgpic && v3d->bgpic->tex!=newtex) {
+		if (newtex)
+			id_us_plus((ID*) newtex);
+		if (v3d->bgpic->tex)
+			v3d->bgpic->tex->id.us--;
+		v3d->bgpic->tex= newtex;
+		
+		allqueue(REDRAWVIEW3D, 0);
+	}
+}
+
+static void load_bgpic_image(char *name)
+{
+	Image *ima;
+	View3D *vd;
+	
+	vd= G.vd;
+	if(vd==0 || vd->bgpic==0) return;
+	
+	ima= add_image(name);
+	if(ima) {
+		if(vd->bgpic->ima) {
+			vd->bgpic->ima->id.us--;
+		}
+		vd->bgpic->ima= ima;
+		
+		free_image_buffers(ima);	/* force read again */
+		ima->ok= 1;
+	}
+	allqueue(REDRAWVIEW3D, 0);
+	
+}
+
+/* this one assumes there is only one global active object in blender...  (for object panel) */
+static float ob_eul[4];	// used for quat too....
+/* this one assumes there is only one editmode in blender...  (for object panel) */
+static float ve_median[4];
+static int ve_median_tot=0;
+
+/* is used for both read and write... */
+static void v3d_editvertex_buts(uiBlock *block, Object *ob, float lim)
+{
+	EditVert *eve;
+	float median[3];
+	int tot;
+	
+	median[0]= median[1]= median[2]= 0.0;
+	tot= 0;
+	
+	if(ob->type==OB_MESH) {
+		eve= G.edve.first;
+		while(eve) {
+			if(eve->f & 1) {
+				tot++;
+				VecAddf(median, median, eve->co);
+			}
+			eve= eve->next;
+		}
+	}
+	else if(ob->type==OB_CURVE || ob->type==OB_SURF) {
+		extern ListBase editNurb; /* editcurve.c */
+		Nurb *nu;
+		BPoint *bp;
+		BezTriple *bezt;
+		int a;
+		
+		nu= editNurb.first;
+		while(nu) {
+			if((nu->type & 7)==1) {
+				bezt= nu->bezt;
+				a= nu->pntsu;
+				while(a--) {
+					if(bezt->f2 & 1) {
+						VecAddf(median, median, bezt->vec[1]);
+						tot++;
+					}
+					else {
+						if(bezt->f1 & 1) {
+							VecAddf(median, median, bezt->vec[0]);
+							tot++;
+						}
+						if(bezt->f3 & 1) {
+							VecAddf(median, median, bezt->vec[2]);
+							tot++;
+						}
+					}
+					bezt++;
+				}
+			}
+			else {
+				bp= nu->bp;
+				a= nu->pntsu*nu->pntsv;
+				while(a--) {
+					if(bp->f1 & 1) {
+						VecAddf(median, median, bp->vec);
+						tot++;
+					}
+					bp++;
+				}
+			}
+			nu= nu->next;
+		}
+	
+	}
+	if(tot==0) return;
+
+	median[0] /= (float)tot;
+	median[1] /= (float)tot;
+	median[2] /= (float)tot;
+	
+	if(block) {	// buttons
+	
+		ve_median_tot= tot;
+		VECCOPY(ve_median, median);
+		
+		uiBlockBeginAlign(block);
+		if(ve_median_tot==1) {
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Vertex X:",	10, 140, 300, 19, &(ve_median[0]), -lim, lim, 10, 0, "");
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Vertex Y:",	10, 120, 300, 19, &(ve_median[1]), -lim, lim, 10, 0, "");
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Vertex Z:",	10, 100, 300, 19, &(ve_median[2]), -lim, lim, 10, 0, "");
+		}
+		else {
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Median X:",	10, 140, 300, 19, &(ve_median[0]), -lim, lim, 10, 0, "");
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Median Y:",	10, 120, 300, 19, &(ve_median[1]), -lim, lim, 10, 0, "");
+			uiDefButF(block, NUM, B_OBJECTPANELMEDIAN, "Median Z:",	10, 100, 300, 19, &(ve_median[2]), -lim, lim, 10, 0, "");
+		}
+		uiBlockEndAlign(block);
+	}
+	else {	// apply
+		
+		VecSubf(median, ve_median, median);
+		
+		if(ob->type==OB_MESH) {
+			eve= G.edve.first;
+			while(eve) {
+				if(eve->f & 1) {
+					VecAddf(eve->co, eve->co, median);
+				}
+				eve= eve->next;
+			}
+		}
+		else if(ob->type==OB_CURVE || ob->type==OB_SURF) {
+			extern ListBase editNurb; /* editcurve.c */
+			Nurb *nu;
+			BPoint *bp;
+			BezTriple *bezt;
+			int a;
+			
+			nu= editNurb.first;
+			while(nu) {
+				if((nu->type & 7)==1) {
+					bezt= nu->bezt;
+					a= nu->pntsu;
+					while(a--) {
+						if(bezt->f2 & 1) {
+							VecAddf(bezt->vec[0], bezt->vec[0], median);
+							VecAddf(bezt->vec[1], bezt->vec[1], median);
+							VecAddf(bezt->vec[2], bezt->vec[2], median);
+						}
+						else {
+							if(bezt->f1 & 1) {
+								VecAddf(bezt->vec[0], bezt->vec[0], median);
+							}
+							if(bezt->f3 & 1) {
+								VecAddf(bezt->vec[2], bezt->vec[2], median);
+							}
+						}
+						bezt++;
+					}
+				}
+				else {
+					bp= nu->bp;
+					a= nu->pntsu*nu->pntsv;
+					while(a--) {
+						if(bp->f1 & 1) {
+							VecAddf(bp->vec, bp->vec, median);
+							tot++;
+						}
+						bp++;
+					}
+				}
+				test2DNurb(nu);
+				testhandlesNurb(nu); /* test for bezier too */
+
+				nu= nu->next;
+			}
+			makeDispList(G.obedit);
+		}
+	}
+}
+
+static void v3d_posearmature_buts(uiBlock *block, Object *ob, float lim)
+{
+	bArmature *arm;
+	Bone *bone;
+
+	arm = get_armature(OBACT);
+	if (!arm)
+		return;
+
+	bone = get_first_selected_bone();
+
+	if (!bone)
+		return;
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "QuatX:",	10, 120, 140, 19, bone->quat, -100.0, 100.0, 10, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "QuatZ:",	10, 100, 140, 19, bone->quat+2, -100.0, 100.0, 10, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "QuatY:",	160, 120, 140, 19, bone->quat+1, -100.0, 100.0, 10, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "QuatW:",	160, 100, 140, 19, bone->quat+3, -100.0, 100.0, 10, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "LocX:",	10, 70, 140, 19, bone->loc, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "LocY:",	10, 50, 140, 19, bone->loc+1, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "locZ:",	10, 30, 140, 19, bone->loc+2, -lim, lim, 100, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "SizeX:",	160, 70, 140, 19, bone->size, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "SizeZ:",	160, 50, 140, 19, bone->size+1, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL2, "SizeZ:",	160, 30, 140, 19, bone->size+2, -lim, lim, 100, 0, "");
+	uiBlockEndAlign(block);
+}
+
+static void v3d_editarmature_buts(uiBlock *block, Object *ob, float lim)
+{
+	EditBone *ebone;
+	
+	ebone= G.edbo.first;
+
+	for (ebone = G.edbo.first; ebone; ebone=ebone->next){
+		if (ebone->flag & BONE_SELECTED)
+			break;
+	}
+
+	if (!ebone)
+		return;
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "RootX:",	10, 70, 140, 19, ebone->head, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "RootY:",	10, 50, 140, 19, ebone->head+1, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "RootZ:",	10, 30, 140, 19, ebone->head+2, -lim, lim, 100, 0, "");
+	uiBlockBeginAlign(block);
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "TipX:",	160, 70, 140, 19, ebone->tail, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "TipY:",	160, 50, 140, 19, ebone->tail+1, -lim, lim, 100, 0, "");
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "TipZ:",	160, 30, 140, 19, ebone->tail+2, -lim, lim, 100, 0, "");
+	uiBlockEndAlign(block);
+	ob_eul[0]= 180.0*ebone->roll/M_PI;
+	uiDefButF(block, NUM, B_ARMATUREPANEL1, "Roll:",	10, 100, 140, 19, ob_eul, -lim, lim, 100, 0, "");
+
+}
+
+
+
+void do_viewbuts(unsigned short event)
+{
+	View3D *vd;
+	Object *ob= OBACT;
+	char *name;
+	
+	vd= G.vd;
+	if(vd==0) return;
+
+	switch(event) {
+	case B_LOADBGPIC:
+		if(vd->bgpic && vd->bgpic->ima) name= vd->bgpic->ima->name;
+		else name= G.ima;
+		
+		if(G.qual==LR_CTRLKEY)
+			activate_imageselect(FILE_SPECIAL, "SELECT IMAGE", name, load_bgpic_image);
+		else
+			activate_fileselect(FILE_SPECIAL, "SELECT IMAGE", name, load_bgpic_image);
+		break;
+		
+	case B_BLENDBGPIC:
+		if(vd->bgpic && vd->bgpic->rect) setalpha_bgpic(vd->bgpic);
+		addqueue(curarea->win, REDRAW, 1);
+		break;
+		
+	case B_BGPICBROWSE:
+		if(vd->bgpic) {
+			if (G.buts->menunr==-2) {
+				activate_databrowse((ID*) vd->bgpic->ima, ID_IM, 0, B_BGPICBROWSE, &G.buts->menunr, do_viewbuts);
+			} else if (G.buts->menunr>0) {
+				Image *newima= (Image*) BLI_findlink(&G.main->image, G.buts->menunr-1);
+
+				if (newima)
+					view3d_change_bgpic_ima(vd, newima);
+			}
+		}
+		break;
+		
+	case B_BGPICCLEAR:
+		if (vd->bgpic)
+			view3d_change_bgpic_ima(vd, NULL);
+		break;
+		
+	case B_BGPICTEX:
+		if (vd->bgpic) {
+			if (G.buts->texnr==-2) {
+				activate_databrowse((ID*) vd->bgpic->tex, ID_TE, 0, B_BGPICTEX, &G.buts->texnr, do_viewbuts);
+			} else if (G.buts->texnr>0) {
+				Tex *newtex= (Tex*) BLI_findlink(&G.main->tex, G.buts->texnr-1);
+				
+				if (newtex)
+					view3d_change_bgpic_tex(vd, newtex);
+			}
+		}
+		break;
+		
+	case B_BGPICTEXCLEAR:
+		if (vd->bgpic)
+			view3d_change_bgpic_tex(vd, NULL);
+		break;
+		
+	case B_OBJECTPANELROT:
+		if(ob) {
+			ob->rot[0]= M_PI*ob_eul[0]/180.0;
+			ob->rot[1]= M_PI*ob_eul[1]/180.0;
+			ob->rot[2]= M_PI*ob_eul[2]/180.0;
+			allqueue(REDRAWVIEW3D, 1);
+		}
+		break;
+		
+	case B_OBJECTPANELMEDIAN:
+		if(ob) {
+			v3d_editvertex_buts(NULL, ob, 1.0);
+			makeDispList(ob);
+			allqueue(REDRAWVIEW3D, 1);
+		}
+		break;
+		
+	case B_ARMATUREPANEL1:
+		{
+			EditBone *ebone, *child;
+			
+			ebone= G.edbo.first;
+			for (ebone = G.edbo.first; ebone; ebone=ebone->next){
+				if (ebone->flag & BONE_SELECTED) break;
+			}
+			if (ebone) {
+				ebone->roll= M_PI*ob_eul[0]/180.0;
+				//	Update our parent
+				if (ebone->parent && ebone->flag & BONE_IK_TOPARENT){
+					VECCOPY (ebone->parent->tail, ebone->head);
+				}
+			
+				//	Update our children if necessary
+				for (child = G.edbo.first; child; child=child->next){
+					if (child->parent == ebone && child->flag & BONE_IK_TOPARENT){
+						VECCOPY (child->head, ebone->tail);
+					}
+				}
+				allqueue(REDRAWVIEW3D, 1);
+			}
+		}
+		break;
+	case B_ARMATUREPANEL2:
+		{
+			bPoseChannel *chan;
+			bArmature *arm;
+			Bone *bone;
+		
+			arm = get_armature(OBACT);
+			if (!arm) return;
+			bone = get_first_selected_bone();
+		
+			if (!bone) return;
+
+			/* This is similar to code in special_trans_update */
+	
+			if (!G.obpose->pose) G.obpose->pose= MEM_callocN(sizeof(bPose), "pose");
+			chan = MEM_callocN (sizeof (bPoseChannel), "transPoseChannel");
+		
+			chan->flag |= POSE_LOC|POSE_ROT|POSE_SIZE;
+			memcpy (chan->loc, bone->loc, sizeof (chan->loc));
+			memcpy (chan->quat, bone->quat, sizeof (chan->quat));
+			memcpy (chan->size, bone->size, sizeof (chan->size));
+			strcpy (chan->name, bone->name);
+			
+			set_pose_channel (G.obpose->pose, chan);
+			allqueue(REDRAWVIEW3D, 1);
+		}
+	}
+}
+
+
+static void view3d_panel_object(short cntrl)	// VIEW3D_HANDLER_OBJECT
+{
+	uiBlock *block;
+	Object *ob= OBACT;
+	float lim;
+	
+	if(ob==NULL) return;
+
+	block= uiNewBlock(&curarea->uiblocks, "view3d_panel_object", UI_EMBOSS, UI_HELV, curarea->win);
+	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | cntrl);
+	uiSetPanelHandler(VIEW3D_HANDLER_OBJECT);  // for close and esc
+	if(uiNewPanel(curarea, block, "Transform Properties", "View3d", 10, 230, 318, 204)==0) return;
+
+	uiDefBut(block, TEX, B_IDNAME, "OB: ",	10,180,140,20, ob->id.name+2, 0.0, 18.0, 0, 0, "");
+	uiDefIDPoinBut(block, test_obpoin_but, B_REDR, "Par:", 160, 180, 140, 20, &ob->parent, "Parent Object"); 
+
+	lim= 1000.0*MAX2(1.0, G.vd->grid);
+
+	if(ob==G.obedit) {
+		if(ob->type==OB_ARMATURE) v3d_editarmature_buts(block, ob, lim);
+		else v3d_editvertex_buts(block, ob, lim);
+	}
+	else if(ob==G.obpose) {
+		v3d_posearmature_buts(block, ob, lim);
+	}
+	else {
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, REDRAWVIEW3D, "LocX:",		10, 140, 140, 19, &(ob->loc[0]), -lim, lim, 100, 0, "");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "LocY:",		10, 120, 140, 19, &(ob->loc[1]), -lim, lim, 100, 0, "");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "LocZ:",		10, 100, 140, 19, &(ob->loc[2]), -lim, lim, 100, 0, "");
+	
+		ob_eul[0]= 180.0*ob->rot[0]/M_PI;
+		ob_eul[1]= 180.0*ob->rot[1]/M_PI;
+		ob_eul[2]= 180.0*ob->rot[2]/M_PI;
+		
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, B_OBJECTPANELROT, "RotX:",	10, 70, 140, 19, &(ob_eul[0]), -lim, lim, 1000, 0, "");
+		uiDefButF(block, NUM, B_OBJECTPANELROT, "RotY:",	10, 50, 140, 19, &(ob_eul[1]), -lim, lim, 1000, 0, "");
+		uiDefButF(block, NUM, B_OBJECTPANELROT, "RotZ:",	10, 30, 140, 19, &(ob_eul[2]), -lim, lim, 1000, 0, "");
+		uiBlockBeginAlign(block);
+		uiDefButF(block, NUM, REDRAWVIEW3D, "SizeX:",		160, 70, 140, 19, &(ob->size[0]), -lim, lim, 100, 0, "");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "SizeY:",		160, 50, 140, 19, &(ob->size[1]), -lim, lim, 100, 0, "");
+		uiDefButF(block, NUM, REDRAWVIEW3D, "SizeZ:",		160, 30, 140, 19, &(ob->size[2]), -lim, lim, 100, 0, "");
+		uiBlockEndAlign(block);
+	}
+}
+
+static void view3d_panel_background(cntrl)	// VIEW3D_HANDLER_BACKGROUND
+{
+	uiBlock *block;
+	View3D *vd;
+	ID *id;
+	char *strp;
+	
+	vd= G.vd;
+
+	block= uiNewBlock(&curarea->uiblocks, "view3d_panel_background", UI_EMBOSS, UI_HELV, curarea->win);
+	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE  | cntrl);
+	uiSetPanelHandler(VIEW3D_HANDLER_BACKGROUND);  // for close and esc
+	if(uiNewPanel(curarea, block, "Background", "View3d", 340, 10, 318, 204)==0) return;
+
+	if(vd->flag & V3D_DISPBGPIC) {
+		if(vd->bgpic==0) {
+			vd->bgpic= MEM_callocN(sizeof(BGpic), "bgpic");
+			vd->bgpic->size= 5.0;
+			vd->bgpic->blend= 0.5;
+		}
+	}
+	uiDefButS(block, TOG|BIT|1, REDRAWVIEW3D, "BackGroundPic",	10,160,150,20 , &vd->flag, 0, 0, 0, 0, "Display a picture in the 3D background");
+	
+	if(vd->bgpic) {
+		
+		uiDefButF(block, NUM, REDRAWVIEW3D, "Size:", 		160,160,150,20, &vd->bgpic->size, 0.1, 250.0, 100, 0, "Set the size for the width of the BackGroundPic");
+		
+		id= (ID *)vd->bgpic->ima;
+		IDnames_to_pupstring(&strp, NULL, NULL, &(G.main->image), id, &(G.buts->menunr));
+		if(strp[0]) {
+			uiBlockBeginAlign(block);
+			uiDefButS(block, MENU, B_BGPICBROWSE, strp, 	10,130,20,20, &(G.buts->menunr), 0, 0, 0, 0, "Browse");
+		
+			if(vd->bgpic->ima)  {
+				uiDefBut(block, TEX,	    0,"BGpic: ",		30,130,260,20,&vd->bgpic->ima->name,0.0,100.0, 0, 0, "The Selected BackGroundPic");
+				uiDefIconBut(block, BUT, B_BGPICCLEAR, ICON_X, 	290,130,20,20, 0, 0, 0, 0, 0, "Remove background image link");
+			}
+			uiBlockEndAlign(block);
+		}
+		MEM_freeN(strp);
+
+		uiDefBut(block, BUT,	    B_LOADBGPIC, "LOAD",	10,100,100,20, 0, 0, 0, 0, 0, "Specify the BackGroundPic");
+		uiDefButF(block, NUMSLI, B_BLENDBGPIC, "Blend:",	120,100,190,20,&vd->bgpic->blend, 0.0,1.0, 0, 0, "Set the BackGroundPic transparency");
+		
+		uiDefButF(block, NUM, B_DIFF, "Center X: ",	10,70,140,20,&vd->bgpic->xof, -20.0,20.0, 10, 2, "Set the BackGroundPic X Offset");
+		uiDefButF(block, NUM, B_DIFF, "Center Y: ",	160,70,140,20,&vd->bgpic->yof, -20.0,20.0, 10, 2, "Set the BackGroundPic Y Offset");
+
+		/* There is a bug here ... (what bug? where? what is this? - zr) */
+		/* (ton) the use of G.buts->texnr is hackish */
+		/* texture block: */
+		id= (ID *)vd->bgpic->tex;
+		IDnames_to_pupstring(&strp, NULL, NULL, &(G.main->tex), id, &(G.buts->texnr));
+		if (strp[0]) 
+			uiDefButS(block, MENU, B_BGPICTEX, strp,			10, 50, 20,20, &(G.buts->texnr), 0, 0, 0, 0, "Select texture for animated backgroundimage");
+		MEM_freeN(strp);
+		
+		if (id) {
+			uiDefBut(block, TEX, B_IDNAME, "TE:",				30,50,260,20, id->name+2, 0.0, 18.0, 0, 0, "");
+			uiDefIconBut(block, BUT, B_BGPICTEXCLEAR, ICON_X, 	290,50,20,20, 0, 0, 0, 0, 0, "Remove background texture link");
+		}
+	}
+}
+
+
+static void view3d_panel_properties(cntrl)	// VIEW3D_HANDLER_SETTINGS
+{
+	uiBlock *block;
+	View3D *vd;
+	
+	vd= G.vd;
+
+	block= uiNewBlock(&curarea->uiblocks, "view3d_panel_properties", UI_EMBOSS, UI_HELV, curarea->win);
+	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE  | cntrl);
+	uiSetPanelHandler(VIEW3D_HANDLER_PROPERTIES);  // for close and esc
+	if(uiNewPanel(curarea, block, "3D Viewport properties", "View3d", 10, 10, 318, 204)==0) return;
+
+
+	uiDefButF(block, NUM, REDRAWVIEW3D, "Grid:",			10, 50, 140, 19, &vd->grid, 0.001, 100.0, 10, 0, "Set the distance between gridlines");
+	uiDefButS(block, NUM, REDRAWVIEW3D, "GridLines:",		160, 50, 140, 19, &vd->gridlines, 0.0, 100.0, 100, 0, "Set the number of gridlines");
+	uiDefButF(block, NUM, REDRAWVIEW3D, "Lens:",			10, 30, 140, 19, &vd->lens, 10.0, 120.0, 100, 0, "Set the lens for the perspective view");
+	
+	uiDefButF(block, NUM, REDRAWVIEW3D, "ClipStart:",		10, 10, 140, 19, &vd->near, vd->grid/10.0, 100.0, 10, 0, "Set startvalue in perspective view mode");
+	uiDefButF(block, NUM, REDRAWVIEW3D, "ClipEnd:",			160, 10, 140, 19, &vd->far, 1.0, 1000.0*vd->grid, 100, 0, "Set endvalue in perspective view mode");
+
+
+}
+
+
+
+static void view3d_blockhandlers(ScrArea *sa)
+{
+	View3D *v3d= sa->spacedata.first;
+	short a;
+	
+	uiFreeBlocksWin(&sa->uiblocks, sa->win);
+
+	for(a=0; a<SPACE_MAXHANDLER; a+=2) {
+	
+		switch(v3d->blockhandler[a]) {
+
+		case VIEW3D_HANDLER_PROPERTIES:
+			view3d_panel_properties(v3d->blockhandler[a+1]);
+			break;
+		case VIEW3D_HANDLER_BACKGROUND:
+			view3d_panel_background(v3d->blockhandler[a+1]);
+			break;
+		case VIEW3D_HANDLER_OBJECT:
+			view3d_panel_object(v3d->blockhandler[a+1]);
+		
+			break;
+		
+		}
+		/* clear action value for event */
+		v3d->blockhandler[a+1]= 0;
+	}
+	uiDrawBlocksPanels(sa, 0);
+
+}
+
 void drawview3dspace(ScrArea *sa, void *spacedata)
 {
+	extern void constline_callback(void);	// editobject.c helpline
 	Base *base;
 	Object *ob;
 	
 	setwinmatrixview3d(0);	/* 0= no pick rect */
-
 	setviewmatrixview3d();
 
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
@@ -830,18 +1532,23 @@ void drawview3dspace(ScrArea *sa, void *spacedata)
 			glClearColor(0.0, 0.0, 0.0, 0.0); 
 		}
 		else {
-			glClearColor(0.45, 0.45, 0.45, 0.0); 
+			float col[3];
+			BIF_GetThemeColor3fv(TH_BACK, col);
+			glClearColor(col[0], col[1], col[2], 0.0); 
 		}
 		glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
 		
 		glLoadIdentity();
 	}
 	else {
-		glClearColor(0.45, 0.45, 0.45, 0.0); 
+		float col[3];
+		BIF_GetThemeColor3fv(TH_BACK, col);
+		glClearColor(col[0], col[1], col[2], 0.0);
 		glClear(GL_COLOR_BUFFER_BIT);
 	}
 	
 	myloadmatrix(G.vd->viewmat);
+	persp(PERSP_STORE);  // store correct view for persp(PERSP_VIEW) calls
 
 	if(G.vd->view==0 || G.vd->persp!=0) {
 		drawfloor();
@@ -957,6 +1664,8 @@ void drawview3dspace(ScrArea *sa, void *spacedata)
 		base= base->next;
 	}
 
+	if(G.moving) constline_callback();
+
 	/* duplis, draw as last to make sure the displists are ok */
 	base= G.scene->base.first;
 	while(base) {
@@ -990,7 +1699,12 @@ void drawview3dspace(ScrArea *sa, void *spacedata)
 
 	if(G.scene->radio) RAD_drawall(G.vd->drawtype>=OB_SOLID);
 	
-	persp(0);
+	if(G.zbuf) {
+		G.zbuf= FALSE;
+		glDisable(GL_DEPTH_TEST);
+	}
+
+	persp(PERSP_WIN);  // set ortho
 	
 	if(G.vd->persp>1) drawviewborder();
 	drawcursor();
@@ -999,19 +1713,20 @@ void drawview3dspace(ScrArea *sa, void *spacedata)
 	ob= OBACT;
 	if(ob!=0 && (U.uiflag & DRAWVIEWINFO)) draw_selected_name(ob->id.name+2);
 	
-	persp(1);
+	draw_area_emboss(sa);
+	
+	/* it is important to end a view in a transform compatible with buttons */
+
+	bwin_scalematrix(sa->win, G.vd->blockscale, G.vd->blockscale, G.vd->blockscale);
+	view3d_blockhandlers(sa);
 
 	curarea->win_swap= WIN_BACK_OK;
 	
-	if(G.zbuf) {
-		G.zbuf= FALSE;
-		glDisable(GL_DEPTH_TEST);
-	}
-
 	if(G.f & (G_VERTEXPAINT|G_FACESELECT|G_TEXTUREPAINT|G_WEIGHTPAINT)) {
 		G.vd->flag |= V3D_NEEDBACKBUFDRAW;
 		addafterqueue(curarea->win, BACKBUFDRAW, 1);
 	}
+	
 }
 
 
@@ -1049,7 +1764,9 @@ void drawview3d_render(struct View3D *v3d)
 	if (v3d->drawtype==OB_TEXTURE && G.scene->world) {
 		glClearColor(G.scene->world->horr, G.scene->world->horg, G.scene->world->horb, 0.0); 
 	} else {
-		glClearColor(0.45, 0.45, 0.45, 0.0);
+		float col[3];
+		BIF_GetThemeColor3fv(TH_BACK, col);
+		glClearColor(col[0], col[1], col[2], 0.0); 
 	}
 	
 	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -1575,11 +2292,10 @@ void inner_play_anim_loop(int init, int mode)
 		audiostream_stop();
 		audiostream_start( CFRA );
 	}
-	else
-	{
+	else {
 		if (U.mixbufsize && (G.scene->audio.flag & AUDIO_SYNC)) CFRA = audiostream_pos();
 		else CFRA++;
-	};	
+	}
 }
 
 int play_anim(int mode)
@@ -1629,22 +2345,6 @@ int play_anim(int mode)
 					else viewmove(0);
 				}
 			}
-			else if(val) {
-				if(event==PAGEUPKEY) {
-					Group *group= G.main->group.first;
-					while(group) {
-						next_group_key(group);
-						group= group->id.next;
-					}
-				}
-				else if(event==PAGEDOWNKEY) {
-					Group *group= G.main->group.first;
-					while(group) {
-						prev_group_key(group);
-						group= group->id.next;
-					}
-				}
-			}
 		}
 		if(event==ESCKEY || event==SPACEKEY) break;
 				
@@ -1672,7 +2372,7 @@ int play_anim(int mode)
 	}
 	
 	/* speed button */
-	allqueue(REDRAWBUTSANIM, 0);
+	// allqueue(REDRAWBUTSOBJECT, 0);
 	/* groups could have changed ipo */
 	allspace(REMAKEIPO, 0);
 	allqueue(REDRAWIPO, 0);
