@@ -1,5 +1,5 @@
 /**
- * $Id: editaction.c,v 1.15 2003/07/20 23:04:08 hos Exp $
+ * $Id: editaction.c,v 1.19 2003/10/22 23:20:44 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -77,7 +77,7 @@
 #include "BIF_toolbox.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"
-#include "BIF_buttons.h"
+#include "BIF_butspace.h"
 #include "BIF_interface.h"
 #include "BIF_editview.h"
 #include "BIF_poseobject.h"
@@ -93,7 +93,6 @@
 
 #include "BDR_editobject.h"
 
-#include "interface.h"
 #include "mydevice.h"
 #include "blendef.h"
 #include "nla.h"
@@ -122,8 +121,6 @@ static void transform_meshchannel_keys(char mode, Key *key);
 static void select_poseelement_by_name (char *name, int select);
 static void hilight_channel (bAction *act, bActionChannel *chan, short hilight);
 static void set_action_key_time (bAction *act, bPoseChannel *chan, int adrcode, short makecurve, float time);
-static void clever_numbuts_meshaction(Key *key, short* mval);
-
 /* Implementation */
 
 short showsliders = 0;
@@ -1792,7 +1789,7 @@ static void delete_actionchannel_keys(void)
 }
 static void delete_actionchannels (void)
 {
-	bConstraintChannel *conchan, *nextconchan;
+	bConstraintChannel *conchan=NULL, *nextconchan;
 	bActionChannel *chan, *next;
 	bAction	*act;
 	int freechan;
@@ -2142,7 +2139,6 @@ void winqreadactionspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 {
 	unsigned short event= evt->event;
 	short val= evt->val;
-	char ascii= evt->ascii;
 	SpaceAction *saction;
 	bAction	*act;
 	int doredraw= 0;
@@ -2168,7 +2164,7 @@ void winqreadactionspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 
 		switch(event) {
 		case UI_BUT_EVENT:
-			do_blenderbuttons(val);
+			do_butspace(val); // abuse!
 			break;
 		case HOMEKEY:
 			do_action_buttons(B_ACTHOME);
@@ -2343,6 +2339,28 @@ void winqreadactionspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			}
 			break;
 
+		case LEFTMOUSE:
+			if (mval[0]>ACTWIDTH){
+				do {
+					getmouseco_areawin(mval);
+					
+					areamouseco_to_ipoco(G.v2d, mval, &dx, &dy);
+					
+					cfra= (int)dx;
+					if(cfra< 1) cfra= 1;
+					
+					if( cfra!=CFRA ) {
+						CFRA= cfra;
+						update_for_newframe();
+						force_draw_plus(SPACE_VIEW3D);
+						force_draw_plus(SPACE_IPO);
+						force_draw_plus(SPACE_BUTS);
+					}
+					
+				} while(get_mbut()&L_MOUSE);
+				break;
+			}
+			/* no break here, we allow leftmouse click too */
 		case RIGHTMOUSE:
 			/* Right clicking in the channel area selects the
 			 * channel or constraint channel
@@ -2402,28 +2420,6 @@ void winqreadactionspace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 			}
 			break;
 
-		case LEFTMOUSE:
-			if (mval[0]>ACTWIDTH){
-				do {
-					getmouseco_areawin(mval);
-					
-					areamouseco_to_ipoco(G.v2d, mval, &dx, &dy);
-					
-					cfra= (int)dx;
-					if(cfra< 1) cfra= 1;
-					
-					if( cfra!=CFRA ) {
-						CFRA= cfra;
-						update_for_newframe();
-						force_draw_plus(SPACE_VIEW3D);
-						force_draw_plus(SPACE_IPO);
-						force_draw_plus(SPACE_BUTS);
-					}
-					
-				} while(get_mbut()&L_MOUSE);
-			}
-			
-			break;
 		case MIDDLEMOUSE:
 		case WHEELUPMOUSE:
 		case WHEELDOWNMOUSE:
@@ -2487,7 +2483,7 @@ int get_nearest_key_num(Key *key, short *mval, float *x) {
 static void clever_keyblock_names(Key *key, short* mval){
     int        but=0, i, keynum;
     char       str[64];
-	float      x, min, max;
+	float      x;
 	KeyBlock   *kb;
 	/* get the keynum cooresponding to the y value
 	 * of the mouse pointer, return if this is
