@@ -1,5 +1,5 @@
 /**
- * $Id: editface.c,v 1.8 2003/06/11 15:43:20 ton Exp $
+ * $Id: editface.c,v 1.14 2003/10/15 19:23:52 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,6 +32,7 @@
 
 
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -86,9 +87,6 @@
 #include "blendef.h"
 #include "render.h"
 
-/*  #include "graphics.h" */
-// #include "blendef.h"
-#include "interface.h"
 #include "TPT_DependKludge.h"
 
 #ifdef NAN_TPT
@@ -105,6 +103,7 @@ void set_lasttface()
 	int a;
 	
 	lasttface= 0;
+	if(OBACT==NULL || OBACT->type!=OB_MESH) return;
 	
 	me= get_mesh(OBACT);
 	if(me==0 || me->tface==0) return;
@@ -517,7 +516,7 @@ void face_select()
 	
 	/* image window redraw */
 	allqueue(REDRAWIMAGE, 0);
-	allqueue(REDRAWBUTSGAME, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWVIEW3D, 0);
 }
 
@@ -937,7 +936,6 @@ void set_faceselect()	/* toggle */
 	}
 
 	allqueue(REDRAWVIEW3D, 0);
-	allqueue(REDRAWBUTSGAME, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
 	allqueue(REDRAWIMAGE, 0);
 	
@@ -1199,7 +1197,7 @@ int face_pick_uv(Object* object, Mesh* mesh, TFace* face, short x, short y, floa
 	 * If face is a quad, there are two triangles to check.
 	 */
 	result = triangle_ray_intersect(v2, v1, v3, org, dir, ab);
-	if ((num_verts == 3) || (num_verts == 4) && (result > 1)) {
+	if ( (num_verts == 3) || ((num_verts == 4) && (result > 1)) ) {
 		/* Face is a triangle or a quad with a hit on the first triangle */
 		face_get_uv(face, 1, 0, 2, ab[0], ab[1], u, v);
 		/* printf("triangle 1, texture (u,v)=(%f, %f)\n", *u, *v); */
@@ -1224,7 +1222,7 @@ void face_draw()
 	TFace *face, *face_old = 0;
 	short xy[2], xy_old[2];
 	//int a, index;
-	Image *img, *img_old = 0;
+	Image *img=NULL, *img_old = NULL;
 	IMG_BrushPtr brush;
 	IMG_CanvasPtr canvas = 0;
 	int rowBytes;
@@ -1362,4 +1360,69 @@ void face_draw()
 	allqueue(REDRAWIMAGE, 0);
 	allqueue(REDRAWHEADERS, 0);
 }
+
+ /* Selects all faces which have the same uv-texture as the active face 
+ * @author	Roel Spruit
+ * @return	Void
+ * Errors:	- Active object not in this layer
+ *		- No active face or active face has no UV-texture			
+ */
+void get_same_uv(void)
+{
+	Object *ob;
+	Mesh *me;
+	TFace *tface;	
+	short a, foundtex=0;
+	Image *ima;
+	char uvname[160];
+	
+	ob = OBACT;
+	if (!(ob->lay & G.vd->lay)) {
+		error("Active object not in this layer!");
+		return;
+	}
+	me = get_mesh(ob);
+	
+		
+	/* Search for the active face with a UV-Texture */
+	tface = me->tface;
+	a = me->totface;
+	while (a--) {		
+		if(tface->flag & TF_ACTIVE){			
+			ima=tface->tpage;
+			if(ima && ima->name){
+				strcpy(uvname,ima->name);			
+				a=0;
+				foundtex=1;
+			}
+		}
+		tface++;
+	}		
+	
+	if(!foundtex) {
+		error("No active face or active face has no UV-texture");
+		return;
+	}
+
+	/* select everything with the same texture */
+	tface = me->tface;
+	a = me->totface;
+	while (a--) {		
+		ima=tface->tpage;
+		if(ima && ima->name){
+			if(!strcmp(ima->name, uvname)){
+				tface->flag |= TF_SELECT;
+			}
+			else tface->flag &= ~TF_SELECT;
+		}
+		else tface->flag &= ~TF_SELECT;
+		tface++;
+	}
+	
+
+	
+	/* image window redraw */
+	allqueue(REDRAWIMAGE, 0);
+	allqueue(REDRAWVIEW3D, 0);
+}
 #endif /* NAN_TPT */
