@@ -1,5 +1,5 @@
 /**
- * $Id: editkey.c,v 1.6 2003/07/23 09:47:22 ton Exp $
+ * $Id: editkey.c,v 1.9 2003/11/19 22:00:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -46,7 +46,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "DNA_curve_types.h"
 #include "DNA_ipo_types.h"
@@ -87,7 +86,6 @@
 #include "blendef.h"
 #include "mydevice.h"
 #include "ipo.h"
-#include "interface.h"
 
 extern ListBase editNurb; /* in editcurve.c */
 
@@ -104,7 +102,7 @@ static IpoCurve *get_key_icu(Key *key, int keynum) {
 	 */
     IpoCurve *icu;
 	if (!(key->ipo)) {
-		key->ipo = get_ipo(key, ID_KE, 1);
+		key->ipo = get_ipo((ID *)key, ID_KE, 1);
 		return NULL;
 	}
 
@@ -356,7 +354,7 @@ void key_to_mesh(KeyBlock *kb, Mesh *me)
 
 
 
-void insert_meshkey(Mesh *me)
+void insert_meshkey(Mesh *me, short offline)
 {
 	Key *key;
 	KeyBlock *kb, *kkb;
@@ -365,7 +363,13 @@ void insert_meshkey(Mesh *me)
 
 	if(me->key==0) {
 		me->key= add_key( (ID *)me);
-		rel = pupmenu("Insert Vertex Keys %t|Relative keys %x1|Absolute keys %x2");
+
+		if (!offline) /* interactive */
+			rel = pupmenu("Insert Vertex Keys %t|"
+										"Relative keys %x1|Absolute keys %x2");
+		else /* we were called from a script */
+			rel = offline;
+
 		switch (rel) {
 		case 1:
 			me->key->type = KEY_RELATIVE;
