@@ -1,5 +1,5 @@
 /**
- * $Id: editscreen.c,v 1.18 2003/07/20 20:25:30 ton Exp $
+ * $Id: editscreen.c,v 1.52 2003/11/18 11:22:17 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -91,13 +91,13 @@
 #include "BIF_toolbox.h"
 #include "BIF_usiblender.h"
 #include "BIF_keyval.h"
+#include "BIF_resources.h"
 
 #include "BSE_edit.h"
 #include "BSE_filesel.h"
 #include "BSE_headerbuttons.h"
 #include "BSE_view.h"
 
-#include "interface.h"
 #include "mydevice.h"
 #include "blendef.h"
 #include "render.h"
@@ -116,11 +116,12 @@ static void testareas(void);
 static void area_autoplayscreen(void);
 static void wait_for_event(void);
 
+
 /* ********* Globals *********** */
 
 static Window *mainwin= NULL;
 static int prefsizx= 0, prefsizy= 0, prefstax= 0, prefstay= 0, start_maximized= 1;
-static short dodrawscreen= 0;
+static short dodrawscreen= 1;
 static ScrArea *areawinar[MAXWIN];
 static ScrArea *g_activearea= NULL;
 short winqueue_break= 0;
@@ -223,12 +224,19 @@ static ScrEdge *screen_find_active_scredge(bScreen *sc, short *mval)
 	
 	for (se= sc->edgebase.first; se; se= se->next) {
 		if (scredge_is_horizontal(se)) {
-			if (abs(mval[1]-se->v1->vec.y)<=EDGEWIDTH2 &&
-					abs(mval[0]-se->v1->vec.x)<=abs(se->v2->vec.x-se->v1->vec.x))
+			short min, max;
+			min= MIN2(se->v1->vec.x, se->v2->vec.x);
+			max= MAX2(se->v1->vec.x, se->v2->vec.x);
+			
+			if (abs(mval[1]-se->v1->vec.y)<=2 && mval[0] >= min && mval[0]<=max)
 				return se;
-		} else {
-			if (abs(mval[0]-se->v1->vec.x)<=EDGEWIDTH2 &&
-					abs(mval[1]-se->v1->vec.y)<=abs(se->v2->vec.y-se->v1->vec.y))
+		} 
+		else {
+			short min, max;
+			min= MIN2(se->v1->vec.y, se->v2->vec.y);
+			max= MAX2(se->v1->vec.y, se->v2->vec.y);
+
+			if (abs(mval[0]-se->v1->vec.x)<=2 && mval[1] >= min && mval[1]<=max)
 				return se;
 		}
 	}
@@ -245,12 +253,13 @@ void areawinset(short win)
 			return;
 		}
 		
+		BIF_SetTheme(curarea);
+		
 		switch(curarea->spacetype) {
 		case SPACE_VIEW3D:
 			G.vd= curarea->spacedata.first;
 			break;
 		case SPACE_IPO:
-			if(G.sipo != curarea->spacedata.first) allqueue(REDRAWBUTSANIM, 0);
 			G.sipo= curarea->spacedata.first;
 			G.v2d= &G.sipo->v2d;
 			break;
@@ -270,6 +279,7 @@ void areawinset(short win)
 		case SPACE_IMAGE:
 			G.sima= curarea->spacedata.first;
 			G.v2d= &G.sima->v2d;
+			break;
 		case SPACE_SOUND:
 			G.ssound= curarea->spacedata.first;
 			G.v2d= &G.ssound->v2d;
@@ -290,30 +300,30 @@ void areawinset(short win)
 	if(win) mywinset(win);
 }
 
-void headerbox(int selcol, int width)
+#define SCR_BACK 0.55
+#define SCR_ROUND 12
+
+void headerbox(ScrArea *area)
 {
-	if(selcol) glClearColor(.75, .75, .75, 0.0);
-	else glClearColor(.65, .65, .65, 0.0);
-	
+	float width= area->winx;
+
+	glClearColor(SCR_BACK, SCR_BACK, SCR_BACK, 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 
-	glColor3ub(0, 0, 0);
-	sdrawbox(0,  0,  width,  HEADERY);
+	if(area_is_active_area(area)) BIF_ThemeColor(TH_HEADER);
+	else BIF_ThemeColor(TH_HEADERDESEL);
 
-	glColor3ub(220, 220, 220);
-	sdrawline(0,  HEADERY-1,  width,  HEADERY-1);
+	/* weird values here... is because of window matrix that centres buttons */
+	if(area->headertype==HEADERTOP) {
+		uiSetRoundBox(3);
+		uiRoundBoxEmboss(-0.5+area->headbutofs, -10.0, width-1.5+area->headbutofs, HEADERY-2.0, SCR_ROUND);
+	}
+	else {
+		uiSetRoundBox(12);
+		uiRoundBoxEmboss(-0.5+area->headbutofs, -3.5, width-1.5+area->headbutofs, HEADERY+10, SCR_ROUND);
+	}
 	
-	glColor3ub(176, 176, 176);
-	sdrawline(0,  HEADERY-2,  width,  HEADERY-2);
-	
-	glColor3ub(128, 128, 128);
-	sdrawline(0,  2,  width,  2);
-	
-	glColor3ub(64, 64, 64);
-	sdrawline(0,  1,  width,  1);
-	
-	glColor3ub(0, 0, 0);
-	sdrawline(0,  0,  width,  0);
+	uiSetRoundBox(15);
 }
 
 int area_is_active_area(ScrArea *area)
@@ -323,36 +333,42 @@ int area_is_active_area(ScrArea *area)
 
 void scrarea_do_headdraw(ScrArea *area)
 {
-	areawinset(area->headwin);
+	if (area->headertype) {
+		areawinset(area->headwin);
+	
+		headerbox(area);
+		
+		/* we make scissor test slightly smaller not to destroy rounded headers */
+		glScissor(area->headrct.xmin+5, area->headrct.ymin, area->winx-10, HEADERY);
+		
+		switch(area->spacetype) {
+		case SPACE_FILE:	file_buttons();		break;
+		case SPACE_INFO:	info_buttons();		break;
+		case SPACE_VIEW3D:	view3d_buttons();	break;
+		case SPACE_IPO:		ipo_buttons();		break;
+		case SPACE_BUTS:	buts_buttons();		break;
+		case SPACE_SEQ:		seq_buttons();		break;
+		case SPACE_IMAGE:	image_buttons();	break;
+		case SPACE_IMASEL:	imasel_buttons();	break;
+		case SPACE_OOPS:	oops_buttons();		break;
+		case SPACE_TEXT:	text_buttons();		break;
+		case SPACE_SOUND:	sound_buttons();	break;
+		case SPACE_ACTION:	action_buttons();	break;
+		case SPACE_NLA:		nla_buttons();		break;
+		}
 
-	headerbox(area_is_active_area(area), area->winx+100);
-
-	switch(area->spacetype) {
-	case SPACE_FILE:	file_buttons();		break;
-	case SPACE_INFO:	info_buttons();		break;
-	case SPACE_VIEW3D:	view3d_buttons();	break;
-	case SPACE_IPO:		ipo_buttons();		break;
-	case SPACE_BUTS:	buts_buttons();		break;
-	case SPACE_SEQ:		seq_buttons();		break;
-	case SPACE_IMAGE:	image_buttons();	break;
-	case SPACE_IMASEL:	imasel_buttons();	break;
-	case SPACE_OOPS:	oops_buttons();		break;
-	case SPACE_TEXT:	text_buttons();		break;
-	case SPACE_SOUND:	sound_buttons();	break;
-	case SPACE_ACTION:	action_buttons();	break;
-	case SPACE_NLA:		nla_buttons();		break;
+		//glScissor(area->winrct.xmin, area->winrct.xmax, area->winx, area->winy);
+		area->head_swap= WIN_BACK_OK;
 	}
-
-	area->head_swap= WIN_BACK_OK;
 }
 void scrarea_do_headchange(ScrArea *area)
 {
 	float ofs= area->headbutofs;
 
 	if (area->headertype==HEADERDOWN) {
-		bwin_ortho2(area->headwin, 0.5+ofs, area->headrct.xmax-area->headrct.xmin-0.5+ofs, +0.6, area->headrct.ymax-area->headrct.ymin+0.6);
-	} else {
-		bwin_ortho2(area->headwin, -0.5+ofs, area->headrct.xmax-area->headrct.xmin-0.5+ofs, -0.5, area->headrct.ymax-area->headrct.ymin-0.5);
+		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -3.375, area->headrct.ymax-area->headrct.ymin-3.375);
+	} else if (area->headertype==HEADERTOP) {
+		bwin_ortho2(area->headwin, -0.375+ofs, area->headrct.xmax-area->headrct.xmin-0.375+ofs, -2.375, area->headrct.ymax-area->headrct.ymin-2.375);
 	}
 }
 
@@ -376,6 +392,7 @@ static void scrarea_change_headertype(ScrArea *sa, int newtype)
 	}
 
 	testareas();
+	mainqenter(DRAWEDGES, 1);
 	winqueue_break= 1;
 }
 
@@ -560,7 +577,6 @@ void splash(void *data, int datasize, char *string)
 	bbuf= IMB_ibImageFromMemory((int *)data, datasize, IB_rect);
 
 	if (bbuf) {
-
 		oldwin = mywinget();
 		mywinset(G.curscreen->mainwin);
 		
@@ -862,6 +878,8 @@ static void screen_dispatch_events(void) {
 	int events_remaining= 1;
 	ScrArea *sa;
 
+	window_make_active(mainwin); // added it here instead of screenmain (ton)
+
 	while (events_remaining) {
 		events_remaining= 0;
 				
@@ -896,10 +914,15 @@ static ScrArea *screen_find_area_for_pt(bScreen *sc, short *mval)
 {
 	ScrArea *sa;
 	
-	for (sa= sc->areabase.first; sa; sa= sa->next)
-		if (BLI_in_rcti(&sa->totrct, mval[0], mval[1]))
-			return sa;
+	/* hotspot area of 1 pixel extra */
 	
+	for (sa= sc->areabase.first; sa; sa= sa->next) {
+		if( sa->totrct.xmin + 1 < mval[0] )
+			if( sa->totrct.ymin + 1 < mval[1] )
+				if( sa->totrct.xmax - 1 > mval[0] )
+					if( sa->totrct.ymax - 1 > mval[1] )
+						return sa;
+	}
 	return NULL;
 }
 
@@ -908,6 +931,8 @@ void screenmain(void)
 	int has_input= 1;
 	int firsttime = 1;
 	
+	window_make_active(mainwin);
+	
 	while (1) {
 		unsigned short event;
 		short val, towin;
@@ -921,9 +946,10 @@ void screenmain(void)
 			event= screen_qread(&val, &ascii);
 		}
 		
-		window_make_active(mainwin);
+		// window_make_active(mainwin); // (only for inputchange, removed, (ton))
 
 		if (event==INPUTCHANGE) {
+			window_make_active(mainwin);
 			has_input= val;
 		}
 		
@@ -934,7 +960,7 @@ void screenmain(void)
 			 * If the main window is not active, deactivate the internal 
 			 * window.
 			 */
-		if (has_input) {
+		if (has_input || g_activearea==NULL || G.curscreen->winakt) {
 			ScrArea *newactarea;
 			int newactwin;
 			short mval[2];
@@ -958,13 +984,18 @@ void screenmain(void)
 				set_cursor(newactarea->cursor);
 				g_activearea= newactarea;
 			}
-
+			/* when you move mouse from header to window, buttons can remain hilited otherwise */
+			if(newactwin != G.curscreen->winakt) {
+				if (g_activearea) scrarea_queue_headredraw(g_activearea);
+			}
 			G.curscreen->winakt= newactwin;
+			
 			if (G.curscreen->winakt) {
 				areawinset(G.curscreen->winakt);
 				set_cursor(choose_cursor(g_activearea));
 			}
-		} else {
+		} 
+		else {
 			if (g_activearea) {
 				scrarea_queue_headredraw(g_activearea);
 			}
@@ -1055,7 +1086,7 @@ void screenmain(void)
 		else if (event==SPACEKEY) {
 			if((G.obedit && G.obedit->type==OB_FONT && g_activearea->spacetype==SPACE_VIEW3D)||g_activearea->spacetype==SPACE_TEXT);
 			else {
-				if(val) toolbox();
+				if(val) toolbox_n();
 				towin= 0;
 			}
 		}
@@ -1074,7 +1105,7 @@ void screenmain(void)
 				towin= 0;
 			}
 		}
-		
+
 		if (towin && event) {
 			if (blenderqread(event, val))
 				addqueue_ext(G.curscreen->winakt, event, val, ascii);
@@ -1090,6 +1121,7 @@ void screenmain(void)
 
 		/* Bizar hack. The event queue has mutated... */
 		if ( (firsttime) && (event == 0) ) {
+			
 			if (G.fileflags & G_FILE_AUTOPLAY) {
 				// SET AUTOPLAY in G.flags for
 				// other fileloads
@@ -1105,11 +1137,11 @@ void screenmain(void)
 				extern char datatoc_splash_jpg[];
 				extern int datatoc_splash_jpg_size;
 
-				if (! ((G.main->versionfile >= G.version)
-				       || G.save_over)) {
+				//if (! ((G.main->versionfile >= G.version)
+				//       || G.save_over)) {
 					splash((void *)datatoc_splash_jpg,
 					       datatoc_splash_jpg_size, NULL);
-				}
+				//}
 			}
 			firsttime = 0;
 		}
@@ -1150,7 +1182,13 @@ void setprefsize(int stax, int stay, int sizx, int sizy)
 	if(sizy<256) sizy= 256;
 
 	if(stax+sizx>scrwidth) sizx= scrwidth-stax;
+
 	if(stay+sizy>scrheight) sizy= scrheight-stay;
+
+#ifdef __APPLE__
+	if(stay+sizy>scrheight-22) sizy= scrheight-22-stay;	// 22 = topbar
+#endif
+
 	if(sizx<320 || sizy<256) {
 		printf("ERROR: illegal prefsize\n");
 		return;
@@ -1349,28 +1387,31 @@ static void removenotused_scredges(void)
 void calc_arearcts(ScrArea *sa)
 {
 
-	if(sa->v1->vec.x>0) sa->totrct.xmin= sa->v1->vec.x+EDGEWIDTH2+1;
+	if(sa->v1->vec.x>0) sa->totrct.xmin= sa->v1->vec.x+1;
 	else sa->totrct.xmin= sa->v1->vec.x;
-	if(sa->v4->vec.x<G.curscreen->sizex-1) sa->totrct.xmax= sa->v4->vec.x-EDGEWIDTH2-1;
+	if(sa->v4->vec.x<G.curscreen->sizex-1) sa->totrct.xmax= sa->v4->vec.x-1;
 	else sa->totrct.xmax= sa->v4->vec.x;
 	
-	if(sa->v1->vec.y>0) sa->totrct.ymin= sa->v1->vec.y+EDGEWIDTH2+1;
+	if(sa->v1->vec.y>0) sa->totrct.ymin= sa->v1->vec.y+1;
 	else sa->totrct.ymin= sa->v1->vec.y;
-	if(sa->v2->vec.y<G.curscreen->sizey-1) sa->totrct.ymax= sa->v2->vec.y-EDGEWIDTH2-1;
+	if(sa->v2->vec.y<G.curscreen->sizey-1) sa->totrct.ymax= sa->v2->vec.y-1;
 	else sa->totrct.ymax= sa->v2->vec.y;
 	
 	sa->winrct= sa->totrct;
+	sa->headrct= sa->totrct;
 	if(sa->headertype) {
-		sa->headrct= sa->totrct;
 		if(sa->headertype==HEADERDOWN) {
-			sa->headrct.ymax= sa->headrct.ymin+HEADERY-1;
+			sa->headrct.ymax= sa->headrct.ymin+HEADERY;
 			sa->winrct.ymin= sa->headrct.ymax+1;
 		}
 		else if(sa->headertype==HEADERTOP) {
-			sa->headrct.ymin= sa->headrct.ymax-HEADERY+1;
+			sa->headrct.ymin= sa->headrct.ymax-HEADERY;
 			sa->winrct.ymax= sa->headrct.ymin-1;
 		}
 	}
+	else {
+		sa->headrct.ymax= sa->headrct.ymin;
+	}
 	if(sa->winrct.ymin>sa->winrct.ymax) sa->winrct.ymin= sa->winrct.ymax;
 	
 	/* for speedup */
@@ -1407,7 +1448,7 @@ static void closeheadwin(ScrArea *sa)
 static void closeareawin(ScrArea *sa)
 {
 	uiFreeBlocksWin(&sa->uiblocks, sa->win);
-
+	
 	if(sa->win) mywinclose(sa->win);
 	sa->win= 0;
 }
@@ -1420,20 +1461,43 @@ static void del_area(ScrArea *sa)
 	freespacelist(&sa->spacedata);
 	
 	uiFreeBlocks(&sa->uiblocks);
+	uiFreePanels(&sa->panels);
 	
 	if(sa==curarea) curarea= 0;
 	if(sa==g_activearea) g_activearea= 0;
 }
 
+/* sa2 to sa1 */
 static void copy_areadata(ScrArea *sa1, ScrArea *sa2)
 {
+	Panel *pa1, *pa2, *patab;
+	
 	sa1->headertype= sa2->headertype;
 	sa1->spacetype= sa2->spacetype;
 	Mat4CpyMat4(sa1->winmat, sa2->winmat);
 	
 	freespacelist(&sa1->spacedata);
-	
 	duplicatespacelist(sa1, &sa1->spacedata, &sa2->spacedata);
+
+	BLI_freelistN(&sa1->panels);
+	duplicatelist(&sa1->panels, &sa2->panels);
+	
+	/* copy pointers */
+	pa1= sa1->panels.first;
+	while(pa1) {
+		
+		patab= sa1->panels.first;
+		pa2= sa2->panels.first;
+		while(patab) {
+			if( pa1->paneltab == pa2) {
+				pa1->paneltab = patab;
+				break;
+			}
+			patab= patab->next;
+			pa2= pa2->next;
+		}
+		pa1= pa1->next;
+	}
 }
 
 static ScrArea *screen_addarea(bScreen *sc, ScrVert *v1, ScrVert *v2, ScrVert *v3, ScrVert *v4, short headertype, short spacetype)
@@ -1605,6 +1669,8 @@ void BIF_wait_for_statechange(void)
 		}
 		statechanged= 0;
 	}
+	else PIL_sleep_ms(3);	/* statechanged can be set '1' while holding mousebutton, causing locks */
+
 }
 void getmouse(short *mval)
 {
@@ -1654,6 +1720,15 @@ void add_to_mainqueue(Window *win, void *user_data, short evt, short val, char a
 	mainqenter_ext(evt, val, ascii);
 }
 
+/* ScrVert ordering in a ScrArea:
+
+2---------3
+|         |
+|         |
+1---------4
+  
+*/
+
 static bScreen *addscreen(char *name)		/* use setprefsize() if you want something else than a full windpw */
 {
 	/* this function sets variabele G.curscreen,
@@ -1705,14 +1780,14 @@ static bScreen *addscreen(char *name)		/* use setprefsize() if you want somethin
 	
 		glMatrixMode(GL_PROJECTION);
 		glLoadIdentity();
-		myortho2(-0.5, sc->sizex-0.5, -0.5, sc->sizey-0.5);
+		myortho2(-0.375, sc->sizex-0.375, -0.375, sc->sizey-0.375);
 		glMatrixMode(GL_MODELVIEW);
 		glLoadIdentity();
 
-		glDrawBuffer(GL_FRONT);
-		glClearColor(.45, .45, .45, 0.0);
+		/* for visual speed, but still needed? */
+		glClearColor(.55, .55, .55, 0.0);
 		glClear(GL_COLOR_BUFFER_BIT);
-		glDrawBuffer(GL_BACK);
+		window_swap_buffers(mainwin);
 		
 		warp_pointer(sc->sizex/2,  sc->sizey/2);
 		
@@ -1794,21 +1869,25 @@ void setscreen(bScreen *sc)
 	test_scale_screen(sc);
 	testareas();
 	
-	for (sa= sc->areabase.first; sa; sa= sa->next) {
+	for(sa= sc->areabase.first; sa; sa= sa->next) {
 		SpaceLink *sl;
 
-		for (sl= sa->spacedata.first; sl; sl= sl->next) {
+		for(sl= sa->spacedata.first; sl; sl= sl->next) {
 			sl->area= sa;
 
-			if (sl->spacetype==SPACE_OOPS) {
-				SpaceOops *soops= (SpaceOops*) sl;
+			if(sl->spacetype==SPACE_OOPS) {
+				SpaceOops *soops= (SpaceOops *) sl;
 
 				/* patch for old files */
-				if (soops->v2d.cur.xmin==soops->v2d.cur.xmax) {
+				if(soops->v2d.cur.xmin==soops->v2d.cur.xmax) {
 					extern void init_v2d_oops(View2D*);
 					init_v2d_oops(&soops->v2d);
 				}
 			}
+			else if(sl->spacetype==SPACE_BUTS) {
+				SpaceButs *sbuts= (SpaceButs *)sl;
+				sbuts->re_align= 1;		// force an align call, maybe new panels were added, also for after file reading
+			}
 		}
 		
 		sa->cursor= CURSOR_STD;
@@ -1893,7 +1972,7 @@ void area_fullscreen(void)	/* with curarea */
 		curarea= old;
 		G.curscreen= oldscreen;	/* needed because of setscreen */
 		
-		/* vopy area */
+		/* copy area */
 		copy_areadata(newa, curarea);
 		
 		curarea->full= oldscreen;
@@ -1953,8 +2032,7 @@ static void copy_screen(bScreen *to, bScreen *from)
 {
 	ScrVert *s1, *s2;
 	ScrEdge *se;
-	ScrArea *sa;
-	ListBase lbase;
+	ScrArea *sa, *saf;
 
 	/* free 'to' */
 	free_screen(to);
@@ -1980,6 +2058,7 @@ static void copy_screen(bScreen *to, bScreen *from)
 	}
 
 	sa= to->areabase.first;
+	saf= from->areabase.first;
 	while(sa) {
 		sa->v1= sa->v1->newv;
 		sa->v2= sa->v2->newv;
@@ -1988,12 +2067,13 @@ static void copy_screen(bScreen *to, bScreen *from)
 		sa->win= 0;
 		sa->headwin= 0;
 		
+		sa->spacedata.first= sa->spacedata.last= NULL;
 		sa->uiblocks.first= sa->uiblocks.last= NULL;
-		
-		duplicatespacelist(sa, &lbase, &sa->spacedata);
-		sa->spacedata= lbase;
+		sa->panels.first= sa->panels.last= NULL;
+		copy_areadata(sa, saf);
 		
 		sa= sa->next;
+		saf= saf->next;
 	}
 	
 	/* put at zero (needed?) */
@@ -2131,11 +2211,11 @@ static short testsplitpoint(ScrArea *sa, char dir, float fac)
 	if(dir=='h') {
 		y= sa->v1->vec.y+ fac*(sa->v2->vec.y- sa->v1->vec.y);
 		
-		if(sa->v2->vec.y==G.curscreen->sizey-1 && sa->v2->vec.y- y < HEADERY+EDGEWIDTH2) 
-			y= sa->v2->vec.y- HEADERY-EDGEWIDTH2;
+		if(sa->v2->vec.y==G.curscreen->sizey-1 && sa->v2->vec.y- y < HEADERY) 
+			y= sa->v2->vec.y- HEADERY;
 
-		else if(sa->v1->vec.y==0 && y- sa->v1->vec.y < HEADERY+EDGEWIDTH2)
-			y= sa->v1->vec.y+ HEADERY+EDGEWIDTH2;
+		else if(sa->v1->vec.y==0 && y- sa->v1->vec.y < HEADERY)
+			y= sa->v1->vec.y+ HEADERY;
 
 		else if(y- sa->v1->vec.y < AREAMINY) y= sa->v1->vec.y+ AREAMINY;
 		else if(sa->v2->vec.y- y < AREAMINY) y= sa->v2->vec.y- AREAMINY;
@@ -2156,7 +2236,7 @@ static short testsplitpoint(ScrArea *sa, char dir, float fac)
 static void splitarea(ScrArea *sa, char dir, float fac)
 {
 	bScreen *sc;
-	ScrArea *newa;
+	ScrArea *newa=NULL;
 	ScrVert *sv1, *sv2;
 	short split;
 	
@@ -2211,6 +2291,11 @@ static void splitarea(ScrArea *sa, char dir, float fac)
 		sa->v2= sv2;
 	}
 	
+	if(sa->spacetype==SPACE_BUTS) {
+		addqueue(sa->win, UI_BUT_EVENT, B_BUTSHOME);
+		addqueue(newa->win, UI_BUT_EVENT, B_BUTSHOME);
+	}
+	
 	/* remove double vertices en edges */
 	removedouble_scrverts();
 	removedouble_scredges();
@@ -2591,6 +2676,7 @@ static void moveareas(ScrEdge *edge)
 	doit= delta= 0;
 	getmouseco_sc(mvalo);
 	draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
+
 	while (!doit) {
 		short val;
 		unsigned short event= extern_qread(&val);
@@ -2606,9 +2692,12 @@ static void moveareas(ScrEdge *edge)
 			delta= CLAMPIS(delta, -smaller, bigger);
 			
 			draw_front_xor_dirdist_line(dir, edge_position+delta, edge_start, edge_end);
-		} else if (event==LEFTMOUSE) {
+
+		} 
+		else if (event==LEFTMOUSE) {
 			doit= 1;
-		} else if (val) {
+		} 
+		else if (val) {
 			if (ELEM(event, ESCKEY, RIGHTMOUSE))
 				doit= -1;
 			else if (ELEM(event, SPACEKEY, RETKEY))
@@ -2748,100 +2837,117 @@ int select_area(int spacetype)
 /* ************  END JOIN/SPLIT/MOVE ************** */
 /* **************** DRAW SCREENEDGES ***************** */
 
-#define EDGE_EXTEND 3
 
-void drawedge(short x1, short y1, short x2, short y2)
+void draw_area_emboss(ScrArea *sa)
 {
-	static unsigned int edcol[EDGEWIDTH]= {0x0, 0x505050, 0x909090, 0xF0F0F0, 0x0};
-	int a;
 	
-	if(x1==x2) {		/* vertical */
-		if (y2<y1) {
-			SWAP(short, y1, y2);
-		}
-		if (y1==0) y1-= EDGE_EXTEND;
-		if (y2==G.curscreen->sizey) y2+= EDGE_EXTEND;
-		
-		x1+= EDGEWIDTH2;
-		x2+= EDGEWIDTH2;
+	/* set transp line */
+	glEnable( GL_BLEND );
+	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
 
-		glBegin(GL_LINES);		
-		for(a=0; a<EDGEWIDTH; a++) {
-			int rounding= abs(a-EDGEWIDTH2);
-			
-			cpack(edcol[a]);
-			glVertex2i(x1-a, y1+rounding);
-			glVertex2i(x2-a, y2-rounding);
-		}
-		glEnd();
+	/* right  */
+	glColor4ub(0,0,0, 80);
+	sdrawline(sa->winx-1, 0, sa->winx-1, sa->winy-1);
+	
+	/* bottom  */
+	if(sa->headertype!=HEADERDOWN) {
+		glColor4ub(0,0,0, 128);
+		sdrawline(0, 0, sa->winx-1, 0);
 	}
-	else {				/* horizontal */
-		if (x2<x1) {
-			SWAP(short, x1, x2);
-		}
-		if (x1==0) x1-= EDGE_EXTEND;
-		if (x2==G.curscreen->sizex) x2+= EDGE_EXTEND;
+	
+	/* top  */
+	if(sa->headertype!=HEADERTOP) {
+		glColor4ub(255,255,255, 96);
+		sdrawline(0, sa->winy-1, sa->winx-1, sa->winy-1);
+	}
+	/* left  */
+	glColor4ub(255,255,255, 80);
+	sdrawline(0, 0, 0, sa->winy);
 
-		y1-= EDGEWIDTH2;
-		y2-= EDGEWIDTH2;
-		
-		glBegin(GL_LINES);
-		for(a=0; a<EDGEWIDTH; a++) {
-			int rounding= abs(a-EDGEWIDTH2);
-			
-			cpack(edcol[a]);
-			glVertex2i(x1+rounding, y1+a);
-			glVertex2i(x2-rounding, y2+a);
-		}
-		glEnd();
+	glDisable( GL_BLEND );
+	
+	
+	/* for test */
+	if(FALSE && sa->spacetype==SPACE_VIEW3D) {
+	cpack(0xA0A0A0);
+	uiSetRoundBox(31);
+	uiRoundBoxEmboss(5.0, 5.0, 25.0, 100.0, 8.0);
+
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA,  GL_ONE_MINUS_SRC_ALPHA); 
+	
+	glRasterPos2f(8.0, 10.0);
+	BIF_draw_icon(ICON_MATERIAL_HLT);
+	glRasterPos2f(8.0, 30.0);
+	BIF_draw_icon(ICON_IPO_HLT);
+	glRasterPos2f(8.0, 50.0);
+	BIF_draw_icon(ICON_HOME);
+	glRasterPos2f(8.0, 70.0);
+	BIF_draw_icon(ICON_BORDERMOVE);
+	
+	glBlendFunc(GL_ONE,  GL_ZERO); 
+	glDisable(GL_BLEND);
 	}
 }
 
-static void drawscredge(ScrEdge *se)
-{
-	bScreen *sc;
-	vec2s *v1, *v2;
-	
-	sc= G.curscreen;
-	
-	v1= &(se->v1->vec);
-	v2= &(se->v2->vec);
-	
-	/* do not draw borders screen */
-	/* bcause of different framebuffer resoltions (PC/SGI etc files) 
-	 * a bit rounding here? should be looked at further...
-	 */
-	se->border= 1;
-	if(v1->x<=1 && v2->x<=1) return;
-	if(v1->x>=sc->sizex-2 && v2->x>=sc->sizex-2) return;
-	if(v1->y<=1 && v2->y<=1) return;
-	if(v1->y>=sc->sizey-2 && v2->y>=sc->sizey-2) return;
-	se->border= 0;
 
-	drawedge(v1->x, v1->y, v2->x, v2->y);
+void drawscredge_area(ScrArea *sa)
+{
+	short x1= sa->v1->vec.x;
+	short y1= sa->v1->vec.y;
+	short x2= sa->v3->vec.x;
+	short y2= sa->v3->vec.y;
+
+	/* this to fill the (undrawn) edge area with back color first */
+	glColor3f(SCR_BACK,SCR_BACK,SCR_BACK);
+	sdrawline(x2, y1, x2, y2);
+	sdrawline(x1, y1, x2, y1);
+	
+	cpack(0x0);
+	
+	/* right border area */
+	if(sa->headertype==HEADERTOP) sdrawline(x2, y1, x2, y2-SCR_ROUND);
+	else if(sa->headertype==HEADERDOWN) sdrawline(x2, y1+SCR_ROUND, x2, y2);
+	else sdrawline(x2, y1, x2, y2);
+	
+	/* left border area */
+	if(sa->headertype==HEADERTOP) sdrawline(x1, y1, x1, y2-SCR_ROUND);
+	else if(sa->headertype==HEADERDOWN) sdrawline(x1, y1+SCR_ROUND, x1, y2);
+	else sdrawline(x1, y1, x1, y2);
+	
+	/* top border area */
+	if(sa->headertype==HEADERTOP) sdrawline(x1+SCR_ROUND, y2, x2-SCR_ROUND, y2);
+	else sdrawline(x1, y2, x2, y2);
+	
+	/* bottom border area */
+	if(sa->headertype==HEADERDOWN) sdrawline(x1+SCR_ROUND, y1, x2-SCR_ROUND, y1);
+	else sdrawline(x1, y1, x2, y1);
+	
 }
 
 void drawscreen(void)
 {
-	ScrEdge *se;
-	
-	mywinset(G.curscreen->mainwin);
-	myortho2(-0.5, (float)G.curscreen->sizex-0.5, -0.6, (float)G.curscreen->sizey-0.6);
+	ScrArea *sa;
 
-	/* two times, because there is no 'win_swap' for this  available */
+	mywinset(G.curscreen->mainwin);
+	myortho2(-0.375, (float)G.curscreen->sizex-0.375, -0.375, (float)G.curscreen->sizey-0.375);
+
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
+	}
+
+	/* this double draw patch seems to be needed for certain sgi's (octane, indigo2) */
 	glDrawBuffer(GL_FRONT);
-	se= G.curscreen->edgebase.first;
-	while(se) {
-		drawscredge(se);
-		se= se->next;
+
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		drawscredge_area(sa);
+		sa= sa->next;
 	}
 
 	glDrawBuffer(GL_BACK);
-	se= G.curscreen->edgebase.first;
-	while(se) {
-		drawscredge(se);
-		se= se->next;
-	}
 }
 
 /* ********************************* */
@@ -2966,7 +3072,7 @@ void set_timecursor(int nr)
 	}
 
 	curcursor= CURSOR_NONE;
-	window_set_custom_cursor(mainwin, mask, bitmap);
+	window_set_custom_cursor(mainwin, mask, bitmap, 7, 7);
 	BIF_renderwin_set_custom_cursor(mask, bitmap);
 }
 
