@@ -1,5 +1,5 @@
 /**
- * $Id: editsima.c,v 1.3 2002/11/25 12:02:05 mein Exp $
+ * $Id: editsima.c,v 1.7 2003/11/19 22:00:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -47,7 +47,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "IMB_imbuf_types.h"
 
@@ -80,7 +79,6 @@
 
 #include "blendef.h"
 #include "mydevice.h"
-#include "interface.h"
 
 static int is_uv_tface_editing_allowed(void)
 {
@@ -219,7 +217,7 @@ static void be_square_tface_uv(Mesh *me)
 	MFace *mface;
 	int a;
 	
-	/* als 1 punt select: doit (met het select punt) */
+	/* if 1 vertex selected: doit (with the selected vertex) */
 	for(a=me->totface, mface= me->mface, tface= me->tface; a>0; a--, tface++, mface++) {
 		if(mface->v4) {
 			if(tface->flag & TF_SELECT) {
@@ -323,7 +321,7 @@ void transform_tface_uv(int mode)
 	else {
 		xim= yim= 256;
 	}
-	/* welke vertices doen mee */
+	/* which vertices are involved */
 	
 	for(a=me->totface, tface= me->tface, mface= me->mface; a>0; a--, tface++, mface++) {
 		if((tface->flag & TF_SELECT) && mface->v3) {
@@ -719,6 +717,7 @@ void mouse_select_sima(void)
 	
 		glDrawBuffer(GL_FRONT);
 		draw_tfaces();
+		glFlush(); // at OSX, a flush pops up the "frontbuffer" (it does a swap, doh!)
 		glDrawBuffer(GL_BACK);
 		
 		std_rmouse_transform(transform_tface_uv);
