@@ -1,5 +1,5 @@
 /**
- * $Id: editview.c,v 1.3 2002/11/25 12:02:05 mein Exp $
+ * $Id: editview.c,v 1.21 2003/11/22 18:38:53 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -34,6 +34,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <math.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -43,20 +44,23 @@
 #include "BLI_winstuff.h"
 #endif
 
-#include "BLI_blenlib.h"
-#include "BLI_arithb.h"
-#include "BLI_editVert.h"
-
 #include "IMB_imbuf.h"
+#include "PIL_time.h"
 
 #include "DNA_armature_types.h"
 #include "DNA_meta_types.h"
+#include "DNA_mesh_types.h"
 #include "DNA_curve_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_view3d_types.h"
+#include "DNA_userdef_types.h"
+
+#include "BLI_blenlib.h"
+#include "BLI_arithb.h"
+#include "BLI_editVert.h"
 
 #include "BKE_utildefines.h"
 #include "BKE_global.h"
@@ -67,7 +71,7 @@
 #include "BIF_mywindow.h"
 #include "BIF_space.h"
 #include "BIF_screen.h"
-#include "BIF_buttons.h"
+#include "BIF_butspace.h"
 #include "BIF_editgroup.h"
 #include "BIF_editmesh.h"
 #include "BIF_editoops.h"
@@ -75,6 +79,7 @@
 #include "BIF_editview.h"
 #include "BIF_glutil.h"
 #include "BIF_editarmature.h"
+#include "BIF_toolbox.h"
 
 #include "BDR_editobject.h"	/* For headerprint */
 #include "BDR_vpaint.h"
@@ -88,8 +93,6 @@
 #include "BSE_drawview.h"
 #include "BSE_editaction.h"
 
-#include "blendertimer.h"
-
 #include "blendef.h"
 #include "mydevice.h"
 
@@ -126,20 +129,20 @@ static char interpret_move(short mcord[][2], int count)
 	
 	if (count <= 10) return ('g');
 
-	/* van short naar float (tekenen is met shorts) */
+	/* from short to float (drawing is with shorts) */
 	for(j=0; j<count; j++) {
 		mouse[j][0]= mcord[j][0];
 		mouse[j][1]= mcord[j][1];
 	}
 	
-	/* nieuwe opzet:
+	/* new method:
 	 * 
-	 * vanuit eindpunten middelpunt met maximale afstand berekenen
-	 * aan de hand van de hoek wordt s / g / r bepaald
+	 * starting from end points, calculate centre with maximum distance
+	 * dependant at the angle s / g / r is defined
 	 */
 	
 
-	/* filteren */
+	/* filter */
 	
 	for( j = 3 ; j > 0; j--){
 		x1 = mouse[1][0];
@@ -154,7 +157,7 @@ static char interpret_move(short mcord[][2], int count)
 		}
 	}
 
-	/* maak directions overzicht */
+	/* make overview of directions */
 	for (i = 0; i <= count - 2; i++){
 		x1 = mouse[i][0] - mouse[i + 1][0];
 		y1 = mouse[i][1] - mouse[i + 1][1];
@@ -174,7 +177,7 @@ static char interpret_move(short mcord[][2], int count)
 		}
 	}
 	
-	/* alle kruisjes naar rechts halen */
+	/* move all crosses to the right */
 	for (i = 7; i>=0 ; i--){
 		if (dir & 128) dir = (dir << 1) + 1;
 		else break;
@@ -185,9 +188,9 @@ static char interpret_move(short mcord[][2], int count)
 		else break;
 	}
 	
-	/* theorie zegt: 1 richting: rechte lijn
-     * meer aaneengesloten richtingen: cirkel
-     * onderbroken en minstens 1 bit gezet in hoogste 4 bits: size
+	/* in theory: 1 direction: straight line
+     * multiple sequential directions: circle
+     * non-sequential, and 1 bit set in upper 4 bits: size
      */
 	switch(dir){
 	case 1:
@@ -220,7 +223,7 @@ static char interpret_move(short mcord[][2], int count)
 		return ('r');
 		break;
 	default:
-		/* bij size moeten minstens een van de hogere bits gezet zijn */
+		/* for size at least one of the higher bits has to be set */
 		if (dir < 16) return ('r');
 		else return ('s');
 	}
@@ -232,11 +235,11 @@ int gesture(void)
 {
 	short mcords[MOVES][2];
 	int i= 1, end= 0, a;
-	unsigned short event;
-	short mval[2], val;
+	unsigned short event=0;
+	short mval[2], val, timer=0;
 	
 	glDrawBuffer(GL_FRONT);
-	persp(0);	/* heeft ortho op pixelnivo */
+	persp(0);	/*  ortho at pixel level */
 	
 	getmouseco_areawin(mval);
 	
@@ -245,8 +248,17 @@ int gesture(void)
 	
 	while(get_mbut()&L_MOUSE) {
 		
-		event= extern_qread(&val);
-	
+		if(qtest()) event= extern_qread(&val);
+		else if(i==1) {
+			/* not drawing yet... check for toolbox */
+			PIL_sleep_ms(10);
+			timer++;
+			if(timer>=10*U.tb_leftmouse) {
+				toolbox_n();
+				return 1;
+			}
+		}
+		
 		switch (event) {
 		case MOUSEY:
 			getmouseco_areawin(mval);
@@ -290,7 +302,7 @@ int gesture(void)
 		
 		return 1;
 	}
-	return 0;
+	else return 0;
 }
 
 void mouse_cursor(void)
@@ -380,7 +392,49 @@ void deselectall(void)	/* is toggle */
 
 }
 
-static void deselectall_ex(Base *b)   /* ALLES deselect behalve b */
+/* selects all objects of a particular type, on currently visible layers */
+void selectall_type(short obtype) 
+{
+	Base *base;
+	
+	base= FIRSTBASE;
+	while(base) {
+		if((base->lay & G.vd->lay) && (base->object->type == obtype)) {
+			base->flag |= SELECT;
+			base->object->flag= base->flag;
+		}
+		base= base->next;
+	}
+
+	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWDATASELECT, 0);
+	allqueue(REDRAWNLA, 0);
+	
+	countall();
+
+}
+/* selects all objects on a particular layer */
+void selectall_layer(int layernum) 
+{
+	Base *base;
+	
+	base= FIRSTBASE;
+	while(base) {
+		if (base->lay == (1<< (layernum -1))) {
+			base->flag |= SELECT;
+			base->object->flag= base->flag;
+		}
+		base= base->next;
+	}
+
+	allqueue(REDRAWVIEW3D, 0);
+	allqueue(REDRAWDATASELECT, 0);
+	allqueue(REDRAWNLA, 0);
+	
+	countall();
+
+}
+static void deselectall_except(Base *b)   /* deselect all except b */
 {
 	Base *base;
 
@@ -388,17 +442,16 @@ static void deselectall_ex(Base *b)   /* ALLES deselect behalve b */
 	while(base) {
 		if (base->flag & SELECT) {
 			if(b!=base) {
-			
 				base->flag &= ~SELECT;
 				base->object->flag= base->flag;
-				draw_object_ext(base);	/* deze test op layer */
 			}
 		}
 		base= base->next;
 	}
-	countall();
 }
 
+#if 0
+/* smart function to sample a rect spiralling outside, nice for backbuf selection */
 static unsigned int samplerect(unsigned int *buf, int size, unsigned int dontdo)
 {
 	Base *base;
@@ -432,7 +485,7 @@ static unsigned int samplerect(unsigned int *buf, int size, unsigned int dontdo)
 			for(b=0;b<tel;b++) {
 
 				if(*buf && *buf<=maxob && *buf!=dontdo) return *buf;
-				if( *buf==dontdo ) retval= dontdo;	/* als alleen kleur dontdo aanwezig is, wel dontdo teruggeven */
+				if( *buf==dontdo ) retval= dontdo;	/* if only color dontdo is available, still return dontdo */
 				
 				buf+= (dirvec[rc][0]+dirvec[rc][1]);
 
@@ -444,6 +497,7 @@ static unsigned int samplerect(unsigned int *buf, int size, unsigned int dontdo)
 	}
 	return retval;
 }
+#endif
 
 #define SELECTSIZE	51
 
@@ -452,12 +506,12 @@ void set_active_base(Base *base)
 	
 	BASACT= base;
 	
-	/* signalen naar buttons */
+	/* signals to buttons */
 	redraw_test_buttons(base);
 
 	set_active_group();
 	
-	/* signaal naar ipo */
+	/* signal to ipo */
 
 	if (base) {
 		allqueue(REDRAWIPO, base->object->ipowin);
@@ -484,12 +538,12 @@ void set_active_object(Object *ob)
 
 void mouse_select(void)
 {
-	Base *base, *startbase=0, *basact=0, *oldbasact;
-	GLuint buffer[MAXPICKBUF];
+	Base *base, *startbase=NULL, *basact=NULL, *oldbasact=NULL;
+	unsigned int buffer[MAXPICKBUF];
 	int temp, a, dist=100;
 	short hits, mval[2];
 
-	/* iedere keer lijst starten vanuit basact */
+	/* always start list from basact */
 	startbase=  FIRSTBASE;
 	if(BASACT && BASACT->next) startbase= BASACT->next;
 
@@ -516,8 +570,8 @@ void mouse_select(void)
 			if(base==0) base= FIRSTBASE;
 			if(base==startbase) break;
 		}
-		
-		/* volledige redraw als */
+
+		/* complete redraw when: */
 		if(G.f & (G_VERTEXPAINT+G_FACESELECT+G_TEXTUREPAINT+G_WEIGHTPAINT)) allqueue(REDRAWVIEW3D, 0);
 		
 	}
@@ -546,8 +600,8 @@ void mouse_select(void)
 	
 	if(basact) {
 		if(G.obedit) {
-			/* alleen select doen */
-			deselectall_ex(BASACT);
+			/* only do select */
+			deselectall_except(basact);
 			basact->flag |= SELECT;
 			draw_object_ext(basact);
 		}
@@ -556,30 +610,29 @@ void mouse_select(void)
 			BASACT= basact;
 			
 			if((G.qual & LR_SHIFTKEY)==0) {
-				deselectall_ex(basact);
+				deselectall_except(basact);
 				basact->flag |= SELECT;
 			}
 			else {
-				if(oldbasact) if(oldbasact != basact) draw_object_ext(oldbasact);
-				
 				if(basact->flag & SELECT) {
 					if(basact==oldbasact)
 						basact->flag &= ~SELECT;
 				}
 				else basact->flag |= SELECT;
 			}
-			
-			/* if((basact->flag & SELECT)==0) BASACT= 0; */
+
+			// copy
 			basact->object->flag= basact->flag;
 			
+			if(oldbasact != basact) {
+				set_active_base(basact);
+			}
+
+			// for visual speed
+			if(oldbasact && oldbasact != basact && (oldbasact->lay & G.vd->lay)) 
+				draw_object_ext(oldbasact);
 			draw_object_ext(basact);
 
-			if(oldbasact != basact) {
-			
-				set_active_base(basact);
-				
-			}
-			
 			if(basact->object->type!=OB_MESH) {
 				if(G.f & G_WEIGHTPAINT) {
 					set_wpaint();	/* toggle */
@@ -591,10 +644,12 @@ void mouse_select(void)
 					set_faceselect();	/* toggle */
 				}
 			}
-			
-			allqueue(REDRAWBUTSGAME, 0);
+			/* also because multiple 3d windows can be open */
+			allqueue(REDRAWVIEW3D, 0);
+
+			allqueue(REDRAWBUTSLOGIC, 0);
 			allqueue(REDRAWDATASELECT, 0);
-			allqueue(REDRAWBUTSCONSTRAINT, 0);
+			allqueue(REDRAWBUTSOBJECT, 0);
 			allqueue(REDRAWACTION, 0);
 			allqueue(REDRAWNLA, 0);
 			allqueue(REDRAWHEADERS, 0);	/* To force display update for the posebutton */
@@ -621,8 +676,7 @@ void borderselect(void)
 	BPoint *bp;
 	MetaElem *ml;
 	struct EditVert *eve;
-	/* was IGLuint */
-	GLuint buffer[MAXPICKBUF];
+	unsigned int buffer[MAXPICKBUF];
 	int a, index;
 	short hits, val, tel;
 
@@ -657,7 +711,7 @@ void borderselect(void)
 				}
 				
 				allqueue(REDRAWBUTSEDIT, 0);
-				allqueue(REDRAWBUTSCONSTRAINT, 0);
+				allqueue(REDRAWBUTSOBJECT, 0);
 				allqueue(REDRAWACTION, 0);
 				allqueue(REDRAWNLA, 0);
 				allqueue(REDRAWVIEW3D, 0);
@@ -670,7 +724,7 @@ void borderselect(void)
 				
 				calc_meshverts_ext();	/* drawobject.c */
 				eve= G.edve.first;
-				while(eve) {
+				while(eve) {		
 					if(eve->h==0 && eve->xs>rect.xmin && eve->xs<rect.xmax) {
 						if(eve->ys>rect.ymin && eve->ys<rect.ymax) {
 							if(val==LEFTMOUSE) eve->f|= 1;
@@ -679,8 +733,7 @@ void borderselect(void)
 					}
 					eve= eve->next;
 				}
-				if(val!=LEFTMOUSE) tekenvertices_ext(0);
-				tekenvertices_ext(1);
+				allqueue(REDRAWVIEW3D, 0);
 				
 			}
 			else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
@@ -783,12 +836,8 @@ void borderselect(void)
 					}
 				}
 				
-					/* XXX, are all these really needed?
-					 * I just copied them from the G.obpose
-					 * OB_ARMATURE section above. - zr
-					 */
 				allqueue(REDRAWBUTSEDIT, 0);
-				allqueue(REDRAWBUTSCONSTRAINT, 0);
+				allqueue(REDRAWBUTSOBJECT, 0);
 				allqueue(REDRAWACTION, 0);
 				allqueue(REDRAWVIEW3D, 0);
 			}
@@ -836,13 +885,16 @@ void borderselect(void)
 				
 				base= base->next;
 			}
+			/* frontbuffer flush */
+			glFinish();
+			
 			allqueue(REDRAWDATASELECT, 0);
 			
-			/* i.v.m. backbufprojektie */
+			/* because backbuf drawing */
 			tel= 1;
 			base= FIRSTBASE;
 			while(base) {
-				/* elke base ivm meerdere windows */
+				/* each base because of multiple windows */
 				base->selcol = ((tel & 0xF00)<<12) 
 					+ ((tel & 0xF0)<<8) 
 					+ ((tel & 0xF)<<4);
@@ -850,7 +902,7 @@ void borderselect(void)
 				base= base->next;
 			}
 			/* new */
-			allqueue(REDRAWBUTSGAME, 0);
+			allqueue(REDRAWBUTSLOGIC, 0);
 			allqueue(REDRAWNLA, 0);
 		}
 		countall();
@@ -889,15 +941,10 @@ void mesh_selectionCB(int selecting, Object *editobj, short *mval, float rad)
 		}
 		eve= eve->next;
 	}
-	if(G.f & (G_FACESELECT+G_DRAWFACES)) {
-		draw_sel_circle(0, 0, 0, 0, 0);	/* signal */
-		force_draw();
-	}
-	else {
-		if(selecting!=LEFTMOUSE) tekenvertices_ext(0);
-		/* altijd geselecteerde vertices tekenen */
-		tekenvertices_ext(1);
-	}
+
+	draw_sel_circle(0, 0, 0, 0, 0);	/* signal */
+	force_draw();
+
 }
 
 
@@ -1039,7 +1086,7 @@ void circle_select(void)
 	
 	while(TRUE) {
 		
-		/* als een renderwindow open is en de muis gaat erin */
+		/* when a renderwindow is open and mouse enters it (activates sometimes) */
 
 		mywinset(curarea->win);
 
@@ -1072,15 +1119,8 @@ void circle_select(void)
 						}
 						eve= eve->next;
 					}
-					if(G.f & (G_FACESELECT+G_DRAWFACES)) {
-						draw_sel_circle(0, 0, 0, 0, 0);	/* signal */
-						force_draw();
-					}
-					else {
-						if(selecting!=LEFTMOUSE) tekenvertices_ext(0);
-						/* altijd geselecteerde vertices tekenen */
-						tekenvertices_ext(1);
-					}
+					draw_sel_circle(0, 0, 0, 0, 0);	/* signal */
+					force_draw();
 				}
 				else if ELEM(G.obedit->type, OB_CURVE, OB_SURF) {
 					
@@ -1193,7 +1233,7 @@ void circle_select(void)
 		}
 	}
 	
-	/* cirkel wissen */
+	/* clear circle */
 	draw_sel_circle(0, mvalo, 0, rad, 1);
 
 	countall();
@@ -1224,6 +1264,9 @@ void set_render_border(void)
 		CLAMP(G.scene->r.border.ymax, 0.0, 1.0);
 		
 		allqueue(REDRAWVIEWCAM, 1);
+		// if it was not set, we do this
+		G.scene->r.mode |= R_BORDER;
+		allqueue(REDRAWBUTSSCENE, 1);
 	}
 }
 
@@ -1285,7 +1328,7 @@ void fly(void)
 		}
 		if(loop==0) break;
 		
-		/* dvec bepalen */
+		/* define dvec */
 		val= mval[0]-cent[0];
 		if(val>20) val-= 20; else if(val< -20) val+= 20; else val= 0;
 		dvec[0]= 0.000001*val*val;
@@ -1308,7 +1351,7 @@ void fly(void)
 			
 			Mat3CpyMat4(mat, G.vd->viewinv);
 			Mat3MulVecfl(mat, dvec);
-			quat= vectoquat(dvec, 5, 1);	/* track en upflag, niet die van de base: cameraview-berekening gebruikt ze niet */
+			quat= vectoquat(dvec, 5, 1);	/* track and upflag, not from the base: camera view calculation does not use that */
 			
 			QuatToEul(quat, G.vd->camera->rot);
 			
