@@ -1,5 +1,5 @@
 /**
- * $Id: filesel.c,v 1.17 2003/07/13 20:16:56 larstiq Exp $
+ * $Id: filesel.c,v 1.28 2003/11/19 22:00:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -55,7 +55,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 #include "BLI_linklist.h"
 #include "BLI_storage_types.h"
 #include "BLI_dynstr.h"
@@ -91,6 +90,7 @@
 #include "BIF_editview.h"
 #include "BIF_space.h"
 #include "BIF_screen.h"
+#include "BIF_resources.h"
 
 #include "BLO_readfile.h"
 
@@ -101,9 +101,9 @@
 #include "mydevice.h"
 #include "blendef.h"
 #include "render.h"
-#include "interface.h"
 #include "nla.h"
 
+
 #if defined WIN32 || defined __BeOS
 	int fnmatch(){return 0;}
 #else
@@ -886,41 +886,37 @@ static void draw_filescroll(SpaceFile *sfile)
 
 	if(scrollrct.ymin+10 >= scrollrct.ymax) return;
 	
-	cpack(0x707070);
+	BIF_ThemeColor(TH_BACK);
 	glRecti(scrollrct.xmin,  scrollrct.ymin,  scrollrct.xmax,  scrollrct.ymax);
 
 	uiEmboss(scrollrct.xmin, scrollrct.ymin, scrollrct.xmax, scrollrct.ymax, 1);
 
-	cpack(0x909090);
+	BIF_ThemeColor(TH_HEADER);
 	glRecti(bar.xmin+2,  bar.ymin+2,  bar.xmax-2,  bar.ymax-2);
 
 	uiEmboss(bar.xmin+2, bar.ymin+2, bar.xmax-2, bar.ymax-2, filescrollselect);
 	
 }
 
-static void regelrect(unsigned int col, int x, int y)
+static void regelrect(int id, int x, int y)
 {
-	cpack(col);
+	if(id & ACTIVE) {
+		if(id & HILITE) BIF_ThemeColorShade(TH_HILITE, 20);
+		else BIF_ThemeColor(TH_HILITE);
+	}
+	else if(id & HILITE) BIF_ThemeColorShade(TH_BACK, 20);
+	else BIF_ThemeColor(TH_BACK);
+	
 	glRects(x-17,  y-3,  x+collumwidth-21,  y+11);
 
 }
 
 static void printregel(SpaceFile *sfile, struct direntry *files, int x, int y)
 {
-	unsigned int boxcol=0;
+	int boxcol=0;
 	char *s;
 
-	switch(files->flags & (HILITE + ACTIVE)) {
-	case HILITE+ACTIVE:
-		boxcol= (0xC09090);
-		break;
-	case HILITE:
-		boxcol= (0x909090);
-		break;
-	case ACTIVE:
-		boxcol= (0xB08080);
-		break;
-	}
+	boxcol= files->flags & (HILITE + ACTIVE);
 
 	if(boxcol) {
 		regelrect(boxcol, x, y);
@@ -955,8 +951,8 @@ static void printregel(SpaceFile *sfile, struct direntry *files, int x, int y)
 		glRects(x-14,  y,  x-8,  y+7);
 	}
 	
-	if(S_ISDIR(files->type)) cpack(0xFFFFFF);
-	else cpack(0x0);
+	if(S_ISDIR(files->type)) BIF_ThemeColor(TH_TEXT_HI);
+	else BIF_ThemeColor(TH_TEXT);
 
 	s = files->string;
 	if(s) {
@@ -1070,7 +1066,7 @@ static void set_active_file(SpaceFile *sfile, int act)
 		glScissor(curarea->winrct.xmin, curarea->winrct.ymin, curarea->winx-12, curarea->winy);
 
 		if( calc_filesel_regel(sfile, old, &x, &y) ) {
-			regelrect(0x717171, x, y);
+			regelrect(0, x, y);
 			printregel(sfile, sfile->filelist+old, x, y);
 		}
 		if( calc_filesel_regel(sfile, newi, &x, &y) ) {
@@ -1098,7 +1094,7 @@ static void draw_filetext(SpaceFile *sfile)
 
 
 	/* box */
-	cpack(0x717171);
+	BIF_ThemeColor(TH_BACK);
 	glRecti(textrct.xmin,  textrct.ymin,  textrct.xmax,  textrct.ymax);
 
 	/* collums */
@@ -1129,24 +1125,25 @@ static void draw_filetext(SpaceFile *sfile)
 	}
 
 	/* clear drawing errors, with text at the right hand side: */
+	BIF_ThemeColor(TH_HEADER);
+	glRecti(textrct.xmax,  textrct.ymin,  textrct.xmax+10,  textrct.ymax);
 	uiEmboss(textrct.xmin, textrct.ymin, textrct.xmax, textrct.ymax, 1);
-	
-	glColor3f(.5625, .5625, .5625);
-	glRecti(textrct.xmax+2,  textrct.ymin,  textrct.xmax+10,  textrct.ymax);
 }
 
 void drawfilespace(ScrArea *sa, void *spacedata)
 {
 	SpaceFile *sfile;
 	uiBlock *block;
+	float col[3];
 	int act, loadbutton;
 	short mval[2];
 	char name[20];
 	char *menu;
 
-	myortho2(-0.5, curarea->winrct.xmax-curarea->winrct.xmin-0.5, -0.5, curarea->winrct.ymax-curarea->winrct.ymin-0.5);
+	myortho2(-0.375, sa->winx-0.375, -0.375, sa->winy-0.375);
 
-	glClearColor(.56, .56, .56, 0.0); 
+	BIF_GetThemeColor3fv(TH_HEADER, col);	// basic undrawn color is border
+	glClearColor(col[0], col[1], col[2], 0.0); 
 	glClear(GL_COLOR_BUFFER_BIT);
 
 	sfile= curarea->spacedata.first;	
@@ -1165,7 +1162,7 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 
 	/* HEADER */
 	sprintf(name, "win %d", curarea->win);
-	block= uiNewBlock(&curarea->uiblocks, name, UI_EMBOSSF, UI_HELV, curarea->win);
+	block= uiNewBlock(&curarea->uiblocks, name, UI_EMBOSS, UI_HELV, curarea->win);
 	
 	uiSetButLock( sfile->type==FILE_MAIN && sfile->returnfunc, NULL);
 
@@ -1199,6 +1196,8 @@ void drawfilespace(ScrArea *sa, void *spacedata)
 	/* others diskfree etc ? */
 	scrarea_queue_headredraw(curarea);	
 	
+	myortho2(-0.375, (float)(sa->winx)-0.375, -0.375, (float)(sa->winy)-0.375);
+	draw_area_emboss(sa);
 	
 	curarea->win_swap= WIN_BACK_OK;
 }
@@ -1529,7 +1528,7 @@ static void filesel_execute(SpaceFile *sfile)
 					}
 				}
 			}
-			sfile->returnfunc((char*) sfile->retval);
+			sfile->returnfunc((char*) (long)sfile->retval);
 		}
 		else {
 			if(strncmp(sfile->title, "SAVE", 4)==0) free_filesel_spec(sfile->dir);
@@ -1797,7 +1796,7 @@ static void fs_fake_users(SpaceFile *sfile)
 
 static int get_hilited_entry(SpaceFile *sfile)
 {
-	int a, count=0;
+	int a;
 
 	for(a=0; a<sfile->totfile; a++) {
 		if(sfile->filelist[a].flags & HILITE) {
@@ -1812,7 +1811,6 @@ void winqreadfilespace(ScrArea *sa, void *spacedata, BWinEvent *evt)
 {
 	unsigned short event= evt->event;
 	short val= evt->val;
-	char ascii= evt->ascii;
 	static int acto=0;
 	SpaceFile *sfile;
 	int act, do_draw= 0, i, test, ret = 0;
@@ -2285,7 +2283,8 @@ static void library_to_filelist(SpaceFile *sfile)
 		if(sfile->libfiledata==0) return;
 	}
 	
-	if (idcode= groupname_to_code(group)) {
+	idcode= groupname_to_code(group);
+	if (idcode) {
 		names= BLO_blendhandle_get_datablock_names(sfile->libfiledata, idcode);
 	} else {
 		names= BLO_blendhandle_get_linkable_groups(sfile->libfiledata);
@@ -2363,7 +2362,7 @@ static void filesel_select_objects(SpaceFile *sfile)
 			}
 
 		}
-		allqueue(REDRAWBUTSRENDER, 0);
+		allqueue(REDRAWBUTSSCENE, 0);
 	}
 }
 
