@@ -1,5 +1,5 @@
 /**
- * $Id: ghostwinlay.c,v 1.12 2003/06/12 10:07:36 ton Exp $
+ * $Id: ghostwinlay.c,v 1.20 2003/11/12 13:32:11 rwenzlaff Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -42,6 +42,9 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "DNA_listBase.h"	
+#include "DNA_userdef_types.h"	/* U.flag & TWOBUTTONMOUSE */
+
 #include "BLI_blenlib.h"
 
 #include "GHOST_C-api.h"
@@ -69,12 +72,13 @@ static GHOST_TStandardCursor convert_cursor(int curs) {
 	default:
 	case CURSOR_STD:		return GHOST_kStandardCursorDefault;
 	case CURSOR_VPAINT:		return GHOST_kStandardCursorRightArrow;
-	case CURSOR_FACESEL:	return GHOST_kStandardCursorRightArrow;
+	case CURSOR_FACESEL:		return GHOST_kStandardCursorRightArrow;
 	case CURSOR_WAIT:		return GHOST_kStandardCursorWait;
 	case CURSOR_EDIT:		return GHOST_kStandardCursorCrosshair;
 	case CURSOR_HELP:		return GHOST_kStandardCursorHelp;
 	case CURSOR_X_MOVE:		return GHOST_kStandardCursorLeftRight;
 	case CURSOR_Y_MOVE:		return GHOST_kStandardCursorUpDown;
+	case CURSOR_PENCIL:		return GHOST_kStandardCursorPencil;
 	}
 }
 
@@ -227,6 +231,10 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 	int scr_w, scr_h;
 
 	winlay_get_screensize(&scr_w, &scr_h);
+	if(G.f & G_DEBUG) {
+		printf("screen size %d %d\n", scr_w, scr_h);	
+		printf("win start %d %d size %d %d\n",  posx,  posy,  sizex,  sizey);
+	}
 	posy= (scr_h-posy-sizey);
 	
 	/* create a fullscreen window on unix by default*/
@@ -239,7 +247,7 @@ Window *window_open(char *title, int posx, int posy, int sizex, int sizey, int s
 		inital_state= GHOST_kWindowStateFullScreen;
 	else
 		inital_state= start_maximized?GHOST_kWindowStateMaximized:GHOST_kWindowStateNormal;
-#else
+#else			// APPLE
 	inital_state= start_maximized?GHOST_kWindowStateMaximized:GHOST_kWindowStateNormal;
 #endif
 #endif
@@ -318,8 +326,9 @@ void window_set_cursor(Window *win, int curs) {
 	}
 }
 
-void window_set_custom_cursor(Window *win, unsigned char mask[16][2], unsigned char bitmap[16][2]) {
-	GHOST_SetCustomCursorShape(win->ghostwin, bitmap, mask, 7, 7);
+void window_set_custom_cursor(Window *win, unsigned char mask[16][2], 
+					unsigned char bitmap[16][2], int hotx, int hoty) {
+	GHOST_SetCustomCursorShape(win->ghostwin, bitmap, mask, hotx, hoty);
 }
 
 void window_make_active(Window *win) {
@@ -361,7 +370,8 @@ static int change_bit(int val, int bit, int to_on) {
 	return to_on?(val|bit):(val&~bit);
 }
 
-static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) {
+static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) 
+{
 	GHOST_TEventType type= GHOST_GetEventType(evt);
 
 	if (type == GHOST_kEventQuit) {
@@ -389,10 +399,10 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) {
 			GHOST_ScreenToClient(win->ghostwin, cd->x, cd->y, &cx, &cy);
 			win->lmouse[0]= cx;
 			win->lmouse[1]= (win->size[1]-1) - cy;
-
+			
 			window_handle(win, MOUSEX, win->lmouse[0]);
 			window_handle(win, MOUSEY, win->lmouse[1]);
-			
+
 			break;
 		}
 		case GHOST_kEventButtonDown:
@@ -405,7 +415,8 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) {
 				if (val) {
 					if (win->commandqual) {
 						bbut= win->faked_mbut= RIGHTMOUSE;
-					} else if (win->lqual & LR_ALTKEY) {
+					} else if ((win->lqual & LR_ALTKEY) && (U.flag & TWOBUTTONMOUSE)) {
+						/* finally, it actually USES the userpref! :) -intrr */
 						bbut= win->faked_mbut= MIDDLEMOUSE;
 					}
 				} else {
@@ -532,9 +543,9 @@ static int event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr private) {
 			window_handle(win, RESHAPE, 1);
 			break;
 		}
-		}
 	}
-
+	}
+	
 	return 1;
 }
 
