@@ -1,5 +1,5 @@
 /**
- * $Id: imasel.c,v 1.4 2003/02/09 15:16:36 phase Exp $
+ * $Id: imasel.c,v 1.7 2003/11/19 22:00:13 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -32,6 +32,7 @@
 
 #include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -49,7 +50,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -550,7 +550,7 @@ void get_next_image(SpaceImaSel *simasel)
 			
 			IMB_converttocmap(ibuf);
 			
-			/* copy ibuf->rect naar ima->pict_rect */ 
+			/* copy ibuf->rect to ima->pict_rect */ 
 			size = ima->dw * ima->dh; if (size > 3968) size = 3968;
 			longtochar(ima->pict_rect, ibuf->rect, size); 
 
@@ -901,8 +901,8 @@ void change_imadir(SpaceImaSel *simasel)
 void check_imasel_copy(SpaceImaSel *simasel)
 {
 
-	/* LET OP: wordt ook gebruikt bij inlezen blender file */
-	/* dingen op nul zetten, opnieuw malloccen etc */
+	/* WATCH IT: also used when reading blender file */
+	/* initialize stuff, malloc, etc */
 	simasel->first_sel_ima	=  0;
 	simasel->hilite_ima	    =  0;
 	simasel->firstdir		=  0;
@@ -916,7 +916,7 @@ void check_imasel_copy(SpaceImaSel *simasel)
 
 void free_imasel(SpaceImaSel *simasel)
 {
-	/* NIET de imasel zelf vrijgeven */
+	/* do not free imasel itself */
 	
 	clear_ima_dir(simasel);
 	IMB_freeImBuf(simasel->cmap);
