@@ -1,5 +1,5 @@
 /**
- * $Id: mywindow.c,v 1.6 2003/05/03 12:17:11 phase Exp $
+ * $Id: mywindow.c,v 1.13 2003/11/19 22:00:14 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 #include "BLI_gsqueue.h"
 
 #include "DNA_screen_types.h"
@@ -84,7 +83,7 @@ static int mainwin_color_depth;
 
 void mywindow_init_mainwin(Window *win, int orx, int ory, int sizex, int sizey)
 {
-	int r, g, b;
+	GLint r, g, b;
 	
 	winlay_mainwindow= win;
 	
@@ -97,7 +96,7 @@ void mywindow_init_mainwin(Window *win, int orx, int ory, int sizex, int sizey)
 	mainwindow.ymax= ory+sizey-1;
 	mainwindow.qevents= NULL;
 
-	myortho2(-0.5, (float)sizex-0.5, -0.6, (float)sizey-0.6);
+	myortho2(-0.5, (float)sizex-0.5, -0.5, (float)sizey-0.5);
 	glLoadIdentity();
 		
 	glGetFloatv(GL_PROJECTION_MATRIX, (float *)mainwindow.winmat);
@@ -249,6 +248,14 @@ void bwin_multmatrix(int winid, float mat[][4])
 	glGetFloatv(GL_MODELVIEW_MATRIX, (float *)win->viewmat);
 }
 
+void bwin_scalematrix(int winid, float x, float y, float z)
+{
+	bWindow *win= bwin_from_winid(winid);
+
+	glScalef(x, y, z);
+	glGetFloatv(GL_MODELVIEW_MATRIX, (float *)win->viewmat);
+}
+
 void bwin_clear_viewmat(int swin)
 {
 	bWindow *win;
@@ -422,6 +429,7 @@ void bwin_ortho(int winid, float x1, float x2, float y1, float y2, float n, floa
 	
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
+
 	glOrtho(x1, x2, y1, y2, n, f);
 
 	glGetFloatv(GL_PROJECTION_MATRIX, (float *)bwin->winmat);
@@ -452,6 +460,9 @@ void myortho(float x1, float x2, float y1, float y2, float n, float f)
 
 void myortho2(float x1, float x2, float y1, float y2)
 {
+	/* prevent opengl from generating errors */
+	if(x1==x2) x2+=1.0;
+	if(y1==y2) y2+=1.0;
 	bwin_ortho(curswin, x1, x2, y1, y2, -1, 1);
 }
 
