@@ -2,7 +2,7 @@
  * 
  * maart 95
  * 
- * $Id: previewrender.c,v 1.5 2003/07/19 20:31:29 ton Exp $
+ * $Id: previewrender.c,v 1.16 2003/11/27 23:41:54 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -73,21 +73,25 @@
 #include "BKE_world.h"
 #include "BKE_texture.h"
 
+#include "BSE_headerbuttons.h"
+
 #include "BIF_gl.h"
 #include "BIF_screen.h"
 #include "BIF_space.h"		/* allqueue */
+#include "BIF_butspace.h"	
 #include "BIF_drawimage.h"	/* rectwrite_part */
 #include "BIF_mywindow.h"
+#include "BIF_interface.h"
 #include "PIL_time.h"
 
 #include "RE_renderconverter.h"
 
-#define PR_RECTX	101
-#define PR_RECTY	101
+#define PR_RECTX	121
+#define PR_RECTY	121
 #define PR_XMIN		10
-#define PR_YMIN		10
-#define PR_XMAX		190
-#define PR_YMAX		190
+#define PR_YMIN		5
+#define PR_XMAX		200
+#define PR_YMAX		195
 
 #define PR_FACY		(PR_YMAX-PR_YMIN-4)/(PR_RECTY)
 
@@ -313,29 +317,52 @@ static void draw_tex_crop(Tex *tex)
 	
 }
 
-void BIF_preview_changed(SpaceButs *sbuts)
+void BIF_all_preview_changed(void)
 {
-	sbuts->cury= 0;
-	addafterqueue(sbuts->area->win, RENDERPREVIEW, 1);
+	ScrArea *sa;
+	SpaceButs *sbuts;
+	
+	sa= G.curscreen->areabase.first;
+	while(sa) {
+		if(sa->spacetype==SPACE_BUTS) {
+			sbuts= sa->spacedata.first;
+			sbuts->cury= 0;
+			addafterqueue(sa->win, RENDERPREVIEW, 1);
+		}
+		sa= sa->next;
+	}
 }
 
-void BIF_previewdraw(SpaceButs *sbuts)
+
+void BIF_preview_changed(SpaceButs *sbuts)
 {
+	/* can be called when no buttonswindow visible */
+	if(sbuts) {
+		sbuts->cury= 0;
+		addafterqueue(sbuts->area->win, RENDERPREVIEW, 1);
+	}
+}
+
+/* is supposed to be called with correct panel offset matrix */
+void BIF_previewdraw(void)
+{
+	SpaceButs *sbuts= curarea->spacedata.first;
+	
 	set_previewrect(sbuts->area->win, PR_XMIN, PR_YMIN, PR_XMAX, PR_YMAX);
 
-	if (sbuts->rect==0 || sbuts->cury==0) {
-		BIF_preview_changed(sbuts);
-	} else {
+	if (sbuts->rect==0) BIF_preview_changed(sbuts);
+	else {
 		int y;
 
 		for (y=0; y<PR_RECTY; y++) {
 			display_pr_scanline(sbuts->rect, y);
 		}
 
-		if (sbuts->mainb==BUTS_TEX) {
+		if (sbuts->mainb==CONTEXT_SHADING && sbuts->tab[CONTEXT_SHADING]==TAB_SHADING_TEX) {
 			draw_tex_crop(sbuts->lockpoin);
 		}
 	}
+	if(sbuts->cury==0) BIF_preview_changed(sbuts);
 }
 
 static void sky_preview_pixel(float lens, int x, int y, char *rect)
@@ -497,10 +524,14 @@ static void halo_preview_pixel(HaloRen *har, int startx, int endx, int y, char *
 
 static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 {
+	uiBlock *block;
 	float ycor;
 	unsigned int *rectot;
 	int afmx, afmy, rectx, recty;
 	
+	block= uiFindOpenPanelBlockName(&curarea->uiblocks, "Preview");
+	if(block==NULL) return;
+
 	/* temps */
 	ycor= R.ycor;
 	rectx= R.rectx;
@@ -517,12 +548,15 @@ static void previewflare(SpaceButs *sbuts, HaloRen *har, unsigned int *rect)
 	R.rectot= rect;
 
 	waitcursor(1);
-
 	RE_renderflare(har);
-
-	BIF_previewdraw(sbuts);
-	
 	waitcursor(0);
+	// not sure why, either waitcursor or renderflare screws up
+	areawinset(curarea->win);
+	
+	uiPanelPush(block);
+	BIF_previewdraw();
+	uiPanelPop(block);
+	
 	
 	/* temps */
 	R.ycor= ycor;
@@ -633,11 +667,7 @@ static float pr2_lamp[3]= {-8.8, -5.6, -1.5};
 static float pr1_col[3]= {0.8, 0.8, 0.8};
 static float pr2_col[3]= {0.5, 0.6, 0.7};
 
-static void shade_preview_pixel(float *vec, 
-								int x, 
-								int y, 
-								char *rect, 
-								int smooth)
+static void shade_preview_pixel(float *vec, int x, int y,char *rect, int smooth)
 {
 	Material *mat;
 	float v1,inp, inprspec=0, isr=0.0, isb=0.0, isg=0.0;
@@ -836,22 +866,52 @@ static void shade_preview_pixel(float *vec,
 
 void BIF_previewrender(SpaceButs *sbuts)
 {
-	Material *mat=0;
-	Tex *tex = NULL;
+	ID *id, *idfrom;
+	Material *mat= NULL;
+	Tex *tex= NULL;
+	Lamp *la= NULL;
+	World *wrld= NULL;
+	LampRen *lar= NULL;
 	Image *ima;
-	Lamp *la;
-	LampRen *lar = NULL;
 	HaloRen har;
 	Object *ob;
-	World *wrld;
+	uiBlock *block;
 	float lens = 0.0, vec[3];
 	int x, y, starty, startx, endy, endx, radsq, xsq, ysq, last = 0;
 	unsigned int *rect;
 
 	if(sbuts->cury>=PR_RECTY) return;
 	
-	if ELEM4(sbuts->mainb, BUTS_MAT, BUTS_TEX, BUTS_LAMP, BUTS_WORLD);
-	else return;
+	/* we safely assume curarea has panel "preview" */
+	/* quick hack for now, later on preview should become uiBlock itself */
+	
+	block= uiFindOpenPanelBlockName(&curarea->uiblocks, "Preview");
+	if(block==NULL) return;
+
+	ob= ((G.scene->basact)? (G.scene->basact)->object: 0);
+	
+	/* we cant trust this global lockpoin.. for example with headerless window */
+	buttons_active_id(&id, &idfrom);
+	G.buts->lockpoin= id;
+
+	if(sbuts->mainb==CONTEXT_SHADING) {
+		int tab= sbuts->tab[CONTEXT_SHADING];
+		
+		if(tab==TAB_SHADING_MAT) 
+			mat= sbuts->lockpoin;
+		else if(tab==TAB_SHADING_TEX) 
+			tex= sbuts->lockpoin;
+		else if(tab==TAB_SHADING_LAMP) {
+			if(ob && ob->type==OB_LAMP) la= ob->data;
+		}
+		else if(tab==TAB_SHADING_WORLD)
+			wrld= sbuts->lockpoin;
+	}
+	else if(sbuts->mainb==CONTEXT_OBJECT) {
+		if(ob && ob->type==OB_LAMP) la= ob->data;
+	}
+	
+	if(mat==NULL && tex==NULL && la==NULL && wrld==NULL) return;
 	
 	har.flarec= 0;	/* below is a test for postrender flare */
 	
@@ -864,10 +924,7 @@ void BIF_previewrender(SpaceButs *sbuts)
 	MTC_Mat4One(R.viewinv);
 	
 	R.osatex= 0;
-	if(sbuts->mainb==BUTS_MAT) {
-		mat= sbuts->lockpoin;
-		if(mat==0) return;
-
+	if(mat) {
 		/* rendervars */
 		init_render_world();
 		init_render_material(mat);
@@ -891,24 +948,21 @@ void BIF_previewrender(SpaceButs *sbuts)
 		
 		if(mat->mode & MA_HALO) init_previewhalo(&har, mat);
 	}
-	else if(sbuts->mainb==BUTS_TEX) {
-		tex= sbuts->lockpoin;
-		if(tex==0) return;
+	else if(tex) {
+
 		ima= tex->ima;
 		if(ima) last= ima->lastframe;
 		init_render_texture(tex);
 		free_unused_animimages();
 		if(tex->ima) {
-			if(tex->ima!=ima) allqueue(REDRAWBUTSTEX, 0);
-			else if(last!=ima->lastframe) allqueue(REDRAWBUTSTEX, 0);
+			if(tex->ima!=ima) allqueue(REDRAWBUTSSHADING, 0);
+			else if(last!=ima->lastframe) allqueue(REDRAWBUTSSHADING, 0);
 		}
 		if(tex->env && tex->env->object) 
 			MTC_Mat4Invert(tex->env->object->imat, tex->env->object->obmat);
 	}
-	else if(sbuts->mainb==BUTS_LAMP) {
-		ob= ((G.scene->basact)? (G.scene->basact)->object: 0);
-		if(ob==0 || ob->type!=OB_LAMP) return;
-		la= ob->data;
+	else if(la) {
+
 		init_render_world();
 		init_render_textures();	/* do not do it twice!! (brightness) */
 		R.totlamp= 0;
@@ -922,9 +976,7 @@ void BIF_previewrender(SpaceButs *sbuts)
 		
 		MTC_Mat3One(lar->imat);
 	}
-	else {
-		wrld= sbuts->lockpoin;
-		if(wrld==0) return;
+	else if(wrld) {
 		
 		lens= 35.0;
 		if(G.scene->camera) {
@@ -958,8 +1010,6 @@ void BIF_previewrender(SpaceButs *sbuts)
 
 	radsq= (PR_RECTX/2)*(PR_RECTY/2);
 	
-	glDrawBuffer(GL_FRONT);
-	
 	if(mat) {
 		if(mat->pr_type==MA_SPHERE) {
 			pr1_lamp[0]= 2.3; pr1_lamp[1]= -2.4; pr1_lamp[2]= -4.6;
@@ -970,13 +1020,17 @@ void BIF_previewrender(SpaceButs *sbuts)
 			pr2_lamp[0]= 1.2; pr2_lamp[1]= -18; pr2_lamp[2]= 3.2;
 		}
 	}
-	
+
+	/* here it starts! */
+	glDrawBuffer(GL_FRONT);
+	uiPanelPush(block);
+
 	for(y=starty; y<endy; y++) {
 		
 		rect= sbuts->rect + 1 + PR_RECTX*sbuts->cury;
 		
 		if(y== -PR_RECTY/2 || y==endy-1);		/* emboss */
-		else if(sbuts->mainb==BUTS_MAT) {
+		else if(mat) {
 			
 			if(mat->mode & MA_HALO) {
 				for(x=startx; x<endx; x++, rect++) {
@@ -1034,12 +1088,12 @@ void BIF_previewrender(SpaceButs *sbuts)
 				}
 			}
 		}
-		else if(sbuts->mainb==BUTS_TEX) {
+		else if(tex) {
 			for(x=startx; x<endx; x++, rect++) {
 				texture_preview_pixel(tex, x, y, (char *)rect);
 			}
 		}
-		else if(sbuts->mainb==BUTS_LAMP) {
+		else if(la) {
 			for(x=startx; x<endx; x++, rect++) {
 				lamp_preview_pixel(lar, x, y, (char *)rect);
 			}
@@ -1063,25 +1117,29 @@ void BIF_previewrender(SpaceButs *sbuts)
 		sbuts->cury++;
 	}
 
-	if(sbuts->cury>=PR_RECTY && sbuts->mainb==BUTS_TEX) 
-		draw_tex_crop(sbuts->lockpoin);
+	if(sbuts->cury>=PR_RECTY && tex) 
+		if (sbuts->tab[CONTEXT_SHADING]==TAB_SHADING_TEX) 
+			draw_tex_crop(sbuts->lockpoin);
 	
 	glDrawBuffer(GL_BACK);
-	BIF_previewdraw(sbuts);
+	/* draw again for clean swapbufers */
+	BIF_previewdraw();
+
+	uiPanelPop(block);
 	
-	if(sbuts->mainb==BUTS_MAT) {
+	if(mat) {
 		end_render_material(mat);
 		for(x=0; x<8; x++) {
 			if(mat->mtex[x] && mat->mtex[x]->tex) end_render_texture(mat->mtex[x]->tex);
 		}	
 	}
-	else if(sbuts->mainb==BUTS_TEX) {
+	else if(tex) {
 		end_render_texture(tex);
 	}
-	else if(sbuts->mainb==BUTS_WORLD) {
+	else if(wrld) {
 		end_render_textures();
 	}
-	else if(sbuts->mainb==BUTS_LAMP) {
+	else if(la) {
 		if(R.totlamp) {
 			if(R.la[0]->org) MEM_freeN(R.la[0]->org);
 			MEM_freeN(R.la[0]);
