@@ -1,5 +1,5 @@
 /**
- * $Id: toolbox.c,v 1.16 2003/07/20 23:04:09 hos Exp $
+ * $Id: toolbox.c,v 1.32 2003/11/22 22:22:16 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -31,6 +31,7 @@
  */
 
 #define PY_TOOLBOX 1
+
 #include <math.h>
 #include <ctype.h>
 #include <stdlib.h>
@@ -53,19 +54,23 @@
 
 #include "BMF_Api.h"
 #include "BIF_language.h"
-
-#include "BLI_blenlib.h"
-#include "BLI_arithb.h"
-#include "BLI_editVert.h"
+#include "BIF_resources.h"
 
 #include "DNA_image_types.h"
 #include "DNA_object_types.h"
+#include "DNA_mesh_types.h"
+#include "DNA_lamp_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_userdef_types.h"
 #include "DNA_view3d_types.h"
 
+#include "BLI_blenlib.h"
+#include "BLI_arithb.h"
+
 #include "BKE_plugin_types.h"
 #include "BKE_utildefines.h"
+#include "BKE_mesh.h"
+#include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
 
@@ -94,13 +99,13 @@
 #include "BSE_editipo.h"
 #include "BSE_buttons.h"
 #include "BSE_filesel.h"
+#include "BSE_edit.h"
+#include "BSE_headerbuttons.h"
 
 #include "IMB_imbuf.h"
 
 #include "mydevice.h"
 #include "blendef.h"
-
-#include "interface.h"
 #include "render.h"
 
 
@@ -167,6 +172,7 @@ void tbox_setinfo(int x, int y)
 				if (addmode==OB_MESH) tbstr= "  MESH";
 				else if(addmode==OB_CURVE) tbstr= "  CURVE";
 				else if(addmode==OB_SURF) tbstr= "  SURF";
+				else if(addmode==OB_MBALL) tbstr= "  META";
 				else tbstr= "ADD";
 				break;
 			case TBOX_MAIN_OBJECT1:		tbstr= "OBJECT";	break;
@@ -240,8 +246,9 @@ void tbox_setinfo(int x, int y)
 					case 0: tbstr= "Mesh";		tbstr1= ">>";	keystr= ">>";	tbval=OB_MESH;										break;
 					case 1: tbstr= "Curve";		tbstr1= ">>";	keystr= ">>";	tbval=OB_CURVE;	;									break;
 					case 2: tbstr= "Surface";	tbstr1= ">>";	keystr= ">>";	tbval=OB_SURF;										break;
-					case 3: tbstr= "Text";		tbstr1= "";		keystr= "";		tbval=OB_FONT;		tbfunc= add_primitiveFont;		break;
-					case 4: tbstr= "MetaBall";	tbstr1= "";		keystr= "";		tbval=OB_MBALL;		tbfunc= add_primitiveMball;		break;
+					case 3: tbstr= "Meta";		tbstr1= ">>";	keystr= ">>";	tbval=OB_MBALL;
+						break;
+					case 4: tbstr= "Text";		tbstr1= "";		keystr= "";		tbval=OB_FONT;		tbfunc= add_primitiveFont;		break;
 					case 5: tbstr= "Empty";		tbstr1= "A";	keystr= "";		tbval=OB_EMPTY;										break;
 					case 6: tbstr= "";			tbstr1= "";		keystr= "";		tbval=0;											break;
 					case 7: tbstr= "Camera";	tbstr1= "A";	keystr= "";		tbval=OB_CAMERA;									break;
@@ -302,13 +309,13 @@ void tbox_setinfo(int x, int y)
 				}
 				if(tbstr1 && tbstr1[0]=='A') tbfunc= add_primitiveCurve;
 			}
-/*			else if(addmode==OB_MBALL) {
+			else if(addmode==OB_MBALL) {
 				switch(y) {
-					case 0: tbstr= "Ball";		tbstr1= "A";	tbval=1; break;
-					case 1: tbstr= "";			tbstr1= "";		break;
-					case 2: tbstr= "";			tbstr1= "";		break;
-					case 3: tbstr= "";			tbstr1= "";		break;
-					case 4: tbstr= "";			tbstr1= "";		break;
+					case 0: tbstr= "Ball";			tbstr1= "A";	tbval=1; break;
+					case 1: tbstr= "Tube";			tbstr1= "A";	tbval=2; break;
+					case 2: tbstr= "Plane";			tbstr1= "A";	tbval=3; break;
+					case 3: tbstr= "Elipsoid";		tbstr1= "A";	tbval=4; break;
+					case 4: tbstr= "Cube";			tbstr1= "A";	tbval=5; break;
 					case 5: tbstr= "";			tbstr1= "";		break;
 					case 6: tbstr= "";			tbstr1= "";		break;
 					case 7: tbstr= "";			tbstr1= "";		break;
@@ -318,7 +325,7 @@ void tbox_setinfo(int x, int y)
 					case 11: tbstr= "Duplicate";tbstr1= "D";	break;
 				}
 				if(tbstr1 && tbstr1[0]=='A') tbfunc= add_primitiveMball;
-			}*/
+			}
 		}
 		
 /* OB TOPICS 1 */
@@ -841,7 +848,7 @@ void tbox_drawelem_text(x, y, type)
 		}
 		
 		glRasterPos2i(x1+5, y1+tbfontyofs);
-		BIF_DrawString(G.font, tbstr, (U.transopts & TR_MENUS), (type==0 || type==2)?0:1);
+		BIF_DrawString(G.font, tbstr, (U.transopts & TR_MENUS));
 		
 		if(keystr && keystr[0]) {
 			if(type & 1) {
@@ -850,12 +857,12 @@ void tbox_drawelem_text(x, y, type)
 				glRecti(x2-len2-2,  y1+2,  x2-3,  y2-2);
 				ColorFunc(TBOXWHITE);
 				glRasterPos2i(x2-len2,  y1+tbfontyofs);
-				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS), 1);
+				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS));
 			}
 			else {
 				ColorFunc(TBOXBLACK);
 				glRasterPos2i(x2-len2,  y1+tbfontyofs);
-				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS), 0);
+				BIF_DrawString(G.font, keystr, (U.transopts & TR_MENUS));
 			}
 		}
 	}
@@ -1111,7 +1118,7 @@ short button(short *var, short min, short max, char *str)
 	if(mval[0]>G.curscreen->sizex) mval[0]= G.curscreen->sizex-10;
 	if(mval[1]>G.curscreen->sizey) mval[1]= G.curscreen->sizey-10;
 
-	block= uiNewBlock(&listb, "button", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
+	block= uiNewBlock(&listb, "button", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|
 				   UI_BLOCK_ENTER_OK);
 
@@ -1145,7 +1152,7 @@ short sbutton(char *var, float min, float max, char *str)
 	if(mval[0]>G.curscreen->sizex) mval[0]= G.curscreen->sizex-10;
 	if(mval[1]>G.curscreen->sizey) mval[1]= G.curscreen->sizey-10;
 
-	block= uiNewBlock(&listb, "button", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
+	block= uiNewBlock(&listb, "button", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
 
 	x1=mval[0]-150; 
@@ -1178,7 +1185,7 @@ short fbutton(float *var, float min, float max, char *str)
 	if(mval[0]>G.curscreen->sizex) mval[0]= G.curscreen->sizex-10;
 	if(mval[1]>G.curscreen->sizey) mval[1]= G.curscreen->sizey-10;
 
-	block= uiNewBlock(&listb, "button", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
+	block= uiNewBlock(&listb, "button", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
 
 	x1=mval[0]-150; 
@@ -1220,24 +1227,27 @@ int movetolayer_buts(unsigned int *lay)
 	x1= pivot[0]-sizex+10; 
 	y1= pivot[1]-sizey/2; 
 
-	block= uiNewBlock(&listb, "button", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
-	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_NUMSELECT|UI_BLOCK_RET_1);
+	block= uiNewBlock(&listb, "button", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_NUMSELECT|UI_BLOCK_ENTER_OK);
 	
 	dx= (sizex-5)/12;
 	dy= sizey/2;
 	
-	for(a=0; a<10; a++) {
+	uiBlockBeginAlign(block);
+	for(a=0; a<5; a++) 
 		uiDefButI(block, TOGR|BIT|a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
-		if(a==4) x1+= 5;
-	}
-	x1-= 5;
-
-	for(a=0; a<10; a++) {
+	for(a=0; a<5; a++) 
 		uiDefButI(block, TOGR|BIT|(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
-		if(a==4) x1+= 5;
-	}
-	x1-= 5;
+	x1+= 5;
 	
+	uiBlockBeginAlign(block);
+	for(a=5; a<10; a++) 
+		uiDefButI(block, TOGR|BIT|a, 0, "",(short)(x1+a*dx),(short)(y1+dy),(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+	for(a=5; a<10; a++) 
+		uiDefButI(block, TOGR|BIT|(a+10), 0, "",(short)(x1+a*dx),(short)y1,(short)dx,(short)dy, lay, 0, 0, 0, 0, "");
+	uiBlockEndAlign(block);
+
+	x1-= 5;
 	uiDefBut(block, BUT, 1, "OK", (short)(x1+10*dx+10), (short)y1, (short)(3*dx), (short)(2*dy), NULL, 0, 0, 0, 0, "");
 
 	uiBoundsBlock(block, 2);
@@ -1273,7 +1283,7 @@ void draw_numbuts_tip(char *str, int x1, int y1, int x2, int y2)
 	temp= 0;
 	while( BIF_GetStringWidth(G.fonts, str+temp, (U.transopts & TR_BUTTONS))>(x2 - x1-24)) temp++;
 	glRasterPos2i(x1+16, y2-30);
-	BIF_DrawString(G.fonts, str+temp, (U.transopts & TR_BUTTONS), 0);
+	BIF_DrawString(G.fonts, str+temp, (U.transopts & TR_BUTTONS));
 }
 
 int do_clever_numbuts(char *name, int tot, int winevent)
@@ -1303,7 +1313,7 @@ int do_clever_numbuts(char *name, int tot, int winevent)
 	x1= mval[0]-sizex/2; 
 	y2= mval[1]+sizey/2;
 	
-	block= uiNewBlock(&listb, "numbuts", UI_EMBOSSX, UI_HELV, G.curscreen->mainwin);
+	block= uiNewBlock(&listb, "numbuts", UI_EMBOSS, UI_HELV, G.curscreen->mainwin);
 	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1|UI_BLOCK_ENTER_OK);
 	
 	/* WATCH IT: TEX BUTTON EXCEPTION */
@@ -1447,12 +1457,13 @@ void clever_numbuts(void)
 		clever_numbuts_imasel();
 	}
 	else if(curarea->spacetype==SPACE_BUTS){
-		clever_numbuts_buts();
+		// clever_numbuts_buts();
 	}
 	else if(curarea->spacetype==SPACE_OOPS) {
 		clever_numbuts_oops();
 	}
 	else if(curarea->spacetype==SPACE_ACTION){
+		void stupid_damn_numbuts_action(void);	// editaction.c
 		stupid_damn_numbuts_action();
 	}
 	else if(curarea->spacetype==SPACE_FILE) {
@@ -1499,3 +1510,575 @@ void replace_names_but(void)
 	}
 	
 }
+
+
+/* ********************** NEW TOOLBOX ********************** */
+
+ListBase tb_listb= {NULL, NULL};
+
+#define TB_TAB	256
+#define TB_ALT	512
+#define TB_CTRL	1024
+#define TB_PAD	2048
+
+typedef struct TBitem {
+	int icon;
+	char *name;
+	int retval;
+	void *poin;
+} TBitem;
+
+static void tb_do_hotkey(void *arg, int event)
+{
+	unsigned short key=0, qual1=0, qual2=0;
+	
+	if(event & TB_CTRL) {
+		qual1= LEFTCTRLKEY;
+		event &= ~TB_CTRL;
+	}
+	if(event & TB_ALT) {
+		qual1= LEFTALTKEY;
+		event &= ~TB_ALT;
+	}
+	
+	if(event & TB_TAB) key= TABKEY;
+	else if(event & TB_PAD) {
+		event &= ~TB_PAD;
+		switch(event) {
+		case '0': key= PAD0; break;
+		case '5': key= PAD5; break;
+		case '/': key= PADSLASHKEY; break;
+		case '.': key= PADPERIOD; break;
+		case '*': key= PADASTERKEY; break;
+		case 'h': key= HOMEKEY; break;
+		case 'u': key= PAGEUPKEY; break;
+		case 'd': key= PAGEDOWNKEY; break;
+		}
+	}
+	else asciitoraw(event, &key, &qual2);
+
+	if(qual1) mainqenter(qual1, 1);
+	if(qual2) mainqenter(qual2, 1);
+	mainqenter(key, 1);
+	mainqenter(key, 0);
+	mainqenter(EXECUTE, 1);
+	if(qual1) mainqenter(qual1, 0);
+	if(qual2) mainqenter(qual2, 0);
+	
+}
+
+/* *************Select ********** */
+
+static TBitem tb_object_select[]= {
+{	0, "Border Select|B", 	'b', NULL},
+{	0, "(De)select All|A", 	'a', NULL},
+{	0, "Linked...|Shift L", 	'L', NULL},
+{	0, "Grouped...|Shift G", 	'G', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_mesh_select[]= {
+{	0, "Border Select|B", 	0, NULL},
+{	0, "SEPR", 				0, NULL},
+{	0, "(De)select All|A", 	2, NULL},
+{	0, "Inverse", 			3, NULL},
+{	0, "SEPR", 				0, NULL},
+{	0, "Face Loop|Shift R", 6, NULL},
+{	0, "Linked Vertices|Ctrl L", 4, NULL},
+{  -1, "", 			0, do_view3d_select_meshmenu}};
+
+
+static TBitem tb_curve_select[]= {
+{	0, "Border Select|B", 	0, NULL},
+{	0, "SEPR", 				0, NULL},
+{	0, "(De)select All|A", 	2, NULL},
+{	0, "Inverse", 			3, NULL},
+{	0, "Row", 			5, NULL},
+{  -1, "", 				0, do_view3d_select_curvemenu}};
+
+static TBitem tb__select[]= {
+{	0, "Border Select|B", 	'b', NULL},
+{	0, "(De)select All|A", 	'a', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+/* *************Edit ********** */
+
+static TBitem tb_edit[]= {
+{	0, "Exit Editmode|Tab", 	TB_TAB, NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+static TBitem tb_edit_hide[]= {
+{	0, "Show Hidden|Alt H", 		TB_ALT|'h', 		NULL},
+{	0, "Hide Selected|H", 			'h', 		NULL},
+{	0, "Hide Deselected|Shift H", 	'H', 		NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+static TBitem tb_curve_edit_seg[]= {
+{	0, "Subdivide", 		0, NULL},
+{	0, "Switch directoin", 	1, NULL},
+{  -1, "", 			0, do_view3d_edit_curve_segmentsmenu}};
+
+static TBitem tb_curve_edit_cv[]= {
+{	0, "Tilt|T", 	't', NULL},
+{	0, "Clear Tilt|Alt T", 			TB_ALT|'t', NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Automatic|Shift H", 		'H', NULL},
+{	0, "Toggle Free/Aligned|H", 	'h', NULL},
+{	0, "Vector|V", 					TB_ALT|'t', NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Make Vertex Parent|Ctrl P", TB_CTRL|'p', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_curve_edit[]= {
+{	0, "Exit Editmode|Tab", 	TB_TAB, NULL},
+{	0, "Extrude|E", 		'e', 		NULL},
+{	0, "Make Segment|F", 	'f', 		NULL},
+{	0, "Toggle Cyclic|F", 	'c', 		NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Control Points", 	0, 		tb_curve_edit_cv},
+{	0, "Segments", 	0, 		tb_curve_edit_seg},
+{	0, "SEPR", 								0, NULL},
+{	0, "Show/Hide", 	0, 		tb_curve_edit_seg},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_mesh_edit_vertex[]= {
+{	0, "Merge...|Alt M", 		5, NULL},
+{	0, "Split|Y", 				4, 		NULL},
+{	0, "Separate|P", 			3, 		NULL},
+{	0, "Smooth|Alt M", 			2, NULL},
+{	0, "Remove Doubles|Alt M", 			1, NULL},
+{	0, "Make Vertex Parent|Ctrl P", 	0, NULL},
+{  -1, "", 			0, do_view3d_edit_mesh_verticesmenu}};
+
+static TBitem tb_mesh_edit_edge[]= {
+{	0, "Make Edge/Face|F", 			5, 		NULL},
+{	0, "Loop Subdivide|Ctrl R", 		4, 		NULL},
+{	0, "Knife Subdivide...|Shift K", 	3, 		NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Subdivide", 			2, 		NULL},
+{	0, "Subdivide Fractal", 	1, 		NULL},
+{	0, "Subdivide Smooth", 		0, 		NULL},
+{  -1, "", 			0, do_view3d_edit_mesh_edgesmenu}};
+
+static TBitem tb_mesh_edit_face[]= {
+{	0, "Make Edge/Face|F", 			'f', 		NULL},
+{	0, "Fill|Shift F", 				'F', 		NULL},
+{	0, "Beaty Fill|Alt F", 			TB_ALT|'f', 		NULL},
+{	0, "Convert to Triangles|Ctrl T", 	TB_CTRL|'t', 		NULL},
+{	0, "Convert to Quads|Alt J", 		TB_ALT|'j', 		NULL},
+{	0, "Flip Triangle Edges|Ctrl F", 	TB_CTRL|'f', 		NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_mesh_edit_normal[]= {
+{	0, "Flip", 				1, 		NULL},
+{	0, "Recalc Inside", 	1, 		NULL},
+{	0, "Recalc Outside", 	1, 		NULL},
+{  -1, "", 			0, do_view3d_edit_mesh_normalsmenu}};
+
+
+static TBitem tb_mesh_edit[]= {
+{	0, "Exit Editmode|Tab", 	TB_TAB, NULL},
+{	0, "Undo|U", 			'u', 		NULL},
+{	0, "Redo|Shift U", 		'U', 		NULL},
+{	0, "Extrude|E", 	'e', 		NULL},
+{	0, "Vertices", 		0, 		tb_mesh_edit_vertex},
+{	0, "Edges", 		0, 		tb_mesh_edit_edge},
+{	0, "Faces", 		0, 		tb_mesh_edit_face},
+{	0, "Normals", 		0, 		tb_mesh_edit_normal},
+{	0, "SEPR", 			0, NULL},
+{	0, "Show/Hide", 	0, 		tb_edit_hide},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_object_ipo[]= {
+{	0, "Show/Hide", 	'k', NULL},
+{	0, "Select Next", 	TB_PAD|'u', NULL},
+{	0, "Select Prev", 	TB_PAD|'d', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_object_edit[]= {
+{	0, "Enter Editmode|Tab", 	TB_TAB, NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Insert Key...|I", 	'i', NULL},
+{	0, "Object Keys", 	0, tb_object_ipo},
+{	0, "SEPR", 								0, NULL},
+{	0, "Boolean...|W", 			'w', NULL},
+{	0, "Join Objects|Ctrl J", 	TB_CTRL|'j', NULL},
+{	0, "Convert Object...|Alt C", TB_ALT|'c', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+/* ************* Type  ********** */
+
+static void tb_do_mesh(void *arg, int event){
+	Mesh *me= get_mesh(OBACT);
+	switch(event) {
+	case 1: duplicate_context_selected(); break;
+	case 2: delete_context_selected(); break;
+	case 3: G.f ^= G_DRAWEDGES; break;
+	case 4: G.f ^= G_DRAWFACES; break;
+	case 5: G.f ^= G_DRAWNORMALS; break;
+	case 6: me->flag ^= ME_SUBSURF; makeDispList(OBACT); break;
+	case 7: me->flag ^= ME_OPT_EDGES; makeDispList(OBACT); break;
+	}
+	addqueue(curarea->win, REDRAW, 1);
+}
+
+static TBitem tb_mesh[]= {
+{	0, "Duplicate|Shift D", 		1, 		NULL},
+{	0, "Delete|X", 					2, 		NULL},
+{	0, "SEPR", 						0, NULL},
+{	0, "Show/Hide Edges", 			3, 		NULL},
+{	0, "Show/Hide Faces", 			4, 		NULL},
+{	0, "Show/Hide Normals", 		5, 		NULL},
+{	0, "SEPR", 						0, NULL},
+{	0, "Subdivision Surface", 		6, 		NULL},
+{	0, "Subd.Surf. Optimal", 		7, 		NULL},
+{  -1, "", 			0, tb_do_mesh}};
+
+static TBitem tb_obdata[]= {
+{	0, "Duplicate|Shift D", 		'D', 		NULL},
+{	0, "Delete|X", 					'x', 		NULL},
+
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_object[]= {
+{	0, "Duplicate|Shift D", 		'D', 		NULL},
+{	0, "Duplicate Linked|Alt D", 	TB_ALT|'D', NULL},
+{	0, "Delete|X", 					'x', 		NULL},
+{	0, "Copy Links...|Ctrl L", 		TB_CTRL|'l', NULL},
+{	0, "Make Single User...|U", 	'u', 		NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Make Parent|Ctrl P", 		TB_CTRL|'p', NULL},
+{	0, "Clear Parent|Alt P", 		TB_ALT|'p', NULL},
+{	0, "Make Track|Ctrl T", 		TB_CTRL|'t', NULL},
+{	0, "Clear Track|Alt T", 		TB_ALT|'t', NULL},
+{	0, "SEPR", 								0, NULL},
+{	0, "Copy Properties...|Ctrl C", TB_CTRL|'c', NULL},
+{	0, "Move to Layer...|M", 		'm', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+/* *************VIEW ********** */
+
+static void tb_do_view_dt(void *arg, int event){
+	G.vd->drawtype= event;
+	addqueue(curarea->win, REDRAW, 1);
+}
+
+static TBitem tb_view_dt[]= {
+{	ICON_BBOX, "Bounding box", 	1, NULL},
+{	ICON_WIRE, "Wireframe", 	2, NULL},
+{	ICON_SOLID, "Solid", 		3, NULL},
+{	ICON_SMOOTH, "Shaded", 		5, NULL},
+{	ICON_POTATO, "Textured", 	5, NULL},
+{  -1, "", 			0, tb_do_view_dt}};
+
+static TBitem tb_view[]= {
+{	0, "Viewport Shading", 			0, tb_view_dt},
+{	0, "SEPR", 						0, NULL},
+{	0, "Ortho/Persp|Pad 5", 	TB_PAD|'5', NULL},
+{	0, "Local View|Pad /", 	TB_PAD|'/', NULL},
+{	0, "Frame All|Home", 		TB_PAD|'h', NULL},
+{	0, "Frame Selected|Pad .", 	TB_PAD|'.', NULL},
+{	0, "Centre Cursor|C", 		'c', NULL},
+{	0, "SEPR", 		0, NULL},
+{	0, "Play Back |Alt A", TB_ALT|'a', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+/* *************TRANSFORM ********** */
+
+static TBitem tb_transform[]= {
+{	0, "Grabber|g", 	'g', NULL},
+{	0, "Rotate|r", 		'r', NULL},
+{	0, "Scale|s", 		's', NULL},
+{	0, "SEPR", 					0, NULL},
+{	ICON_MENU_PANEL, "Properties|n", 'n', NULL},
+{	0, "Snap...|Shift S", 		'S', NULL},
+{	0, "SEPR", 					0, NULL},
+{	0, "Clear Location", 		TB_ALT|'g', NULL},
+{	0, "Clear Rotation", 		TB_ALT|'r', NULL},
+{	0, "Clear Size", 			TB_ALT|'s', NULL},
+{	0, "Apply Rot/Size|Ctrl A", TB_CTRL|'a', NULL},
+{	0, "Apply Deform|Shift Ctrl A", 	TB_CTRL|'A', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+static TBitem tb_transform_editmode1[]= {
+{	0, "Grabber|g", 	'g', NULL},
+{	0, "Rotate|r", 		'r', NULL},
+{	0, "Scale|s", 		's', NULL},
+{	0, "SEPR", 					0, NULL},
+{	0, "Shrink/Fatten|Alt S", TB_ALT|'s', NULL},
+{	0, "Shear|Ctrl S", TB_CTRL|'s', NULL},
+{	0, "Warp|Shift W", 	'W', NULL},
+{	0, "SEPR", 					0, NULL},
+{	ICON_MENU_PANEL, "Properties|n", 'n', NULL},
+{	0, "Snap...|Shift S", 		'S', NULL},
+{	0, "Proportional Edit|O", 	'o', 		NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+static TBitem tb_transform_editmode2[]= {
+{	0, "Grabber|g", 	'g', NULL},
+{	0, "Rotate|r", 		'r', NULL},
+{	0, "Scale|s", 		's', NULL},
+{	0, "SEPR", 					0, NULL},
+{	ICON_MENU_PANEL, "Properties|n", 'n', NULL},
+{	0, "Snap...|Shift S", 		'S', NULL},
+{  -1, "", 			0, tb_do_hotkey}};
+
+
+/* *************ADD ********** */
+
+static TBitem addmenu_mesh[]= {
+{	0, "Plane", 	0, NULL},
+{	0, "Cube", 		1, NULL},
+{	0, "Circle", 	2, NULL},
+{	0, "UVsphere", 	3, NULL},
+{	0, "Icosphere", 4, NULL},
+{	0, "Cylinder", 	5, NULL},
+{	0, "Tube", 		6, NULL},
+{	0, "Cone", 		7, NULL},
+{	0, "Grid", 		8, NULL},
+{	0, "Monkey", 	9, NULL},
+{  -1, "", 			0, do_info_add_meshmenu}};
+
+static TBitem addmenu_curve[]= {
+{	0, "Bezier Curve", 	0, NULL},
+{	0, "Bezier Circle", 1, NULL},
+{	0, "NURBS Curve", 	2, NULL},
+{	0, "NURBS Circle", 	3, NULL},
+{	0, "Path", 			4, NULL},
+{  -1, "", 			0, do_info_add_curvemenu}};
+
+static TBitem addmenu_surf[]= {
+{	0, "NURBS Curve", 	0, NULL},
+{	0, "NURBS Circle", 	1, NULL},
+{	0, "NURBS Surface", 2, NULL},
+{	0, "NURBS Tube", 	3, NULL},
+{	0, "NURBS Sphere", 	4, NULL},
+{	0, "NURBS Donut", 	5, NULL},
+{  -1, "", 			0, do_info_add_surfacemenu}};
+
+static TBitem addmenu_meta[]= {
+{	0, "Meta Ball", 	0, NULL},
+{	0, "Meta Tube", 	1, NULL},
+{	0, "Meta Plane", 	2, NULL},
+{	0, "Meta Ellipsoid", 3, NULL},
+{	0, "Meta Cube", 	4, NULL},
+{  -1, "", 			0, do_info_add_metamenu}};
+
+static TBitem addmenu_armature[]= {
+{	0, "Bones", 	8, NULL},
+{  -1, "", 			0, do_info_addmenu}};
+
+
+static TBitem tb_add[]= {
+{	0, "Mesh", 		0, addmenu_mesh},
+{	0, "Curve", 	1, addmenu_curve},
+{	0, "Surface", 	2, addmenu_surf},
+{	0, "MBall", 	3, addmenu_meta},
+{	0, "Text", 		4, NULL},
+{	0, "Empty", 	5, NULL},
+{	0, "SEPR", 		0, NULL},
+{	0, "Camera", 	6, NULL},
+{	0, "Lamp", 		7, NULL},
+{	0, "SEPR", 		0, NULL},
+{	0, "Armature", 	8, NULL},
+{	0, "Lattice", 	9, NULL},
+{  -1, "", 			0, do_info_addmenu}};
+
+static TBitem tb_empty[]= {
+{	0, "Nothing...", 	0, NULL},
+{  -1, "", 		0, NULL}};
+
+
+
+static uiBlock *tb_makemenu(void *arg)
+{
+	static int counter=0;
+	TBitem *item= arg, *itemt;
+	uiBlock *block;
+	int yco= 0;
+	char str[10];
+	
+	if(arg==NULL) return NULL;
+	
+	sprintf(str, "tb %d\n", counter++);
+	block= uiNewBlock(&tb_listb, str, UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetCol(block, TH_MENU_ITEM);
+
+	// last item has do_menu func, has to be stored in each button
+	itemt= item;
+	while(itemt->icon != -1) itemt++;
+	uiBlockSetButmFunc(block, itemt->poin, NULL);
+
+	// now make the buttons
+	while(item->icon != -1) {
+
+		if(strcmp(item->name, "SEPR")==0) {
+			uiDefBut(block, SEPR, 0, "", 0, yco-=6, 50, 6, NULL, 0.0, 0.0, 0, 0, "");
+		}
+		else if(item->icon) {
+			uiDefIconTextBut(block, BUTM, 1, item->icon, item->name, 0, yco-=20, 50, 19, NULL, 0.0, 0.0, 0, item->retval, "");
+		}
+		else if(item->poin) {
+			uiDefIconTextBlockBut(block, tb_makemenu, item->poin, ICON_RIGHTARROW_THIN, item->name, 0, yco-=20, 50, 19, "");
+		}
+		else {
+			uiDefIconTextBut(block, BUTM, 1, ICON_BLANK1, item->name, 0, yco-=20, 50, 19, NULL, 0.0, 0.0, 0, item->retval, "");
+		}
+		item++;
+	}
+	uiTextBoundsBlock(block, 50);
+	
+	/* direction is also set in the function that calls this */
+	uiBlockSetDirection(block, UI_RIGHT|UI_CENTRE);
+
+	return block;
+}
+
+static int tb_mainx= 0, tb_mainy= -5;
+static void store_main(void *arg1, void *arg2)
+{
+	tb_mainx= (int)arg1;
+	tb_mainy= (int)arg2;
+}
+
+void toolbox_n(void)
+{
+	uiBlock *block;
+	uiBut *but;
+	TBitem *menu1=NULL, *menu2=NULL, *menu3=NULL; 
+	TBitem *menu4=NULL, *menu5=NULL, *menu6=NULL;
+	int dx;
+	short event, mval[2], tot=0;
+	char *str1=NULL, *str2=NULL, *str3=NULL, *str4=NULL, *str5=NULL, *str6=NULL;
+	
+	mywinset(G.curscreen->mainwin); // we go to screenspace
+	
+	block= uiNewBlock(&tb_listb, "toolbox", UI_EMBOSSP, UI_HELV, G.curscreen->mainwin);
+	uiBlockSetFlag(block, UI_BLOCK_LOOP|UI_BLOCK_REDRAW|UI_BLOCK_RET_1);
+	uiBlockSetCol(block, TH_MENU_ITEM);
+	
+	dx= 65;
+	
+	/* select context for main items */
+	if(curarea->spacetype==SPACE_VIEW3D) {
+		/* standard menu */
+		menu1= tb_object; str1= "Object";
+		menu2= tb_add; str2= "Add";
+		menu3= tb_object_select; str3= "Select";
+		menu4= tb_object_edit; str4= "Edit";
+		menu5= tb_transform; str5= "Transform";
+		menu6= tb_view; str6= "View";
+		
+		if(G.obedit) {
+			if(G.obedit->type==OB_MESH) {
+				menu1= tb_mesh; str1= "Mesh";
+				menu2= addmenu_mesh; 
+				menu3= tb_mesh_select;
+				menu4= tb_mesh_edit; 
+				menu5= tb_transform_editmode1;
+			}
+			else if(G.obedit->type==OB_CURVE) {
+				menu1= tb_obdata; str1= "Curve";
+				menu2= addmenu_curve;
+				menu3= tb_curve_select;
+				menu4= tb_curve_edit;
+				menu5= tb_transform_editmode1;
+			}
+			else if(G.obedit->type==OB_SURF) {
+				menu1= tb_obdata; str1= "Surface";
+				menu2= addmenu_surf; 
+				menu3= tb_curve_select;
+				menu4= tb_curve_edit;
+				menu5= tb_transform_editmode1;
+			}
+			else if(G.obedit->type==OB_MBALL) {
+				menu1= tb_obdata; str1= "Meta";
+				menu2= addmenu_meta;
+				menu3= tb__select;
+				menu4= tb_edit;
+				menu5= tb_transform_editmode2;
+			}
+			else if(G.obedit->type==OB_ARMATURE) {
+				menu1= tb_obdata;str1= "Armature";
+				menu2= addmenu_armature;
+				menu3= tb__select;
+				menu4= tb_edit;
+				menu5= tb_transform_editmode2;
+			}
+			else if(G.obedit->type==OB_LATTICE) {
+				menu1= tb_empty;str1= "Lattice";
+				menu2= tb_empty;
+				menu3= tb__select;
+				menu4= tb_edit;
+				menu5= tb_transform_editmode1;
+			}
+			
+		}
+		else {
+		}
+		tot= 6;
+	}
+	
+	getmouseco_sc(mval);
+	
+	/* create the main buttons menu */
+	if(tot==6) {
+	
+		/* check if it fits */
+		if(mval[0]-1.5*dx+tb_mainx < 6) mval[0]= 6 + 1.5*dx -tb_mainx;
+		else if(mval[0]+1.5*dx+tb_mainx > G.curscreen->sizex-6) 
+			mval[0]= G.curscreen->sizex-6-1.5*dx-tb_mainx;
+
+		if(mval[1]-20+tb_mainy < 6) mval[1]= 6+20 -tb_mainy;
+		else if(mval[1]+20+tb_mainy > G.curscreen->sizey-6) 
+			mval[1]= G.curscreen->sizey-6-20-tb_mainy;
+	
+		but=uiDefBlockBut(block, tb_makemenu, menu1, str1,	mval[0]-1.5*dx+tb_mainx,mval[1]+tb_mainy, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_TOP|UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)dx, (void *)-5);
+
+		but=uiDefBlockBut(block, tb_makemenu, menu2, str2,	mval[0]-0.5*dx+tb_mainx,mval[1]+tb_mainy, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_TOP);
+		uiButSetFunc(but, store_main, (void *)0, (void *)-5);
+
+		but=uiDefBlockBut(block, tb_makemenu, menu3, str3,	mval[0]+0.5*dx+tb_mainx,mval[1]+tb_mainy, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_TOP|UI_MAKE_LEFT);
+		uiButSetFunc(but, store_main, (void *)-dx, (void *)-5);
+
+		but=uiDefBlockBut(block, tb_makemenu, menu4, str4,	mval[0]-1.5*dx+tb_mainx,mval[1]+tb_mainy-20, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_DOWN|UI_MAKE_RIGHT);
+		uiButSetFunc(but, store_main, (void *)dx, (void *)5);
+
+		but=uiDefBlockBut(block, tb_makemenu, menu5, str5,	mval[0]-0.5*dx+tb_mainx,mval[1]+tb_mainy-20, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_DOWN);
+		uiButSetFunc(but, store_main, (void *)0, (void *)5);
+
+		but=uiDefBlockBut(block, tb_makemenu, menu6, str6,	mval[0]+0.5*dx+tb_mainx,mval[1]+tb_mainy-20, dx, 19, "");
+		uiButSetFlag(but, UI_MAKE_DOWN|UI_MAKE_LEFT);
+		uiButSetFunc(but, store_main, (void *)-dx, (void *)5);
+	}
+	
+	uiBoundsBlock(block, 2);
+	event= uiDoBlocks(&tb_listb, 0);
+	
+	mywinset(curarea->win);
+}
+
+void toolbox_n_add(void)
+{
+	tb_mainx= 0;
+	tb_mainy= -5;
+	toolbox_n();
+}
+
