@@ -1,5 +1,5 @@
 /**
- * $Id: view.c,v 1.6 2003/07/11 20:02:52 ton Exp $
+ * $Id: view.c,v 1.17 2003/11/19 22:00:14 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -48,7 +48,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 
 #include "DNA_object_types.h"
 #include "DNA_screen_types.h"
@@ -84,25 +83,6 @@
 
 #define TRACKBALLSIZE  (1.1)
 
-void persp3d(View3D *v3d, int a)
-{
-	ScrArea *area= v3d->area;
-	
-		/* only 3D windows */
-	if(a== 0) {
-		bwin_get_winmatrix(area->win, area->winmat);
-		glMatrixMode(GL_MODELVIEW);
-		bwin_ortho2(area->win, -0.5, (float)(area->winx)-.05, -0.5, (float)(area->winy)-0.5);
-		glLoadIdentity();
-	}
-	else if(a== 1) {
-		glMatrixMode(GL_PROJECTION);
-		bwin_load_winmatrix(area->win, area->winmat);
-		glMatrixMode(GL_MODELVIEW);
-		bwin_load_viewmatrix(area->win, v3d->viewmat);
-	}
-}
-
 void persp_general(int a)
 {
 	/* for all window types, not 3D */
@@ -113,7 +93,7 @@ void persp_general(int a)
 		glPushMatrix();
 		glMatrixMode(GL_MODELVIEW);
 
-		myortho2(-0.5, ((float)(curarea->winx))-0.5, -0.5, ((float)(curarea->winy))-0.5);
+		myortho2(-0.375, ((float)(curarea->winx))-0.375, -0.375, ((float)(curarea->winy))-0.375);
 		glLoadIdentity();
 	}
 	else if(a== 1) {
@@ -129,18 +109,23 @@ void persp(int a)
 	/* only 3D windows */
 
 	if(curarea->spacetype!=SPACE_VIEW3D) persp_general(a);
-	else if(a== 0) {
+	else if(a == PERSP_STORE) {		// only store
 		glMatrixMode(GL_PROJECTION);
-		mygetmatrix(curarea->winmat);
+		mygetmatrix(G.vd->winmat1);	
 		glMatrixMode(GL_MODELVIEW);
-		myortho2(-0.5, (float)(curarea->winx)-.05, -0.5, (float)(curarea->winy)-0.5);
+		mygetmatrix(G.vd->viewmat1); 
+	}
+	else if(a== PERSP_WIN) {		// only set
+		myortho2(-0.375, (float)(curarea->winx)-0.375, -0.375, (float)(curarea->winy)-0.375);
 		glLoadIdentity();
 	}
-	else if(a== 1) {
+	else if(a== PERSP_VIEW) {
 		glMatrixMode(GL_PROJECTION);
-		myloadmatrix(curarea->winmat);
-		glMatrixMode(GL_MODELVIEW);
-		myloadmatrix(G.vd->viewmat);
+		myloadmatrix(G.vd->winmat1); // put back
+		Mat4CpyMat4(curarea->winmat, G.vd->winmat1); // to be sure? 
+		glMatrixMode(GL_MODELVIEW); 
+		myloadmatrix(G.vd->viewmat); // put back
+		
 	}
 }
 
@@ -225,6 +210,22 @@ void project_short_noclip(float *vec, short *adr)
 	}
 }
 
+void project_float(float *vec, float *adr)
+{
+	float vec4[4];
+	
+	adr[0]= 3200.0;
+	VECCOPY(vec4, vec);
+	vec4[3]= 1.0;
+	
+	Mat4MulVec4fl(G.vd->persmat, vec4);
+
+	if( vec4[3]>0.1 ) {
+		adr[0]= (curarea->winx/2.0)+(curarea->winx/2.0)*vec4[0]/vec4[3];	
+		adr[1]= (curarea->winy/2.0)+(curarea->winy/2.0)*vec4[1]/vec4[3];
+	}
+}
+
 int boundbox_clip(float obmat[][4], BoundBox *bb)
 {
 	/* return 1: draw */
@@ -521,8 +522,10 @@ void viewmove(int mode)
 			scrarea_do_windraw(curarea);
 			screen_swapbuffers();
 		}
-		else BIF_wait_for_statechange();
-
+		else {
+			BIF_wait_for_statechange();
+		}
+		
 		/* this in the end, otherwise get_mbut does not work on a PC... */
 		if( !(get_mbut() & (L_MOUSE|M_MOUSE))) break;
 	}
@@ -709,6 +712,26 @@ void setviewmatrixview3d()
 	}
 }
 
+void setcameratoview3d()
+{
+	Object *ob;
+	float dvec[3];
+
+	ob= G.vd->camera;
+	dvec[0]= G.vd->dist*G.vd->viewinv[2][0];
+	dvec[1]= G.vd->dist*G.vd->viewinv[2][1];
+	dvec[2]= G.vd->dist*G.vd->viewinv[2][2];					
+	VECCOPY(ob->loc, dvec);
+	VecSubf(ob->loc, ob->loc, G.vd->ofs);
+	G.vd->viewquat[0]= -G.vd->viewquat[0];
+	if (ob->transflag & OB_QUAT) {
+		QUATCOPY(ob->quat, G.vd->viewquat);
+	} else {
+		QuatToEul(G.vd->viewquat, ob->rot);
+	}
+	G.vd->viewquat[0]= -G.vd->viewquat[0];
+}
+
 /* IGLuint-> GLuint*/
 short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short y2)
 {
@@ -731,6 +754,8 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 		rect.ymin= y1;
 		rect.ymax= y2;
 	}
+	/* get rid of overlay button matrix */
+	persp(PERSP_VIEW);
 	setwinmatrixview3d(&rect);
 	Mat4MulMat4(G.vd->persmat, G.vd->viewmat, curarea->winmat);
 	
@@ -775,6 +800,7 @@ short selectprojektie(unsigned int *buffer, short x1, short y1, short x2, short
 		G.zbuf= 0;
 		glDisable(GL_DEPTH_TEST);
 	}
+	persp(PERSP_WIN);
 
 	return hits;
 }
