@@ -1,5 +1,5 @@
 /**
- * $Id: vpaint.c,v 1.6 2003/07/05 14:24:05 ton Exp $
+ * $Id: vpaint.c,v 1.9 2003/12/02 18:39:32 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -50,7 +50,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_arithb.h"
-#include "BLI_editVert.h"
 #include "MTC_matrixops.h"
 
 #include "DNA_mesh_types.h"
@@ -454,7 +453,7 @@ void sample_vpaint()	/* frontbuf */
 	Gvp.b= cp[2];
 	Gvp.b /= 255.0;
 
-	allqueue(REDRAWBUTSGAME, 0);
+	allqueue(REDRAWBUTSEDIT, 0);
 }
 
 void init_vertexpaint()
@@ -799,6 +798,7 @@ void weight_paint(void)
 
 //	if(me->tface==NULL && me->mcol==NULL) return;
 	
+	persp(PERSP_VIEW);
 	/* imat for normals */
 	Mat4MulMat4(mat, ob->obmat, G.vd->viewmat);
 	Mat4Invert(imat, mat);
@@ -957,6 +957,7 @@ void vertex_paint()
 
 	if(me->tface==NULL && me->mcol==NULL) return;
 	
+	persp(PERSP_VIEW);
 	/* imat for normals */
 	Mat4MulMat4(mat, ob->obmat, G.vd->viewmat);
 	Mat4Invert(imat, mat);
@@ -1124,7 +1125,6 @@ void set_wpaint(void)		/* toggle */
 	
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
-	allqueue(REDRAWBUTSGAME, 0);
 	
 	if(G.f & G_WEIGHTPAINT) {
 		setcursor_space(SPACE_VIEW3D, CURSOR_VPAINT);
@@ -1168,7 +1168,6 @@ void set_vpaint(void)		/* toggle */
 	
 	allqueue(REDRAWVIEW3D, 0);
 	allqueue(REDRAWBUTSEDIT, 0);
-	allqueue(REDRAWBUTSGAME, 0);
 	
 	if(G.f & G_VERTEXPAINT) {
 		setcursor_space(SPACE_VIEW3D, CURSOR_VPAINT);
