@@ -1,5 +1,5 @@
 /* 
- * $Id: creator.c,v 1.21 2003/07/19 20:14:07 ton Exp $
+ * $Id: creator.c,v 1.30 2003/11/25 18:14:00 ton Exp $
  *
  * ***** BEGIN GPL/BL DUAL LICENSE BLOCK *****
  *
@@ -30,6 +30,7 @@
  * ***** END GPL/BL DUAL LICENSE BLOCK *****
  */
 #include <stdlib.h>
+#include <string.h>
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -51,8 +52,10 @@
 
 #include "BKE_utildefines.h"
 #include "BKE_blender.h"
+#include "BKE_font.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
+#include "BKE_packedFile.h"
 #include "BKE_scene.h"
 
 #include "BIF_gl.h"
@@ -69,18 +72,21 @@
 #include "BLO_writefile.h"
 #include "BLO_readfile.h"
 
-#include "BPY_extern.h"  // for init of blender python extension
-
 #include "BSE_headerbuttons.h" // for BIF_read_homefile
 
 #include "BDR_drawmesh.h"
 
+#include "IMB_imbuf.h"	// for quicktime_init
+
 #include "RE_renderconverter.h"
 
+#include "BPY_extern.h"
+
 #include "playanim_ext.h"
 #include "mydevice.h"
 #include "render.h"
 #include "nla.h"
+#include "datatoc.h"
 
 /* for passing information between creator and gameengine */
 #include "SYS_System.h"
@@ -88,22 +94,11 @@
 #include <signal.h>
 
 #ifdef __FreeBSD__
-  #ifndef __OpenBSD__
-    #include <floatingpoint.h>
-    #include <sys/rtprio.h>
-  #endif
+# include <sys/types.h>
+# include <floatingpoint.h>
+# include <sys/rtprio.h>
 #endif
 
-#ifdef WITH_QUICKTIME
-#ifdef _WIN32
-#include <QTML.h>
-#include <Movies.h>
-#elif defined(__APPLE__)
-#undef NDEBUG
-#include <QuickTime/Movies.h>
-#endif /* __APPLE__ */
-#endif /* WITH_QUICKTIME */
-
 // from buildinfo.c
 extern char * build_date;
 extern char * build_time;
@@ -213,7 +208,8 @@ int main(int argc, char **argv)
 		
 		argc= 1;
 		
-		setprefsize(100, 100, 800, 600);
+		/* 40 + 684 + (headers) 22 + 22 = 768, the powerbook screen height */
+		setprefsize(120, 40, 850, 684);
 		
 		winlay_get_screensize(&scr_x, &scr_y);
 		winlay_process_events(0);
@@ -319,8 +315,10 @@ int main(int argc, char **argv)
 	/* for all platforms, even windos has it! */
 	if(G.background) signal(SIGINT, blender_esc);	/* ctrl c out bg render */
 
-
-	RE_init_render_data();	/* must be called here because R.winpos from default file */
+		/* background render uses this font too */
+	BKE_font_register_builtin(datatoc_Bfont, datatoc_Bfont_size);
+		/* must be called here because R.winpos from default file */
+	RE_init_render_data();
 	
 	if(G.background==0) {
 		for(a=1; a<argc; a++) {
@@ -431,27 +429,12 @@ int main(int argc, char **argv)
 	RE_init_filt_mask();
 	
 #ifdef WITH_QUICKTIME
-#ifdef _WIN32
-        if (InitializeQTML(0) != noErr)
-            G.have_quicktime = FALSE;
-        else
-            G.have_quicktime = TRUE;
-#endif /* _WIN32 */
 
-        /* Initialize QuickTime */
-#if defined(_WIN32) || defined (__APPLE__)
-        if (EnterMovies() != noErr)
-            G.have_quicktime = FALSE;
-        else
-#endif /* _WIN32 || __APPLE__ */
-#ifdef __linux__
-			/* inititalize quicktime codec registry */
-			lqt_registry_init();
-#endif
-			G.have_quicktime = TRUE;
+	quicktime_init();
+
 #endif /* WITH_QUICKTIME */
-
-		/* OK we are ready for it */
+		
+	/* OK we are ready for it */
 
 	for(a=1; a<argc; a++) {
 		if (G.afbreek==1) break;
